import { ApplicationsModule } from './../manage-applications/applications.module';
import { DynamicReportModule } from './../reports/dynamic-report/dynamic-report.module';
import { Module } from '@nestjs/common';
import { AppController } from './app.controller';
import { AppService } from './app.service';
import { LoginModule } from '../iam/authentication/login.module';
import { APP_GUARD, Reflector } from '@nestjs/core';
import { JwtAuthGuard } from '../iam/jwt-auth-guard/jwt-auth-guard';
import { JwtAuthGuardModule } from '../iam/jwt-auth-guard/jwt-auth-guard.module';
import { ConfigModule } from '@nestjs/config';
import {
  DatabaseRegistry,
  DrivineModule,
  DrivineModuleOptions,
} from '@liberation-data/drivine';
import { BootstrapApplicationModule } from '../bootstrap-application/boodstrap.module';

@Module({
  imports: [
    ConfigModule.forRoot(),
    DrivineModule.withOptions(<DrivineModuleOptions>{
      connectionProviders: [DatabaseRegistry.buildOrResolveFromEnv()],
    }),
    LoginModule,
    JwtAuthGuardModule,
    DynamicReportModule,
    // ApplicationsModule,
    BootstrapApplicationModule,
  ],
  controllers: [AppController],
  providers: [
    AppService,
    {
      provide: APP_GUARD,
      useFactory: (ref) => new JwtAuthGuard(ref),
      inject: [Reflector],
    },
  ],
})
export class AppModule {}
