import {
  ArgumentsHost,
  Catch,
  ExceptionFilter,
  HttpException,
  Logger,
} from '@nestjs/common';
import { Response } from 'express';

/**
 * To catch http exception from all modules
 */
@Catch(HttpException)
export class HttpExceptionFilter implements ExceptionFilter {
  private readonly logger = new Logger(HttpExceptionFilter.name);

  constructor() {}

  catch(exception: HttpException, host: ArgumentsHost) {
    const httpResponse = host.switchToHttp().getResponse<Response>();
    const status = exception.getStatus();
    const response = JSON.stringify(exception.getResponse());
    const responseJSON = JSON.parse(response);

    const errorObject = {
      message: responseJSON.message,
      error: responseJSON.error,
    };

    let errorResponse = httpResponse.status(status).send(errorObject);

    return errorResponse;
  }
}

/**
 * @summary
 * Error object for all modules
 */
interface Error {
  response: any;
}

//Error Message Format

export const errorMessage = (error: Error) => {
  const errorObject = {
    statusCode: Number,
    message: String,
    error: String,
  };

  errorObject['statusCode'] = error.response.status
    ? error.response.status
    : 500;
  errorObject['message'] = error.response.data.error_description
    ? error.response.data.error_description
    : 'Internal Server Error';
  errorObject['error'] = error.response.data.error
    ? error.response.data.error
    : 'Internal Server Error';

  return errorObject;
};
