import { Body, Controller, HttpStatus, Logger, Post } from '@nestjs/common';
import { ApiOperation, ApiTags } from '@nestjs/swagger';
import { ILogin } from './login.controller.interface';
import { LoginService } from './login.service';
import { LoginMockService } from './login.mock.service';
import { UnProtected } from '../jwt-auth-guard/jwt-auth-guard';
import { LoginResponse } from '../../shared entities/dto/login-response.dto';
import { LoginRequest } from '../../shared entities/dto/login-request.dto';
import { AuthenticateUser } from '../entity/authentication-user.entity';

/**
 *
 */
@ApiTags('Login')
@Controller()
export class LoginController implements ILogin {
  private readonly logger = new Logger(LoginController.name);
  private readonly loginService: LoginService;
  private readonly loginMockService: LoginMockService;

  constructor(
    private readonly _loginService: LoginService,
    private readonly _loginMockService: LoginMockService
  ) {
    this.loginService = _loginService;
    this.loginMockService = _loginMockService;
  }

  /**
   *
   * @param request
   */
  @ApiOperation({
    summary: 'Create user sessions',
  })
  @Post('/v1/login')
  @UnProtected()
  async login(@Body() request: LoginRequest): Promise<LoginResponse> {
    const username = request.username;
    const password = request.password;
    let authenticateUser: AuthenticateUser;

    if (request.useMockData) {
      authenticateUser = await this.loginMockService.authenticateUser(
        username,
        password
      );
    } else {
      authenticateUser = await this.loginService.authenticateUser(
        username,
        password
      );
    }

    let response = new LoginResponse();
    response.user = authenticateUser;

    return response;
  }
}
