import { ILoginRequest } from './login-request.interface.dto';
import { ApiProperty } from '@nestjs/swagger';
import { IsEmail, IsNotEmpty } from 'class-validator';

/**
 *
 */
export class LoginRequest implements ILoginRequest {
  /**
   * Email of the user
   *
   * @example
   * johnmatthew@gmail.com
   *
   */
  @ApiProperty({
    required: true,
    example: 'johnmatthew@gmail.com',
    description: 'Email of the user',
    default: '',
  })
  email: string;

  /**
   * Username of the user
   *
   * @example
   * john.matthew
   */
  @ApiProperty({
    required: true,
    example: 'john.matthew',
    description: 'Username of the user',
    default: '',
  })
  @IsNotEmpty()
  username: string;

  /**
   * Password of the user
   *
   * @example
   * secret
   */
  @ApiProperty({
    required: true,
    example: 'secret',
    description: 'Username of the user',
    default: '',
  })
  @IsNotEmpty()
  password: string;

 /**
   * Default parameter to identify service
   *
   * @example
   * true/false
   */
  @ApiProperty({
    required: false,
    example: 'true',
    description: 'Mock data field to identify mock service',
    default: '',
  })
  useMockData:boolean;
}
