import { Logger, ValidationPipe } from '@nestjs/common';
import { NestFactory } from '@nestjs/core';
import { AppModule } from './app/app.module';
import { DocumentBuilder, SwaggerModule } from '@nestjs/swagger';
import { HttpExceptionFilter } from './exception.filter';

async function bootstrap() {
  const app = await NestFactory.create(AppModule);

  //Enable Http Exception Filter
  app.useGlobalFilters(new HttpExceptionFilter());

  //Enable CORS
  app.enableCors();

  //Set Global Prefix
  const globalPrefix = '/api';
  app.setGlobalPrefix(globalPrefix);

  //Swagger Configuration
  const config = new DocumentBuilder()
    .setTitle('Hash8 Platform API 2.0')
    .setDescription(
      'Enables developers to build, launch and manage IoT solutions.'
    )
    .setVersion('1.0')
    .build();
  const document = SwaggerModule.createDocument(app, config);

  //Configure Swagger URL
  SwaggerModule.setup('/api/docs', app, document);

  //Use Validators
  app.useGlobalPipes(
    new ValidationPipe({
      // disableErrorMessages: true,
      // whitelist: true
      forbidNonWhitelisted: true,
    })
  );

  const port = process.env.PORT || 42748;
  await app.listen(port);
  Logger.log(
    `🚀 Application is running on: http://localhost:${port}/${globalPrefix}`
  );
}

bootstrap();
