import {
  Controller,
  Get,
  Post,
  Body,
  Patch,
  Param,
  Delete,
  Query,
  Logger,
} from '@nestjs/common';
import { ApiOperation, ApiTags } from '@nestjs/swagger';
import { v4 as uuidv4 } from 'uuid';
import { ViewEntityResponseDto } from '../shared/view-entity.res.dto';
import { ApplicationsService } from './applications.service';
import { CreateApplicationRequestDto } from './dto/create-application.req.dto';
import { CreateApplicationResponsetDto } from './dto/create-application.res.dto';
import { UpdateApplicationDto } from './dto/update-application.dto';
import { ViewApplicationsRequestDto } from './dto/view-applications.req.dto';
import { ApplicationEnum } from './entity/application.enum';

@ApiTags('applications')
@Controller('applications')
// @UseFilters(new HttpExceptionFilter())
export class ApplicationsController {
  private readonly logger = new Logger(ApplicationsController.name);
  constructor(private readonly service: ApplicationsService) {}

  @ApiOperation({ summary: 'Create a new application' })
  @Post()
  async create(
    @Body() createRequestDto: CreateApplicationRequestDto
  ): Promise<object> {
    //Initialize Request DTP
    createRequestDto.uuid = uuidv4();
    createRequestDto.state = ApplicationEnum.ENABLED;
    createRequestDto.createdTime = new Date().toISOString();
    createRequestDto.modifiedTime = new Date().toISOString();

    //Create Tenant Object in database
    let response = await this.service.create(createRequestDto);

    //Convert JSON objects to Tenant Entities
    // const tenants: Tenant[] = tenantObjects.map((tenant: Object) => {
    //   return new Tenant(tenant);
    // });

    //Create Response DTO
    let createResponseDto: CreateApplicationResponsetDto =
      new CreateApplicationResponsetDto(createRequestDto, response);

    return createResponseDto;
  }

  @ApiOperation({ summary: 'Read all applications' })
  @Get()
  async findAll(
    @Query() viewRequestDto: ViewApplicationsRequestDto
  ): Promise<object> {
    //Get Tenant Objects from database, result will be a JSON
    let objects = await this.service.findAll(viewRequestDto);

    //Create Response DTO
    let viewResponseDto: ViewEntityResponseDto = new ViewEntityResponseDto(
      viewRequestDto,
      objects
    );

    return viewResponseDto;
  }

  @ApiOperation({ summary: 'Read a specific application' })
  @Get(':id')
  findOne(@Param('id') id: string) {
    return this.service.findOne(+id);
  }

  @Patch(':id')
  update(
    @Param('id') id: string,
    @Body() updateTenantDto: UpdateApplicationDto
  ) {
    return this.service.update(+id, updateTenantDto);
  }

  @Delete(':id')
  remove(@Param('id') id: string) {
    return this.service.remove(+id);
  }
}
