import {
  CypherStatement,
  InjectCypher,
  InjectPersistenceManager,
  PersistenceManager,
  QuerySpecification,
  Transactional,
} from '@liberation-data/drivine';
import {
  Injectable,
  InternalServerErrorException,
  Logger,
} from '@nestjs/common';
import { CreateApplicationRequestDto } from './dto/create-application.req.dto';
import { UpdateApplicationDto } from './dto/update-application.dto';
import { ViewApplicationsRequestDto } from './dto/view-applications.req.dto';

@Injectable()
export class ApplicationsService {
  //Initialize Logged
  private readonly logger = new Logger(ApplicationsService.name);

  //Initialize Neo4J Persistence Manager
  persistenceManager: PersistenceManager;

  //Initialize Cypher Queries
  viewQuery: CypherStatement;
  createQuery: CypherStatement;

  //Inject Persistence Manager when constructor is called
  //Inject Cypher Query when the constructor is called
  constructor(
    @InjectPersistenceManager() _persistenceManager: PersistenceManager,
    @InjectCypher(__dirname, '../../neo4j/applications/view-applications')
    _viewQuery: CypherStatement,
    @InjectCypher(__dirname, '../../neo4j/applications/create-applications')
    _createQuery: CypherStatement
  ) {
    this.persistenceManager = _persistenceManager;
    this.viewQuery = _viewQuery;
    this.createQuery = _createQuery;
  }

  @Transactional()
  async create(createRequestDto: CreateApplicationRequestDto): Promise<any> {
    const createQuery = new QuerySpecification()
      .withStatement(this.createQuery)
      .bind(createRequestDto);
    try {
      let response = await this.persistenceManager.query(createQuery);
      this.logger.debug(`Created ${JSON.stringify(response)}`);
      return response;
    } catch (error) {
      this.logger.error(`Neo4J Database Error ${error}`);
      throw new InternalServerErrorException();
    }
  }

  @Transactional()
  async findAll(viewRequestDto: ViewApplicationsRequestDto): Promise<any> {
    const viewAllQuery = new QuerySpecification()
      .withStatement(this.viewQuery)
      .bind(viewRequestDto);
    try {
      let objects = await this.persistenceManager.query(viewAllQuery);
      this.logger.debug(JSON.stringify(objects));
      return objects;
    } catch (error) {
      this.logger.error(`Neo4J Database Error ${error}`);
      throw new InternalServerErrorException();
    }
  }

  findOne(id: number) {
    return `This action returns a #${id} tenant`;
  }

  update(id: number, _updateTenantDto: UpdateApplicationDto) {
    return `This action updates a #${id} tenant`;
  }

  remove(id: number) {
    return `This action removes a #${id} tenant`;
  }
}
