import { ApiProperty } from '@nestjs/swagger';
import { IsEnum, IsNotEmpty } from 'class-validator';
import { CreateEntityRequestDto } from '../../shared/create-entity.req.dto';
import { ApplicationEnum } from '../entity/application.enum';

/**
 * Create Tenant DTO
 */
export class CreateApplicationRequestDto extends CreateEntityRequestDto {
  /**
   * Name of the application, should be unique
   *
   * @example
   * Fleet Management Application
   */
  @ApiProperty({
    required: true,
    description: 'Name of the application, should be unique',
    default: 'Fleet Management Application',
  })
  @IsNotEmpty()
  applicationName: string;

  /**
   * State of the application
   *
   * @examples
   * ENABLED
   * DISABLED
   */
  @ApiProperty({
    required: true,
    description: 'State of the application',
    default: 'ENABLED',
  })
  @IsEnum(ApplicationEnum)
  state: ApplicationEnum;
}
