import { ApiProperty } from '@nestjs/swagger';
import { Entity } from '../../shared entities/entity.entity';
import { ApplicationEnum } from './application.enum';

/**
 * Application refers to the IoT solution.
 *
 * A single tenant can host multiple applications or can choose to host one application.

 */
export class Application extends Entity {
  /**
   * Name of the application, should be unique
   *
   * @example
   * Fleet Management Application
   */
  @ApiProperty({
    required: false,
    description: 'Name of the application, should be unique',
    default: '',
  })
  applicationName: string;

  /**
   * State of the application
   *
   * @examples
   * ENABLED
   * DISABLED
   */
  @ApiProperty({
    required: false,
    description: 'State of the application',
    default: '',
  })
  state: ApplicationEnum;

  applicationCode: string;
  applicationDescription: string;
  tenantUUID: string;

  constructor(partial: Partial<Application>) {
    super(partial);
    Object.assign(this, partial);
  }
}
