import { ApiProperty } from '@nestjs/swagger';

/**
 * DTO Entity for API's to call
 */
export class CreateEntityRequestDto {
  /**
   * Version 4 UUID
   *
   * Will be the key identifier for the entity in the database
   *
   * @example
   * 7233a760-519b-4fe8-af4a-1d64fba9bf14
   */
  @ApiProperty({
    required: false,
    description: 'Version 4 UUID',
    default: '',
  })
  uuid: string;

  /**
   * Used to track when this object was created
   *
   * @example
   * let date = new Date();
   * Tue Nov 05 1985 06:23:20 GMT+0530 (IST)
   */
  @ApiProperty({
    required: false,
    description: 'Used to track when this object was created',
    default: '',
  })
  createdTime: string;

  /**
   * Used to track when this object was modified
   *
   * @example
   * let date = new Date();
   * Tue Nov 05 1985 06:23:20 GMT+0530 (IST)
   */
  @ApiProperty({
    required: false,
    description: 'Used to track when this object was modified',
    default: '',
  })
  modifiedTime: string;

  /**
   * Used to track who created this object
   *
   * @example
   * System or John Matthew
   */
  @ApiProperty({
    required: false,
    description: 'Used to track who created this object',
    default: '',
  })
  createdBy: string;

  /**
   * Used to track who modified this object
   *
   * @example
   * System or John Matthew
   */
  @ApiProperty({
    required: false,
    description: 'Used to track who modified this object',
    default: '',
  })
  modifiedBy: string;
}
