import { ApiProperty } from '@nestjs/swagger';
import { CreateEventRequestDto } from './create-event.req.dto';

/**
 * DTO Event for API's to call
 */
export class CreateEventResponseDto {
  /**
   * Version 4 UUID
   *
   * Will be the key identifier for the entity in the database
   *
   * @example
   * 7233a760-519b-4fe8-af4a-1d64fba9bf14
   */
  @ApiProperty({
    required: false,
    description: 'Version 4 UUID',
    default: '',
  })
  uuid: string;

  /**
   * Used to track when this event was created
   *
   * @example
   * let date = new Date();
   * Tue Nov 05 1985 06:23:20 GMT+0530 (IST)
   */
  @ApiProperty({
    required: false,
    description: 'Used to track when this event was created',
    default: '',
  })
  eventTime: string;

  /**
   * Event Code
   *
   * @example
   * EV-121611
   */
  @ApiProperty({
    required: false,
    description: 'Event Code',
    default: 'EV-121611',
  })
  eventCode: string;

  /**
   * Event Description
   *
   * @example
   * Battery Low
   */
  @ApiProperty({
    required: false,
    description: 'Event Description',
    default: 'Battery Low',
  })
  eventDescription: string;

  /**
   * Payload that contains event details
   *
   * @example
   * System or John Matthew
   */
  @ApiProperty({
    required: false,
    description: 'Payload that contains event details',
    default: '',
  })
  payload: object;

  //TODO
  data: Event[];

  constructor(_request: CreateEventRequestDto, event: any) {
    this.data = event;
  }
}
