import { ApiProperty } from '@nestjs/swagger';

/**
 * All events will inherit this Event Object
 */
export abstract class Event {
  constructor(partial: Partial<Event>) {
    Object.assign(this, partial);
  }

  /**
   * Version 4 UUID
   *
   * Will be the key identifier for the event in the database
   *
   * @example
   * 7233a760-519b-4fe8-af4a-1d64fba9bf14
   */
  @ApiProperty({
    required: false,
    description: 'Version 4 UUID',
    default: '',
  })
  uuid: string;

  /**
   * Event code that identifies the event
   *
   * Will be the key identifier for the event
   *
   * @example
   * EV-016192
   */
  @ApiProperty({
    required: true,
    description: 'Event Code',
    default: '',
  })
  eventCode: string;

  /**
   * Event description
   *
   * @example
   * Battery Low
   */
  @ApiProperty({
    required: false,
    description: 'Event Description',
    default: 'Battery Low',
  })
  eventDescription: string;

  /**
   * Time of the event
   *
   * @example
   * new Date().toISOString()
   */
  @ApiProperty({
    required: false,
    description: 'Event Time',
  })
  eventTime: string;

  /**
   * Payload data with the details of the event
   *
   * @example
   * new Date().toISOString()
   */
  @ApiProperty({
    required: false,
    description: 'Event Payload',
  })
  eventPayload: object;
}
