import { Type } from 'class-transformer';
import { IsNumber, Max, Min } from 'class-validator';
import { Entity } from './entity.entity';
import { ViewEntityRequestDto } from './view-entity.req.dto';

/**
 * Basic View Report DTO that will be inherited by individual entities
 *
 */
export class ViewEntityResponseDto {
  /**
   * Search Query
   *
   * Will search the entity for given search term
   *
   * @example
   * bluebird
   */
  q: string;

  /**
   * Number of records to rtrieve from the database
   *
   * @example
   * 20
   */
  @Type(() => Number)
  @IsNumber()
  @Min(1)
  @Max(500)
  limit: number;

  /**
   * Number of records to skip as part of pagination
   *
   * If you want to skip 2 pages and view 3rd page, and if the limit is 20, then set offset to 40.
   * Default will always be 0.
   *
   * @example
   * 40
   */
  @Type(() => Number)
  @IsNumber()
  @Min(0)
  @Max(10000)
  offset: number;

  /**
   * Sort column and sort order
   *
   * Specify the column to be sorted along with ascending or descending
   *
   * @example
   * modifiedTime desc
   */
  sort: string;

  /**
   * Total number of records returned
   *
   * @example
   * 26
   */
  @Type(() => Number)
  @IsNumber()
  totalRecords: number;

  //TODO
  data: Entity[];

  constructor(request: ViewEntityRequestDto, entity: any) {
    this.q = request.q;
    this.limit = request.limit;
    this.offset = request.offset;
    this.sort = request.sort;
    this.data = entity;
  }
}
