/* eslint-disable @angular-eslint/use-lifecycle-interface */
import { Component, isDevMode } from '@angular/core';
import { BreakpointService } from '@hash8io/core-web';
import { AppStoreActions, AppStoreSelectors } from '@hash8io/core-web';
import { Store } from '@ngrx/store';
import { ColDef } from 'ag-grid-community';
import { Observable } from 'rxjs';
import { WelcomeComponent } from './welcome.component';
import packageJson from './../../../../package.json';
import { STORE_MOCK_OBJ } from './app-store-data/store-data';
import { REGISTRATION_FORM } from '@hash8io/registration-web';
import { environment } from './../environments/environment';
import { TemplateService } from '@hash8io/events-web';

@Component({
  selector: 'hash8-app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss'],
})
export class AppComponent {
  layout = {
    lg: {
      areas:
        'header header | toolbar toolbar | sidebar banner  | sidebar content | sidebar footer',
      gap: '0px',
      rows: '52px 42px auto 1fr 36px',
      columns: '240px 1fr',
      alignRows: 'start stretch',
      alignColumns: 'start stretch',
    },
    md: {
      areas:
        'header header | toolbar toolbar | banner banner | sidebar sidebar | content content | footer footer',
      gap: '0px',
      rows: '52px 42px auto 0 1fr 36px',
      columns: '240px 1fr',
      alignRows: 'start stretch',
      alignColumns: 'start stretch',
    },
    sm: {
      areas:
        'header header | toolbar toolbar | banner banner | sidebar sidebar | content content | footer footer',
      gap: '0px',
      rows: '52px 42px auto 0 1fr 36px',
      columns: '240px 1fr',
      alignRows: 'start stretch',
      alignColumns: 'start stretch',
    },
    xs: {
      areas:
        'header header | toolbar toolbar | banner banner | sidebar sidebar | content content | footer footer',
      gap: '0px',
      rows: '52px 42px auto 0 1fr 36px',
      columns: '240px 1fr',
      alignRows: 'start stretch',
      alignColumns: 'start stretch',
    },
  };

  columnDefs: ColDef[] = [];
  rowData = [];

  welcomeComponent = WelcomeComponent;
  preferences!: Observable<any>;
  branding!: Observable<any>;
  devices!: Observable<any>;
  RouterRef = 'router';
  HttpRef = 'http';
  constructor(
    private breakpointService: BreakpointService,
    private store: Store,
    private templateService: TemplateService
  ) {}

  steps = {
    steps: [
      {
        label: 'Organization',
        fields: [
          {
            fieldGroup: [
              {
                key: 'organizationName',
                type: '#text',
                props: {
                  label: 'Organization Name',
                  required: true,
                  hideLabel: true,
                  // align: 'row',
                },
                extras: {
                  labelOverInput: true,
                },
              },
              {
                key: 'address01',
                type: '#text',
                props: {
                  label: 'Address 01',
                  required: false,
                  hideLabel: true,
                  // align: 'row',
                },
                extras: {
                  labelOverInput: true,
                },
              },
              {
                key: 'country',
                type: '#text',
                props: {
                  label: 'Country',
                  hideLabel: true,
                  required: false,
                  // align: 'row',
                },
                extras: {
                  labelOverInput: true,
                },
              },
              {
                key: 'state',
                type: '#text',
                props: {
                  label: 'State',
                  required: false,
                  // async: true,
                  // labelProp: 'country',
                  // valueProp: 'id',
                  // apiEndpoint:
                  //   'https://63239166bb2321cba91c006d.mockapi.io/country',
                  hideLabel: true,
                  // align: 'row',
                },
                extras: {
                  labelOverInput: true,
                },
              },
              {
                key: 'city',
                type: '#text',
                props: {
                  label: 'City',
                  required: false,
                  // async: true,
                  // labelProp: 'state',
                  // valueProp: 'id',
                  // apiEndpoint:
                  //   'https://63239166bb2321cba91c006d.mockapi.io/country',
                  hideLabel: true,
                  // align: 'row',
                },
                extras: {
                  labelOverInput: true,
                },
              },

              {
                key: 'zipCode',
                type: '#number',
                props: {
                  label: 'Zip Code',
                  mask: '0{5}',
                  hideLabel: true,
                  // align: 'row',
                },
                extras: {
                  labelOverInput: true,
                },
              },
            ],
          },
        ],
      },
      {
        label: 'Registration',
        fields: [
          {
            fieldGroup: [
              {
                key: 'organizationName',
                type: '#text',
                props: {
                  label: 'Organization Name',
                  required: true,
                  hideLabel: true,
                  // align: 'row',
                },
                extras: {
                  labelOverInput: true,
                },
              },
              {
                key: 'address01',
                type: '#text',
                props: {
                  label: 'Address 01',
                  required: false,
                  hideLabel: true,
                  // align: 'row',
                },
                extras: {
                  labelOverInput: true,
                },
              },
              {
                key: 'country',
                type: '#text',
                props: {
                  label: 'Country',
                  hideLabel: true,
                  required: false,
                  // align: 'row',
                },
                extras: {
                  labelOverInput: true,
                },
              },
              {
                key: 'state',
                type: '#text',
                props: {
                  label: 'State',
                  required: false,
                  // async: true,
                  // labelProp: 'country',
                  // valueProp: 'id',
                  // apiEndpoint:
                  //   'https://63239166bb2321cba91c006d.mockapi.io/country',
                  hideLabel: true,
                  // align: 'row',
                },
                extras: {
                  labelOverInput: true,
                },
              },
              {
                key: 'city',
                type: '#text',
                props: {
                  label: 'City',
                  required: false,
                  // async: true,
                  // labelProp: 'state',
                  // valueProp: 'id',
                  // apiEndpoint:
                  //   'https://63239166bb2321cba91c006d.mockapi.io/country',
                  hideLabel: true,
                  // align: 'row',
                },
                extras: {
                  labelOverInput: true,
                },
              },

              {
                key: 'zipCode',
                type: '#number',
                props: {
                  label: 'Zip Code',
                  mask: '0{5}',
                  hideLabel: true,
                  // align: 'row',
                },
                extras: {
                  labelOverInput: true,
                },
              },
            ],
          },
        ],
      },
    ],
    options: {
      submitLabel: 'Sign up',
    },
  };

  register = [
    {
      label: 'Personal data',
      fields: [
        {
          key: 'firstname',
          type: 'input',
          templateOptions: {
            label: 'First name',
            required: true,
          },
        },
        {
          key: 'age',
          type: 'input',
          templateOptions: {
            type: 'number',
            label: 'Age',
            required: true,
          },
        },
      ],
    },
    {
      label: 'Destination',
      fields: [
        {
          key: 'country',
          type: 'input',
          templateOptions: {
            label: 'Country',
            required: true,
          },
        },
      ],
    },
    {
      label: 'Day of the trip',
      fields: [
        {
          key: 'day',
          type: 'input',
          templateOptions: {
            type: 'date',
            label: 'Day of the trip',
            required: true,
          },
        },
      ],
    },
  ];

  ngOnInit() {
    const tpl = this.templateService.processTemplate(
      {
        name: 'My name is {{data.name}}',
        address: '{%data.address%}',
      },
      {
        name: 'John',
        address: {
          city: 'New York',
          state: 'New York',
          country: 'USA',
        },
      }
    );
    // console.log('%capp.component.ts line:332 object', 'color: #007acc;', tpl);

    //Set Application Version
    // this.store.dispatch(
    //   AppStoreActions.SET_APP_VERSION({ version: packageJson.version })
    // );

    this.breakpointService.breakpoints.subscribe((breakpoints) => {
      // console.log(breakpoints);
    });
    this.store.dispatch(
      AppStoreActions.SET_INITIAL_STORE_OBJ({ storeObj: STORE_MOCK_OBJ })
    );
    this.branding = this.store.select(AppStoreSelectors.selectBranding);

    if (localStorage.getItem('app-selected')) {
      const selectApprovedApplications = this.store.select(
        AppStoreSelectors.selectApprovedApplications
      );

      selectApprovedApplications.subscribe((data: any) => {
        data.forEach((x: any) => {
          if (x.id === localStorage.getItem('app-selected')) {
            this.store.dispatch(
              AppStoreActions.SET_APP_NAVIGATION({
                navigationMenu: x.navigationMenu,
              })
            );
            // this.nav?.subscribe((data: any) => {
            //   console.log("Data", this.nav)
            // })
          }
        });
      });
    }
  }

  onGridReady(params: any) {
    this.devices.subscribe((devices) => {
      this.columnDefs = devices.columns;
      this.rowData = devices.data;
    });
  }

  turnOnDarkMode() {
    this.store.dispatch(AppStoreActions.SET_DARK_MODE({ darkMode: true }));
  }

  turnOffDarkMode() {
    this.store.dispatch(AppStoreActions.SET_LIGHT_MODE({ darkMode: false }));
  }

  changeData() {
    this.store.dispatch(AppStoreActions.LOAD_DEVICES({}));
  }

  updateCommands() {
    const updateCommandsEvent = [
      {
        event: 'pageLoad',
        method: 'reloadCommandsConfiguration',
        params: {
          apiUrl: 'http://localhost:3000/commandsList',
        },
        type: '',
      },
    ];

    this.store.dispatch({
      type: '[Events] Process events',
      events: updateCommandsEvent,
    });
  }

  openMapOverlay() {
    const locationEvent = [
      {
        event: 'click',
        method: 'openDialog',
        params: {
          maxWidth: '900px',
          width: '850px',
          autoFocus: false,
          data: {
            type: 'map',
            title: {
              label: '{{data._selectedRow.reeferName}}',
              iconUrl: 'local_shipping',
              showBorder: true,
            },
            content: [
              {
                style: 'padding-top: 14px; padding-left: 14px',
              },
              // {
              //   type: 'template',
              //   props: {
              //     template: '<div class="flex place-items-center space-x-3 pb-2"><div class="text-gray-500">Location: </div><div class="flex"><i class="material-icons text-base text-gray-600 w-5">place_on</i><div class="font-semibold">Jessup, PA 18434</div></div>'
              //   }
              // },

              {
                type: 'map',
                props: {
                  fields: [
                    {
                      label: 'Location',
                      key: 'location',
                      data: '{{data._selectedRow.lastKnownLocation}}',
                      iconUrl: 'place_on',
                      labelClass: 'pb-1 font-semibold',

                      iconClass: 'w-4 text-base text-gray-600',
                    },
                    {
                      label: 'Last reported time',
                      data: '2023-10-14T20:00:18Z',
                      key: 'lastReportedTime',
                      labelClass: 'pb-1 font-semibold',

                      iconClass: 'w-4 text-base text-gray-600',
                    },
                    {
                      label: 'Status',
                      key: 'status',
                      labelClass: 'pb-1 font-semibold',

                      iconClass: 'w-4 text-base text-gray-600',

                      statusFields: {
                        movingStatus: {
                          field: 'reeferMovingStatus',
                          options: [
                            {
                              value: 'Begin Motion',
                              label: 'BEGIN MOTION',
                              color: 'blue',
                              labelColor: 'white',
                              colorMode: 'dark',
                              iconUrl: 'local_shipping',
                              iconColor: 'blue',
                              hideLabel: true,
                              tooltip: 'Begin Motion',
                              style: 'none',
                            },
                            {
                              value: 'Stopped',
                              label: 'STOPPED',
                              color: 'red',
                              labelColor: 'white',
                              colorMode: 'light',
                              iconUrl: 'local_shipping',
                              iconColor: 'red',
                              hideLabel: true,
                              tooltip: 'Stopped',
                              style: 'none',
                            },
                            {
                              value: 'In Trip',
                              label: 'IN TRIP',
                              color: 'green',
                              labelColor: 'white',
                              colorMode: 'dark',
                              iconUrl: 'local_shipping',
                              iconColor: 'green',
                              hideLabel: true,
                              tooltip: 'In Trip',
                              style: 'none',
                            },
                            {
                              value: null,
                              label: '-',
                              color: '',
                              iconUrl: 'local_shipping',
                              iconColor: '#C2CFD9',
                              colorMode: 'light',
                              labelColor: 'black',
                              hideLabel: true,
                              tooltip: 'No Data Available',
                              style: 'none',
                            },
                          ],
                        },
                        reeferStatus: {
                          field: 'reeferStatus',
                          options: [
                            {
                              value: 'Reefer ON',
                              label: 'REEFER ON',
                              color: 'green',
                              labelColor: 'green',
                              colorMode: 'dark',
                              iconUrl: 'power_settings_new',
                              iconColor: 'green',
                              hideLabel: true,
                              tooltip: 'Reefer On',
                              style: 'none',
                            },
                            {
                              value: 'Reefer OFF',
                              label: 'REEFER OFF',
                              color: 'red',
                              labelColor: 'red',
                              colorMode: 'dark',
                              iconUrl: 'power_settings_new',
                              iconColor: 'red',
                              hideLabel: true,
                              tooltip: 'Reefer Off',
                              style: 'none',
                            },
                            {
                              value: 'Switched OFF Timed (48)',
                              label: 'Switched OFF Timed (48)',
                              color: 'blue',
                              labelColor: 'white',
                              colorMode: 'light',
                              iconUrl: 'power_settings_new',
                              iconColor: 'blue',
                              hideLabel: true,
                              tooltip: 'Reefer Off - 48H',
                              style: 'none',
                            },
                            {
                              value: null,
                              label: '-',
                              color: '',
                              iconUrl: 'power_settings_new',
                              iconColor: '#C2CFD9',
                              colorMode: 'light',
                              labelColor: 'black',
                              hideLabel: true,
                              tooltip: 'No Data Available',
                              style: 'none',
                            },
                          ],
                        },
                        alertStatus: {
                          field: 'alarmSeverity',
                          options: [
                            {
                              value: 'CRITICAL',
                              label: 'CRITICAL',
                              color: 'red',
                              labelColor: 'gray',
                              colorMode: 'light',
                              iconUrl: 'report_problem',
                              iconColor: 'red',
                              hideLabel: true,
                              tooltip: 'Critical Alarm(s) Present',
                              style: 'none',
                            },
                            {
                              value: 'WARNING',
                              label: 'WARNING',
                              color: 'orange',
                              labelColor: 'gray',
                              colorMode: 'light',
                              iconUrl: 'report_problem',
                              iconColor: 'orange',
                              hideLabel: true,
                              tooltip: 'Warning Alarm(s) Present',
                              style: 'none',
                            },
                            {
                              value: 'INFO',
                              label: 'INFO',
                              color: 'blue',
                              labelColor: 'gray',
                              colorMode: 'light',
                              iconUrl: 'report_problem',
                              iconColor: 'blue',
                              hideLabel: true,
                              tooltip: 'Info Alarm(s) Present',
                              style: 'none',
                            },
                            {
                              value: 'NO ALARMS',
                              label: 'NO ALARMS',
                              color: 'gray',
                              labelColor: 'gray',
                              colorMode: 'light',
                              iconUrl: 'report_problem',
                              iconColor: 'green',
                              hideLabel: true,
                              tooltip: 'No Alerts Present',
                              style: 'none',
                            },
                            {
                              value: null,
                              label: '-',
                              color: '',
                              iconUrl: 'report_problem',
                              iconColor: '#C2CFD9',
                              colorMode: 'light',
                              labelColor: 'black',
                              hideLabel: true,
                              tooltip: 'No Data Avaliable',
                              style: 'none',
                            },
                            {
                              value: undefined,
                              label: '-',
                              color: '',
                              iconUrl: 'report_problem',
                              iconColor: '#C2CFD9',
                              colorMode: 'light',
                              labelColor: 'black',
                              hideLabel: true,
                              tooltip: 'No Data Avaliable',
                              style: 'none',
                            },
                          ],
                        },
                        batteryPower: {
                          field: 'batteryPower',
                          options: [
                            {
                              value: 'Battery Low',
                              label: 'BATTERY LOW',
                              color: 'orange',
                              labelColor: 'orange',
                              colorMode: 'dark',
                              iconUrl: 'battery_charging_full',
                              iconColor: 'orange',
                              hideLabel: true,
                              tooltip: 'Battery is Low',
                              style: 'none',
                            },
                            {
                              value: 'Battery Critical',
                              label: 'BATTERY CRITICAL',
                              color: 'red',
                              labelColor: 'red',
                              colorMode: 'dark',
                              iconUrl: 'battery_charging_full',
                              iconColor: 'red',
                              hideLabel: true,
                              tooltip: 'Battery is Critical',
                              style: 'none',
                            },
                            {
                              value: 'Battery OK',
                              label: 'BATTERY OK',
                              color: 'green',
                              labelColor: 'gray',
                              colorMode: 'light',
                              iconUrl: 'battery_charging_full',
                              iconColor: 'green',
                              hideLabel: true,
                              tooltip: 'Battery is Ok',
                              style: 'none',
                            },
                            {
                              value: null,
                              label: '-',
                              color: '',
                              iconUrl: 'battery_charging_full',
                              iconColor: '#C2CFD9',
                              colorMode: 'light',
                              labelColor: 'black',
                              hideLabel: true,
                              tooltip: 'No Data Available',
                              style: 'none',
                            },
                          ],
                        },
                      },
                    },
                  ],
                  markerOptions: {
                    draggable: false,
                    animation: false, // google.maps.Animation.DROP,
                    clickable: false,
                    visible: true,
                    icon: {
                      url: 'assets/Reefer_Marker_Black.svg',
                    },
                    latitudeField: 'latitude',
                    longitudeField: 'longitude',
                  },
                  mapOptions: {
                    height: '400px',
                    mapTypeId: 'satellite',
                    tilt: 0,
                    mapTypeControl: true,
                    mapTypeControlOptions: {
                      style: google.maps.MapTypeControlStyle.HORIZONTAL_BAR,
                      position: google.maps.ControlPosition.TOP_RIGHT,
                    },
                    rotateControl: false,
                    zoom: 17,
                    zoomControl: true,
                    zoomControlOptions: {
                      position: 8, //google.maps.ControlPosition.RIGHT_TOP,
                    },
                    scaleControl: false,
                    streetViewControl: true,
                    streetViewControlOptions: {
                      position: google.maps.ControlPosition.RIGHT_CENTER,
                    },
                    center: { lat: '41.467636', lng: '-75.555382' },
                  },
                  markers: [{ lat: 41.467636, lng: -75.555382 }],
                },
              },
            ],
          },
        },
      },
    ];
    this.store.dispatch({
      type: '[Events] Process events',
      events: locationEvent,
    });
  }
}
