import { NgModule } from '@angular/core';
import { BrowserModule } from '@angular/platform-browser';
import { StoreModule } from '@ngrx/store';

import { AppComponent } from './app.component';
import { AppRoutingModule } from './app-routing.module';
import { WelcomeComponent } from './welcome.component';
import { ApplicationStoreModule, CoreWebModule } from '@hash8io/core-web';
import { StoreDevtoolsModule } from '@ngrx/store-devtools';
import { FormsModule } from '@angular/forms';
import { AgGridModule } from 'ag-grid-angular';
import { LayoutModule } from '@hash8io/core-web/layout';
import { BannerModule } from '@hash8io/core-web/banner';
import { DashboardModule } from './dashboard/dashboard.module';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';

import { DarkModeTogglerModule } from '@hash8io/core-web/dark-mode-toggler';
import { AppHeaderModule } from '@hash8io/core-web/app-header';
import { SidebarModule } from '@hash8io/core-web/sidebar';
import { AppLayoutModule } from './layout/layout.module';
import { EffectsModule } from '@ngrx/effects';
import { DynamicContentWebModule } from '@hash8io/dynamic-content-web';
import { LoadingBarLibraryModule } from '@hash8io/core-web/loading-bar';
import { ChartWebModule } from '@hash8io/chart-web';
import { ChartPageModule } from './chart-page/chart-page.module';
import { ComponentCreatorModule } from './component-creator/component-creator.module';

import { DataGridStoreWebModule } from '@hash8io/data-grid-web';
//Tryout
import { CtaButtonModule } from '@hash8io/core-web/cta-button';
import { EventsWebModule } from '@hash8io/events-web';

import { NgOtpInputModule } from 'ng-otp-input';

import { FormsWebModule } from '@hash8io/forms-web';

// import { DataReportStoreWebModule } from '@hash8io/data-report-web';

import { MoveEntityStoreModule } from '@hash8io/move-entity-web';

import { CommandsStoreModule } from '@hash8io/commands-web';

import { ENVIRONMENT } from '@hash8io/environment';
import { environment } from '../environments/environment';
import { TreeViewModule } from '@hash8io/tree-view-web';
import {
  APPLICATION_UUID_SKIP_URLS,
  AUTH_SKIP_URLS,
  AuthInterceptorProviders,
  AuthInterceptorService,
} from '@hash8io/authentication-web/login';
import { HTTP_INTERCEPTORS } from '@angular/common/http';
@NgModule({
  declarations: [AppComponent],
  imports: [
    BrowserModule,
    BrowserAnimationsModule,
    CoreWebModule,
    FormsModule,
    WelcomeComponent,
    AppRoutingModule,
    AgGridModule,
    LayoutModule,
    SidebarModule,
    BannerModule,
    DashboardModule,
    DarkModeTogglerModule,
    ApplicationStoreModule,
    AppHeaderModule,
    AppLayoutModule,
    CtaButtonModule,
    DynamicContentWebModule,
    MoveEntityStoreModule,
    EffectsModule.forRoot([]),
    StoreModule.forRoot(
      {},
      {
        runtimeChecks: {
          strictActionImmutability: false,
          strictActionSerializability: true,
          strictActionTypeUniqueness: true,
          strictActionWithinNgZone: true,
          strictStateImmutability: true,
          strictStateSerializability: true,
        },
      }
    ),
    StoreDevtoolsModule.instrument({
      maxAge: 25, // Retains last 25 states
      logOnly: true, // Restrict extension to log-only mode
      autoPause: true, // Pauses recording actions and state changes when the extension window is not open
    }),
    // RouterModule.forRoot([], { initialNavigation: 'enabledBlocking' }),
    LoadingBarLibraryModule,
    ChartWebModule,
    ChartPageModule,
    ComponentCreatorModule,
    DataGridStoreWebModule,
    EventsWebModule,
    NgOtpInputModule,
    FormsWebModule,
    // DataReportStoreWebModule
    TreeViewModule.forRoot(),
    CommandsStoreModule
  ],
  exports: [],
  providers: [
    { provide: ENVIRONMENT, useValue: environment },
    // AuthInterceptorProviders,
    // {
    //   provide: AUTH_SKIP_URLS,
    //   useValue: ['/api/v1/login', '/api/v1/tags', '/api/v1/assets'], // Configure skip URLs here
    // },
    // {
    //   provide: APPLICATION_UUID_SKIP_URLS,
    //   useValue: ['api/v1/users'], // Configure skip URLs here
    // },
  ],
  bootstrap: [AppComponent],
})
export class AppModule { }
