import { Dictionary } from '@hash8io/core-web';
import { FormModel } from '@hash8io/forms-web';
import { distinctUntilChanged, map, startWith } from 'rxjs';

export const CHART_FORM: FormModel = {
  model: {
    series: [{ data: [] }],
  },
  fields: [
    {
      type: '#select',
      key: 'dictinory',
      props: {
        hideLabel: false,
        label: 'Data Dictionary',
        align: 'row',
        placeholder: 'Data Dictionary',
        options: [
          { value: 1, label: 'Organization' },
          { value: 2, label: 'Sales' },
          { value: 3, label: 'Users' },
        ],
      },
      extras: {
        labelOverInput: false,
      },
      defaultValue: 1,
    },
    {
      type: '#select',
      key: 'dataSourceOption',
      props: {
        hideLabel: false,
        label: 'Data Source Type',
        align: 'row',
        placeholder: 'Data Source',
        options: [
          { value: 1, label: 'Real API Server' },
          { value: 2, label: 'Mock API Server' },
        ],
      },
      extras: {
        labelOverInput: false,
      },
      defaultValue: 2,
    },
    {
      type: '#group',
      fieldGroupClassName: 'flex w-full',
      fieldGroup: [
        {
          type: '#template',
          className: 'w-full',
          template: `<div class='w-full'>&nbsp;</div>`,
        },
        {
          type: '#select',
          key: 'dataSource',
          className: 'w-full',
          props: {
            hideLabel: true,
            label: 'Data Source',
            placeholder: 'Data Source',
            required: true,
            options: [
              { value: 1, label: 'Organization' },
              { value: 2, label: 'Sales' },
              { value: 3, label: 'Users' },
            ],
            // change: (field, $event) => {
            //   console.log(field.formControl?.value);
            //   const mockData = MockServer.LOOKUP.find(
            //     (d) => d.value === field.formControl?.value
            //   );
            //   console.log(mockData);
            //   if (mockData) {
            //     GRID_FORM.model.rowData = mockData.ref || [];
            //   }
            // },
          },
          expressions: {
            hide: 'model.dataSourceOption == 1',
          },
          extras: {
            labelOverInput: false,
          },
        },
      ],
    },

    {
      type: '#template',
      template: `<hr class="mb-5">`,
    },
    {
      type: '#template',
      template: `<h5 class="forms-subheading p-0 mt-5 mb-2">Properties</h5>`,
    },
    {
      key: 'series',
      type: '#repeat-grid',
      props: {
        addText: 'Add Segement',
        heading: ['Segment Name', 'Chart Type'],
        addField: true,
        removeField: true,
      },
      fieldArray: {
        fieldGroup: [
          {
            type: '#text',
            key: 'name',
            props: {
              placeholder: 'Segment Name',
              hideLabel: true,
            },
          },
          {
            type: '#select',
            key: 'type',
            props: {
              hideLabel: true,
              placeholder: 'Type',
              required: true,
              options: [
                { value: 'line', label: 'Line' },
                { value: 'bar', label: 'Bar' },
                { value: 'area', label: 'Area' },
                { value: 'pie', label: 'Pie' },
              ],
            },
          },
          // {
          //   type: '#text',
          //   key: 'data',
          //   props: {
          //     placeholder: 'Data',
          //     hideLabel: true,
          //   },
          // },
        ],
      },
    },
    {
      key: 'xaxis',
      fieldGroup: [
        {
          type: '#select',
          key: 'field',
          props: {
            align: 'row',
            label: 'X-axis Field',
            hideLabel: false,
            required: true,
            options: [],
            dependentField: 'dictinory',
          },
          hooks: {
            onInit: (field) => {
              if (field && field?.props) {
                console.log(field);
                const parentField = field.props['dependentField'] || '';
                const parentFormControl =
                  field.parent?.parent?.formControl?.get(parentField);
                parentFormControl?.valueChanges
                  .pipe(
                    startWith(parentFormControl.value),
                    distinctUntilChanged(),
                    map((filterId) => {
                      if (field && field?.props) {
                        if (filterId) {
                          const dictinory = Dictionary.LOOKUP.find(
                            (d) => d.value === parentFormControl.value
                          );
                          console.log(dictinory);
                          if (dictinory) {
                            field.props.options = dictinory.ref || [];
                          }
                        } else {
                          field.props.options = [];
                        }
                      }
                    })
                  )
                  .subscribe();
              }
            },
          },
          extras: {
            labelOverInput: true,
          },
        },
      ],
    },
    {
      key: 'yaxis',
      fieldGroup: [
        {
          type: '#select',
          key: 'field',
          props: {
            align: 'row',
            label: 'Y-axis Field',
            hideLabel: false,
            required: true,
            dependentField: 'dictinory',
          },
          hooks: {
            onInit: (field) => {
              if (field && field?.props) {
                console.log(field);
                const parentField = field.props['dependentField'] || '';
                const parentFormControl =
                  field.parent?.parent?.formControl?.get(parentField);
                parentFormControl?.valueChanges
                  .pipe(
                    startWith(parentFormControl.value),
                    distinctUntilChanged(),
                    map((filterId) => {
                      if (field && field?.props) {
                        if (filterId) {
                          const dictinory = Dictionary.LOOKUP.find(
                            (d) => d.value === parentFormControl.value
                          );
                          console.log(dictinory);
                          if (dictinory) {
                            field.props.options = dictinory.ref || [];
                          }
                        } else {
                          field.props.options = [];
                        }
                      }
                    })
                  )
                  .subscribe();
              }
            },
          },
          extras: {
            labelOverInput: true,
          },
        },
      ],
    },
  ],
  options: {
    submitLabel: 'Preview',
    hasReset: true,
    // submitLabelClass: ['w-full'],
  },
};
