import { Dictionary } from '@hash8io/core-web';
import { FormModel } from '@hash8io/forms-web';
import { distinctUntilChanged, map, startWith } from 'rxjs';

export const GRID_FORM: FormModel = {
  fields: [
    {
      type: '#select',
      key: 'dictinory',
      props: {
        hideLabel: false,
        label: 'Data Dictionary',
        align: 'row',
        placeholder: 'Data Dictionary',
        options: [
          { value: 1, label: 'Organization' },
          { value: 2, label: 'Sales' },
          { value: 3, label: 'Users' },
        ],
      },
      extras: {
        labelOverInput: false,
      },
      defaultValue: 1,
    },
    {
      type: '#select',
      key: 'dataSourceOption',
      props: {
        hideLabel: false,
        label: 'Data Source Type',
        align: 'row',
        placeholder: 'Data Source',
        options: [
          { value: 1, label: 'Real API Server' },
          { value: 2, label: 'Mock API Server' },
        ],
      },
      extras: {
        labelOverInput: false,
      },
      defaultValue: 2,
    },
    {
      type: '#group',
      fieldGroupClassName: 'flex w-full',
      fieldGroup: [
        {
          type: '#template',
          className: 'w-full',
          template: `<div class='w-full'>&nbsp;</div>`,
        },
        {
          type: '#select',
          key: 'dataSource',
          className: 'w-full',
          props: {
            hideLabel: true,
            label: 'Data Source',
            placeholder: 'Data Source',
            required: true,
            options: [
              { value: 1, label: 'Organization' },
              { value: 2, label: 'Sales' },
              { value: 3, label: 'Users' },
            ],
            // change: (field, $event) => {
            //   console.log(field.formControl?.value);
            //   const mockData = MockServer.LOOKUP.find(
            //     (d) => d.value === field.formControl?.value
            //   );
            //   console.log(mockData);
            //   if (mockData) {
            //     GRID_FORM.model.rowData = mockData.ref || [];
            //   }
            // },
          },
          expressions: {
            hide: 'model.dataSourceOption == 1',
          },
          extras: {
            labelOverInput: false,
          },
        },
      ],
    },

    {
      type: '#template',
      template: `<hr class="mb-5">`,
    },
    {
      type: '#template',
      template: `<h5 class="forms-subheading p-0 mt-5 mb-2">Properties</h5>`,
    },
    {
      key: 'title',
      fieldGroup: [
        {
          type: '#text',
          key: 'label',
          props: {
            align: 'row',
            label: 'Grid Name',
            hideLabel: false,
            required: true,
          },
          extras: {
            labelOverInput: true,
          },
        },
        // {
        //   type: '#text',
        //   key: 'icon',
        //   className: 'w-full',
        //   props: {
        //     label: 'Icon',
        //     hideLabel: true,
        //   },
        //   extras: {
        //     labelOverInput: true,
        //   },
        // },
      ],
    },
    {
      key: 'columns',
      type: '#repeat-grid',
      props: {
        addText: 'Add Field',
        heading: ['Field', 'Column Label', 'Sortable', 'Filterable'],
        addField: true,
        removeField: true,
      },
      fieldArray: {
        fieldGroup: [
          {
            type: '#select',
            key: 'field',
            props: {
              placeholder: 'Field',
              required: true,
              hideLabel: true,
              dependentField: 'dictinory',
              // valueProp: 'field',
              // labelProp: 'field',
              change: (field, $event) => {
                field?.parent?.formControl
                  ?.get('headerName')
                  ?.setValue($event.source.triggerValue);
              },
            },
            hooks: {
              onInit: (field) => {
                if (field && field?.props) {
                  const parentField = field.props['dependentField'] || '';
                  const parentFormControl =
                    field.parent?.parent?.parent?.formControl?.get(parentField);
                  parentFormControl?.valueChanges
                    .pipe(
                      startWith(parentFormControl.value),
                      distinctUntilChanged(),
                      map((filterId) => {
                        if (field && field?.props) {
                          if (filterId) {
                            const dictinory = Dictionary.LOOKUP.find(
                              (d) => d.value === parentFormControl.value
                            );
                            console.log(dictinory);
                            if (dictinory) {
                              field.props.options = dictinory.ref || [];
                            }
                          } else {
                            field.props.options = [];
                          }
                        }
                      })
                    )
                    .subscribe();
                }
              },
            },
          },
          {
            type: '#text',
            key: 'headerName',
            props: {
              placeholder: 'Column Label',
              required: true,
              hideLabel: true,
            },
            hooks: {
              onInit: (field) => {
                console.log(field);
              },
            },
          },
          {
            type: '#checkbox',
            key: 'sortable',
            props: {
              hideLabel: true,
              placeholder: 'Sortable',
              options: [
                { value: true, label: 'True' },
                { value: false, label: 'False' },
              ],
            },
            defaultValue: true,
          },
          {
            type: '#checkbox',
            key: 'filter',
            props: {
              hideLabel: true,
              placeholder: 'Filterable',
            },
            defaultValue: true,
          },
          // {
          //   type: '#select',
          //   key: 'hide',
          //   props: {
          //     hideLabel: true,
          //     placeholder: 'Hide',
          //     options: [
          //       { value: true, label: 'True' },
          //       { value: false, label: 'False' },
          //     ],
          //   },
          //   defaultValue: true,
          // },
        ],
      },
    },
  ],
  options: {
    submitLabel: 'Preview',
    hasReset: true,
    // submitLabelClass: ['w-full'],
  },
};
