import { Component, OnInit } from '@angular/core';
import { Model, BreakpointService, AppStoreSelectors } from '@hash8io/core-web';
import { Store } from '@ngrx/store';
import { LoadingBarService } from '@ngx-loading-bar/core';
import { Observable } from 'rxjs';
import { Actions } from '../mock/actions';
import { Breadcrumb } from '../mock/breadcrumb';
import { Menu } from '../mock/menu';
import { Snapshot } from '../mock/snapshot';
import { Title } from '../mock/title';
import { Toolbar } from '../mock/toolbar';

@Component({
  selector: 'hash8-app-components-page',
  templateUrl: './components-page.component.html',
  styleUrls: ['./components-page.component.scss'],
})
export class ComponentsPageComponent implements OnInit {
  navigations: Model.Menu[] = [...Menu];
  navigation2: Model.Menu[] = [
    { type: Model.MenuType.Heading, label: 'Settings' },
    ...Actions,
  ];
  toolbar: Model.Toolbar = Toolbar;
  breadcrumbs: Model.Breadcrumb[] = Breadcrumb;
  actions: Model.Action[] = Actions;
  title: Model.Title = Title;

  snapshot: Model.SnapshotModel = Snapshot;
  reportUUID!: string;

  breakpoint: any;
  branding!: Observable<any>;

  options = {
    showLabel: false,
    stroked: true,
  };

  tileLayout: Model.TileLayout = {
    columns: 12,
    rows: 'auto',
    gap: '12px',
    tiles: {
      lg: [],
    },
  };

  dynamicTopBottomContent: Model.Content[] = [
    {
      type: 'text',
      props: {
        type: 'text',
        label: 'Last Reported Time',
        value: '06/08/2022 03:10 AM (EST)',
        key: 'lastReportedTime',
        align: 'column',
      },
    },
    {
      type: 'text',
      props: {
        type: 'tag',
        key: 'status',
        label: 'Reporting Status',
        options: {
          color: 'green',
        },
        value: 'Reporting',
        align: 'column',
      },
    },
    {
      type: 'text',
      props: {
        type: 'text',
        label: 'Last Reported Time',
        value: '06/08/2022 03:10 AM (EST)',
        key: 'lastReportedTime',
        align: 'column',
      },
    },
    {
      type: 'text',
      props: {
        type: 'tag',
        key: 'status',
        label: 'Reporting Status',
        options: {
          color: 'green',
        },
        value: 'Reporting',
        align: 'column',
      },
    },

    {
      type: 'text',
      props: {
        type: 'text',
        label: 'Location',
        value: '2101 Richland Avenue, Houston, Texas 77077, USA',
        key: 'location',
        align: 'column',
      },
    },
  ];

  dynamicLeftRightContent: Model.Content[] = [
    {
      type: 'text',
      props: {
        type: 'text',
        label: 'Last Reported Time',
        value: '06/08/2022 03:10 AM (EST)',
        key: 'lastReportedTime',
      },
    },
    {
      type: 'text',
      props: {
        type: 'tag',
        key: 'status',
        label: 'Reporting Status',
        value: 'Reporting',
      },
    },
    {
      type: 'text',
      props: {
        type: 'text',
        label: 'Last Reported Time',
        value: '06/08/2022 03:10 AM (EST)',
        key: 'lastReportedTime',
      },
    },
    {
      type: 'text',
      props: {
        type: 'tag',
        key: 'status',
        label: 'Reporting Status',
        value: 'Reporting',
      },
    },

    {
      type: 'text',
      props: {
        type: 'text',
        label: 'Location',
        value: '2101 Richland Avenue, Houston, Texas 77077, USA',
        key: 'location',
      },
    },
  ];

  dynamicRowContent: Model.Content[] = [
    {
      type: 'row',
      props: [
        {
          type: 'text',
          props: {
            type: 'text',
            label: 'Last Reported Time',
            value: '06/08/2022 03:10 AM (EST)',
            key: 'lastReportedTime',
            align: 'column',
          },
        },
        {
          type: 'text',
          props: {
            type: 'tag',
            key: 'status',
            label: 'Reporting Status',
            align: 'column',
            value: 'Reporting',
          },
        },
      ],
    },
    {
      type: 'row',
      props: [
        {
          type: 'text',
          props: {
            type: 'text',
            label: 'Location',
            value: '2101 Richland Avenue, Houston, Texas 77077, USA',
            key: 'location',
            align: 'column',
          },
        },
        {
          type: 'text',
          props: {
            label: 'Message ID',
            value: '#10024',
            key: 'messageID',
            align: 'column',
          },
        },
      ],
    },
    {
      type: 'row',
      props: [
        {
          type: 'text',
          props: {
            type: 'tag',
            key: 'status',
            label: 'Reporting Status',
            align: 'column',
            value: 'Reporting',
          },
        },
        {
          type: 'text',
          props: {
            type: 'text',
            label: 'Last Reported Time',
            value: '06/08/2022 03:10 AM (EST)',
            key: 'lastReportedTime',
            align: 'column',
          },
        },
      ],
    },
  ];

  dynamicKpiContent: Model.Content[] = [
    {
      type: 'row',
      props: [
        {
          type: 'kpi',
          props: {
            label: 'Liquid Level',
            value: '0.06 inches',
          },
        },
        {
          type: 'kpi',
          props: {
            label: 'Liquid Volume',
            value: '15.70 litres',
          },
        },
      ],
    },
    {
      type: 'kpi',
      props: {
        label: 'Liquid Volume',
        value: '15.70 litres',
      },
    },
    {
      type: 'kpi',
      props: {
        label: 'Temperature',
        value: '13• C',
      },
    },
  ];

  cardHeader: Model.Toolbar = {
    title: {
      label: 'My Profile',
    },
  };

  cardHeader2: Model.Toolbar = {
    title: {
      label: 'Privacy & Personalization',
      iconUrl: 'security',
    },
  };

  cardHeader3: Model.Toolbar = {
    title: {
      label: 'Contact Info',
      iconUrl: 'assignment_ind',
    },
    actions: [
      {
        label: 'Edit Contact Details',
        targetUrl: '',
        type: Model.MenuType.Link,
        iconUrl: 'edit',
      },
    ],
  };

  cardHeader4: Model.Toolbar = {
    title: {
      label: 'Sign In',
      iconUrl: 'key',
    },
  };
  loader = this.loadingBar.useRef();

  constructor(private _breakpoint: BreakpointService, private store: Store,private loadingBar: LoadingBarService,) {
    this._breakpoint.active$.subscribe((breakpoint) => {
      this.breakpoint = breakpoint === 'xs' ? 'sm' : breakpoint;
    });
  }

  ngOnInit(): void {
    this.branding = this.store.select(AppStoreSelectors.selectBranding);
    this.loader.start()
  }
}
