import { Model } from '@hash8io/core-web';
import { DialogDataModel } from '@hash8io/dialog-web';
import { FormModel } from '@hash8io/forms-web';

export const PAGE_TITLE: Model.Title = {
  label: 'Demo Page',
  iconUrl: 'language',
};

export const PAGE_NAV: Model.Menu[] = [
  {
    label: 'Reports Grid',
    iconUrl: 'layers',
    type: Model.MenuType.Link,
    visible: true,
    clickable: true,
    targetUrl: '/demo/reports',
  },
  {
    label: 'Cascade Grid',
    iconUrl: 'layers',
    type: Model.MenuType.Link,
    visible: true,
    clickable: true,
    targetUrl: '/demo/cascade',
  },
  {
    label: 'Forms',
    iconUrl: 'layers',
    type: Model.MenuType.Link,
    visible: true,
    clickable: true,
    targetUrl: '/demo/forms',
  },
  {
    label: 'Dialog',
    iconUrl: 'layers',
    type: Model.MenuType.Link,
    visible: true,
    clickable: true,
    targetUrl: '/demo/dialog',
  },
  {
    label: 'Cards',
    iconUrl: 'layers',
    type: Model.MenuType.Link,
    visible: true,
    clickable: true,
    targetUrl: '/demo/cards',
  },
];

export const PAGE_TOOLBAR: Model.Toolbar = {
  breadcrumbs: [
    {
      label: 'Home',
      targetUrl: '/demo',
    },
    {
      label: 'All Pages',
      targetUrl: '',
      value: 'Breadcrumb Value',
    },
  ],
  actions: [
    {
      label: 'Invite Users',
      iconUrl: 'person',
      type: Model.MenuType.Link,
      visible: true,
      clickable: true,
      targetUrl: '/demo/cards',
      isPrimaryAction: true,
    },

    {
      label: 'Revoke Invites',
      iconUrl: 'layers',
      type: Model.MenuType.Link,
      visible: true,
      clickable: true,
      targetUrl: '/demo/cards',
    },
    {
      label: 'Delete Invites',
      iconUrl: 'delete',
      type: Model.MenuType.Link,
      visible: true,
      clickable: true,
      targetUrl: '/demo/cards',
    },
  ],
  options: {
    showSearch: false,
  },
};

export const PAGE_FORMS: FormModel = {
  title: {
    label: 'Page Forms',
  },
  fields: [
    {
      type: '#template',
      template: '<h5 class="form-heading">Sign Up</h5>',
    },
    {
      type: '#text',
      key: 'firstName',
      props: {
        label: 'First Name',
      },
    },
    {
      fieldGroupClassName: 'flex place-items-center gap-2',
      fieldGroup: [
        {
          type: '#number',
          key: 'age',
          props: {
            label: 'Age',
          },
        },
        {
          type: '#decimal',
          key: 'currency',
          className: 'w-full',
          props: {
            label: 'Amount',
          },
        },
      ],
    },
    {
      type: '#template',
      template: '<h5 class="form-subheading">Address Details</h5>',
    },
    {
      type: '#template',
      template:
        "<p class='form-description'>Please provide your personal details. We won't share anywhere.</p>",
    },
    {
      type: '#textarea',
      props: {
        label: 'Address',
      },
    },
    {
      type: '#select',
      props: {
        label: 'Country',
        options: [
          {
            value: 1,
            label: 'India',
          },
          {
            value: 2,
            label: 'US',
          },
        ],
      },
    },

    {
      type: '#password',
    },
    {
      type: '#phoneNumber',
    },
    {
      type: '#select-async',
      key: 'userAsync',
      props: {
        label: 'User Async',
        labelProp: 'first_name',
        description: 'Selec a user to view task',
        valueProp: 'id',
        required: true,
        apiEndpoint: 'https://63239166bb2321cba91c006d.mockapi.io/users',
        async: true,
        align: 'row',
      },
    },
    {
      type: '#select-async',
      key: 'taskAsync',
      props: {
        label: 'Task',
        labelProp: 'task',
        valueProp: 'id',
        required: true,
        apiEndpoint: 'https://63239166bb2321cba91c006d.mockapi.io/users',
        async: true,
        dependentField: 'userAsync',
        dependentFieldKey: 'tasks',
        align: 'row',
      },
      expressions: {
        'props.disabled': '!model.userAsync',
      },
    },
    {
      type: '#textbox-async',
      key: 'name',
      props: {
        label: 'Name',
        apiEndpoint: 'https://63239166bb2321cba91c006d.mockapi.io/users',
        apiKey: 'first_name',
      },
    },
    {
      type: '#repeat-grid',
      key: 'users',
      props: {
        heading: ['User', 'User Name'],
      },
      fieldArray: {
        fieldGroup: [
          {
            type: '#select-async',
            key: 'userAsyncs',
            props: {
              label: 'User Async',
              labelProp: 'first_name',
              placeholder: 'Select User',
              valueProp: 'id',
              required: true,
              hideLabel: true,
              apiEndpoint: 'https://63239166bb2321cba91c006d.mockapi.io/users',
              async: true,
            },
            extras: {
              labelOverInput: true,
            },
          },
          {
            type: '#textbox-async',
            key: 'name',
            props: {
              label: 'Name',
              hideLabel: true,
              apiEndpoint: 'https://63239166bb2321cba91c006d.mockapi.io/users',
              apiKey: 'first_name',
            },
          },
        ],
      },
    },
  ],
};

export const PAGE_FORMS2: FormModel = {
  type: 'steps',
  fields: [
    // {
    //   type: '#template',
    //   template: '<h5 class="form-heading">Sign Up</h5>',
    // },
    {
      type: 'stepper',
      fieldGroup: [
        {
          props: { label: 'Organization Details' },
          fieldGroup: [
            {
              type: '#text',
              key: 'firstName',
              props: {
                label: 'First Name',
              },
            },
            {
              fieldGroupClassName: 'flex place-items-center gap-2',
              fieldGroup: [
                {
                  type: '#number',
                  key: 'age',
                  props: {
                    label: 'Age',
                  },
                },
                {
                  type: '#decimal',
                  key: 'currency',
                  className: 'w-full',
                  props: {
                    label: 'Amount',
                  },
                },
              ],
            },
          ],
        },
        {
          props: { label: 'Personal Details' },
          fieldGroup: [
            {
              type: '#textarea',
              props: {
                label: 'Address',
              },
            },
            {
              type: '#select',
              props: {
                label: 'Country',
                options: [
                  {
                    value: 1,
                    label: 'India',
                  },
                  {
                    value: 2,
                    label: 'US',
                  },
                ],
              },
            },

            {
              type: '#password',
            },
            {
              type: '#phoneNumber',
            },
            {
              type: '#select-async',
              key: 'userAsync',
              props: {
                label: 'User Async',
                labelProp: 'first_name',
                description: 'Selec a user to view task',
                valueProp: 'id',
                required: true,
                apiEndpoint:
                  'https://63239166bb2321cba91c006d.mockapi.io/users',
                async: true,
                align: 'row',
              },
            },
            {
              type: '#select-async',
              key: 'taskAsync',
              props: {
                label: 'Task',
                labelProp: 'task',
                valueProp: 'id',
                required: true,
                apiEndpoint:
                  'https://63239166bb2321cba91c006d.mockapi.io/users',
                async: true,
                dependentField: 'userAsync',
                dependentFieldKey: 'tasks',
                align: 'row',
              },
              expressions: {
                'props.disabled': '!model.userAsync',
              },
            },
            {
              type: '#textbox-async',
              key: 'name',
              props: {
                label: 'Name',
                apiEndpoint:
                  'https://63239166bb2321cba91c006d.mockapi.io/users',
                apiKey: 'first_name',
              },
            },
          ],
        },
        {
          props: { label: 'Extra Details' },
          fieldGroup: [
            {
              type: '#repeat-grid',
              key: 'users',
              props: {
                heading: ['User', 'User Name'],
              },
              fieldArray: {
                fieldGroup: [
                  {
                    type: '#select-async',
                    key: 'userAsyncs',
                    props: {
                      label: 'User Async',
                      labelProp: 'first_name',
                      placeholder: 'Select User',
                      valueProp: 'id',
                      required: true,
                      hideLabel: true,
                      apiEndpoint:
                        'https://63239166bb2321cba91c006d.mockapi.io/users',
                      async: true,
                    },
                    extras: {
                      labelOverInput: true,
                    },
                  },
                  {
                    type: '#textbox-async',
                    key: 'name',
                    props: {
                      label: 'Name',
                      hideLabel: true,
                      apiEndpoint:
                        'https://63239166bb2321cba91c006d.mockapi.io/users',
                      apiKey: 'first_name',
                    },
                  },
                ],
              },
            },
          ],
        },
      ],
    },
  ],
};

export const PAGE_DIALOG: DialogDataModel = {
  title: {
    label: 'Invite Users',
  },
  content: [
    // {
    //   type: 'form',
    //   props: {
    //     type: 'steps',
    //     fields: PAGE_FORMS2.fields,
    //   },
    // },
    {
      type: 'form',
      props: {
        fields: PAGE_FORMS.fields,
      },
    },
  ],
};

export const PAGE_CARD: Model.Card = {
  header: {
    title: PAGE_TITLE,
    actions: [
      {
        label: 'Poll Location',
        type: Model.MenuType.Link,
      },
    ],
  },
  content: [
    {
      type: 'text',
      props: {
        label: 'Device Serial Number',
        value: '28741',
        key: 'lastReportedTime',
        align: 'column',
      },
    },
    {
      type: 'text',
      props: {
        label: 'Device Serial Number',
        value: '28741',
        key: 'lastReportedTime',
        align: 'column',
      },
    },
    {
      type: 'text',
      props: {
        label: 'Device Serial Number',
        value: '28741',
        key: 'lastReportedTime',
        align: 'column',
      },
    },
    {
      type: 'row',
      props: [
        {
          type: 'kpi',
          props: {
            label: 'Device Serial Number',
            value: '28741',
            key: 'lastReportedTime',
            align: 'column',
          },
        },
        {
          type: 'kpi',
          props: {
            label: 'Device Serial Number',
            value: '28741',
            key: 'lastReportedTime',
            align: 'column',
          },
        },
      ],
    },
  ],
};

export const PAGE_CARD2 = {
  content: [
    {
      type: 'form',
      props: {
        type: 'steps',
        fields: PAGE_FORMS2.fields,
      },
    },
  ],
};
