import { DialogDataModel } from '@hash8io/dialog-web';
import { FormsWebComponent } from '@hash8io/forms-web';

export const DIALOG_FORM: DialogDataModel = {
  title: {
    label: 'Dynamic API Forms',
    iconUrl: 'apartment',
  },
  content: [
    {
      type: 'form',
      props: {
        fields: [
          {
            template: `<h5 class="forms-subheading">API Dropdowns</h5>
        <p class="forms-description mb-4">Individual select dropdowns loaded from API endpoint.</p>`,
          },
          {
            key: 'driver',
            type: '#select-async',
            focus: true,
            props: {
              label: 'Driver',
              options: [],
              valueProp: 'id',
              labelProp: 'name',
              async: true,
              required: false,
              align: 'row',
              // apiEndpoint: 'https://my.api.mockaroo.com/driver.json?key=a77bb3b0',
              apiEndpoint: 'https://63239166bb2321cba91c006d.mockapi.io/users',
            },
          },
          {
            key: 'country',
            type: '#select-async',
            props: {
              label: 'Country',
              options: [],
              valueProp: 'id',
              labelProp: 'country',
              async: true,
              required: false,
              align: 'row',
              // apiEndpoint: 'https://my.api.mockaroo.com/country.json?key=a77bb3b0',
              apiEndpoint:
                'https://63239166bb2321cba91c006d.mockapi.io/country',
            },
            extras: {
              hintBelowInput: true,
            },
            defaultValue: '4',
          },
          {
            key: 'timezone',
            type: '#select-async',
            props: {
              label: 'Timezone',
              options: [],
              valueProp: 'id',
              labelProp: 'timezone',
              async: true,
              required: false,
              align: 'row',
              // apiEndpoint: 'https://my.api.mockaroo.com/country.json?key=a77bb3b0',
              apiEndpoint: 'https://63239166bb2321cba91c006d.mockapi.io/states',
            },
            extras: {
              hintBelowInput: true,
            },
            defaultValue: '55',
          },
          {
            type: '#select-async',
            key: 'colorCode',
            props: {
              label: 'Color Code',
              async: true,
              labelProp: 'color',
              valueProp: 'id',
              required: false,
              align: 'row',
              apiEndpoint:
                'https://63239166bb2321cba91c006d.mockapi.io/country',
            },
            defaultValue: '7',
          },
          {
            template:
              '<h5 class="forms-subheading p-0 mb-4">Drilldown API Dropdowns</h5>',
          },
          {
            key: 'user',
            type: '#select-async',
            props: {
              label: 'Users',
              placeholder: 'Select a User',
              options: [],
              valueProp: 'id',
              labelProp: 'first_name',
              async: true,
              required: false,
              align: 'row',
              apiEndpoint: 'https://63239166bb2321cba91c006d.mockapi.io/users',
            },
            // defaultValue: 3,
          },
          {
            key: 'task',
            type: '#select-async',
            props: {
              label: "User's Task",
              options: [],
              valueProp: 'id',
              labelProp: 'task',
              description: 'Select a user to list tasks',
              async: true,
              align: 'row',
              required: true,
              // apiEndpoint: 'https://my.api.mockaroo.com/tasks.json?key=a77bb3b0',
              apiEndpoint: 'https://63239166bb2321cba91c006d.mockapi.io/users',
              dependentField: 'user',
              dependentFieldKey: 'tasks',
            },
            expressions: {
              'props.disabled': '!model.user',
            },
          },
          {
            template:
              ' <h5 class="forms-subheading p-0 mt-4 mb-4">Check Unique value with API</h5>',
          },
          {
            type: '#textbox-async',
            key: 'username',
            props: {
              label: 'Username',
              align: 'row',
              required: true,
              async: true,
              description:
                'Pick a unique username for your account. (Try: Bret, Antonette, Samantha)',
              apiEndpoint: 'https://jsonplaceholder.typicode.com/users',
              apiKey: 'username',
              errorMessage: `username is already taken.`,
            },
          },
          {
            type: '#textbox-async',
            key: 'email',
            className: 'block mb-2',
            props: {
              label: 'Email',
              align: 'row',
              required: true,
              async: true,
              description: 'Try: Nathan@yesenia.net, Sherwood@rosamond.me',
              apiEndpoint: 'https://jsonplaceholder.typicode.com/users',
              apiKey: 'email',
              errorMessage: `email is already regisitered.`,
            },
          },
          {
            type: '#textbox-async',
            key: 'newuser',
            className: 'block mb-2',
            props: {
              label: 'Mock User Name',
              align: 'row',
              required: true,
              async: true,
              description: 'Try: Marian, Larry',
              apiEndpoint: 'https://63239166bb2321cba91c006d.mockapi.io/users',
              apiKey: 'first_name',
              errorMessage: `name already exist.`,
            },
          },
        ],
      },
    },
  ],
};

export const DIALOG_ORGANIZATION: DialogDataModel = {
  title: {
    label: 'Repeat Grid Form',
    iconUrl: 'apartment',
  },
  customComponents: {
    form: FormsWebComponent,
  },
  content: [
    {
      type: 'form',
      props: {
        fields: [
          {
            key: 'organizations',
            type: '#repeat-grid',
            props: {
              addText: 'Add Organization',
              heading: ['Organization Name', 'Country', 'Organization Type'],
              addField: true,
              removeField: true,
              maxItems: 5,
            },
            defaultValue: [undefined, undefined, undefined],
            fieldArray: {
              fieldGroup: [
                {
                  type: '#text',
                  key: 'organizationName',
                  props: {
                    placeholder: 'Organization Name',
                    required: true,
                    hideLabel: true,
                  },
                },
                {
                  type: '#select-async',
                  key: 'country',
                  props: {
                    label: 'Country',
                    required: true,
                    async: true,
                    labelProp: 'country',
                    valueProp: 'id',
                    apiEndpoint:
                      'https://63239166bb2321cba91c006d.mockapi.io/country',
                    hideLabel: true,
                  },
                  extras: {
                    labelOverInput: true,
                  },
                  defaultValue: '7',
                },
                {
                  type: '#select',
                  key: 'organizationType',
                  props: {
                    hideLabel: true,
                    placeholder: 'Organization Type',
                    options: [
                      { value: 1, label: 'RT' },
                      { value: 2, label: 'Teezle' },
                      { value: 3, label: 'Autocavo' },
                      { value: 4, label: 'Hash8' },
                    ],
                  },
                },
              ],
            },
          },
        ],
      },
    },
  ],
};
