import { DialogDataModel } from '@hash8io/dialog-web';

export const DIALOG_REGISTRATION: DialogDataModel = {
  title: {
    label: 'Signup',
    iconUrl: 'person',
  },
  content: [
    {
      type: 'form',
      props: {
        fields: [
          {
            template: `<p class="forms-description text-base mb-3">Sign up to Hash8 IoT system</p>`,
          },
          {
            fieldGroupClassName: 'lg:flex gap-3',
            fieldGroup: [
              {
                type: '#text',
                className: 'w-full',
                props: {
                  label: 'First Name',
                  required: true,
                  hideLabel: true,
                },
                extras: {
                  labelOverInput: true,
                },
              },
              {
                type: '#text',
                className: 'w-full',
                key: 'lastName',
                props: {
                  label: 'Last Name',
                  required: true,
                  hideLabel: true,
                },
                extras: {
                  labelOverInput: true,
                },
              },
            ],
          },
          {
            type: '#text',
            key: 'email',
            props: {
              label: 'Email',
              type: 'email',
              readonly: true,
              disabled: true,
              hideLabel: true,
            },
            extras: {
              labelOverInput: true,
            },
            defaultValue: 'johnmathew@gmail.com',
            expressions: {
              // 'props.disabled': '!model.username',
            },
          },
          {
            type: '#textbox-async',
            key: 'username',
            props: {
              label: 'Username',
              required: true,
              hideLabel: true,
              apiEndpoint: 'https://jsonplaceholder.typicode.com/users',
              apiKey: 'username',
              errorMessage: `username is already taken.`,
              minLength: 3,
            },
            extras: {
              labelOverInput: true,
            },
          },
          {
            type: '#phoneNumber',
            props: {
              hideLabel: true,
            },
            extras: {
              labelOverInput: true,
            },
          },
          {
            validators: {
              validation: [
                {
                  name: 'fieldMatch',
                  options: { errorPath: 'confirmPassword' },
                },
              ],
            },
            fieldGroup: [
              {
                type: '#password',
                key: 'password',
                props: {
                  hideLabel: true,
                },
                extras: {
                  labelOverInput: true,
                },
              },
              {
                type: '#password',
                key: 'confirmPassword',
                props: {
                  label: 'Confirm Password',
                  hideLabel: true,
                },
                extras: {
                  labelOverInput: true,
                },
              },
            ],
          },
          {
            template: `<p class="text-xs -mt-1 mb-2 opacity-50">Note: Your password should meet all the following conditions.<br>
    * Atleast one upper case letter.<br>
    * Atleast one number.<br>
    * Min.8 characters in length.</p>`,
          },
        ],
      },
    },
  ],
};
