import { CascadeReportWebComponent } from '@hash8io/cascade-report-web';
import { DataReportWebComponent } from '@hash8io/data-report-web';
import { DialogDataModel } from '@hash8io/dialog-web';

export const DIALOG_REPORT: DialogDataModel = {
  title: {
    label: 'Reports',
    iconUrl: 'article',
  },
  content: [
    {
      type: 'report',
      props: {
        reportUUID: 101,
        showToolbar: true,
      },
    },
  ],
  customComponents: {
    report: DataReportWebComponent,
    // cascade: CascadeReportWebComponent,
  },
};

export const DIALOG_CASCADE_REPORT: DialogDataModel = {
  title: {
    label: 'Cascade Reports',
    iconUrl: 'article',
  },
  content: [
    {
      type: 'cascade',
      props: {
        cascadeUUID: 'CR-1000',
        layout: 'horizontal',
      },
    },
  ],
  customComponents: {
    cascade: CascadeReportWebComponent,
  },
};

export const CREATE_SALES_CHANNEL_DIALOG_FORM2 = {
  title: {
    label: 'Create Sales Channel',
    iconUrl: 'person',
    showBorder: true,
  },
  content: [
    {
      type: 'form',
      props: {
        fields: [
          {
            type: '#textbox-async',
            key: 'email',
            className: 'block mb-1',
            props: {
              type: 'email',
              label: 'Email',
              align: 'row',
              required: true,
              async: true,
              hideLabel: false,
              apiEndpoint: '/api/v1/email-unique-status',
              apiKey: 'email',
              errorMessage: `email already exists`,
              valueCheck: true,
            },
            validators: {
              validation: [{ name: 'email' }],
            },
          },
          {
            type: '#textbox-async',
            key: 'organizationName',
            className: 'block mb-1',
            props: {
              label: 'Organization Name',
              align: 'row',
              required: true,
              async: true,
              hideLabel: false,

              apiEndpoint: '/api/v1/organizations',
              apiKey: 'organizationName',
              errorMessage: `organization name already exists`,
              valueCheck: true,
            },
          },
          {
            type: '#text',
            key: 'userRoleName',
            props: {
              label: 'User Role',
              readonly: true,
              disabled: true,
              hideLabel: false,
              align: 'row',
              description: '',
            },
            extras: {
              hintBelowInput: true,
            },
            defaultValue: 'Sales Channel Owner',
          },
          // {
          //   type: '#template',
          //   template: '<div class="h-1"></div>',
          // },
          {
            type: '#template',
            template: '<hr class="mt-0">',
          },

          {
            key: 'applications',
            type: '#checkbox-multiple',
            props: {
              label: 'Applications enabled for Reselling',
              align: 'row',
              required: true,
              valueProp: 'applicationUUID',
              labelProp: 'applicationName',
              async: true,
              apiEndpoint: `/api/v1/organizations/{{data.localStorage.organizationUUID}}/applications`,
              params: {
                bodyParams: [
                  {
                    key: 'userUUID',
                  },
                ],
              },
              options: [
                // { applicationUUID: 'liftiq', applicationName: 'LiftiQ' },
                // { applicationUUID: 'reeferiq', applicationName: 'ReeferiQ' },
                // { applicationUUID: 'cloudiq', applicationName: 'CloudiQ' },
                // { applicationUUID: 'installiq', applicationName: 'InstalliQ' },
                // { applicationUUID: 'tankiq', applicationName: 'TankiQ' },
              ],
            },
          },
          // {
          //   key: 'applications',
          //   type: '#checkbox-multiple',
          //   props: {
          //     label: 'Applications enabled for Reselling',
          //     align: 'row',
          //     required: true,
          //     valueProp: 'applicationUUID',
          //     labelProp: 'applicationName',
          //     async: false,
          //     apiEndpoint: `/api/v1/organizations/{{data.localStorage.organizationUUID}}/applications`,
          //     params: {
          //       bodyParams: [
          //         {
          //           key: 'userUUID',
          //         },
          //       ],
          //     },
          //     options: [
          //       { applicationUUID: 'liftiq', applicationName: 'LiftiQ' },
          //       { applicationUUID: 'reeferiq', applicationName: 'ReeferiQ' },
          //       { applicationUUID: 'cloudiq', applicationName: 'CloudiQ' },
          //       { applicationUUID: 'installiq', applicationName: 'InstalliQ' },
          //       { applicationUUID: 'tankiq', applicationName: 'TankiQ' },
          //     ],
          //   },
          // },
          // {
          //   type: '#template',
          //   template:
          //     '<p class="mt-2 text-sm"><span class="text-base material-icons">info</span>&nbsp;Note: The <b>"Sales Channel Owner"</b> will get the access to control and monitor the selected applications.</p>',
          // },
          {
            type: '#template',
            template:
              '<div class=" text-sm text-gray-600" ><span class="text-base material-icons">info</span> The <span class="font-bold">"Sales Channel Owner"</span> will get the access to control and monitor the selected applications.</div>',
          },
          // {
          //   type: '#template',
          //   template: '<div class="h-0.5"></div>',
          // },
          // {
          //   key: 'applicationNames',
          //   type: '#select-async',
          //   focus: false,
          //   props: {
          //     label: 'Application',
          //     options: [],
          //     valueProp: 'applicationName',
          //     labelProp: 'applicationName',
          //     async: true,
          //     required: true,
          //     align: 'row',
          //     hideLabel: false,
          //     apiEndpoint: `/api/v1/applications`,
          //     params: {
          //       bodyParams: [
          //         {
          //           key: 'userUUID',
          //         },
          //       ],
          //     },
          //   },
          // },
        ],
        options: {
          submit: {
            label: 'Send Invitation',
            events: [
              {
                event: 'click',
                method: 'postAPI',
                params: {
                  apiUrl: '/api/v1/sales-channel-invitationssss',
                  bodyParams: [
                    {
                      key: 'userUUID',
                    },
                  ],
                },
                events: [
                  {
                    event: 'success',
                    method: 'showToast',
                    enabled:
                      'data[0].message === "Successfully invited sales channel"',
                    params: {
                      message: 'User Invited Successfully',
                    },
                    type: '',
                  },
                  {
                    event: 'failure',
                    method: 'showErrorToast',
                    enabled:
                      'data[0].message !== "Successfully invited sales channel"',

                    params: {
                      message: 'Error on sending user invitation',
                    },
                    type: '',
                  },
                  {
                    event: 'success',
                    method: 'closeDialog',
                    enabled:
                      'data[0].message === "Successfully invited sales channel"|| data[0].message !== "Successfully invited sales channel"',
                  },
                  {
                    event: 'success',
                    method: 'refreshDataGrid',
                    enabled:
                      'data[0].message === "Successfully invited sales channel"',

                    params: {
                      gridName: 'salesChannelsDataGrid',
                    },
                  },
                ],
              },
            ],
          },
          submitLabel: 'Invite User',
          cancel: {
            label: 'Cancel',
            events: [
              {
                event: 'click',
                method: 'closeDialog',
                params: {},
                type: '',
              },
            ],
          },
        },
      },
    },
  ],
};

export const REPEAT_FORM_DEFAULT_USER_FORM = {
  title: {
    label: 'Invite User',
    iconUrl: 'person',
    showBorder: true,
  },
  content: [
    {
      type: 'form',
      props: {
        fields: [
          {
            type: '#textbox-async',
            key: 'email',
            className: 'block mb-2',
            props: {
              type: 'email',
              label: 'Email',
              align: 'row',
              required: true,
              async: true,
              hideLabel: false,
              // apiKey: 'email',
              // errorMessage: `email is already regisitered.`,
              apiEndpoint: '/api/v1/email-unique-status',
              apiKey: 'email',
              errorMessage: `email already exists`,
              valueCheck: true,
            },
            validators: {
              validation: [{ name: 'email' }],
            },
          },
          {
            key: 'userRole',
            type: '#select-async',
            focus: false,
            props: {
              label: 'User Roles',
              options: [],
              valueProp: 'userRoleName',
              labelProp: 'userRoleName',
              async: true,
              required: true,
              align: 'row',
              hideLabel: false,
              // apiEndpoint: 'https://my.api.mockaroo.com/driver.json?key=a77bb3b0',
              apiEndpoint: `/api/v1/invite/user-roles?label=MyOrganization`,
              params: {},
            },
            extras: {
              labelOverInput: false,
            },
          },
          {
            type: '#select',
            key: 'accountName',
            defaultValue: 'Main',
            disabled: true,
            className: 'disable-form-select-option',
            props: {
              label: 'Account Name',
              hideLabel: false,
              disabled: true,
              align: 'row',
              options: [{ value: 'Main', label: 'Main' }],
            },
          },
          {
            key: 'applications',
            type: '#repeat-grid',
            props: {
              addText: 'Add Application',
              heading: ['', 'Applications *', 'User Roles *'],
              addField: false,
              removeField: false,
              // maxItems:1
            },
            defaultValue: [
              {
                selected: false,
                applicationName: 'LiftiQ',
                userRole: 'Tenant Admin',
              },
              {
                selected: false,
                applicationName: 'ReeferiQ',
                userRole: 'Tenant Admin',
              },
              {
                selected: false,
                applicationName: 'CloudiQ',
                userRole: 'Tenant Admin',
              },
            ],
            fieldArray: {
              fieldGroup: [
                {
                  type: '#checkbox',
                  key: 'selected',
                  className: 'block mb-2',
                  props: {
                    type: 'checkbox',
                    label: '',
                    align: 'row',
                    required: false,
                    async: false,
                    hideLabel: true,
                  },
                  validators: {
                    // validation: [{ name: 'email' }],
                  },
                },
                {
                  type: '#text',
                  key: 'applicationName',
                  className: 'block mb-2',
                  props: {
                    type: 'text',
                    label: 'Application',
                    align: 'row',
                    readonly: true,
                    disabled: false,
                    required: false,
                    async: false,
                    hideLabel: true,
                  },
                  validators: {
                    // validation: [{ name: 'email' }],
                  },
                },
                {
                  key: 'userRole',
                  type: '#select-async',
                  focus: false,
                  props: {
                    label: 'User Roles',
                    options: [],
                    valueProp: 'userRoleName',
                    labelProp: 'userRoleName',
                    async: true,
                    required: false,
                    align: 'row',
                    hideLabel: true,
                    // apiEndpoint: 'https://my.api.mockaroo.com/driver.json?key=a77bb3b0',
                    apiEndpoint: `/api/v1/invite/user-roles?label=MyOrganization`,
                    params: {},
                  },
                  expressions: {
                    'props.required': 'model.application',
                  },
                },
                // {
                //   type: '#select',
                //   key: 'accountName',
                //   defaultValue: 'Main',
                //   disabled: true,
                //   className: 'disable-form-select-option',

                //   props: {
                //     hideLabel: true,
                //     placeholder: 'Account Name',
                //     disabled: true,

                //     options: [{ value: 'Main', label: 'Main' }],
                //   },
                // },
              ],
            },
          },
        ],
        options: {
          submit: {
            label: 'Send Invitation',
            events: [
              {
                // event: 'submit',
                // method: 'getLocalStorage',
                // params: {
                //   storeParams: [
                //     {
                //       key: 'userUUID',
                //     },
                //   ],
                // },
                // events: [
                //   {
                event: 'click',
                method: 'postAPI',
                params: {
                  apiUrl: '/api/v1/users/invitationsss',
                  bodyParams: [
                    {
                      key: 'organizationUUID',
                      value: '',
                    },
                    {
                      key: 'userUUID',
                    },
                  ],
                },
                events: [
                  {
                    event: 'success',
                    method: 'showToast',
                    params: {
                      message: 'User Invited Successfully',
                    },
                    type: '',
                  },
                  {
                    event: 'failure',
                    method: 'showErrorToast',
                    params: {
                      message: 'Error on sending user invitation',
                    },
                    type: '',
                  },
                  {
                    event: 'success',
                    method: 'closeDialog',
                  },

                  {
                    event: 'success',
                    method: 'refreshDataGrid',
                    params: {
                      gridName: 'usersDataGrid',
                    },
                  },
                ],
                //   },
                // ],
              },
            ],
          },
          submitLabel: 'Send Invitation',
          cancel: {
            label: 'Cancel',
            events: [
              {
                event: 'click',
                method: 'closeDialog',
                params: {},
                type: '',
              },
            ],
          },
        },
      },
    },
  ],
};

export const INVITE_USER_FORM = {
  title: {
    label: 'Invite User',
    iconUrl: 'person',
    showBorder: true,
  },
  content: [
    {
      type: 'form',
      props: {
        fields: [
          {
            type: '#textbox-async',
            key: 'email',
            className: 'block mb-2',
            props: {
              type: 'email',
              label: 'Email',
              align: 'row',
              required: true,
              async: true,
              hideLabel: false,
              // apiKey: 'email',
              // errorMessage: `email is already regisitered.`,
              apiEndpoint: '/api/v1/email-unique-status',
              apiKey: 'email',
              errorMessage: `email already exists`,
              valueCheck: true,
            },
            validators: {
              validation: [{ name: 'email' }],
            },
          },
          {
            key: 'userRole',
            type: '#select-async',
            focus: false,
            props: {
              label: 'User Roles',
              options: [],
              valueProp: 'userRoleName',
              labelProp: 'userRoleName',
              async: true,
              required: true,
              align: 'row',
              hideLabel: false,
              // apiEndpoint: 'https://my.api.mockaroo.com/driver.json?key=a77bb3b0',
              apiEndpoint: `/api/v1/invite/user-roles?label={{data.label0}}`,

              params: {},
              description: '',
            },
            extras: {
              labelOverInput: false,
              hintBelowInput: true,
            },
            expressions: {},
          },
          {
            key: 'accountName',
            type: '#select-async',
            focus: false,
            props: {
              label: 'Account',
              options: [],
              valueProp: 'uuid',
              labelProp: 'accountName',
              async: true,
              required: true,
              align: 'row',
              hideLabel: false,
              apiEndpoint: `/api/v1/organizations/accounts?viewType=Dropdown&organizationUUID=e520d1b4-437c-4d48-a0ed-13489ab52074`,
              params: {},
              description: '',
            },
            extras: {
              labelOverInput: false,
              hintBelowInput: true,
            },
            expressions: {},
          },
          // {
          //   type: '#select',
          //   key: 'accountName',
          //   defaultValue: 'Main',
          //   disabled: true,
          //   appearance: 'legacy',
          //   className: 'no-border disable-form-select-option',
          //   props: {
          //     label: 'Account',
          //     hideLabel: false,
          //     disabled: true,
          //     align: 'row',
          //     options: [{ value: 'Main', label: 'Main' }],
          //   },
          // },
          {
            key: 'applicationName',
            type: '#checkbox-multiple',
            props: {
              label: 'Applications',
              align: 'row',
              required: true,
              className: 'disable-form-select-option',
              // valueProp: 'applicationName',
              // labelProp: 'applicationName',
              async: false,
              disabled: true,
              apiEndpoint: `/api/v1/applications`,
              params: {
                bodyParams: [
                  {
                    key: 'userUUID',
                  },
                ],
              },
              options: [
                { value: 'liftiq', label: 'LiftiQ' },
                { value: 'reeferiq', label: 'ReeferiQ' },
                { value: 'tankiq', label: 'TankiQ' },
              ],
            },
            defaultValue: {
              liftiq: true,
              reeferiq: false,
              tankiq: true,
            },
          },
          // {
          //   type: '#template',
          //   template:
          //     '<p class="mt-2 text-sm">Note: The <b>"Organization Member"</b> will only get the access to monitor the selected applications.</p>',
          // },
          {
            type: '#template',
            template:
              '<div class=" text-sm text-gray-600" ><span class="text-base material-icons">info</span> Note: The <b class="font-bold">"Organization Member"</b> will only get the access to monitor the selected applications.</div>',
          },
        ],
        options: {
          submit: {
            label: 'Send Invitation',
            events: [
              {
                // event: 'submit',
                // method: 'getLocalStorage',
                // params: {
                //   storeParams: [
                //     {
                //       key: 'userUUID',
                //     },
                //   ],
                // },
                // events: [
                //   {
                event: 'click',
                method: 'postAPI',
                params: {
                  apiUrl: '/api/v1/users/invitationsss',
                  bodyParams: [
                    {
                      key: 'organizationUUID',
                      value: '',
                    },
                    {
                      key: 'userUUID',
                    },
                  ],
                },
                events: [
                  {
                    event: 'success',
                    method: 'showToast',
                    params: {
                      message: 'User Invited Successfully',
                    },
                    type: '',
                  },
                  {
                    event: 'failure',
                    method: 'showErrorToast',
                    params: {
                      message: 'Error on sending user invitation',
                    },
                    type: '',
                  },
                  {
                    event: 'success',
                    method: 'closeDialog',
                  },

                  {
                    event: 'success',
                    method: 'refreshDataGrid',
                    params: {
                      gridName: 'usersDataGrid',
                    },
                  },
                ],
                //   },
                // ],
              },
            ],
          },
          submitLabel: 'Send Invitation',
          cancel: {
            label: 'Cancel',
            events: [
              {
                event: 'click',
                method: 'closeDialog',
                params: {},
                type: '',
              },
            ],
          },
        },
      },
    },
  ],
};

export const TENANT_CREATE_CUSTOMER_DIALOG_FORM = {
  title: {
    label: 'Create Customer',
    iconUrl: 'corporate_fare',
    showBorder: true,
  },
  content: [
    {
      type: 'form',
      props: {
        name: 'createCustomerForm',
        fields: [
          {
            key: 'applicationProvider',
            type: '#select-async',
            focus: false,
            props: {
              label: 'Application Provider',
              options: [],
              valueProp: 'uuid',
              labelProp: 'organizationName',
              async: true,
              required: true,
              align: 'row',
              hideLabel: false,
              apiEndpoint: `/api/v1/application-providers`,
              params: {
                bodyParams: [
                  {
                    key: 'uuid',
                  },
                ],
              },
            },
          },
          {
            type: '#textbox-async',
            key: 'email',
            className: 'block mb-1',
            props: {
              type: 'email',
              label: 'Email',
              align: 'row',
              required: true,
              async: true,
              hideLabel: false,
              // apiKey: 'email',
              // errorMessage: `email is already regisitered.`,
              apiEndpoint: '/api/v1/email-unique-status',
              apiKey: 'email',
              errorMessage: `email already exists`,
              valueCheck: true,
            },
            validators: {
              validation: [{ name: 'email' }],
            },
          },
          {
            type: '#textbox-async',
            key: 'organizationName',
            className: 'block mb-1',
            props: {
              label: 'Organization Name',
              align: 'row',
              required: true,
              async: true,
              hideLabel: false,
              apiEndpoint: '/api/v1/organizations',
              apiKey: 'organizationName',
              errorMessage: `Organization name already exists`,
              valueCheck: true,
            },
          },
          {
            type: '#text',
            key: 'userRoleName',
            props: {
              label: 'User Role',
              readonly: true,
              disabled: true,
              hideLabel: false,
              align: 'row',
              description: '',
            },
            extras: {
              hintBelowInput: true,
            },
            defaultValue: 'Organization Owner',
          },
          {
            type: '#template',
            template: '<hr class="mt-0">',
            hideExpression: '!model.applicationProvider',
          },
          // {
          //   key: 'appl',
          //   type: '#select-async',
          //   props: {
          //     label: 'Applications',
          //     options: [],
          //     valueProp: 'applicationUUID',
          //     labelProp: 'applicationName',
          //     async: true,
          //     align: 'row',
          //     required: true,
          //     // apiEndpoint: 'https://my.api.mockaroo.com/tasks.json?key=a77bb3b0',
          //     apiEndpoint: '/api/v1/organizations',
          //     dependentField: 'applicationProvider',
          //     dependentFieldKey: 'applications',
          //   },
          //   expressions: {},
          // },
          {
            key: 'applications',
            type: '#checkbox-multiple',
            props: {
              label: 'Applications',
              valueProp: 'applicationName',
              labelProp: 'applicationName',
              async: true,
              align: 'row',
              required: true,
              // apiEndpoint: 'https://my.api.mockaroo.com/tasks.json?key=a77bb3b0',
              apiEndpoint: '/api/v1/organizations',
              options: [],
              dependentField: 'applicationProvider',
              dependentFieldKey: 'applications',
            },
            hideExpression: '!model.applicationProvider',
            expressions: {},
          },
          // {
          //   key: 'applications',
          //   type: '#checkbox-multiple',
          //   props: {
          //     label: 'Applications',
          //     align: 'row',
          //     required: true,
          //     valueProp: 'applicationUUID',
          //     labelProp: 'applicationName',
          //     async: true,
          //     apiEndpoint: `/api/v1/organizations`,
          //     // params: {
          //     //   bodyParams: [
          //     //     {
          //     //       key: 'userUUID',
          //     //       value: '{{data.localStorage.userUUID}}',
          //     //     },
          //     //     // {
          //     //     //   key: 'applicationProviderUUID',
          //     //     //   value: '{{data.createCustomerForm.applicationProvider}}',
          //     //     // },
          //     //   ],
          //     // },
          //     options: [
          //       // { value: 'liftiq', label: 'LiftiQ' },
          //       // { value: 'reeferiq', label: 'ReeferiQ' },
          //       // { value: 'cloudiq', label: 'CloudiQ' },
          //       // { value: 'installiq', label: 'InstalliQ' },
          //       // { value: 'tankiq', label: 'TankiQ' },
          //     ],
          //     dependentField: 'applicationProvider',
          //     dependentFieldKey: 'applications',
          //   },
          //   // hideExpression: '!model.applicationProvider',
          //   // expressions: {
          //   //   'props.disabled': '!model.applicationProvider',

          //   // },
          // },
          {
            type: '#template',
            template: `<div class=" text-sm text-gray-600" ><span class="text-base material-icons">info</span> The <span class="font-bold">Organization Owner</span> Customer will get the access to control and monitor the selected applications.</div>`,
            hideExpression: '!model.applicationProvider',
          },
        ],
        options: {
          submit: {
            label: 'Send Invitation',
            events: [
              {
                event: 'click',
                method: 'postAPI',
                params: {
                  apiUrl: '/api/v1/customer-invitationsss',
                  bodyParams: [
                    {
                      key: 'userUUID',
                    },
                    {
                      key: 'userRoleName',
                      value: 'Organization Owner',
                    },
                  ],
                },
                events: [
                  {
                    event: 'success',
                    method: 'showToast',
                    enabled:
                      'data[0].message === "Successfully invited customer"',
                    params: {
                      message: 'User Invited Successfully',
                    },
                    type: '',
                  },
                  {
                    event: 'failure',
                    method: 'showErrorToast',
                    enabled:
                      'data[0].message !== "Successfully invited customer"',

                    params: {
                      message: 'Error on sending user invitation',
                    },
                    type: '',
                  },
                  {
                    event: 'success',
                    method: 'closeDialog',
                    enabled:
                      'data[0].message === "Successfully invited customer"|| data[0].message !== "Successfully invited customer"',
                  },
                  {
                    event: 'success',
                    method: 'refreshDataGrid',
                    enabled:
                      'data[0].message === "Successfully invited customer"',

                    params: {
                      gridName: 'customerDataGrid',
                    },
                  },
                ],
              },
            ],
          },
          submitLabel: 'Invite User',
          cancel: {
            label: 'Cancel',
            events: [
              {
                event: 'click',
                method: 'closeDialog',
                params: {},
                type: '',
              },
            ],
          },
        },
      },
    },
  ],
};
