/* eslint-disable @typescript-eslint/no-empty-function */
/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
import { HttpClient } from '@angular/common/http';
import { Component, OnInit } from '@angular/core';
import { ReportGridSelectors } from '@hash8io/common-store-web/report-grid-store';
import { Model, BreakpointService } from '@hash8io/core-web';
import { FormOptions, FormFields } from '@hash8io/forms-web';
import { Store } from '@ngrx/store';
import { Subject, takeUntil } from 'rxjs';

@Component({
  selector: 'hash8-app-all-form-elements',
  templateUrl: './all-form-elements.component.html',
  styleUrls: ['./all-form-elements.component.scss'],
})
export class AllFormElementsComponent implements OnInit {
  title: Model.Title = {
    label: 'All Form Elements',
    // iconUrl: 'list_alt',
  };

  model: any = {};

  options: FormOptions = {
    formState: {
      text: '',
    },
  };

  existingUsers = ['user1', 'user2', 'user3'];

  fields: FormFields[] = [
    // {
    //   type: '#template',
    //   template: '<h5 class="forms-subheading p-0 mt-5 mb-2">Form Elements</h5>',
    // },
    {
      type: '#template',
      template: '<h5 class="forms-subheading p-0 mt-5 mb-2">Text Element</h5>',
    },
    {
      type: 'addEntity',
      key: 'assets',
      props: {
        type: 'addEntity',
        label: 'Assets',
        showLabel: true,
        selectionEntitiesConfig: {
          name: 'FleetSelectionAssetsDataGrid',
          apiUrl: `/api/mock/selectionAssets?organizationUUID={{data.orgFilterUUIDs}}`,
          // apiUrl: `/api/v1/users?organizationUUID={{data.orgFilterUUIDs}}`,
          // organizationFilterConfig: this.orgConfig,
        },
        selectedEntitiesConfig: {
          name: 'FleetSelectedAssetsDataGrid',
          apiUrl: `/api/mock/assetsSelected?organizationUUID={{data.orgFilterUUIDs}}`,
          // apiUrl: `/api/v1/users?organizationUUID={{data.orgFilterUUIDs}}`,
          // organizationFilterConfig: this.orgConfig,
        },
      },
    },
    {
      type: '#text',
      props: {
        description: 'Description',
        label: 'Text',
        align: 'row',
      },
    },
    {
      type: '#text',
      props: {
        description: '',
        label: 'Text with Prefix',
        align: 'row',
        prefix: 'DL-',
        mask: '0{6}',
      },
    },
    {
      type: '#text',
      key: 'text1',
      props: {
        description: '',
        label: 'Text with Minlength and Maxlength',
        align: 'row',
        minLength: 8,
        maxLength: 12,
      },
    },
    {
      type: '#template',
      template: '<hr>',
    },
    {
      type: '#template',
      template:
        '<h5 class="forms-subheading p-0 mt-5 mb-2">Number Element</h5>',
    },
    {
      type: '#number',
      props: {
        label: 'Number',
        align: 'row',
      },
    },
    {
      type: '#number',
      key: 'number2',
      props: {
        description: '',
        label: 'Number with Min and Max',
        align: 'row',
        min: 18,
        max: 50,
      },
    },
    {
      type: '#decimal',
      props: {
        label: 'Decimal Number',
        align: 'row',
      },
    },
    {
      type: '#template',
      template: '<hr>',
    },
    {
      type: '#template',
      template:
        '<h5 class="forms-subheading p-0 mt-5 mb-2">Textarea Element</h5>',
    },
    {
      type: '#textarea',
      props: {
        label: 'Textarea',
        align: 'row',
      },
    },
    {
      type: '#template',
      template: '<hr>',
    },
    {
      type: '#template',
      template:
        '<h5 class="forms-subheading p-0 mt-5 mb-2">Checkbox Element</h5>',
    },
    {
      type: '#checkbox',
      props: {
        text: 'Accept terms and conditions',
        align: 'row',
        label: 'Checkbox',
      },
    },
    {
      type: '#checkbox-multiple',
      className: 'mt-6',
      props: {
        label: 'Multiple Checkbox',
        align: 'row',
        options: [
          { value: 1, label: 'RT' },
          { value: 2, label: 'Teezle' },
          { value: 3, label: 'Autocavo' },
          { value: 4, label: 'Hash8' },
        ],
      },
    },
    {
      type: '#template',
      template: '<hr class="mt-3">',
    },
    {
      type: '#template',
      template:
        '<h5 class="forms-subheading p-0 mt-5 mb-2">Select Element</h5>',
    },
    {
      type: '#select',
      props: {
        align: 'row',
        options: [
          { value: 1, label: 'RT' },
          { value: 2, label: 'Teezle' },
          { value: 3, label: 'Autocavo' },
          { value: 4, label: 'Hash8' },
        ],
      },
    },
    {
      type: '#select',
      key: 'select1',
      props: {
        align: 'row',
        label: 'Select with Group',
        // required: true,
        options: [
          {
            label: 'Teezle',
            group: [
              { label: 'Chennai', value: '1' },
              { label: 'Hosur', value: '2' },
            ],
          },
          {
            label: 'Hash8',
            group: [{ label: 'Denkanikottai', value: '3' }],
          },
        ],
      },
    },
    {
      type: '#select-multiple',
      props: {
        align: 'row',
        options: [
          { value: 'A', label: 'Option 1' },
          { value: 'B', label: 'Option 2' },
          { value: 'C', label: 'Option 3' },
          { value: 'D', label: 'Option 4', disabled: true },
        ],
      },
    },
    {
      type: '#template',
      template: '<hr>',
    },
    {
      type: '#template',
      template:
        '<h5 class="forms-subheading p-0 mt-5 mb-2">Toggle Element</h5>',
    },
    {
      type: '#toggle',
      props: {
        align: 'row',
        text: 'Accept terms and conditions',
      },
    },
    {
      type: '#template',
      template: '<hr class="mt-6">',
    },
    {
      type: '#template',
      template:
        '<h5 class="forms-subheading p-0 mt-5 mb-2">Slider Element</h5>',
    },
    {
      type: '#slider',
      className: 'mt-5',
      defaultValue: 40,
      props: {
        align: 'row',
      },
    },
    {
      type: '#template',
      template: '<hr>',
    },
    {
      type: '#template',
      template: '<h5 class="forms-subheading p-0 mt-5 mb-2">Radio Element</h5>',
    },
    {
      type: '#radio',
      props: {
        align: 'row',
        options: [
          { value: 1, label: 'Option 1' },
          { value: 2, label: 'Option 2' },
          { value: 3, label: 'Option 3' },
          { value: 4, label: 'Option 4', disabled: true },
        ],
      },
    },

    // {
    //   type: '#date',
    //   props: {
    //     align: 'row',}
    // },
    // {
    //   type: '#email',
    //   props: {
    //     align: 'row',
    //   },
    // },
    // {
    //   type: '#phoneNumber',
    //   props: {
    //     align: 'row',
    //   },
    // },
    {
      type: '#template',
      template: '<hr class="mt-6">',
    },
    {
      type: '#template',
      template:
        '<h5 class="forms-subheading p-0 mt-5 mb-2">Password Element</h5>',
    },
    {
      type: '#password',
      props: {
        align: 'row',
      },
    },
    {
      type: '#template',
      template: '<hr>',
    },
    {
      type: '#template',
      template:
        '<h5 class="forms-subheading p-0 mt-5 mb-2">Datepicker Element</h5>',
    },
    {
      type: '#group',
      fieldGroup: [
        {
          key: 'Datepicker',
          type: 'datepicker',
          wrappers: ['panel', 'form-field'],
          props: {
            label: 'Datepicker',
            placeholder: 'Datepicker',
            hideLabel: false,
            align: 'row',
            // description: 'Description',
            // required: true,
          },
        },
      ],
    },
    {
      type: '#template',
      template: '<hr>',
    },
    {
      type: '#template',
      template: '<h5 class="forms-subheading p-0 mt-5 mb-2">Show Hide</h5>',
    },
    {
      key: 'awesome',
      type: '#checkbox',
      defaultValue: true,
      props: {
        label: '',
        appearance: 'standard',
        hideLabel: true,
      },
      expressions: {
        'props.disabled': 'formState.awesomeIsForced',
        'props.label': ({ options: any }) => {
          if (this.model?.whyNot && this.model?.whyNot !== '') {
            return 'Forms is really awesome...';
          } else {
            return 'Is Organization default to all entity? (uncheck this and see what happens)';
          }
        },
      },
    },
    {
      key: 'whyNot',
      type: '#textarea',
      props: {
        label: 'Please provide some suggestions?',
        placeholder: 'Type in here...',
      },
      expressions: {
        hide: 'model.awesome',
        'props.disabled': 'formState.awesomeIsForced',
      },
    },
    {
      type: '#template',
      template: '<hr>',
    },
    {
      type: '#template',
      template:
        '<h5 class="forms-subheading p-0 mt-5 mb-2">Field Matching</h5>',
    },

    {
      // type: 'match',
      validators: {
        validation: [
          { name: 'fieldMatch', options: { errorPath: 'confirmPassword' } },
        ],
      },
      type: '#group',
      fieldGroup: [
        {
          key: 'password',
          type: '#password',
          props: {
            label: 'Password',
            placeholder: 'Must be at least 8 characters',
            required: true,
            align: 'row',
          },
        },
        {
          key: 'confirmPassword',
          type: '#password',
          props: {
            label: 'Confirm Password',
            placeholder: 'Please re-enter your password',
            required: true,
            align: 'row',
          },
        },
      ],
    },
    {
      type: '#template',
      template: '<hr>',
    },
    {
      type: '#template',
      template:
        '<h5 class="forms-subheading p-0 mt-5 mb-2">Matching Field using API</h5>',
    },
    {
      type: '#textbox-async',
      key: 'username',
      props: {
        label: 'Username',
        align: 'row',
        required: true,
        async: true,
        description:
          'Pick a unique username for your account. (Try: Bret, Antonette, Samantha)',
        apiEndpoint: 'https://jsonplaceholder.typicode.com/users',
        apiKey: 'username',
        errorMessage: `username is already taken.`,
      },
    },
    {
      type: '#template',
      template: '<hr>',
    },
    {
      type: '#template',
      template:
        '<h5 class="forms-subheading p-0 mt-5 mb-2">More Field Types</h5>',
    },
    {
      type: '#ip-address',
      props: {
        align: 'row',
      },
    },
    {
      type: '#color',
      wrappers: ['panel', 'form-field'],
      key: 'color3',
      props: {
        align: 'row',
        description: '',
      },
    },
    {
      type: '#odometer',
      props: {
        label: 'Vehicle Odometer (Mask)',
        align: 'row',
      },
    },
    {
      type: '#template',
      template: '<hr>',
    },

    {
      type: '#template',
      template:
        '<h5 class="forms-subheading p-0 mt-5 mb-2">Form Field Alignments</h5>',
    },

    {
      type: '#template',
      template:
        '<h5 class="forms-subheading p-0 mt-5 mb-2">Top Label and Bottom Element</h5>',
    },
    {
      type: '#phoneNumber',
      key: 'newPhoneNumber',
      className: 'w-full',
      props: {
        label: 'Office Phone Number',
        align: 'column',
      },
    },
    {
      type: '#phoneNumber',
      key: 'newPhoneNumber',
      props: {
        label: 'New Phone Number',
        prefix: '+1 ',
        description: 'US Phone numbers only',
      },
    },
    {
      type: '#template',
      template:
        '<h5 class="forms-subheading p-0 mt-5 mb-2">Left Label and Right Element</h5>',
    },
    {
      type: '#phoneNumber',
      key: 'newPhoneNumber',
      className: 'w-full',
      props: {
        label: 'Office Phone Number',
        align: 'row',
      },
    },
    {
      type: '#phoneNumber',
      key: 'newPhoneNumber',
      props: {
        label: 'New Phone Number',
        mask: '000-000-0000',
        align: 'row',
        description: 'US Phone numbers only',
      },
    },
    {
      type: '#phoneNumber',
      key: 'newPhoneNumber',
      props: {
        label: 'New Phone Number',
        mask: '000-000-0000',
        align: 'row',
        hideLabel: false,
        description: 'US Phone numbers only',
      },
      extras: {
        labelOverInput: false,
        hintBelowInput: true,
      },
    },
  ];

  constructor(
    private _breakpoint: BreakpointService,
    private http: HttpClient,
    private store: Store
  ) {
    this._breakpoint.active$.subscribe((breakpoint) => {
      const _breakpoint = breakpoint as keyof Model.Breakpoint;
    });
  }

  private readonly destroySubscription$ = new Subject<void>();

  ngOnInit(): void {
    let resourcesAdded: any[] = [];
    let resourcesRemoved: any[] = [];

    // this.store
    //   .select(
    //     ReportGridSelectors.selectAddedRows('FleetSelectedAssetsDataGrid')
    //   )
    //   .pipe(takeUntil(this.destroySubscription$))
    //   .subscribe((res: any) => {
    //     console.log('ADDED ROWS', res.addedRows);
    //     resourcesAdded = resourcesAdded.filter(
    //       (x: any) => x.uuid != res.addedRows[0].uuid
    //     );
    //     if (
    //       resourcesRemoved.length == 0 ||
    //       resourcesRemoved.filter((x: any) => x.uuid === res.addedRows[0].uuid)
    //         .length === 0
    //     ) {
    //       resourcesAdded.push({
    //         uuid: res.addedRows[0].uuid,
    //         action: 'add',
    //         fleetName: res.addedRows[0].fleetName,
    //       });
    //     } else {
    //       resourcesRemoved = resourcesRemoved.filter(
    //         (x: any) => x.uuid != res.addedRows[0].uuid
    //       );
    //     }

    //     console.log('resourcesAdded', resourcesAdded);
    //     console.log('resourcesRemoved', resourcesRemoved);
    //   });

    // this.store
    //   .select(
    //     ReportGridSelectors.selectRemovedRows('FleetSelectedAssetsDataGrid')
    //   )
    //   .pipe(takeUntil(this.destroySubscription$))
    //   .subscribe((res: any) => {
    //     console.log('Removed ROWS', res.removedRows);
    //     resourcesRemoved = resourcesRemoved.filter(
    //       (x) => x.uuid != res.removedRows[0].uuid
    //     );
    //     if (
    //       resourcesAdded.length === 0 ||
    //       resourcesAdded.filter((x: any) => x.uuid === res.removedRows[0].uuid)
    //         .length === 0
    //     ) {
    //       resourcesRemoved.push({
    //         uuid: res.removedRows[0].uuid,
    //         action: 'removed',
    //         fleetName: res.removedRows[0].fleetName,
    //       });
    //     } else {
    //       resourcesAdded = resourcesAdded.filter(
    //         (x: any) => x.uuid != res.removedRows[0].uuid
    //       );
    //     }

    //     console.log('resourcesAdded RM', resourcesAdded);
    //     console.log('resourcesRemoved RM', resourcesRemoved);
    //   });
  }

  ngOnDestroy() {
    this.destroySubscription$.next();
    this.destroySubscription$.complete();
  }
  modelChange(event: any) {
    // console.log(event);
  }

  onSubmit(model: any) {
    console.log(model);
  }
}
