/* eslint-disable @typescript-eslint/no-empty-function */
/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
import { HttpClient } from '@angular/common/http';
import { Component, OnInit } from '@angular/core';
import { AuthenticationService } from '@hash8io/authentication-web/login';
import { FormFields, FormsWebService } from '@hash8io/forms-web';
import { map, Observable } from 'rxjs';

@Component({
  selector: 'hash8-app-asynchronous-form-page',
  templateUrl: './asynchronous-form-page.component.html',
  styleUrls: ['./asynchronous-form-page.component.scss'],
})
export class AsynchronousFormPageComponent implements OnInit {
  model = {};
  fields: FormFields[] = [
    {
      type: '#template',
      template: '<h3 class="forms-heading">API Form elements</h3>',
    },
    {
      type: '#template',
      template: `<h5 class="forms-subheading">API Dropdowns</h5>
        <p class="forms-description mb-4">Individual select dropdowns loaded from API endpoint.</p>`,
    },
    {
      key: 'driver',
      type: '#select-async',
      props: {
        label: 'Driver',
        options: [],
        valueProp: 'id',
        labelProp: 'name',
        async: true,
        required: false,
        align: 'row',
        // apiEndpoint: 'https://my.api.mockaroo.com/driver.json?key=a77bb3b0',
        apiEndpoint: 'https://63239166bb2321cba91c006d.mockapi.io/users',
      },
    },
    {
      type: '#template',
      template: '<hr>',
    },
    {
      type: '#template',
      template:
        '<h5 class="forms-subheading p-0 mb-4">Drilldown API Dropdowns</h5>',
    },
    {
      key: 'user',
      type: '#select-async',
      props: {
        label: 'Users',
        placeholder: 'Select a User',
        options: [],
        valueProp: 'id',
        labelProp: 'first_name',
        async: true,
        required: false,
        align: 'row',
        apiEndpoint: 'https://63239166bb2321cba91c006d.mockapi.io/users',
      },
      // defaultValue: 3,
    },
    {
      key: 'task',
      type: '#select-async',
      props: {
        label: "User's Task",
        options: [],
        valueProp: 'id',
        labelProp: 'task',
        description: 'Select a user to list tasks',
        async: true,
        align: 'row',
        required: true,
        // apiEndpoint: 'https://my.api.mockaroo.com/tasks.json?key=a77bb3b0',
        apiEndpoint: 'https://63239166bb2321cba91c006d.mockapi.io/users',
        dependentField: 'user',
        dependentFieldKey: 'tasks',
      },
      expressions: {
        'props.disabled': '!model.user',
      },
    },
    {
      type: '#template',
      template: '<hr>',
    },
    {
      type: '#template',
      template:
        ' <h5 class="forms-subheading p-0 mt-4 mb-4">Check Unique value with API</h5>',
    },
    {
      type: '#textbox-async',
      key: 'username',
      props: {
        label: 'Username',
        align: 'row',
        required: true,
        async: true,
        description:
          'Pick a unique username for your account. (Try: Bret, Antonette, Samantha)',
        apiEndpoint: 'https://jsonplaceholder.typicode.com/users',
        apiKey: 'username',
        errorMessage: `username is already taken.`,
      },
    },
  ];

  constructor(
    private http: HttpClient,
    private authService: AuthenticationService,
    private formService: FormsWebService
  ) {}

  ngOnInit(): void {}

  loadOptions() {
    this.authService
      .login({ username: 'jerry', password: '23434' })
      .subscribe();
  }

  loadDropdown(): Observable<any[]> {
    return this.http.get('https://reqres.in/api/users?page=1').pipe(
      map((response: any) => {
        return response.data;
      })
    );
  }

  getTodos(id?: any): Observable<any[]> {
    // https://jsonplaceholder.typicode.com/todos/
    return this.http.get('https://jsonplaceholder.typicode.com/todos').pipe(
      map((response: any) => {
        const options = response.filter(
          (team: { userId: any }) => team.userId === id
        );
        // if (!options.find((option) => sportId === option.id)) {
        //   field.formControl?.setValue(null);
        // }
        return options;
      })
    );
  }

  checkUserNameExist(username: string): Observable<boolean> {
    return this.http.get('https://jsonplaceholder.typicode.com/users').pipe(
      map((response: any) => {
        const options = response.filter(
          (users: { username: string }) => users.username === username
        );
        return options.length === 0;
      })
    );
  }
}
