import { FormModel } from '@hash8io/forms-web';

export const REGISTRATION_FORM: FormModel = {
  fields: [
    {
      type: '#template',
      template: `<h3 class="forms-heading">Sign Up</h3>
      <h5 class="forms-subheading">Type 1</h5>
      <p class="forms-description text-base mb-3">Sign up to Hash8 IoT system</p>`,
    },
    {
      type: '#group',
      fieldGroupClassName: 'lg:flex gap-3',
      fieldGroup: [
        {
          type: '#text',
          className: 'w-full',
          props: {
            label: 'First Name',
            required: true,
            hideLabel: true,
          },
          extras: {
            labelOverInput: true,
          },
        },
        {
          type: '#text',
          className: 'w-full',
          props: {
            label: 'Last Name',
            required: true,
            hideLabel: true,
          },
          extras: {
            labelOverInput: true,
          },
        },
      ],
    },
    {
      type: '#text',
      key: 'email',
      props: {
        label: 'Email',
        type: 'email',
        readonly: true,
        disabled: true,
        hideLabel: true,
      },
      extras: {
        labelOverInput: true,
      },
      defaultValue: 'johnmathew@gmail.com',
      expressions: {
        // 'props.disabled': '!model.username',
      },
    },
    {
      type: '#textbox-async',
      key: 'username',
      props: {
        label: 'Username',
        required: true,
        hideLabel: true,
        apiEndpoint: 'https://jsonplaceholder.typicode.com/users',
        apiKey: 'username',
        errorMessage: `username is already taken.`,
        minLength: 3,
      },
      extras: {
        labelOverInput: true,
      },
    },
    {
      type: '#phoneNumber',
      props: {
        hideLabel: true,
      },
      extras: {
        labelOverInput: true,
      },
    },
    {
      validators: {
        validation: [
          { name: 'fieldMatch', options: { errorPath: 'confirmPassword' } },
        ],
      },
      type: '#group',
      fieldGroup: [
        {
          type: '#password',
          key: 'password',
          props: {
            hideLabel: true,
          },
          extras: {
            labelOverInput: true,
          },
        },
        {
          type: '#password',
          key: 'confirmPassword',
          props: {
            label: 'Confirm Password',
            hideLabel: true,
          },
          extras: {
            labelOverInput: true,
          },
        },
      ],
    },
    {
      type: '#template',
      template: `<p class="text-xs -mt-1 mb-2 opacity-50">Note: Your password should meet all the following conditions.<br>
    * Atleast one upper case letter.<br>
    * Atleast one number.<br>
    * Min.8 characters in length.</p>`,
    },
  ],
  options: {
    submitLabel: 'Register Form',
    hasReset: false,
    submitLabelClass: ['w-full'],
  },
};

export const REGISTRATION_FORM2: FormModel = {
  fields: [
    {
      type: '#template',
      template: `<h3 class="forms-heading">Registration</h3>
      <h5 class="forms-subheading">Type 2</h5>
        <p class="forms-description text-base mb-3">Sign up to Hash8 IoT system</p>`,
    },
    {
      type: '#group',
      fieldGroupClassName: 'lg:flex gap-3',
      fieldGroup: [
        {
          type: '#text',
          className: 'w-full',
          props: {
            label: 'First Name',
            required: true,
          },
        },
        {
          type: '#text',
          className: 'w-full',
          props: {
            label: 'Last Name',
            required: true,
          },
        },
      ],
    },
    {
      type: '#text',
      key: 'email',
      props: {
        label: 'Email',
        type: 'email',
        readonly: true,
        disabled: true,
      },
      defaultValue: 'johnmathew@gmail.com',
      expressions: {
        // 'props.disabled': '!model.username',
      },
    },
    {
      type: '#textbox-async',
      key: 'username',
      props: {
        label: 'Username',
        required: true,
        apiEndpoint: 'https://jsonplaceholder.typicode.com/users',
        apiKey: 'username',
        errorMessage: `username is already taken.`,
        minLength: 3,
      },
    },
    {
      type: '#phoneNumber',
    },
    {
      type: '#group',
      validators: {
        validation: [
          { name: 'fieldMatch', options: { errorPath: 'confirmPassword' } },
        ],
      },
      fieldGroup: [
        {
          type: '#password',
          key: 'password',
        },
        {
          type: '#password',
          key: 'confirmPassword',
          props: {
            label: 'Confirm Password',
          },
        },
      ],
    },
    {
      type: '#template',
      template: `<p class="text-xs -mt-1 mb-2 opacity-50">Note: Your password should meet all the following conditions.<br>
    * Atleast one upper case letter.<br>
    * Atleast one number.<br>
    * Min.8 characters in length.</p>`,
    },
  ],
};

export const STEPPER_FORM: FormModel = {
  fields: [
    {
      type: '#template',
      template: `<h3 class="forms-heading mb-1">Create your Sales Channel</h3>
      <p class="forms-description text-base mb-3">Please provide your organization details.</p>`,
    },
    {
      type: 'stepper',
      fieldGroup: [
        {
          type: '#group',
          props: { label: 'Organization' },
          fieldGroup: [
            {
              key: 'salesChannelName',
              type: '#text',
              props: {
                label: 'Sales Channel Name',
                required: true,
                hideLabel: false,
                // align: 'row',
              },
              extras: {
                // labelOverInput: true,
              },
            },
            {
              key: 'address',
              type: '#text',
              props: {
                label: 'Address Line 1',
                required: true,
                hideLabel: false,
                // align: 'row',
              },
              extras: {
                // labelOverInput: true,
              },
            },
            {
              key: 'address2',
              type: '#text',
              props: {
                label: 'Address Line 2',
                hideLabel: false,
                // align: 'row',
              },
              extras: {
                // labelOverInput: true,
              },
            },
            {
              key: 'country',
              type: '#select-async',
              props: {
                label: 'Country',
                required: true,
                async: true,
                labelProp: 'country',
                valueProp: 'id',
                apiEndpoint:
                  'https://63239166bb2321cba91c006d.mockapi.io/country',
                hideLabel: false,
                // align: 'row',
              },
              extras: {
                // labelOverInput: true,
              },
            },
            {
              key: 'state',
              type: '#select-async',
              props: {
                label: 'State',
                required: true,
                async: true,
                labelProp: 'state',
                valueProp: 'id',
                apiEndpoint:
                  'https://63239166bb2321cba91c006d.mockapi.io/country',
                hideLabel: false,
                // align: 'row',
              },
              extras: {
                // labelOverInput: true,
              },
            },
            {
              key: 'city',
              type: '#select-async',
              props: {
                label: 'City',
                async: true,
                required: true,
                labelProp: 'city',
                valueProp: 'id',
                apiEndpoint:
                  'https://63239166bb2321cba91c006d.mockapi.io/country',
                hideLabel: false,
                // align: 'row',
              },
              extras: {
                // labelOverInput: true,
              },
            },
            {
              key: 'zipcode',
              type: '#number',
              props: {
                label: 'Zip Code',
                mask: '0{5}',
                hideLabel: false,
                // align: 'row',
              },
              extras: {
                // labelOverInput: true,
              },
            },
          ],
        },
        {
          props: { label: 'Profile Details' },
          fieldGroup: [
            {
              fieldGroupClassName: 'lg:flex gap-3',
              fieldGroup: [
                {
                  type: '#text',
                  key: 'firstName',
                  className: 'w-full',
                  props: {
                    label: 'First Name',
                    required: true,
                  },
                },
                {
                  type: '#text',
                  className: 'w-full',
                  key: 'lastName',
                  props: {
                    label: 'Last Name',
                    required: true,
                  },
                },
              ],
            },
            {
              type: '#text',
              key: 'email',
              props: {
                label: 'Email',
                type: 'email',
                readonly: true,
                // disabled: true,
              },
              defaultValue: 'johnmathew@gmail.com',
              expressions: {
                // 'props.disabled': '!model.username',
              },
            },
            {
              type: '#textbox-async',
              key: 'username',
              props: {
                label: 'Username',
                required: true,
                apiEndpoint: 'https://jsonplaceholder.typicode.com/users',
                apiKey: 'username',
                errorMessage: `username is already taken.`,
                minLength: 3,
              },
            },
            {
              type: '#phoneNumber',
              key: 'phoneNumber'
            },
            {
              validators: {
                validation: [
                  {
                    name: 'fieldMatch',
                    options: { errorPath: 'confirmPassword' },
                  },
                ],
              },
              fieldGroup: [
                {
                  type: '#password',
                  key: 'password',
                },
                {
                  type: '#password',
                  key: 'confirmPassword',
                  props: {
                    label: 'Confirm Password',
                  },
                },
              ],
            },
            {
              type: '#template',
              template: `<p class="text-xs -mt-1 mb-2 opacity-50">Note: Your password should meet all the following conditions.<br>
    * Atleast one upper case letter.<br>
    * Atleast one number.<br>
    * Min.8 characters in length.</p>`,
            },
          ],
        },
        {
          props: { label: 'Confirmation' },
          fieldGroup: [
            // {
            //   key: 'day',
            //   type: 'input',
            //   props: {
            //     type: 'date',
            //     label: 'Day of the trip',
            //     required: true,
            //   },
            // },
            {
              key: 'repeat_grid_type_3',
              type: '#repeat-grid',
              props: {
                addText: 'Add New Sales Channel',
                heading: [
                  'Service Name',
                  'Odometer',
                  'Service Interval',
                  'Disabled Name',
                ],
                addField: true,
                removeField: false,
              },
              fieldArray: {
                fieldGroup: [
                  {
                    type: '#label',
                    key: 'name',
                    props: {
                      required: true,
                      hideLabel: true,
                    },
                  },
                  {
                    type: '#odometer',
                    props: {
                      required: true,
                      hideLabel: true,
                    },
                  },
                  {
                    type: '#text',
                    key: 'owner_email',
                    props: {
                      placeholder: 'Service Interval',
                      required: true,
                      hideLabel: true,
                    },
                  },
                  {
                    type: '#label',
                    key: 'name1',
                    props: {
                      required: true,
                      hideLabel: true,
                      disabled: true,
                    },
                  },
                ],
              },
            },
          ],
        },
      ],
    },
  ],
};
