import { Component, OnInit } from '@angular/core';
import {
  BreakpointService,
  AppStoreSelectors,
  AppStoreActions,
} from '@hash8io/core-web';
import { Store } from '@ngrx/store';
import { map, Observable } from 'rxjs';
import { MOCK_MEGAMENU_ASSET, MOCK_MEGAMENU_REPORTS, STORE_MOCK_OBJ } from '../../app-store-data/store-data';

@Component({
  selector: 'hash8-app-app-layout',
  templateUrl: './app-layout.component.html',
  styleUrls: ['./app-layout.component.scss'],
})
export class AppLayoutComponent implements OnInit {
  preferences!: Observable<any>;
  branding!: Observable<any>;
  devices!: Observable<any>;
  appVersion!: Observable<any>;
  header!: Observable<any>;

  layout = {
    lg: {
      areas:
        'header header | toolbar toolbar | sidebar banner  | sidebar content | sidebar footer',
      gap: '0px',
      rows: '52px 0px auto 1fr 36px',
      columns: '240px 1fr',
      alignRows: 'start stretch',
      alignColumns: 'start stretch',
    },
    md: {
      areas:
        'header header | toolbar toolbar | banner banner | sidebar sidebar | content content | footer footer',
      gap: '0px',
      rows: '52px 0px auto 0 1fr 36px',
      columns: '240px 1fr',
      alignRows: 'start stretch',
      alignColumns: 'start stretch',
    },
    sm: {
      areas:
        'header header | toolbar toolbar | banner banner | sidebar sidebar | content content | footer footer',
      gap: '0px',
      rows: '52px 42px auto 0 1fr 36px',
      columns: '240px 1fr',
      alignRows: 'start stretch',
      alignColumns: 'start stretch',
    },
    xs: {
      areas:
        'header header | toolbar toolbar | banner banner | sidebar sidebar | content content | footer footer',
      gap: '0px',
      rows: '52px 0px auto 0 1fr 36px',
      columns: '240px 1fr',
      alignRows: 'start stretch',
      alignColumns: 'start stretch',
    },
  };

  constructor(
    private breakpointService: BreakpointService,
    private store: Store
  ) { }

  ngOnInit() {
    this.breakpointService.breakpoints.subscribe((breakpoints) => {
      // console.log(breakpoints);
    });

    localStorage.setItem('userUUID', '11f1120c-8cb3-41ac-9460-e69c6370809f')
    localStorage.setItem('userUUID', '7a6d577d-aa42-42fc-8d73-0c6d440a21a3')



    // console.log("Entering app layout component")
    this.store.dispatch(
      AppStoreActions.SET_INITIAL_STORE_OBJ({ storeObj: STORE_MOCK_OBJ })
    );

    this.branding = this.store.select(AppStoreSelectors.selectBranding);
    this.header = this.store.select(AppStoreSelectors.selectHeader)

    if (localStorage.getItem('app-selected')) {

      // console.log("Get data from local storage")

      const selectApprovedApplications = this.store.select(AppStoreSelectors.selectApprovedApplications)

      selectApprovedApplications.subscribe((data: any) => {

        data.forEach((x: any) => {
          if (x.id === localStorage.getItem('app-selected')) {
            // console.log("Assign navigation in navigation menu")
            // this.store.dispatch(AppStoreActions.SET_APP_NAVIGATION({ navigationMenu: x.navigationMenu }))
            // const value = this.store.select(AppStoreSelectors.selectNavigations)
            // value.subscribe((data: any) => {
            //   console.log("Value", data)
            // })
          }
        })
      });

    }



  }
}
