import { Component, OnInit, ViewChild } from '@angular/core';
import {
  CIRCLE_OPTIONS,
  CUSTOM_POLYLINE_OPTIONS,
  DASHED_POLYLINE_OPTIONS,
  EMPTY_GOOGLE_MAP_OPTIONS,
  GOOGLE_MAP_OPTIONS,
  MARKER_OPTIONS,
  MULTIPLE_CIRCLE_DATA,
  MULTIPLE_MARKER_DATA,
  MULTIPLE_POLYGON_DATA,
  MULTIPLE_POLYLINE_DATA,
  NEW_MULTIPLE_CIRCLE_DATA,
  NEW_MULTIPLE_MARKER_DATA,
  NEW_MULTIPLE_POLYGON_DATA,
  NEW_MULTIPLE_POLYLINE_DATA,
  POLYGON_OPTIONS,
  POLYLINE_OPTIONS,
  UPDATED_MULTIPLE_POLYGON_DATA,
  UPDATED_MULTIPLE_POLYLINE_DATA,
} from './map-mock.data';
import { MapInfoWindow, MapMarker } from '@angular/google-maps';

@Component({
  selector: 'hash8-app-map-page',
  templateUrl: './map-page.component.html',
  styleUrls: ['./map-page.component.scss'],
})
export class MapPageComponent implements OnInit {
  // Info Window
  // @ViewChild(MapInfoWindow) infoWindow!: MapInfoWindow;
  @ViewChild(MapInfoWindow) infoWindow: any;
  infoWindowObj: any;

  // Google Map Options
  googleMapOptions: google.maps.MapOptions = GOOGLE_MAP_OPTIONS;
  emptyGoogleMapOptions: google.maps.MapOptions = EMPTY_GOOGLE_MAP_OPTIONS;
  mapTypeId: any = 'roadmap';
  clickedPoint: any;
  mapHover: any;

  // Marker Config
  markerOptions: google.maps.MarkerOptions = MARKER_OPTIONS;
  multipleMarkerConfig: any;

  // Polyline Config
  polylineOptions: google.maps.PolylineOptions = POLYLINE_OPTIONS;
  dashedPolylineOptions: google.maps.PolylineOptions = DASHED_POLYLINE_OPTIONS;
  customPolylineOptions: google.maps.PolylineOptions = CUSTOM_POLYLINE_OPTIONS;
  multiplePolylineConfig: any;

  // Polygon Config
  polygonOptions: google.maps.PolygonOptions = POLYGON_OPTIONS;
  multiplePolygonConfig: any;

  // Circle Config
  circleOptions: google.maps.CircleOptions = CIRCLE_OPTIONS;
  multipleCircleConfig: any;

  // CURD Operations Config
  addedMarkerCount = -1;
  addedPolylineCount = -1;
  addedPolygonCount = -1;
  addedCircleCount = -1;

  map1: any;
  map2: any;

  constructor() {
    this.multipleMarkerConfig = this.mapMarkerFormat(
      MULTIPLE_MARKER_DATA,
      'latitude',
      'longitude'
    );
    this.multiplePolylineConfig = this.mapPolylineFormat(
      MULTIPLE_POLYLINE_DATA,
      'latitude',
      'longitude'
    );
    this.multiplePolygonConfig = this.mapPolygonFormat(
      MULTIPLE_POLYGON_DATA,
      'latitude',
      'longitude'
    );
    this.multipleCircleConfig = this.mapCircleFormat(
      MULTIPLE_CIRCLE_DATA,
      'latitude',
      'longitude'
    );

    console.log('this.multiplePolylineConfig', this.multiplePolylineConfig);
    console.log('this.multipleMarkerConfig', this.multipleMarkerConfig);
  }

  // eslint-disable-next-line @angular-eslint/no-empty-lifecycle-method
  ngOnInit(): void {
    //
    this.map1 = {
      mapOptions: {
        name: 'usersMap',
        mapMode: 'hybrid',
        hideZoomControl: false,
        hideMapLayerControl: true,
        zoom: 15,
        height: '100%',
      },
    };

    this.map2 = {
      mapOptions: {
        name: 'assetMap',
        mapMode: 'roadmap',
        hideZoomControl: true,
        hideMapLayerControl: false,
        height: '100%',
      },
    };
  }

  mapTypeChange(event: any) {
    this.mapTypeId = event.value;
  }

  clickedMapPoint(event: any) {
    this.clickedPoint = event.latLng.toJSON();
  }

  mouseMove(event: any) {
    this.mapHover = event.latLng.toJSON();
  }

  // Format Data - MARKER, POLYLINE, POLYGON, CIRCLE
  mapMarkerFormat(data: any, latField: string, lngField: string) {
    let geofenceId: any;
    const markerFormat = data.map((item: any) => {
      return {
        position: {
          lat: item[latField],
          lng: item[lngField],
          ...item,
        },
        geofenceId: item.geofenceId,
        ...this.markerOptions,
      };
    });

    return markerFormat;
  }

  mapPolylineFormat(data: any, latField: string, lngField: string) {
    const polylineConfig: any[] = [];
    let geofenceId: any;
    data.map((item: any) => {
      const polylineFormat = item.map((item: any) => {
        geofenceId = item.geofenceId;
        return {
          lat: item[latField],
          lng: item[lngField],
          ...item,
        };
      });
      const dataFormat = {
        ...this.polylineOptions,
        path: polylineFormat,
        geofenceId,
      };
      polylineConfig.push(dataFormat);
    });
    return polylineConfig;
  }

  mapPolygonFormat(data: any, latField: string, lngField: string) {
    const polygonConfig: any[] = [];
    let geofenceId: any;
    data.map((item: any) => {
      const polygonFormat = item.map((item: any) => {
        geofenceId = item.geofenceId;
        return {
          lat: item[latField],
          lng: item[lngField],
          ...item,
        };
      });
      const dataFormat = {
        ...this.polygonOptions,
        paths: polygonFormat,
        geofenceId,
      };
      polygonConfig.push(dataFormat);
    });
    return polygonConfig;
  }

  mapCircleFormat(data: any, latField: string, lngField: string) {
    const circleConfig = data.map((item: any) => {
      return {
        center: {
          lat: item[latField],
          lng: item[lngField],
        },
        radius: item.radius,
        geofenceId: item.geofenceId,
        ...this.circleOptions,
      };
    });
    return circleConfig;
  }

  // ------------------------- MAP CONTROL / Events - POLYLINE -----------------------------------------
  basicPolyline() {
    this.multiplePolylineConfig = this.multiplePolylineConfig.map(
      (item: any) => {
        return {
          ...item,
          ...this.polylineOptions,
        };
      }
    );
  }

  dashedPolyline() {
    this.multiplePolylineConfig = this.multiplePolylineConfig.map(
      (item: any) => {
        return {
          ...item,
          ...this.dashedPolylineOptions,
        };
      }
    );
  }

  customPolyline() {
    this.multiplePolylineConfig = this.multiplePolylineConfig.map(
      (item: any) => {
        return {
          ...item,
          ...this.customPolylineOptions,
        };
      }
    );
  }

  changeColorPolyline() {
    this.multiplePolylineConfig = this.multiplePolylineConfig.map(
      (item: any) => {
        return {
          ...item,
          strokeColor: 'red',
        };
      }
    );
  }

  clickablePolyline() {
    this.multiplePolylineConfig = this.multiplePolylineConfig.map(
      (item: any) => {
        return {
          ...item,
          clickable: true,
        };
      }
    );
  }

  visibilityPolyline() {
    this.multiplePolylineConfig = this.multiplePolylineConfig.map(
      (item: any) => {
        if (item.visible == true) {
          return {
            ...item,
            visible: false,
          };
        } else {
          return {
            ...item,
            visible: true,
          };
        }
      }
    );
  }

  polylineClick(event: any, polyline: any) {
    console.log('POLYLINE CLICKED', polyline._options._value);
  }

  // ------------------------- MAP CONTROL / Events - POLYGON -----------------------------------------
  basicPolygon() {
    this.multiplePolygonConfig = this.multiplePolygonConfig.map((item: any) => {
      return {
        ...item,
        ...this.polygonOptions,
      };
    });
  }

  changeColorPolygon() {
    this.multiplePolygonConfig = this.multiplePolygonConfig.map((item: any) => {
      return {
        ...item,
        strokeColor: 'red',
        fillColor: 'red',
      };
    });
  }

  clickablePolygon() {
    this.multiplePolygonConfig = this.multiplePolygonConfig.map((item: any) => {
      return {
        ...item,
        clickable: true,
      };
    });
  }

  visibilityPolygon() {
    this.multiplePolygonConfig = this.multiplePolygonConfig.map((item: any) => {
      if (item.visible == true) {
        return {
          ...item,
          visible: false,
        };
      } else {
        return {
          ...item,
          visible: true,
        };
      }
    });
  }

  polygonClick(event: any, polygon: any) {
    console.log('POLYGON CLICKED', polygon._options._value);
  }

  // ------------------------- MAP CONTROL / Events - CIRCLE -----------------------------------------
  basicCircle() {
    this.multipleCircleConfig = this.multipleCircleConfig.map((item: any) => {
      return {
        ...item,
        ...this.circleOptions,
      };
    });
  }

  changeColorCircle() {
    this.multipleCircleConfig = this.multipleCircleConfig.map((item: any) => {
      return {
        ...item,
        strokeColor: 'red',
        fillColor: 'red',
      };
    });
  }

  clickableCircle() {
    this.multipleCircleConfig = this.multipleCircleConfig.map((item: any) => {
      return {
        ...item,
        clickable: true,
      };
    });
  }

  visibilityCircle() {
    this.multipleCircleConfig = this.multipleCircleConfig.map((item: any) => {
      if (item.visible == true) {
        return {
          ...item,
          visible: false,
        };
      } else {
        return {
          ...item,
          visible: true,
        };
      }
    });
  }

  circleClick(event: any, circle: any) {
    console.log('CIRCLE CLICKED', circle._options._value);
  }

  // ------------------------- CURD Operations - MARKER -----------------------------------------
  addMarker() {
    this.addedMarkerCount++;
    const obj = this.mapMarkerFormat(
      NEW_MULTIPLE_MARKER_DATA,
      'latitude',
      'longitude'
    );
    const newData = obj[this.addedMarkerCount];
    this.multipleMarkerConfig = [...this.multipleMarkerConfig, { ...newData }];
  }

  updateMarker() {
    const assetIndex = this.multipleMarkerConfig.findIndex(
      (obj: any) => obj.geofenceId == 'GE-MA-101'
    );
    const obj = this.mapMarkerFormat(
      NEW_MULTIPLE_MARKER_DATA,
      'latitude',
      'longitude'
    );

    const updatedData = {
      ...this.multipleMarkerConfig[assetIndex],
      position: {
        ...obj[assetIndex].position,
      },
    };
    this.multipleMarkerConfig[assetIndex] = updatedData;
    console.log(this.multipleMarkerConfig);
  }

  removeMarker() {
    this.multipleMarkerConfig = this.multipleMarkerConfig.filter(
      (obj: any) => obj.geofenceId != 'GE-MA-101'
    );
  }

  // ------------------------- CURD Operations - POLYLINE -----------------------------------------
  addPolyline() {
    this.addedPolylineCount++;

    const path = NEW_MULTIPLE_POLYLINE_DATA[this.addedPolylineCount];
    const newData = {
      ...this.polylineOptions,
      geofenceId: 'GE' + this.addedPolylineCount,
      path: path,
    };
    this.multiplePolylineConfig = [...this.multiplePolylineConfig, newData];
  }

  updatePolyline() {
    const assetIndex = this.multiplePolylineConfig.findIndex(
      (obj: any) => obj.geofenceId == 'GE101'
    );

    const updatedData = {
      ...this.polylineOptions,
      path: UPDATED_MULTIPLE_POLYLINE_DATA,
      geofenceId: 'GE101',
    };

    this.multiplePolylineConfig[assetIndex] = updatedData;
  }

  removePolyline() {
    this.multiplePolylineConfig = this.multiplePolylineConfig.filter(
      (obj: any) => obj.geofenceId != 'GE101'
    );
  }

  // ------------------------- CURD Operations - POLYGON -----------------------------------------

  addPolygon() {
    this.addedPolygonCount++;
    const paths = NEW_MULTIPLE_POLYGON_DATA[this.addedPolygonCount];
    const newData = {
      ...this.polygonOptions,
      geofenceId: 'GE' + this.addedPolygonCount,
      paths: paths,
    };
    this.multiplePolygonConfig = [...this.multiplePolygonConfig, newData];
  }

  updatePolygon() {
    const assetIndex = this.multiplePolygonConfig.findIndex(
      (obj: any) => obj.geofenceId == 'GE101'
    );
    const updatedData = {
      ...this.polygonOptions,
      paths: UPDATED_MULTIPLE_POLYGON_DATA,
      strokeColor: 'green',
      fillColor: 'green',
      geofenceId: 'GE101',
    };
    this.multiplePolygonConfig[assetIndex] = updatedData;
  }

  removePolygon() {
    this.multiplePolygonConfig = this.multiplePolygonConfig.filter(
      (obj: any) => obj.geofenceId != 'GE101'
    );
  }

  // ------------------------- CURD Operations - CIRCLE -----------------------------------------

  addCircle() {
    this.addedCircleCount++;
    const obj = this.mapCircleFormat(
      NEW_MULTIPLE_CIRCLE_DATA,
      'latitude',
      'longitude'
    );
    const newData = obj[this.addedCircleCount];
    this.multipleCircleConfig = [...this.multipleCircleConfig, { ...newData }];
  }

  updateCircle() {
    const assetIndex = this.multipleCircleConfig.findIndex(
      (obj: any) => obj.geofenceId == 'GE101'
    );
    const obj = this.mapCircleFormat(
      NEW_MULTIPLE_CIRCLE_DATA,
      'latitude',
      'longitude'
    );
    const updatedData = {
      ...this.multipleCircleConfig[assetIndex],
      strokeColor: 'green',
      fillColor: 'green',
      center: {
        ...obj[assetIndex].center,
      },
    };
    this.multipleCircleConfig[assetIndex] = updatedData;
  }

  removeCircle() {
    this.multipleCircleConfig = this.multipleCircleConfig.filter(
      (obj: any) => obj.geofenceId != 'GE101'
    );
  }

  // ------------------------- Info Window - Marker, Polyline, Polygon, Circle ------------------------

  openMarkerInfoWindow(marker: any) {
    this.infoWindow.open(marker);

    this.infoWindowObj = {
      uuid: marker._options.position.uuid,
      geofenceId: marker._options.geofenceId,
      accountId: marker._options.position.accountId,
      accountName: marker._options.position.accountName,
      status: marker._options.position.status,
    };
    console.log('Marker Clicked', marker._options);
    // console.log(this.infoWindowObj);
  }

  openPolylineInfoWindow(polyline: any) {
    this.infoWindow.open(polyline);
    this.infoWindowObj = {
      mapElement: 'POLYLINE INFO',
    };
  }

  openPolygonInfoWindow(polygon: any) {
    this.infoWindow.open(polygon);
    this.infoWindowObj = {
      mapElement: 'POLYGON INFO',
    };
  }

  openCircleInfoWindow(circle: any) {
    this.infoWindow.open(circle);
    this.infoWindowObj = {
      mapElement: 'CIRCLE INFO',
    };
  }
}
