/* eslint-disable @typescript-eslint/no-empty-function */
/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
import { HttpClient } from '@angular/common/http';
import { Component, OnInit } from '@angular/core';
import { TreeviewConfig } from '@hash8io/tree-view-web';
import { map } from 'rxjs';

@Component({
  selector: 'hash8-app-platform-map-view',
  templateUrl: './map-view.component.html',
  styleUrls: ['./map-view.component.scss'],
})
export class MapViewComponent implements OnInit {
  // Map options
  markerOptions = {
    draggable: false,
    animation: false, // google.maps.Animation.DROP,
    clickable: true,
    visible: true,
    icon: 'assets/asset-marker.svg',
    latitudeField: 'latitude',
    longitudeField: 'longitude',
  };
  mapOptions = {
    name: 'assetStatusMap',
    hideZoomControl: false,
    hideMapLayerControl: false,
    drawerSize: 'w-96',
    hideDrawingControl: true,
    hasGeofenceCreate: false,
    hasGeofenceEdit: false,
    preLoadAllGeofences: true,
    geofenceListApiUrl: 'http://localhost:3000/geofences',
    // height: '75vh',
    // mapTypeId: 'roadmap',
    // tilt: 0,
    // mapTypeControl: true,
    // mapTypeControlOptions: {
    //   style: google.maps.MapTypeControlStyle.HORIZONTAL_BAR,
    //   position: google.maps.ControlPosition.TOP_RIGHT,
    // },
    // zoom: 3,
    // zoomControl: true,
    // zoomControlOptions: {
    //   position: 8, //google.maps.ControlPosition.RIGHT_TOP,
    // },
    // scaleControl: false,
    // streetViewControl: true,
    // streetViewControlOptions: {
    //   position: google.maps.ControlPosition.RIGHT_CENTER,
    // },
    // fullscreenControl: true,
    // fullscreenControlOptions: {
    //   position: google.maps.ControlPosition.RIGHT_TOP,
    // },
    // center: { lat: 36.2016755, lng: -113.7796949 },
  };
  markers!: any;

  infoWindowFields = [
    {
      label: 'Reefer Name',
      key: 'reeferName',
      iconUrl: '',
      labelClass: 'pb-1 font-semibold text-base',
      iconClass: '',
      visible: true,
    },
    {
      label: 'Location',
      key: 'lastKnownLocation',
      iconUrl: 'place_on',
      labelClass: 'pb-1',
      iconClass: 'w-5 text-base text-gray-600',
      visible: true,
    },
    {
      label: 'Last reported time',
      key: 'lastReportedTime',
      data: '23/09/2023 01:37 PM',
      iconUrl: 'schedule',
      labelClass: 'pb-1',
      iconClass: 'w-5 text-base text-gray-600',
      visible: true,
    },
    {
      label: 'Status',
      key: 'status',
      labelClass: 'pb-1 font-semibold',
      iconClass: 'w-4 text-base text-gray-600',
      statusFields: {
        movingStatus: {
          field: 'reeferMovingStatus',
          options: [
            {
              value: 'Begin Motion',
              label: 'BEGIN MOTION',
              color: 'blue',
              labelColor: 'white',
              colorMode: 'dark',
              iconUrl: 'local_shipping',
              iconColor: 'blue',
              hideLabel: true,
              tooltip: 'Begin Motion',
              style: 'none',
            },
            {
              value: 'Stopped',
              label: 'STOPPED',
              color: 'red',
              labelColor: 'white',
              colorMode: 'light',
              iconUrl: 'local_shipping',
              iconColor: 'red',
              hideLabel: true,
              tooltip: 'Stopped',
              style: 'none',
            },
            {
              value: 'In Trip',
              label: 'IN TRIP',
              color: 'green',
              labelColor: 'white',
              colorMode: 'dark',
              iconUrl: 'local_shipping',
              iconColor: 'green',
              hideLabel: true,
              tooltip: 'In Trip',
              style: 'none',
            },
            {
              value: null,
              label: '-',
              color: '',
              iconUrl: 'local_shipping',
              iconColor: '#C2CFD9',
              colorMode: 'light',
              labelColor: 'black',
              hideLabel: true,
              tooltip: 'No Data Available',
              style: 'none',
            },
          ],
        },
        reeferStatus: {
          field: 'reeferStatus',
          options: [
            {
              value: 'Reefer ON',
              label: 'REEFER ON',
              color: 'green',
              labelColor: 'green',
              colorMode: 'dark',
              iconUrl: 'power_settings_new',
              iconColor: 'green',
              hideLabel: true,
              tooltip: 'Reefer On',
              style: 'none',
            },
            {
              value: 'Reefer OFF',
              label: 'REEFER OFF',
              color: 'red',
              labelColor: 'red',
              colorMode: 'dark',
              iconUrl: 'power_settings_new',
              iconColor: 'red',
              hideLabel: true,
              tooltip: 'Reefer Off',
              style: 'none',
            },
            {
              value: 'Switched OFF Timed (48)',
              label: 'Switched OFF Timed (48)',
              color: 'blue',
              labelColor: 'white',
              colorMode: 'light',
              iconUrl: 'power_settings_new',
              iconColor: 'blue',
              hideLabel: true,
              tooltip: 'Reefer Off - 48H',
              style: 'none',
            },
            {
              value: null,
              label: '-',
              color: '',
              iconUrl: 'power_settings_new',
              iconColor: '#C2CFD9',
              colorMode: 'light',
              labelColor: 'black',
              hideLabel: true,
              tooltip: 'No Data Available',
              style: 'none',
            },
          ],
        },
        alertStatus: {
          field: 'alarmSeverity',
          options: [
            {
              value: 'CRITICAL',
              label: 'CRITICAL',
              color: 'red',
              labelColor: 'gray',
              colorMode: 'light',
              iconUrl: 'report_problem',
              iconColor: 'red',
              hideLabel: true,
              tooltip: 'Critical Alarm(s) Present',
              style: 'none',
            },
            {
              value: 'WARNING',
              label: 'WARNING',
              color: 'orange',
              labelColor: 'gray',
              colorMode: 'light',
              iconUrl: 'report_problem',
              iconColor: 'orange',
              hideLabel: true,
              tooltip: 'Warning Alarm(s) Present',
              style: 'none',
            },
            {
              value: 'INFO',
              label: 'INFO',
              color: 'blue',
              labelColor: 'gray',
              colorMode: 'light',
              iconUrl: 'report_problem',
              iconColor: 'blue',
              hideLabel: true,
              tooltip: 'Info Alarm(s) Present',
              style: 'none',
            },
            {
              value: 'NO ALARMS',
              label: 'NO ALARMS',
              color: 'gray',
              labelColor: 'gray',
              colorMode: 'light',
              iconUrl: 'report_problem',
              iconColor: 'green',
              hideLabel: true,
              tooltip: 'No Alerts Present',
              style: 'none',
            },
            {
              value: null,
              label: '-',
              color: '',
              iconUrl: 'report_problem',
              iconColor: '#C2CFD9',
              colorMode: 'light',
              labelColor: 'black',
              hideLabel: true,
              tooltip: 'No Data Avaliable',
              style: 'none',
            },
            {
              value: undefined,
              label: '-',
              color: '',
              iconUrl: 'report_problem',
              iconColor: '#C2CFD9',
              colorMode: 'light',
              labelColor: 'black',
              hideLabel: true,
              tooltip: 'No Data Avaliable',
              style: 'none',
            },
          ],
        },
        batteryPower: {
          field: 'batteryPower',
          options: [
            {
              value: 'Battery Low',
              label: 'BATTERY LOW',
              color: 'orange',
              labelColor: 'orange',
              colorMode: 'dark',
              iconUrl: 'battery_charging_full',
              iconColor: 'orange',
              hideLabel: true,
              tooltip: 'Battery is Low',
              style: 'none',
            },
            {
              value: 'Battery Critical',
              label: 'BATTERY CRITICAL',
              color: 'red',
              labelColor: 'red',
              colorMode: 'dark',
              iconUrl: 'battery_charging_full',
              iconColor: 'red',
              hideLabel: true,
              tooltip: 'Battery is Critical',
              style: 'none',
            },
            {
              value: 'Battery OK',
              label: 'BATTERY OK',
              color: 'green',
              labelColor: 'gray',
              colorMode: 'light',
              iconUrl: 'battery_charging_full',
              iconColor: 'green',
              hideLabel: true,
              tooltip: 'Battery is Ok',
              style: 'none',
            },
            {
              value: null,
              label: '-',
              color: '',
              iconUrl: 'battery_charging_full',
              iconColor: '#C2CFD9',
              colorMode: 'light',
              labelColor: 'black',
              hideLabel: true,
              tooltip: 'No Data Available',
              style: 'none',
            },
          ],
        },
      },
    },
  ];

  orgFilterConfig!: any;

  // orgValues !: any;

  organizationFilterConfig = {
    hasAllCheckBox: true,
    hasFilter: false,
    hasCollapseExpand: true,
    decoupleChildFromParent: true,
    maxHeight: 400,
    configuration: {
      name: 'organizationFilter',
      label: 'All Organization',
      selection: 'multiple',
      visible: true,
      //Filter data
      apiUrl: `/api/v1/users/b4bf2c49-f214-47e5-bad4-fe56e1d76f3e/organizations?include=All_Organizations`,

      //dataSource - Array
      events: [
        {
          event: 'click',
          method: 'reloadReport',
          params: {
            gridName: 'tagsHierarchyDataGrid',
            apiUrl:
              'api/v1/tags?userUUID=b4bf2c49-f214-47e5-bad4-fe56e1d76f3e&organizationUUID={{data.orgFilterUUIDs}}&applicationUUID={{data.localStorage.applicationUUID}}',
          },
        },
      ],
    },
  };
  constructor(private http: HttpClient) {
    this.loadAssets();
  }

  ngOnInit() {
    this.orgFilterConfig = TreeviewConfig.create(
      this.organizationFilterConfig || {}
    );
  }

  loadAssets() {
    this.http
      .get('http://localhost:3000/reeferStatus', {})
      .pipe(map((res: any) => res.data?.rowData || []))
      .subscribe((data) => {
        this.markers = data;
      });
  }

  orgValues(event: any) {
    console.log('Event', event);
  }

  onValueChange(value: any) {
    // this.mapTypeId = value;
  }
}
