import { Model } from '@hash8io/core-web';
import { Actions } from './actions';
import { Breadcrumb } from './breadcrumb';
import { Layout, Layout2, TileLayout } from './layout';
import { ReportMenu } from './reports';
import { Title } from './title';
import { Toolbar } from './toolbar';

export const Snapshot: Model.SnapshotModel = {
  layout: TileLayout,
  config: {
    cards: {
      details: {
        header: {
          title: {
            label: 'Tank Details',
            iconUrl: 'info',
          },
          actions: [
            {
              label: 'Poll Location',
              targetUrl: '/tank-history',
              visible: true,
              clickable: true,
              type: Model.MenuType.Link,
              iconUrl: 'layers',
              configApiUrl: '',
              apiUrl: '',
              hasSubMenu: false,
              isPrimaryAction: false,
            },
            {
              label: 'Change Tank Name',
              targetUrl: '/netusage',
              visible: true,
              clickable: true,
              type: Model.MenuType.Link,
              iconUrl: 'edit',
              configApiUrl: '',
              apiUrl: '',
              hasSubMenu: false,
            },
          ],
        },
        content: [
          {
            type: 'text',
            props: {
              row: {
                type: 'text',
                label: 'Device Serial Number',
                value: '28741',
                key: 'lastReportedTime',
              },
            },
          },
          {
            type: 'text',
            props: {
              row: {
                type: 'tag',
                key: 'status',
                label: 'Reporting status',
                options: {
                  color: 'green',
                },
                value: 'Reporting',
              },
            },
          },

          {
            type: 'text',
            props: {
              row: {
                type: 'text',
                label: 'Location',
                value: '2101 Richland Avenue, Houston, Texas 77077, USA',
                key: 'location',
              },
            },
          },
          {
            type: 'report',
            props: {
              reportUUID: '300',
              showToolbar: false,
            },
          },
        ],
        footer: undefined,
        options: {
          showTitle: true,
          // showActions: true,
          showSearch: false,
          showSettings: false,
          showBreadcrumb: false,
          actionOptions: {
            showLabel: false,
            stroked: false,
          },
          hasBorder: true,
        },
        type: Model.CardType.LeftRight,
      },
      status: {
        header: {
          title: {
            label: 'Tank Properties',
            iconUrl: 'description',
          },
          actions: [
            {
              label: 'Change Tank Settings',
              targetUrl: '/tank-history',
              visible: true,
              clickable: true,
              type: Model.MenuType.Link,
              iconUrl: 'layers',
              configApiUrl: '',
              apiUrl: '',
              hasSubMenu: false,
              isPrimaryAction: false,
            },
          ],
        },
        content: [
          {
            label: 'Usage Type',
            key: 'usageType',
            type: 'text',
            props: {
              row: {
                label: 'Device Serial Number',
                value: '28741',
              },
            },
          },
          {
            key: 'tankShape',
            label: 'Tank Shape',
            options: {
              color: 'green',
            },
          },
          {
            label: 'Tank Height',
            key: '',
          },
          {
            label: 'Tank Diameter',
            key: '',
          },
          {
            label: 'Tank Capacity',
            key: '',
          },
          {
            label: 'Liquid Type',
            key: '',
          },
          {
            label: 'Specific Gravity',
            key: '',
          },
        ],
        footer: undefined,
        options: {
          showTitle: true,
          showSearch: false,
          actionOptions: {
            showLabel: false,
            stroked: false,
          },
        },
        type: Model.CardType.TopBottom,
      },
      deviceDetails: {
        header: {
          title: {
            label: 'Device Details',
            iconUrl: 'memory',
          },
          actions: [
            {
              label: 'Deactivate Device',
              targetUrl: '/netusage',
              visible: true,
              clickable: true,
              type: Model.MenuType.Link,
              iconUrl: 'edit',
              configApiUrl: '',
              apiUrl: '',
              hasSubMenu: false,
            },
          ],
        },
        content: [
          {
            type: 'breadcrumb',
            props: {
              breadcrumbs: [
                {
                  label: 'My Organizations',
                  targetUrl: '/myorganization',
                  value: '',
                },
                {
                  label: 'Organization',
                  targetUrl: '/organization',
                  value: 'Autocavo',
                },
              ],
            },
          },
          {
            type: 'actions',
            props: {
              actions: [
                {
                  label: 'Change Tank Settings',
                  targetUrl: '/fill',
                  visible: true,
                  clickable: true,
                  type: Model.MenuType.Link,
                  iconUrl: 'settings',
                  configApiUrl: '',
                  apiUrl: '',
                  hasSubMenu: false,
                  isPrimaryAction: false,
                },
              ],
            },
          },
          {
            label: 'Sensor Type',
            key: '',
            type: 'title',
            props: {
              title: {
                label: 'Welcome to Title',
                iconUrl: 'layers',
              },
            },
          },
          {
            label: 'Device ID',
            key: '',
            type: 'text',
            props: {
              row: {
                label: 'Device ID',
                value: 'K87123',
              },
            },
          },
          {
            label: 'Device Model',
            key: '',
            type: 'report',
            props: {
              reportUUID: 100,
              showToolbar: false,
            },
          },

          {
            label: 'Sensor Offset',
            key: '',
          },
          {
            label: 'Battery',
            key: '',
          },
        ],
        options: {
          showTitle: true,
          showSearch: false,
          showActions: false,
          actionOptions: {
            showLabel: false,
            stroked: false,
          },
        },
      },
    },
  },
  title: Title,
  toolbar: {
    breadcrumbs: Breadcrumb,
    actions: Actions,
    settings: Actions,
  },
  //   toolbar: Toolbar,
  reports: ReportMenu,
};
