/* eslint-disable @typescript-eslint/no-empty-function */
/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
import { Component, OnInit } from '@angular/core';
import { CascadeReportWebComponent } from '@hash8io/cascade-report-web';
import { Model } from '@hash8io/core-web';
import { BreadcrumbComponent } from '@hash8io/core-web/breadcrumb';
import { DarkModeTogglerComponent } from '@hash8io/core-web/dark-mode-toggler';
import { ReportMenu } from 'libs/snapshot-web/src/lib/mock-data/reports';

@Component({
  selector: 'hash8-app-snapshot2',
  templateUrl: './snapshot2.component.html',
  styleUrls: ['./snapshot2.component.scss'],
})
export class Snapshot2Component implements OnInit {
  title: Model.Title = {
    label: 'Watch Snapshot',
    iconUrl: 'layers',
  };

  toolbar: Model.Toolbar = {
    breadcrumbs: [
      {
        label: 'All Watches',
        targetUrl: '/watches',
      },
      {
        label: 'Watch',
        targetUrl: '',
        value: 'WATCH78234',
      },
    ],
    actions: [
      {
        label: 'Find Watch',
        iconUrl: 'navigation',
        type: Model.MenuType.Link,
        isPrimaryAction: true,
      },
    ],
    options: {
      showSearch: true,
    },
  };

  layout: Model.TileLayout = {
    columns: 12,
    rows: 'auto',
    gap: '12px',
    autoFlowColumns: false,
    tiles: {
      lg: [
        {
          column: 4,
          row: 1,
          area: 'details',
        },
        {
          column: 4,
          row: 1,
          area: 'location',
        },
        {
          column: 4,
          row: 1,
          area: 'device',
        },
        {
          column: 4,
          row: 1,
          area: 'kid',
        },
        {
          column: 8,
          row: 1,
          area: 'status',
        },
        {
          column: 12,
          row: 1,
          area: 'reports',
        },
        {
          column: 12,
          row: 1,
          area: 'cascade',
        },
      ],
      md: [
        {
          column: 6,
          row: 1,
          area: 'details',
        },
        {
          column: 6,
          row: 1,
          area: 'location',
        },
        {
          column: 6,
          row: 1,
          area: 'device',
        },
        {
          column: 6,
          row: 1,
          area: 'kid',
        },
        {
          column: 12,
          row: 1,
          area: 'status',
        },
        {
          column: 12,
          row: 1,
          area: 'reports',
        },
        {
          column: 12,
          row: 1,
          area: 'cascade',
        },
      ],
      sm: [
        {
          column: 12,
          row: 1,
          area: 'details',
        },
        {
          column: 12,
          row: 1,
          area: 'location',
        },
        {
          column: 12,
          row: 1,
          area: 'device',
        },
        {
          column: 12,
          row: 1,
          area: 'kid',
        },
        {
          column: 12,
          row: 1,
          area: 'status',
        },
        {
          column: 12,
          row: 1,
          area: 'reports',
        },
        {
          column: 12,
          row: 1,
          area: 'cascade',
        },
      ],
    },
  };

  config: Model.SnapshotConfig = {
    cards: {
      details: {
        header: {
          title: {
            label: 'Watch Details',
            iconUrl: 'info',
          },
          actions: [
            {
              label: 'Poll Location',
              targetUrl: '/tank-history',
              visible: true,
              clickable: true,
              type: Model.MenuType.Link,
              iconUrl: 'layers',
              configApiUrl: '',
              apiUrl: '',
              hasSubMenu: false,
              isPrimaryAction: false,
            },
          ],
        },
        content: [
          {
            type: 'row',
            props: [
              {
                type: 'text',
                props: {
                  type: 'text',
                  label: 'Watch Name',
                  value: 'WATCH72312',
                  align: 'column',
                },
              },
              {
                type: 'text',
                props: {
                  type: 'text',
                  label: 'Phone Number',
                  value: '(987) 6543 3210',
                  align: 'column',
                },
              },
            ],
          },

          {
            type: 'text',
            props: {
              type: 'text',
              label: 'Last Reported Time',
              value: '06/08/2022 03:10 AM (EST)',
              key: 'lastReportedTime',
              align: 'column',
            },
          },
        ],
      },
      location: {
        header: {
          title: {
            label: 'Location',
            iconUrl: 'places',
          },
        },
        content: [
          {
            type: 'text',
            props: {
              type: 'text',
              label: 'Last Location',
              value: '2101 Richland Avenue, Houston, Texas 77077, USA',
            },
          },
          {
            type: 'text',
            props: {
              type: 'text',
              label: 'GPS Coordinates',
              value: '13.923422, 80.234122',
            },
          },
        ],
      },
      device: {
        header: {
          title: {
            label: 'Device Details',
            iconUrl: 'memory',
          },
        },
        content: [
          {
            type: 'darkmode',
            props: {
              // darkMode: true,
            },
          },
        ],
      },
      kid: {
        header: {
          title: {
            label: 'Kid Details',
            iconUrl: 'child_care',
          },
          actions: [
            {
              label: 'Call Kid',
              type: Model.MenuType.Link,
              targetUrl: '',
              iconUrl: '',
            },
            {
              label: 'Send Message to Kid',
              type: Model.MenuType.Link,
              targetUrl: '',
              iconUrl: '',
            },
          ],
        },
        content: [
          {
            type: 'row',
            props: [
              {
                type: 'kpi',
                props: {
                  label: 'Devices',
                  value: 34,
                },
              },
              {
                type: 'kpi',
                props: {
                  label: 'Watch',
                  value: 34,
                },
              },
              {
                type: 'kpi',
                props: {
                  label: 'Users',
                  value: 34,
                },
              },
            ],
          },
          {
            type: 'kpi',
            props: {
              label: 'New Messages',
              value: 34,
            },
          },
          {
            type: 'grid',
            props: {
              reportUUID: 300,
              showToolbar: false,
            },
          },
        ],
      },
      status: {
        header: {
          title: {
            label: 'Recent Geofences Alerts',
            iconUrl: 'crisis_alert',
          },
        },
        content: [
          {
            type: 'breadcrumb',
            props: {
              breadcrumbs: [
                {
                  label: 'All Locations',
                  targetUrl: '/watches',
                },
                {
                  label: 'Location',
                  targetUrl: '',
                  value: 'Loc001',
                },
              ],
            },
          },
          {
            type: 'row',
            props: [
              // {
              //   type: 'grid',
              //   props: {
              //     reportUUID: 300,
              //     showToolbar: false,
              //   },
              // },
              {
                type: 'grid',
                props: {
                  reportUUID: 100,
                  showToolbar: false,
                },
              },
            ],
          },
        ],
      },
      reports: {
        header: {
          title: {
            label: 'Reports',
            iconUrl: 'insights',
          },
        },
        content: [
          {
            type: 'report-card',
            props: {
              reportNavigations: [
                {
                  label: 'Reports',
                  type: Model.MenuType.Heading,
                },
                {
                  label: 'Organizations',
                  targetUrl: '/snapshot/2',
                  visible: true,
                  clickable: true,
                  type: Model.MenuType.Link,
                  iconUrl: 'layers',
                  configApiUrl: '',
                  apiUrl: '',
                  hasSubMenu: false,
                  params: '987',
                },
                {
                  label: 'Users',
                  targetUrl: '/snapshot/2',
                  visible: true,
                  clickable: true,
                  type: Model.MenuType.Link,
                  iconUrl: 'layers',
                  configApiUrl: '',
                  apiUrl: '',
                  hasSubMenu: false,
                  params: '100',
                },
                {
                  label: 'Sales Channel',
                  targetUrl: '/snapshot/2',
                  visible: true,
                  clickable: true,
                  type: Model.MenuType.Link,
                  iconUrl: 'layers',
                  configApiUrl: '',
                  apiUrl: '',
                  hasSubMenu: false,
                  params: '101',
                },
                {
                  label: 'Customers',
                  targetUrl: '/snapshot/2',
                  visible: true,
                  clickable: true,
                  type: Model.MenuType.Link,
                  iconUrl: 'layers',
                  configApiUrl: '',
                  apiUrl: '',
                  hasSubMenu: false,
                  params: '102',
                },
              ],
            },
          },
        ],
      },
      cascade: {
        header: {
          title: {
            label: 'Cascade Reports',
            iconUrl: 'insights',
          },
        },
        content: [
          {
            type: 'cascade',
            props: {
              cascadeUUID: 'CR-1000',
              layout: 'horizontal',
            },
          },
        ],
      },
    },
  };

  componentLookup = {
    breadcrumb: BreadcrumbComponent,
    cascade: CascadeReportWebComponent,
    darkmode: DarkModeTogglerComponent,
  };

  constructor() {}

  ngOnInit(): void {}
}
