import { Model } from "@hash8io/core-web";
import { LabelComponent } from "libs/core-web/label/src/lib/label.component";
import { DataGridModel } from "libs/data-grid-web/src/lib/models/data-grid";

export const TankStatus: DataGridModel = {
  columnDefs: [
    //Maintenance Status - Grid Text Component
    {
      headerName: 'Maintenance Status',
      field: 'maintenanceStatus',
      cellRendererParams: {
        fieldType: 'text',
        defaultColor: 'gray',
        options: [
          {
            label: 'Overdue',
            value: 'overdue',
            iconUrl: 'error',
            color: 'red',
            //     colorExpression:
            //       `if(data.liquidLevel <50){
            //   return 'blue'
            // }`
          },
          {
            label: 'Due Soon',
            value: 'duesoon',
            iconUrl: 'warning',
            color: 'yellow',
            //       colorExpression:
            //         `if(data.liquidLevel <25 ){
            //   return 'blue'
            // }`
          },
          {
            label: 'Due',
            value: 'due',
            color: 'orange',
            iconUrl: 'warning'
          },
          {
            label: 'OK',
            value: 'ok',
            color: 'green',
            iconUrl: 'check'
          }
        ]
      }
    },
    //Maintenance Status Icon - Icon Component
    {
      headerName: 'Maintenance Status Icon',
      field: 'maintenanceStatusIcon',

      cellRendererParams: {
        fieldType: 'icon',
        options: [
          {
            label: 'Overdue',
            value: 'overdue',
            iconUrl: 'error',
            color: 'red'
          },
          {
            label: 'Due Soon',
            value: 'duesoon',
            iconUrl: 'warning',
            color: 'yellow'
          },
          {
            label: 'Due',
            value: 'due',
            color: 'orange',
            iconUrl: 'warning'
          },
          {
            label: 'OK',
            value: 'ok',
            color: 'green',
            iconUrl: 'check'
          }
        ]
      }
    },
    //Liquid Level - Grid Text Component
    {
      headerName: 'Liquid Level',
      field: 'liquidLevel',
      cellRendererParams: {
        fieldType: 'text',

        defaultIconUrl: 'water_drop',
        defaultIconColor: '#9d7725',

        default: {
          iconUrl: 'water_drop',
          iconColor: '#9d7725',
        }
      }
    },

    //Capacity - Progress component
    {
      colId: 'a6678112-eadc-4bd3-a4f3-65d2655aedd7',
      filter: true,
      hide: false,
      sortable: true,
      field: 'capacity',
      headerName: 'Capacity',
      enableRowGroup: true,
      cellRendererParams: {
        fieldType: 'progress',
        defaultColor: 'brown',
        colorExpression:
          `if(data?.capacity?.percentage > 0  && data?.capacity?.percentage <= 10){
          return 'red'
        }else if(data?.capacity?.percentage > 10  && data?.capacity?.percentage <= 20){
          return 'orange'
        }else if(data?.capacity?.percentage > 20  && data?.capacity?.percentage <= 30){
          return 'green'
        }`,
      }
    },
    //Reason code - Label component
    {
      headerName: 'Reason Code',
      field: 'reasonCode',
      cellRendererParams: {
        fieldType: 'label',
        defaultColor: 'gray',
        options: [

          {
            label: 'Scheduled Message',
            value: 'Scheduled Message',
            color: 'pink'
          },
          {
            label: 'Heartbeat',
            value: 'Heartbeat',
            color: 'blue'
          },
          {
            label: 'Sensor Report',
            value: 'Sensor Report',
            color: 'orange'
          }
        ]
      }
    },
    {
      headerName: 'Action',
      field: 'action',
      cellRendererParams: {
        fieldType: 'action',
        showLabel: false,
        stroked: false,
        columnData: false,
        options: [
          {
            label: 'Send Command',
            targetUrl: '/tank-history',
            visible: true,
            clickable: true,
            type: Model.MenuType.Link,
            iconUrl: 'layers',
            configApiUrl: '',
            apiUrl: '',
            hasSubMenu: false,
          },
          {
            label: 'Change Tank Name',
            targetUrl: '/netusage',
            visible: true,
            clickable: true,
            type: Model.MenuType.Link,
            iconUrl: 'edit',
            configApiUrl: '',
            apiUrl: '',
            hasSubMenu: false,
          },
          {
            label: 'Change Tank Image',
            targetUrl: '/fill',
            visible: true,
            clickable: true,
            type: Model.MenuType.Link,
            iconUrl: 'add_photo_alternate',
            configApiUrl: '',
            apiUrl: '',
            hasSubMenu: false,
          },
          {
            label: 'Change Tank Settings',
            targetUrl: '/fill',
            visible: true,
            clickable: true,
            type: Model.MenuType.Link,
            iconUrl: 'settings',
            configApiUrl: '',
            apiUrl: '',
            hasSubMenu: false,
            isPrimaryAction: false,
          },
          {
            label: 'Change Target Injection Rate',
            targetUrl: '/fill',
            visible: true,
            clickable: true,
            type: Model.MenuType.Link,
            iconUrl: 'filter_center_focus',
            configApiUrl: '',
            apiUrl: '',
            hasSubMenu: false,
          },
          {
            label: 'Change Refill Point',
            targetUrl: '/fill',
            visible: true,
            clickable: true,
            type: Model.MenuType.Link,
            iconUrl: 'layers',
            configApiUrl: '',
            apiUrl: '',
            hasSubMenu: false,
          },
          {
            label: 'Poll Location',
            targetUrl: '/fill',
            visible: true,
            clickable: true,
            type: Model.MenuType.Link,
            iconUrl: 'add_location_alt',
            configApiUrl: '',
            apiUrl: '',
            hasSubMenu: false,
          },

        ]

      }
    }
  ],
  rowData: [
    {
      maintenanceStatus: 'duesoon',
      maintenanceStatusIcon: 'duesoon',
      liquidLevel: '10',
      reasonCode: 'Heartbeat',
      capacity: {
        percentage: 25,
        label: '234.34'
      },
      action: [
        {
          label: 'Send Command',
          targetUrl: '/tank-history',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'layers',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },
        {
          label: 'Change Tank Name',
          targetUrl: '/netusage',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'edit',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },
        {
          label: 'Change Tank Image',
          targetUrl: '/fill',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'add_photo_alternate',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },
        {
          label: 'Change Tank Settings',
          targetUrl: '/fill',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'settings',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
          isPrimaryAction: false,
        },
        {
          label: 'Change Target Injection Rate',
          targetUrl: '/fill',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'filter_center_focus',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },
        {
          label: 'Change Refill Point',
          targetUrl: '/fill',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'layers',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },
        {
          label: 'Poll Location',
          targetUrl: '/fill',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'add_location_alt',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },

      ]

    },
    {
      maintenanceStatus: 'Overdue',
      maintenanceStatusIcon: 'Overdue',
      liquidLevel: '40',
      reasonCode: 'Heartbeat',
      capacity: {
        percentage: 15,
        label: '234.34'
      },
      action: [
        {
          label: 'Send Command',
          targetUrl: '/tank-history',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'layers',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },
        {
          label: 'Change Tank Name',
          targetUrl: '/netusage',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'edit',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },
        {
          label: 'Change Tank Image',
          targetUrl: '/fill',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'add_photo_alternate',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },
        {
          label: 'Change Tank Settings',
          targetUrl: '/fill',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'settings',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
          isPrimaryAction: false,
        },
        {
          label: 'Change Target Injection Rate',
          targetUrl: '/fill',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'filter_center_focus',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },
        {
          label: 'Change Refill Point',
          targetUrl: '/fill',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'layers',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },
        {
          label: 'Poll Location',
          targetUrl: '/fill',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'add_location_alt',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },

      ]

    },
    {
      maintenanceStatus: 'OK',
      maintenanceStatusIcon: 'OK',
      liquidLevel: '20',
      reasonCode: 'Heartbeat',
      capacity: {
        percentage: 10,
        label: '234.34'
      },
      action: [
        {
          label: 'Send Command',
          targetUrl: '/tank-history',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'layers',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },
        {
          label: 'Change Tank Name',
          targetUrl: '/netusage',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'edit',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },
        {
          label: 'Change Tank Image',
          targetUrl: '/fill',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'add_photo_alternate',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },
        {
          label: 'Change Tank Settings',
          targetUrl: '/fill',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'settings',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
          isPrimaryAction: false,
        },
        {
          label: 'Change Target Injection Rate',
          targetUrl: '/fill',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'filter_center_focus',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },
        {
          label: 'Change Refill Point',
          targetUrl: '/fill',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'layers',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },
        {
          label: 'Poll Location',
          targetUrl: '/fill',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'add_location_alt',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },

      ]

    },
    {
      maintenanceStatus: 'duesoon',
      maintenanceStatusIcon: 'duesoon',
      liquidLevel: '30',
      reasonCode: 'Heartbeat',
      capacity: {
        percentage: 15,
        label: '234.34'
      },
      action: [
        {
          label: 'Send Command',
          targetUrl: '/tank-history',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'layers',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },
        {
          label: 'Change Tank Name',
          targetUrl: '/netusage',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'edit',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },
        {
          label: 'Change Tank Image',
          targetUrl: '/fill',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'add_photo_alternate',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },
        {
          label: 'Change Tank Settings',
          targetUrl: '/fill',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'settings',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
          isPrimaryAction: false,
        },
        {
          label: 'Change Target Injection Rate',
          targetUrl: '/fill',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'filter_center_focus',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },
        {
          label: 'Change Refill Point',
          targetUrl: '/fill',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'layers',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },
        {
          label: 'Poll Location',
          targetUrl: '/fill',
          visible: true,
          clickable: true,
          type: Model.MenuType.Link,
          iconUrl: 'add_location_alt',
          configApiUrl: '',
          apiUrl: '',
          hasSubMenu: false,
        },

      ]

    }
  ],
};

export const Hierarchy_Mock = [{
  columnDefs: [
    {
      headerName: 'Accounts',
      field: 'accounts',
    },
  ],
  rowData: [{
    accounts: ["A"]
  },
  {
    accounts: ["A", "B"]

  }]
}]

export const ALERTS_MY_SUBSCRIPTION_REPORT_CONFIG = {
  title: 'My Subscriptions',
  icon: 'layer',
  reportUUID: 606,
  // apiUrl: `/api/v1/assets?organizationUUID=${this.organizationUUID}&userUUID=${this.userUUID}`,
  // apiUrl: '/api/v1/user-event-registrations',
  apiUrl: '/api/v1/user-event-group-registrations',
  options: {
    showTitle: false,
    showBreadcrumb: false,
    showSearch: true,
    showSettings: true,
    showActions: true,
    editable: false,
    showLegends: true,
  },
  breadcrumbs: [
    // {
    //   urlPath: '',
    //   label: 'Tanks',
    //   value: '',
    // },
    // {
    //   urlPath: '',
    //   label: 'All',
    //   value: '',
    // },
  ],
  style: {
    height: '450px',
  },
  settings: [
    {
      label: ' Export to CSV',
      iconUrl: 'file_download',
      urlPath: '',
      isActive: true,
      events: [
        {
          type: 'click',
          method: 'exportToCSV',
          params: {},
          events: [
            {
              type: 'success',
              method: 'showToast',
              params: {
                message: 'Exported to CSV',
              },
            },
            {
              type: 'failure',
              method: 'showToast',
              params: {
                message: '',
              },
            },
          ],
        },
      ],
    },
    {
      label: ' Export to Excel',
      iconUrl: 'file_download',
      urlPath: '',
      isActive: true,
      events: [
        {
          type: 'click',
          method: 'exportToExcel',
          params: {},
          events: [
            {
              type: 'success',
              method: 'showToast',
              params: {
                message: 'Exported to Excel',
              },
            },
            {
              type: 'failure',
              method: 'showToast',
              params: {
                message: '',
              },
            },
          ],
        },
      ],
    },
  ],

  count: {
    label: 'Subscriptions',
    value: '',
  },
};
