import { Component, OnInit } from '@angular/core';
import { AppStoreSelectors } from '@hash8io/core-web';
import { Store } from '@ngrx/store';

@Component({
  selector: 'hash8-platform-forgot-password',
  templateUrl: './forgot-password.component.html',
  styleUrls: ['./forgot-password.component.scss'],
})
export class ForgotPasswordComponent implements OnInit {
  forgotPasswordForm: any = {
    fields: [
      {
        template: `<h4 class="text-sm mb-4 flex justify-center">To reset password, please enter the email address associated with your account</h4>`,
      },
      {
        type: '#text',
        key: 'email',
        className: 'email-padding block',
        props: {
          label: 'Email',
          type: 'email',
          required: true,
          hideLabel: true,
        },
        extras: {
          labelOverInput: true,
        },
        expressions: {
          // 'props.disabled': '!model.username',
        },
        validators: {
          validation: [{ name: 'email' }],
        },
      },
      // {
      //   type: '#textbox-async',
      //   key: 'email',
      //   className: 'email-padding block',
      //   props: {
      //     type: 'email',
      //     label: 'Email',
      //     required: true,
      //     // async: true,
      //     hideLabel: true,
      //     // apiKey: 'email',
      //     // apiEndpoint: '/api/v1/email-unique-status',
      //     apiKey: 'email',
      //     // errorMessage: `email already exists`,
      //     // valueCheck: true,
      //   },
      //   extras: {
      //     labelOverInput: true,
      //   },
      //   validators: {
      //     validation: [{ name: 'email' }],
      //   },
      // },
      {
        template: `<hr class ="mb-2 mt-2"/>`,
      },
    ],
    options: {
      submit: {
        label: 'Send Password Reset Link',
        events: [
          {
            event: 'click',
            method: 'callAPI',
            params: {
              method: 'post',
              noAccessToken: true,
              apiUrl: '/api/v1/user-forgot-password-email-initializations',
              bodyParams: [
                {
                  key: 'email',
                  value: '{{data.formData.email}}',
                },
              ],
            },
            events: [
              {
                event: 'success',
                method: 'showErrorToast',
                enabled: 'data.message === "Invalid email address"',
                params: {
                  message: 'Invalid Email Address',
                },
                type: '',
              },
              {
                event: 'success',
                method: 'showErrorToast',
                enabled:
                  'data.message === "Failed to send password reset link"',
                params: {
                  message:
                    'Something went wrong. Failed to send password reset link.',
                },
                type: '',
              },
              {
                event: 'failure',
                method: 'showErrorToast',
                params: {
                  message:
                    'Something went wrong. Failed to send password reset link.',
                },
                type: '',
              },
              {
                event: 'success',
                method: 'goToPage',
                enabled:
                  'data.message === "Password reset link sent successfully"',
                params: {
                  urlPath: '/password-reset-link-sent',
                },
                type: '',
              },
            ],
          },
        ],
      },
      hasReset: false,
      submitLabel: 'Send Password Reset Link',

      // cancel: {
      //   label: 'Return to Login',
      // },
    },
  };
  model!: any;
  constructor(private store: Store) {}

  ngOnInit(): void {}

  modelChange(event: any) {}
  onSubmit(model: any) {
    //
  }
}
