import { Component, OnInit } from '@angular/core';
import { AppStoreSelectors } from '@hash8io/core-web';
import { Store } from '@ngrx/store';
@Component({
  selector: 'hash8-platform-reset-password',
  templateUrl: './reset-password.component.html',
  styleUrls: ['./reset-password.component.scss'],
})
export class ResetPasswordComponent implements OnInit {
  resetForm: any = {
    fields: [
      {
        validators: {
          validation: [
            {
              name: 'fieldMatch',
              options: { errorPath: 'confirmPassword' },
            },
          ],
        },
        fieldGroup: [
          {
            type: '#password',
            key: 'newPassword',
            props: {
              label: 'New Password',
              hideLabel: false,
              required: true,
              align: 'row',
            },
            extras: {
              labelOverInput: false,
            },
            validators: {
              validation: [{ name: 'password' }],
            },
          },
          {
            type: '#password',
            key: 'confirmPassword',
            props: {
              label: 'Confirm Password',
              hideLabel: false,
              required: true,
              align: 'row',
            },
            extras: {
              labelOverInput: false,
            },
          },
        ],
      },
      {
        template: `<p class="text-xs -mt-1 mb-2 opacity-50">Note: Password should meet all the following conditions.<br>
      * Atleast one upper case letter.<br>
      * Atleast one number.<br>
      * Min.8 characters in length.</p>`,
      },
    ],
    options: {
      submit: {
        label: 'Reset Password',
        events: [
          {
            event: 'success',
            method: 'putAPI',
            params: {
              noAccessToken: true,
              apiUrl: '/api/v1/user-reset-password',
              bodyParams: [
                {
                  key: 'uuid',
                },
                {
                  key: 'userUUID',
                },
              ],
            },
            // event: 'click',
            // method: 'callAPI',
            // params: {
            //   method: 'put',
            //   apiUrl: '/api/v1/user-reset-password',
            //   bodyParams: [
            //     {
            //       key: 'uuid',
            //     },
            //     {
            //       key: 'userUUID',
            //     },
            //   ],
            // },
            events: [
              {
                event: 'success',
                method: 'showErrorToast',
                enabled: 'data.message === "Failed to reset password"',
                params: {
                  message: 'Something went wrong. Failed to reset password.',
                },
                type: '',
              },
              {
                event: 'success',
                method: 'showErrorToast',
                enabled:
                  'data.message === "Invalid password: must not be equal to previous password"',
                params: {
                  message:
                    'Old and New Passwords cannot be similar, please change accordingly',
                },
                type: '',
              },
              {
                event: 'failure',
                method: 'showErrorToast',
                enabled: 'data.message === "Failed to reset password"',
                params: {
                  message: 'Something went wrong. Failed to reset password.',
                },
                type: '',
              },
              {
                event: 'success',
                method: 'goToPage',
                enabled: 'data.message === "Password successfully reset"',
                params: {
                  urlPath: '/login',
                },
                type: '',
              },
              {
                event: 'success',
                method: 'showToast',
                enabled: 'data.message === "Password successfully reset"',
                params: {
                  type: 'success',
                  message:
                    'Your password has been reset successfully. Please sign in with your new credentials',
                },
              },
            ],
          },
        ],
      },
      hasReset: false,
    },
  };

  model!: any;
  appSegments!: any;
  forgotPasswordStoreData!: any;
  constructor(private store: Store) {}

  ngOnInit(): void {
    localStorage.clear();

    this.appSegments = this.store.select(
      AppStoreSelectors.selectApplicationSegments
    );

    this.appSegments?.subscribe((data: any) => {
      this.forgotPasswordStoreData = data?.resetPassword;
    });

    this.store.dispatch({
      type: '[Events] Process events',
      events: this.forgotPasswordStoreData?.events,
    });
  }
  modelChange(event: any) {}
  onSubmit(model: any) {
    //
  }
}
