import { FormModel } from '@hash8io/forms-web';
import { FormlyFieldConfig } from '@ngx-formly/core';

export const LOGIN_FORM: FormModel = {
  fields: [
    {
      type: '#text',
      key: 'username',
      className: 'w-full',
      props: {
        label: 'Username / Email',
        required: true,
        hideLabel: true,
      },
      extras: {
        labelOverInput: true,
      },
    },
    {
      type: '#password',
      key: 'password',
      className: 'w-full',
      props: {
        label: 'Password',
        required: true,
        hideLabel: true,
      },
      extras: {
        labelOverInput: true,
      },
    },
    {
      type: '#group',
      fieldGroupClassName: 'flex w-full justify-between',
      fieldGroup: [
        {
          type: '#checkbox',
          key: 'rememberMe',
          props: {
            text: 'Remember Me',
            hideLabel: true,
            color: 'primary'
          },
          extras: {
            labelOverInput: true,
          },
        },
        {
          template: `<a href="/" class="hover:underline hover:text-primary text-xs">Forgot Password?</a>`,
        },
      ],
    },
  ],
  options: {
    submitLabel: 'Sign In',
    hasReset: false,
    submitLabelClass: ['w-full'],
    submitLabelColor: 'primary'

  }
}

export const loginFormFields: FormlyFieldConfig[] = [
  {
    key: 'username',
    type: 'input',
    focus: true,
    templateOptions: {
      label: 'Username / Email',
      required: true,
      appearance: 'outline',
    },
    validation: {
      messages: {
        required: 'Please enter username/email address.',
      },
    },
  },
  {
    key: 'password',
    type: 'password',
    templateOptions: {
      type: 'password',
      label: 'Password',
      required: true,
      appearance: 'outline',
    },
    validation: {
      messages: {
        required: 'Please enter password.',
      },
    },
  },
];
