import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { AppStoreActions, AppStoreSelectors } from '@hash8io/core-web';
import { Store } from '@ngrx/store';
import { map, Observable, of } from 'rxjs';

@Injectable({
  providedIn: 'root',
})
export class AuthenticationService {
  apiUrl = '';
  isAuthenticate = false;

  constructor(
    private http: HttpClient,
    private store: Store,
    private router: Router
  ) {
    this.store
      .select(AppStoreSelectors.selectApplicationSegments)
      .subscribe((response) => {
        this.apiUrl = response.login?.apiUrl;
      });
  }

  login(user: any) {
    let endPoint = '';
    let redirectUrl = '';
    let events: any;
    this.store
      .select(AppStoreSelectors.selectApplicationSegments)
      .pipe(map((segments) => segments.login))
      .subscribe((login) => {
        redirectUrl = login.redirectUrl;
        endPoint = login.apiUrl;
        events = login?.events;
      });
    return this.http.post(`${this.apiUrl}`, user).pipe(
      map(
        (response: any) => {
          if (response?.data?.organizationUUID) {
            localStorage.setItem(
              'organizationUUID',
              response?.data?.organizationUUID
            );
            localStorage.setItem(
              'organizationName',
              response?.data?.organizationName
            );
          } else {
            localStorage.setItem(
              'organizationUUID',
              '183e3ad2-c3c3-45ac-b12c-c20d59cbdc22'
            );
          }

          if (response?.data?.userRole) {
            localStorage.setItem('userRole', response?.data?.userRole);
          }

          if (response?.data?.accountUUID) {
            localStorage.setItem('accountUUID', response?.data?.accountUUID);
          } else {
            localStorage.setItem(
              'accountUUID',
              'c6b28722-7139-4658-9075-e57b546ec994'
            );
          }
          if (response?.data?.access_token) {
            events?.forEach((x: any) => {
              const functionName = x?.method;

              if (functionName === 'clearValue') {
                this.clearValue(x?.params);
              }
              if (functionName === 'redirectUrl') {
                // this.redirectUrl(x?.params)
                window.location.href = redirectUrl;
              }
              // else if (functionName === 'setValue') {
              //   this.setValue(x?.params)
              // }
            });

            // this.router.navigateByUrl(redirectUrl);
            // this.isAuthenticate = true;
            // localStorage.setItem('Authenticated', 'true')
            // return of(true)
          }
          return response;
        },
        (error: any) => {
          console.log();
          return error;
          // return of(false)
        }
      )
    );
  }

  clearValue(params: any) {
    localStorage.removeItem(params.key);
  }

  redirectUrl(params: any) {
    this.router.navigateByUrl(params.url);
  }

  setValue(params: any) {
    localStorage.setItem(params?.key, params?.value);
  }

  logout() {
    localStorage.clear();
    this.store.dispatch(AppStoreActions.CLEAR_AUTH_TOKEN());
    this.router.navigate(['/login']);
  }
}
