import { Inject, Injectable, InjectionToken } from '@angular/core';
import {
  HTTP_INTERCEPTORS,
  HttpRequest,
  HttpHandler,
  HttpEvent,
  HttpInterceptor,
} from '@angular/common/http';
import { Observable } from 'rxjs';

@Injectable({
  providedIn: 'any',
})
export class AuthInterceptorService implements HttpInterceptor {
  // private skipUrls: string[] = [];
  constructor(
    @Inject(AUTH_SKIP_URLS) private skipUrls: string[],
    @Inject(APPLICATION_UUID_SKIP_URLS)
    private applicationUUIDSkipUrls: string[]
  ) {}

  // @Inject(APPLICATION_UUID_SKIP_URLS)
  // private applicationUUIDSkipUrls?: string[];

  setSkipUrls(urls: string[]) {
    this.skipUrls = urls;
  }

  addSkipUrls(urls: string[]) {
    this.skipUrls = [...this.skipUrls, ...urls];
  }

  intercept(
    request: HttpRequest<any>,
    next: HttpHandler
  ): Observable<HttpEvent<any>> {
    // Check if the request URL is in the skip list
    if (this.shouldSkipAuthorization(request.url)) {
      return next.handle(request);
    }

    // Modify the request, e.g., adding an authorization header
    let modifiedRequest = request.clone({
      // setHeaders: {
      //   Authorization: 'Bearer ' + localStorage.getItem('access_token'),
      // },
      setParams: {
        authUUID: localStorage.getItem('access_token') || '',
      },
    });

    if (this.shouldSkipApplicationParams(modifiedRequest.url)) {
      return next.handle(modifiedRequest);
    }

    modifiedRequest = modifiedRequest.clone({
      setParams: {
        applicationUUIDs: 'APPLICATION_UUID',
      },
    });

    // Continue the request by passing the modified request to the next interceptor or to the HTTP client
    return next.handle(modifiedRequest);

    // // Modify the request, e.g., adding an authorization header
    // const modifiedRequest = request.clone({
    //   setParams: {
    //     applicationUUID: localStorage.getItem('access_token') || '',
    //   },
    // });

    // // Continue the request by passing the modified request to the next interceptor or to the HTTP client
    // return next.handle(modifiedRequest);
  }

  private shouldSkipAuthorization(url: string): boolean {
    alert(url + ' - ' + this.skipUrls.some((skipUrl) => url.includes(skipUrl)));
    return this.skipUrls.some((skipUrl) => url.includes(skipUrl));
  }

  private shouldSkipApplicationParams(url: string): boolean {
    alert(
      url + ' ->>> ' + this.skipUrls.some((skipUrl) => url.includes(skipUrl))
    );
    return (
      this.applicationUUIDSkipUrls?.some((skipUrl) => url.includes(skipUrl)) ||
      false
    );
  }
}

export const AuthInterceptorProviders = [
  {
    provide: HTTP_INTERCEPTORS,
    useClass: AuthInterceptorService,
    multi: true,
  },
];

export const AUTH_SKIP_URLS = new InjectionToken<string[]>('authSkipUrls');
export const APPLICATION_UUID_SKIP_URLS = new InjectionToken<string[]>(
  'applicationUUIDSkipUrls'
);
