/* eslint-disable @nrwl/nx/enforce-module-boundaries */
import { Injectable } from '@angular/core';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { map, exhaustMap, catchError } from 'rxjs/operators';
import { of } from 'rxjs';
import { Store } from '@ngrx/store';
import { LoginStoreActions } from '.';
import { AuthenticationService } from '../services/authentication.service';
import { TokenService } from '../services/token/token.service';
import { DecodeTokenService } from '../services/decode-token/decode-token.service';

@Injectable()
export class LoginStoreEffects {
  constructor(
    private actions$: Actions,
    private authenticationService: AuthenticationService,
    private tokenService: TokenService,
    private decodeTokenService: DecodeTokenService,
    private store: Store,
  ) { }

  userLogin$ = createEffect(() => {
    return this.actions$.pipe(
      ofType(LoginStoreActions.AUTH_LOGIN),
      exhaustMap((user) =>
        this.authenticationService.login(user).pipe(
          map((response: any) => {
            this.store.dispatch(LoginStoreActions.SET_AUTH_TOKEN({ response }));
            this.store.dispatch(LoginStoreActions.DECODE_AUTH_TOKEN({ response }))
            // this.loader.complete();
            return LoginStoreActions.AUTH_SUCCESS(response.data);
          }),
          catchError((error) => {
            this.store.dispatch(LoginStoreActions.AUTH_LOGIN_ERROR());
            this.store.dispatch(LoginStoreActions.AUTH_LOGIN_TRIES());
            // this.loader.complete();
            return of(LoginStoreActions.AUTH_FAILED(error.error));
          })
        )
      )
    );
  });

  setAuthTokens$ = createEffect(() => {
    return this.actions$.pipe(
      ofType(LoginStoreActions.SET_AUTH_TOKEN),
      exhaustMap((token) => {
        this.tokenService.setAuthToken(token);
        return of(LoginStoreActions.SET_AUTH_TOKEN_SUCCESS());
      }),
      catchError((error) => {
        return of(LoginStoreActions.SET_AUTH_TOKEN_FAILED());
      })
    );
  });

  decodeAuthTokens$ = createEffect(() => {
    return this.actions$.pipe(
      ofType(LoginStoreActions.DECODE_AUTH_TOKEN),
      exhaustMap((token) => {
        const decodedToken = this.decodeTokenService.decodeToken(token);
        return of(LoginStoreActions.DECODE_AUTH_TOKEN_SUCCESS({ decodedToken }));
      }),
      catchError((error) => {
        return of(LoginStoreActions.DECODE_AUTH_TOKEN_FAILED());
      })
    );
  });

  // decodeAuthTokens$ = createEffect(() => {
  //   return this.actions$.pipe(
  //     ofType(LoginStoreActions.DECODE_AUTH_TOKEN),
  //     exhaustMap((token) =>
  //       this.decodeTokenService.decodeToken(token).pipe(
  //         map((response: any) => {
  //           return LoginStoreActions.DECODE_AUTH_TOKEN_SUCCESS(response);
  //         }),
  //         catchError((error) => {
  //           console.log('Error Message', error.error.message);
  //           this.store.dispatch(LoginStoreActions.AUTH_LOGIN_ERROR());
  //           this.store.dispatch(LoginStoreActions.AUTH_LOGIN_TRIES());
  //           return of(LoginStoreActions.AUTH_FAILED(error));
  //         })
  //       )
  //     )
  //   );
  // });



  // clearAuthTokens$ = createEffect(() => {
  //   return this.actions$.pipe(
  //     ofType(LoginStoreActions.CLEAR_AUTH_TOKEN),
  //     exhaustMap(() => {
  //       return of(LoginStoreActions.AUTH_FAILED({}));
  //     })
  //   );
  // });
}
