import { AppStoreState } from '@hash8io/core-web';
import { createReducer, on } from '@ngrx/store';
import { LoginStoreActions } from '.';

export const reducer = createReducer(
  AppStoreState.initialState,

  // Login
  on(LoginStoreActions.AUTH_LOGIN, (state, { user }) => ({
    ...state,
    loginState: {
      state: 'initiated',
      isAuthenticated: false,
      isLoading: true,
      error: false,
      errorMessage: '',
    },
  })),

  on(LoginStoreActions.AUTH_LOGIN_INPROGRESS, (state) => ({
    ...state,
    loginState: {
      ...state.loginState,
      state: 'inprogress',
    },
  })),
  on(LoginStoreActions.AUTH_SUCCESS, (state, response) => {
    return {
      ...state,
      loginState: {
        ...state.loginState,
        state: 'loggedIn',
        isAuthenticated: true,
        loaded: true,
      },
    };
  }),
  on(LoginStoreActions.AUTH_FAILED, (state, error) => {
    return {
      ...state,
      loginState: {
        ...state.loginState,
        state: 'loggedFailed',
        loaded: true,
        error: true,
        errorMessage: error?.message,
      },
    };
  }),
  on(LoginStoreActions.AUTH_LOGIN_TRIES, (state) => ({
    ...state,
    loginState: {
      ...state.loginState,
      state: 'loginDenied',
      invalidTriesCount: state.loginState.invalidTriesCount + 1,
    },
  })),
  on(LoginStoreActions.DECODE_AUTH_TOKEN_SUCCESS, (state, response) => ({
    ...state,
    userDetails: {
      uuid: response.decodedToken.userUUID,
      firstName: response.decodedToken.given_name,
      lastName: response.decodedToken.family_name,
      username: response.decodedToken.name,
      email: response.decodedToken.email,
    },
  }))
);
