import { Component, Input, OnInit } from '@angular/core';
import { UntypedFormGroup } from '@angular/forms';
import { FormFields, FormOptions } from '@hash8io/forms-web';
import { FormlyFieldConfig } from '@ngx-formly/core';
import { changePasswordConfig } from './change-password-config';

@Component({
  selector: 'hash8-platform-change-password',
  templateUrl: './change-password.component.html',
  styleUrls: ['./change-password.component.scss'],
})
export class ChangePasswordComponent implements OnInit {
  @Input() changePasswordCard: any;

  form = new UntypedFormGroup({});
  model: any = {};
  options: any = {
    submit: {
      label: 'Update',
      events: [
        {
          event: 'success',
          method: 'putAPI',
          params: {
            apiUrl: '/api/v1/user-change-password',
            bodyParams: [],
          },
          events: [
            {
              event: 'success',
              method: 'showToast',
              enabled: 'data.message === "Password updated successfully"',
              params: {
                message: 'Password has been changed successfully.',
              },
            },
            {
              event: 'success',
              method: 'showErrorToast',
              enabled: 'data.message !== "Password updated successfully"',
              params: {
                message: 'Something went wrong. Failed to change Password.',
              },
            },
            {
              event: 'failure',
              method: 'showErrorToast',
              enabled: 'data.message !== "Password updated successfully"',
              params: {
                message: 'Something went wrong. Failed to change Password.',
              },
            },
            {
              event: 'success',
              method: 'reloadPage',
            },
          ],
        },
      ],
    },
    submitLabel: 'Update',
    // cancel: {
    //   label: 'Reset',
    //   events: [
    //     {
    //       event: 'click',
    //       method: 'goToPage',
    //       params: {
    //         urlPath: '/displayPreferences',
    //       },
    //     },
    //   ],
    // },
    hasReset: true,
    resetForm: true,
  };

  fields: any = [
    {
      key: 'currentPassword',
      type: '#password',
      props: {
        label: 'Current Password',
        placeholder: 'Please enter your current password',
        required: true,
        align: 'row',
      },
    },
    {
      // type: '#group',
      validators: {
        validation: [
          { name: 'fieldMatch', options: { errorPath: 'confirmPassword' } },
        ],
      },
      fieldGroup: [
        {
          key: 'newPassword',
          type: '#password',
          props: {
            label: 'New Password',
            placeholder: 'Please enter your new password',
            required: true,
            align: 'row',
          },
        },
        {
          key: 'confirmPassword',
          type: '#password',
          props: {
            label: 'Confirm Password',
            placeholder: 'Please re-enter your password',
            required: true,
            align: 'row',
          },
        },
      ],
    },
  ];
  fields1: FormlyFieldConfig[] = [
    {
      key: 'currentPassword',
      type: 'password',
      templateOptions: {
        label: 'Current Password',
        type: 'password',
        appearance: 'outline',
        required: true,
        pattern: '^(?=.*?[A-Z])(?=.*?[0-9]).{8,}$',
      },
      validation: {
        messages: {
          required: 'Please enter current password.',
          pattern: 'Please enter valid password.',
        },
      },
    },
    {
      validators: {
        validation: [
          { name: 'match', options: { errorPath: 'passwordConfirm' } },
        ],
      },
      fieldGroup: [
        {
          key: 'newPassword',
          type: 'password',
          templateOptions: {
            type: 'password',
            label: 'New Password',
            appearance: 'outline',
            required: true,
            pattern: '^(?=.*?[A-Z])(?=.*?[0-9]).{8,}$',
          },
          validation: {
            messages: {
              required: 'Please enter new password.',
              pattern: 'Please enter valid password.',
            },
          },
        },
        {
          key: 'passwordConfirm',
          type: 'password',
          templateOptions: {
            type: 'password',
            label: 'Confirm Password',
            appearance: 'outline',
            required: true,
            pattern: '^(?=.*?[A-Z])(?=.*?[0-9]).{8,}$',
          },
          validation: {
            messages: {
              required: 'Please enter confirm password.',
              pattern: 'Please enter valid password.',
            },
          },
        },
      ],
    },
  ];
  // eslint-disable-next-line @typescript-eslint/no-empty-function
  constructor() {}
  // eslint-disable-next-line @angular-eslint/no-empty-lifecycle-method, @typescript-eslint/no-empty-function
  ngOnInit(): void {}
  modelChange(event: any) {
    // console.log(event);
  }

  onSubmit(model: any) {
    // console.log(model);
  }
}
