import { Component, Input, OnInit } from '@angular/core';
import { UserProfileService } from '../../service/user-profile.service';
import { userProfileNavigationConfig } from '../../user-profile-config';
import { regionalSettingConfig, timeZoneList } from './regional-setting-config';
import { FormFields } from '@hash8io/forms-web';

// import moment from 'moment-timezone';

@Component({
  selector: 'hash8-platform-regional-setting',
  templateUrl: './regional-setting.component.html',
  styleUrls: ['./regional-setting.component.scss'],
})
export class RegionalSettingComponent implements OnInit {
  @Input() regionalSettingCard!: any;
  edit = false;

  timezones: any[] = [];

  fields!: FormFields[];
  options!: any;

  // regionalSettings!: any;

  // eslint-disable-next-line @typescript-eslint/no-empty-function
  constructor(private userProfileService: UserProfileService<any>) {
    // const tzArray = moment.tz.names();
    // for (const names of tzArray) {
    //   this.timezones.push({
    //     value: names,
    //     label: `(GMT${moment().tz(names).format('Z')}) ${names}`,
    //   });
    //}

    this.fields = [
      {
        key: 'dateFormat',
        type: '#select',
        focus: false,
        defaultValue: 'MM/dd/yy',
        props: {
          label: 'Date Format',
          // options: [
          //   { value: 'MM/dd/yy', label: 'mm/dd/yy (06/15/23)' },
          //   { value: 'M/d/yy', label: ' m/d/yy (6/15/23)' },
          //   { value: 'MMM d, y', label: 'mmm d, y (Jun 15, 2023)' },
          //   { value: 'MMMM d, y', label: 'mmmm d, y (June 15, 2023)' },
          //   { value: 'MMM d', label: 'mmm d, y (Jun 15)' },
          //   { value: 'MMMM d', label: 'mmmm d, y (June 15)' },
          //   {
          //     value: 'EEEE, MMMM d, y',
          //     label: 'eeee, mmmm d, y (Saturday, June 15, 2019)',
          //   },
          // ],
          options: [
            { value: 'MM/dd/yy', label: '06/15/23' },
            { value: 'M/d/yy', label: ' 6/15/23' },
            { value: 'MMM d, y', label: 'Jun 15, 2023' },
            { value: 'MMMM d, y', label: 'June 15, 2023' },
            { value: 'MMM d', label: 'Jun 15' },
            { value: 'MMMM d', label: 'June 15' },
            {
              value: 'EEEE, MMMM d, y',
              label: 'Saturday, June 15, 2019',
            },
          ],
          align: 'row',
        },
        extras: {
          labelOverInput: false,
        },
      },
      {
        key: 'timeFormat',
        type: '#select',
        focus: false,
        defaultValue: 'h:mm a',
        props: {
          label: 'Time Format',
          options: [
            { value: 'h:mm a', label: '10:54 PM' },
            { value: 'h:mm:ss a', label: '10:54:25 PM' },
            { value: 'h:mm a zzzz', label: '10:54 PM GMT+05:30' },
            { value: 'h:mm:ss a zzzz', label: '10:54:25 PM GMT+05:30' },
            { value: 'HH:mm', label: '15:00' },
            { value: 'HH:mm zzzz', label: '15:00 GMT+05:30' },
            { value: 'HH:mm:ss zzzz', label: '15:00:25 GMT+05:30' },
          ],
          align: 'row',
        },
        extras: {
          labelOverInput: false,
        },
      },
      {
        key: 'timeZone',
        type: '#select',
        focus: false,
        defaultValue: 'America/New_York',
        props: {
          label: 'Timezone',
          // options: [
          //   { value: 'EDT', label: 'America/New_york' },
          //   { value: 'PST', label: 'America/Los_Angeles' },
          //   { value: 'IST', label: 'Asia/Kolkata' },
          //   { value: 'JST', label: 'Asia/Tokyo' },
          // ],
          options: timeZoneList,
          align: 'row',
        },
        extras: {
          labelOverInput: false,
        },
      },
      // {
      //   key: 'timezone',
      //   type: '#select-async',
      //   props: {
      //     label: 'Timezone',
      //     description: 'Default value id:4',
      //     options: [],
      //     valueProp: 'id',
      //     labelProp: 'timezone',
      //     async: true,
      //     required: false,
      //     align: 'row',
      //     // apiEndpoint: 'https://my.api.mockaroo.com/country.json?key=a77bb3b0',
      //     apiEndpoint: 'https://63239166bb2321cba91c006d.mockapi.io/states',
      //   },
      //   extras: {
      //     hintBelowInput: true,
      //   },
      //   defaultValue: '55',
      // },
    ];

    this.options = {
      submit: {
        label: 'Update',
        events: [
          {
            event: 'success',
            method: 'putAPI',
            params: {
              apiUrl: '/api/v1/userDisplaySettings',
              bodyParams: [],
            },
            events: [
              {
                event: 'success',
                method: 'showToast',
                params: {
                  message: 'Display Settings Updated Successfully.',
                },
              },
              {
                event: 'failure',
                method: 'showErrorToast',
                params: {
                  message: 'Display Settings Update Not Successful.',
                },
              },
              {
                event: 'failure',
                method: 'updateStore',
                enabled: 'data.formData',
                params: {
                  type: '[Application] Set User Display Settings',
                  userDisplaySettings: {
                    timeFormat: '{{data.formData.timeFormat}}',
                    dateFormat: '{{data.formData.dateFormat}}',
                    timeZone: '{{data.formData.timeZone}}',
                  },
                },
              },
              {
                event: 'success',
                method: 'reloadPage',
              },
            ],
          },
        ],
      },
      submitLabel: 'Update',
      // cancel: {
      //   label: 'Reset',
      //   events: [
      //     {
      //       event: 'click',
      //       method: 'goToPage',
      //       params: {
      //         urlPath: '/displayPreferences',
      //       },
      //     },
      //   ],
      // },
      hasReset: true,
      resetForm: true,
    };
  }

  // eslint-disable-next-line @angular-eslint/no-empty-lifecycle-method
  ngOnInit(): void {
    const state = history.state;
    this.edit = state['editable'];

    //TO update Default Value:
    // this.regionalSettingCard;
  }
}
