/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
/* eslint-disable @typescript-eslint/no-empty-function */
import { Component, OnInit } from '@angular/core';
import { AppStoreActions, AppStoreSelectors, Model } from '@hash8io/core-web';
import { UserProfileService } from './service/user-profile.service';
import {
  regionalSettingsAction,
  unitOfMeasurementAction,
  userProfileAction,
  userProfileNavigationConfig,
  usesrProfileTitle,
} from './user-profile-config';
import { FormFields } from '@hash8io/forms-web';
import { FormGroup } from '@angular/forms';
import { Store } from '@ngrx/store';
import {
  editRegionalSettingFields,
  editRegionalSettingsOptions,
} from './components/regional-setting/regional-setting-config';
import {
  editMetricFields,
  editMetricOptions,
} from './components/unit-of-measurement/unit-of-measurement-config';
import { changePasswordConfig } from './components/change-password/change-password-config';

@Component({
  selector: 'hash8-platform-user-profile',
  templateUrl: './user-profile.component.html',
  styleUrls: ['./user-profile.component.scss'],
})
export class UserProfileComponent implements OnInit {
  form = new FormGroup({});

  fields: FormFields[] = [
    {
      type: '#text',
      className: 'w-full',
      key: 'firstName',
      defaultValue: 'Karthick',
      props: {
        label: 'First Name',
        align: 'row',
      },
      // extras: {
      //   labelOverInput: true,
      // },
    },
    {
      type: '#text',
      className: 'w-full',
      key: 'lastName',
      defaultValue: 'T',
      props: {
        label: 'Last Name',
        align: 'row',
      },
      // extras: {
      //   labelOverInput: fal,
      // },
    },
    {
      type: '#text',
      key: 'email',
      props: {
        label: 'Email',
        type: 'email',
        // required: true,
        readonly: true,
        align: 'row',

        // disabled: true,
        hideLabel: false,
      },
      // extras: {
      //   labelOverInput: true,
      // },
      defaultValue: 'johnmathew@gmail.com',
      expressions: {
        // 'props.disabled': '!model.username',
      },
    },
    {
      type: '#phoneNumber',
      key: 'phoneNumber',
      props: {
        hideLabel: false,
        required: false,
        enableSearch: true,
        intlTelInput: true,
        align: 'row',
        readonly: true,
        // format: 'national',
      },
      defaultValue: '+917401099234',

      // extras: {
      //   labelOverInput: true,
      // },
    },
  ];

  options = {
    submit: {
      label: 'Update',
      events: [
        // {
        //   event: 'click',
        //   method: 'setLocalStorage',
        //   params: {
        //     storeParams: [
        //       {
        //         key: 'organizationName',
        //         store: true,
        //         storeName: 'organizationName',
        //       },
        //     ],
        //   },
        // },
        {
          // events: [
          //   {
          event: 'success',
          method: 'postAPI',
          params: {
            apiUrl: '/api/v1/users',
            bodyParams: [
              {
                key: 'invitationUUID',
              },
            ],
          },
          events: [
            {
              event: 'success',
              method: 'showToast',
              params: {
                type: 'information',
                message:
                  'Registration successful. Please sign in with your credentials.',
              },
            },
            {
              event: 'success',
              method: 'goToPage',
              params: {
                urlPath: '/login',
              },
              type: '',
            },
          ],
          //   },
          // ],
        },
      ],
    },
    submitLabel: 'Update',
    cancel: {
      label: 'Cancel',
      // events: [
      //   {
      //     event: 'click',
      //     method: '',
      //     params: {},
      //     type: '',
      //   },
      // ],
    },
    // hasReset: true,
  };

  navigations = userProfileNavigationConfig;
  title = usesrProfileTitle;
  selectedTab: any = 'My Profile';
  tileLayout: Model.TileLayout = {
    columns: 12,
    rows: 'auto',
    gap: '12px',
    tiles: {
      lg: [],
    },
  };

  userProfile!: any;
  displayPreference!: any;

  profileData!: any;
  constructor(
    private userProfileService: UserProfileService<any>,
    private store: Store
  ) {}

  ngOnInit(): void {
    const userUUId = localStorage.getItem('userUUID');

    this.userProfile = userProfileNavigationConfig.find((key: any) => {
      return key;
    });

    this.userProfileService
      .getMyProfileData(this.userProfile?.apiUrl, userUUId)
      .subscribe((data: any) => {
        //My Profile
        this.profileData = data;
        this.profileData.userProfile['actions'] = userProfileAction;

        //Display Preferences
        const displayApiUrl = `/api/v1/user-display-settings/`;
        this.userProfileService
          .getMyProfileData(displayApiUrl) //API URL Static
          .subscribe((data: any) => {
            this.displayPreference = data;

            //Add Default Value - Regional Settings
            editRegionalSettingFields[0].defaultValue =
              this.displayPreference?.regionalSettings?.content[0]?.props
                ?.value || 'MM/dd/yyyy';
            editRegionalSettingFields[1].defaultValue =
              this.displayPreference?.regionalSettings?.content[1]?.props
                ?.value || 'hh:mm a';
            editRegionalSettingFields[2].defaultValue =
              this.displayPreference?.regionalSettings?.content[2]?.props
                ?.value || 'America/New_York';

            //Add Default Value - Units
            editMetricFields[0].defaultValue =
              this.displayPreference?.unitOfMeasurements?.content[0]?.props
                ?.value || 'Inches';
            editMetricFields[1].defaultValue =
              this.displayPreference?.unitOfMeasurements?.content[1]?.props
                ?.value || 'Miles';
            editMetricFields[2].defaultValue =
              this.displayPreference?.unitOfMeasurements?.content[2]?.props
                ?.value || 'Celsius';
            editMetricFields[3].defaultValue =
              this.displayPreference?.unitOfMeasurements?.content[3]?.props
                ?.value || 'Bar';
            editMetricFields[4].defaultValue =
              this.displayPreference?.unitOfMeasurements?.content[4]?.props
                ?.value || 'Gallons';
            editMetricFields[5].defaultValue =
              this.displayPreference?.unitOfMeasurements?.content[5]?.props
                ?.value || 'Pounds';

            //Update in Store
            this.store.dispatch(
              AppStoreActions.SET_USER_DISPLAY_SETTINGS({
                userDisplaySettings: {
                  dateFormat:
                    this.displayPreference?.regionalSettings?.content[0]?.props
                      ?.value || 'MM/dd/yyyy',
                  timeFormat:
                    this.displayPreference?.regionalSettings?.content[1]?.props
                      ?.value || 'hh:mm a',
                  timeZone:
                    this.displayPreference?.regionalSettings?.content[2]?.props
                      ?.value || 'America/New_York',

                  //Units
                  level:
                    this.displayPreference?.unitOfMeasurements?.content[0]
                      ?.props?.value || 'Inches',
                  distance:
                    this.displayPreference?.unitOfMeasurements?.content[1]
                      ?.props?.value || 'Miles',
                  temperature:
                    this.displayPreference?.unitOfMeasurements?.content[2]
                      ?.props?.value || 'Celsius',
                  pressure:
                    this.displayPreference?.unitOfMeasurements?.content[3]
                      ?.props?.value || 'Bar',
                  quantity:
                    this.displayPreference?.unitOfMeasurements?.content[4]
                      ?.props?.value || 'Gallons',
                  weight:
                    this.displayPreference?.unitOfMeasurements?.content[5]
                      ?.props?.value || 'Pounds',
                },
              })
            );

            this.displayPreference.regionalSettings['actions'] =
              regionalSettingsAction;
            this.displayPreference.regionalSettings['editableFields'] =
              editRegionalSettingFields;
            this.displayPreference.regionalSettings['editOptions'] =
              editRegionalSettingsOptions;
            this.displayPreference.unitOfMeasurements['actions'] =
              unitOfMeasurementAction;
            this.displayPreference.unitOfMeasurements['editableFields'] =
              editMetricFields;
            this.displayPreference.unitOfMeasurements['editOptions'] =
              editMetricOptions;
          });

        //Change Password
        this.profileData.changePassword['title'] = changePasswordConfig?.title;
        this.profileData.changePassword['actions'] =
          changePasswordConfig?.actions;

        //-------------------------Display Preference---------------------------

        // this.store
        //   .select(AppStoreSelectors.selectDisplaySettings)
        //   .subscribe((data: any) => {

        // });
        //-------------------------------------------------------------------------------------

        //TEMPORARY. TO-DO Update from branding
        const firstName =
          this.profileData.userProfile?.content[0]?.props?.value || '';
        const lastName =
          this.profileData.userProfile?.content[1]?.props?.value || '';

        this.store.dispatch(
          AppStoreActions.SET_USER_DETAILS({
            userDetails: {
              firstName: firstName,
              lastName: lastName,
            },
          })
        );
      });
  }

  navChanged($event: any) {
    const Label = $event.label ? $event.label : $event.value; //Event based change Bug Fix
    this.selectedTab = Label.toString();
  }
}
