import { Injectable } from '@angular/core';

@Injectable({
  providedIn: 'root',
})
export class BreadcrumbMapWebService {
  constructor() {
    //
  }

  //  Single Breadcrumb Data Format based on breadcrub type
  singleBreadcrumb(
    data: any,
    latitudeField: any,
    longitudeField: any,
    breadcrumbOptions: any
  ) {
    
    const startLocationObj = { ...data[0], markerType: 'startLocationMarker' };
    const endLocationObj = {
      ...data[data.length - 1],
      markerType: 'endLocationMarker',
    };
    const locationObj = data.slice(1, -1);

    const middleLocationObj = locationObj.map((item: any) => {
      return {
        ...item,
        markerType: 'middleLocationMarker',
      };
    });

    const dataFormat = [
      { ...startLocationObj },
      ...middleLocationObj,
      { ...endLocationObj },
    ];

    const options = this.breadcrumbOptions(breadcrumbOptions);
    const polylineFormat = dataFormat.map((item: any) => {
      return {
        lat: item[latitudeField],
        lng: item[longitudeField],
        ...item,
      };
    });
    const breadcrumbConfig = {
      path: polylineFormat,
      ...options,
    };

    const markerConfig = dataFormat.map((item: any) => {
      let markerOptions: any = '';

      if (
        breadcrumbOptions.showMarker == true ||
        breadcrumbOptions.showStartAndEndLocationMarkerOnly == true
      ) {
        markerOptions = this.markerOptions(breadcrumbOptions, item);
      } else {
        const marker = {
          icon: {
            path: '',
          },
        };
        markerOptions = marker;
      }

      return {
        position: {
          lat: item[latitudeField],
          lng: item[longitudeField],
          ...item,
        },
        ...markerOptions,
      };
    });

    return { breadcrumbConfig, markerConfig };
  }

  //  Multiple Breadcrumb Data Format based on breadcrub type
  multipleBreadcrumb(
    data: any,
    latitudeField: any,
    longitudeField: any,
    breadcrumbOptions: any
  ) {
    const dataFormat: any = [];

    data.map((item: any) => {
      const startLocationObj = {
        ...item[0],
        markerType: 'startLocationMarker',
      };
      const endLocationObj = {
        ...item[item.length - 1],
        markerType: 'endLocationMarker',
      };
      const locationObj = item.slice(1, -1);
      const middleLocationObj = locationObj.map((item: any) => {
        return {
          ...item,
          markerType: 'middleLocationMarker',
        };
      });

      const data = [
        { ...startLocationObj },
        ...middleLocationObj,
        { ...endLocationObj },
      ];
      dataFormat.push(data);
    });
    // console.log('dataFormat',dataFormat);

    const options = this.breadcrumbOptions(breadcrumbOptions);

    const polylineFormat: any[] = [];

    dataFormat.map((item: any) => {
      const format = item.map((item: any) => {
        return {
          lat: item[latitudeField],
          lng: item[longitudeField],
          ...item,
        };
      });
      polylineFormat.push(format);
    });

    const breadcrumbConfig = {
      path: polylineFormat,
      breadcrumbOptions: options,
    };

    const markerConfig: any[] = [];

    dataFormat.map((item: any) => {
      const format = item.map((item: any) => {
        const markerOptions = this.markerOptions(breadcrumbOptions, item);

        return {
          position: {
            lat: item[latitudeField],
            lng: item[longitudeField],
            ...item,
          },
          ...markerOptions,
        };
      });
      markerConfig.push(format);
    });

    return { breadcrumbConfig, markerConfig };
  }

  // Marker Options
  markerOptions(options: any, item: any) {
    const markerOptions = {
      ...options.controlOptions,
      animation: google.maps.Animation.DROP,
      icon: '',
      label: '',
    };

    const markerLabel = {
      text: '\ue530',
      fontFamily: 'Material Icons',
      color: '#ffffff',
      fontSize: '18px',
    };

    if (item.markerType == 'startLocationMarker') {
      const markerIcon = {
        path: options.startLocationMarkerSVG
          ? options.startLocationMarkerSVG
          : '',
        strokeColor: options.startLocationMarkerSVGColor
          ? options.startLocationMarkerSVGColor
          : '',
        strokeWeight: options.startLocationMarkerStrokeWeight
          ? options.startLocationMarkerStrokeWeight
          : 4,
        strokeOpacity: options.startLocationMarkerStrokeOpacity
          ? options.startLocationMarkerStrokeOpacity
          : 1,
        scale: options.startLocationMarkerStrokeScale
          ? options.startLocationMarkerStrokeScale
          : 3,
      };
      markerOptions.icon = options.startLocationMarkerIcon
        ? options.startLocationMarkerIcon
        : options.startLocationMarkerSVG
        ? markerIcon
        : '';
      if (options.showMarkerLabel == true) {
        markerOptions.label = options.startLocationMarkerLabel
          ? options.startLocationMarkerLabel
          : markerLabel;
      } else {
        markerOptions.label = '';
      }
    } else if (item.markerType == 'middleLocationMarker') {
      const markerIcon = {
        path: options.middleLocationMarkerSVG
          ? options.middleLocationMarkerSVG
          : '',
        strokeColor: options.middleLocationMarkerSVGColor
          ? options.middleLocationMarkerSVGColor
          : 'blue',
        strokeWeight: options.middleLocationMarkerStrokeWeight
          ? options.middleLocationMarkerStrokeWeight
          : 4,
        strokeOpacity: options.middleLocationMarkerStrokeOpacity
          ? options.middleLocationMarkerStrokeOpacity
          : 1,
        scale: options.middleLocationMarkerStrokeScale
          ? options.middleLocationMarkerStrokeScale
          : 3,
      };

      if (options.showStartAndEndLocationMarkerOnly == true) {
        markerIcon.path = '';
        markerOptions.icon = markerIcon;
      } else {
        markerOptions.icon = options.middleLocationMarkerIcon
          ? options.middleLocationMarkerIcon
          : options.middleLocationMarkerSVG
          ? markerIcon
          : '';

        if (options.showMarkerLabel == true) {
          markerOptions.label = options.middleLocationMarkerLabel
            ? options.middleLocationMarkerLabel
            : '';
        } else {
          markerOptions.label = '';
        }
      }
    } else if (item.markerType == 'endLocationMarker') {
      const markerIcon = {
        path: options.endLocationMarkerSVG ? options.endLocationMarkerSVG : '',
        strokeColor: options.endLocationMarkerSVGColor
          ? options.endLocationMarkerSVGColor
          : '',
        strokeWeight: options.endLocationMarkerStrokeWeight
          ? options.endLocationMarkerStrokeWeight
          : 4,
        strokeOpacity: options.endLocationMarkerStrokeOpacity
          ? options.endLocationMarkerStrokeOpacity
          : 1,
        scale: options.endLocationMarkerStrokeScale
          ? options.endLocationMarkerStrokeScale
          : 3,
      };
      markerOptions.icon = options.endLocationMarkerIcon
        ? options.endLocationMarkerIcon
        : options.endLocationMarkerSVG
        ? markerIcon
        : '';
      if (options.showMarkerLabel == true) {
        markerOptions.label = options.endLocationMarkerLabel
          ? options.endLocationMarkerLabel
          : markerLabel;
      } else {
        markerOptions.label = '';
      }
    } else {
      //
    }

    return markerOptions;
  }

  // Breadcrumb Options
  breadcrumbOptions(options: any) {
    let icons: any;
    let strokeOpacity: any;

    if (options?.showBreadcrumb == true) {
      switch (options?.lineStyle) {
        case 'line':
          strokeOpacity = 1.0;
          icons = [
            {
              icon: {},
            },
          ];
          break;
        case 'dashed':
          strokeOpacity = 0;
          icons = [
            {
              icon: {
                path: 'M 0,-1 0,1',
                strokeOpacity: 1,
                scale: 3,
              },
              offset: '0',
              repeat: '20px',
            },
          ];
          break;
        case 'arrow':
          strokeOpacity = 1.0;
          icons = [
            {
              icon: {
                path: google.maps.SymbolPath.FORWARD_OPEN_ARROW,
              },
              offset: '20%',
              repeat: '200px',
            },
          ];
          break;
      }
    } else {
      strokeOpacity = 0;
    }

    const breadcrumbOptions = {
      ...options?.controlOptions,
      icons: icons,
      strokeColor: options.lineColor ? options.lineColor : 'blue',
      strokeOpacity: strokeOpacity,
      strokeWeight: 2,
    };
    return breadcrumbOptions;
  }
}
