import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { ToastService } from '@hash8io/core-web';
import { catchError, map, of } from 'rxjs';
import { CASCADE_CONFIG_MOCK } from '../mock-data/cascade-config.mock';

@Injectable({
  providedIn: 'root',
})
export class CascadeReportWebService<T> {
  constructor(
    private http: HttpClient,
    private route: Router,
    private toast: ToastService
  ) {}
  authenticationToken = localStorage.getItem('access_token');

  getCascadeReportData(
    apiEndPoint: string,
    uuid?: any,
    layerApiUrl?: any,
    queryParams?: any
  ) {
    const options = {
      headers: new HttpHeaders({
        'Content-Type': 'application/json',
        Authorization: 'Bearer ' + this.authenticationToken,
      }),
      params: queryParams ? queryParams : '',
    };

    if (apiEndPoint) {
      return this.http.get<T[]>(apiEndPoint, options).pipe(
        catchError((error) => {
          if (error?.status === 401) {
            this.route.navigateByUrl('/login');
            this.toast.message('Session Expired');
          }
          return of([]);
        }),
        map((res: any) => {
          if (res.data) {
            const data = res.data;
            data['children'] = data['reportConfigurations'];
            delete data['reportConfigurations'];

            data['layerApiUrl'] = layerApiUrl ? layerApiUrl : '';

            return data;
          } else {
            return 'No Data';
          }
        })
      );
    } else {
      return this.http.get<T[]>(apiEndPoint, options).pipe((data): any => {
        return of((CASCADE_CONFIG_MOCK as any)[uuid]);
      });
    }
  }
}
