import {
  Component,
  EventEmitter,
  Input,
  OnInit,
  Output,
  ViewChild,
} from '@angular/core';
// import { Hash8ChartOptions } from 'libs/chart-web/src/lib/models/chart-web.model';
import { ChartComponent } from 'ng-apexcharts';

import { Hash8ChartOptions, ChartWebService } from '@hash8io/chart-web';
import { Model } from '@hash8io/core-web';

// import { Hash8ChartOptions } from '../../../src/lib/models/chart-web.model';
// import { ChartWebService } from '../../../src/lib/services/chart-web.service';

@Component({
  selector: 'hash8-platform-area-chart',
  templateUrl: './area-chart.component.html',
  styleUrls: ['./area-chart.component.scss'],
})
export class AreaChartComponent implements OnInit {
  
  @ViewChild('chart', { static: false }) chart!: ChartComponent;
  @Input() chartOptions!: Hash8ChartOptions;
  @Input() showHeader = true;
  @Input() title: any;
  @Input() subtitle: any;
  @Output() dataClick: any = new EventEmitter<any>();

  chartObj: any;

  constructor(public chartWebService: ChartWebService) {
    //
  }

  ngOnInit(): void {
    this.chartObj = this.chartWebService.chartFormat(this.chartOptions);
    const chartClickEvent = {
      events: {
        markerClick: (e?: any, chart?: any, config?: any) => {
          this.chartData('click', config);
        },
        dataPointSelection: (e?: any, chart?: any, config?: any) => {
          console.log('DATA POINT SELECTION');
          this.chartData('click', config);
        },
        beforeMount: (chartContext?: any, config?: any) => {
          this.chartData('beforeMount', config);
        },
        mounted: (chartContext?: any, config?: any) => {
          this.chartData('mount', config);
        },
        updated: (chartContext?: any, config?: any) => {
          this.chartData('updated', config);
        },
        xAxisLabelClick: (event?: any, chartContext?: any, config?: any) => {
          this.chartData('xaxisLabelClick', config);
        },
      },
    };
    this.chartObj.chart = { ...this.chartObj.chart, ...chartClickEvent };
  }

  chartData(event: any, config: any) {
    const chartType = this.chartObj.chart.type;
    if (event == 'click') {
      const data = this.chartWebService.chartClickEvent(
        event,
        config,
        this.chartOptions,
        chartType
      );
      this.dataClick.emit({ event, data });
    } else {
      this.dataClick.emit({ event });
    }
  }
}
