import { Injectable, Type } from '@angular/core';
import {
  annotationsCommonFactor,
  chartCommonFactor,
  colorsCommonFactor,
  dataLabelsCommonFactor,
  fillCommonFactor,
  forecastCommonFactor,
  gridCommonFactor,
  legendCommonFactor,
  markersCommonFactor,
  noDataCommonFactor,
  plotOptionsCommonFactor,
  responsiveCommonFactor,
  statesCommonFactor,
  strokeCommonFactor,
  subtitleCommonFactor,
  themeCommonFactor,
  titleCommonFactor,
  tooltipCommonFactor,
  xaxisCommonFactor,
  yaxisCommonFactor,
} from '../chart-common-data/chart-common-factor.data';
import { AppStoreSelectors } from '@hash8io/core-web';
import { Store } from '@ngrx/store';

@Injectable({
  providedIn: 'root',
})
export class ChartWebService {
  chartThemeData = {
    fontFamily: '',
    mode: 'light',
  };


  constructor(private store: Store) {
    this.store.select(AppStoreSelectors.selectTheme).subscribe((theme) => {
      this.chartThemeData.fontFamily = theme.fontFamily;
      this.chartThemeData.mode = theme.defaultMode;
    });
  }

  // line | bar | area
  chartFormat(options: any) {
    // Iterate series array object and form x and y coordinates
    const chartData: any = {
      type: '',
    };
    const lineCoordinateData: any = [];
    let lineCoordinate: any = {};
    options.series.map((item: any) => {

      if (options?.xaxis?.field || options?.yaxis?.field) {
        lineCoordinate = item.data.map((coordinate: any) => {
          return {
            x: coordinate[options?.xaxis?.field],
            y: coordinate[options?.yaxis?.field],
          };
        });
      } else if (!options?.xaxis?.field || !options?.yaxis?.field) {
        lineCoordinate = item.data.map((coordinate: any) => {
          return {
            x: coordinate[item?.xaxis?.field],
            y: coordinate[item?.yaxis?.field],
          };
        });
      }

      const dataFormat = {
        name: item.name ? item.name : '',
        type: item.type ? item.type : undefined,
        color: item.color ? item.color : undefined,
        data: lineCoordinate,
      };
      chartData.type = options?.chart?.type ? options?.chart?.type : item.type;
      lineCoordinateData.push(dataFormat);
    });

    // call common factor function
    const commonFactor = this.commonFactor(options, chartData);

    // Make Chart Whole object with coordinates and common factors
    const chartConfig = {
      series: lineCoordinateData,
      chart: {
        ...chartCommonFactor,
        ...chartData,
        ...this.chartThemeData,
        ...options.chart,
      },
      labels: options.labels ? options.labels : '',
      ...commonFactor,
    };
    return chartConfig;
  }

  // pie | donut | radialBar | polarArea
  circleChartFormat(options: any) {
    // Iterate series array object and form x and y coordinates

    let circleCoordinateData: any; // Series Data
    let labels: any;
    const circleChartData: any = {
      type: '',
    };
    options.series.map((item: any) => {
      const circleCoordinate = item.data.map((coordinate: any) => {
        return {
          x: coordinate[options.xaxis.field],
          y: coordinate[options.yaxis.field],
        };
      });

      circleChartData.type = options?.chart?.type ? options?.chart?.type : item.type;
      circleCoordinateData = circleCoordinate.map((item: any) => item.y);
      labels = circleCoordinate.map((item: any) => item.x);
    });

    // call common factor function
    const commonFactor = this.commonFactor(options, circleChartData);

    // Make Chart Whole object with coordinates and common factors
    const chartConfig = {
      series: circleCoordinateData,
      chart: {
        ...chartCommonFactor,
        ...circleChartData,
        ...this.chartThemeData,
        ...options.chart,
      },
      labels: labels,
      ...commonFactor,
    };
    return chartConfig;
  }

  // treemap
  tree_HeatMap_Radar_ChartFormat(options: any) {
    // Iterate series array object and form x and y coordinates

    const chartCoordinateData: any = [];
    const chartData: any = {
      type: '',
    };
    options.series.map((item: any) => {
      const chartCoordinate = item.data.map((coordinate: any) => {
        return {
          x: coordinate[options.xaxis.field],
          y: coordinate[options.yaxis.field],
        };
      });

      const dataFormat = {
        name: item.name ? item.name : '',
        color: item.color ? item.color : undefined,
        data: chartCoordinate,
      };

      chartCoordinateData.push(dataFormat);
      chartData.type = options?.chart?.type ? options?.chart?.type : item.type;
    });

    // call common factor function
    const commonFactor = this.commonFactor(options, chartData);

    // Make Chart Whole object with coordinates and common factors
    const chartConfig = {
      series: chartCoordinateData,
      chart: {
        ...chartCommonFactor,
        ...chartData,
        ...this.chartThemeData,
        ...options.chart,
      },
      labels: options.labels ? options.labels : '',
      ...commonFactor,
    };
    return chartConfig;
  }

  // boxplot ~ boxplot : [ minimum, Q1, Median, Q3, Maximum ]
  boxPlotChartFormat(options: any) {
    // Iterate series array object and form x and y coordinates
    const boxPlotCoordinateData: any = [];
    const boxPlotChartData: any = {
      type: '',
    };
    options.series.map((item: any) => {
      const boxPlotCoordinate = item.data.map((coordinate: any) => {
        const data: any = {
          x: coordinate[options.xaxis.field],
          y1: coordinate[
            options.yaxis.field ? options.yaxis.field[0] : undefined
          ],
          y2: coordinate[
            options.yaxis.field ? options.yaxis.field[1] : undefined
          ],
          y3: coordinate[
            options.yaxis.field ? options.yaxis.field[2] : undefined
          ],
          y4: coordinate[
            options.yaxis.field ? options.yaxis.field[3] : undefined
          ],
          y5: coordinate[
            options.yaxis.field ? options.yaxis.field[4] : undefined
          ],
        };

        return {
          x: data.x,
          y: [data.y1, data.y2, data.y3, data.y4, data.y5],
        };
      });

      const dataFormat = {
        name: item.name ? item.name : '',
        type: item.type ? item.type : undefined,
        color: item.color ? item.color : undefined,
        data: boxPlotCoordinate,
      };
      boxPlotChartData.type = options?.chart?.type ? options?.chart?.type : item.type;
      boxPlotCoordinateData.push(dataFormat);
    });

    // call common factor function
    const commonFactor = this.commonFactor(options, boxPlotChartData);

    // Make Chart Whole object with coordinates and common factors
    const chartConfig = {
      series: boxPlotCoordinateData,
      chart: {
        ...chartCommonFactor,
        ...boxPlotChartData,
        ...this.chartThemeData,
        ...options.chart,
      },
      labels: { ...options.labels },
      ...commonFactor,
    };
    return chartConfig;
  }

  // candlestick ~ candlestick : 	[ open, high, low, close ]
  candleStickChartFormat(options: any) {
    // Iterate series array object and form x and y coordinates
    const candleStickCoordinateData: any = [];
    const candleStickChartData: any = {
      type: '',
    };
    options.series.map((item: any) => {
      const candleStickCoordinate = item.data.map((coordinate: any) => {
        const data: any = {
          x: coordinate[options.xaxis.field],
          y1: coordinate[
            options.yaxis.field ? options.yaxis.field[0] : undefined
          ],
          y2: coordinate[
            options.yaxis.field ? options.yaxis.field[1] : undefined
          ],
          y3: coordinate[
            options.yaxis.field ? options.yaxis.field[2] : undefined
          ],
          y4: coordinate[
            options.yaxis.field ? options.yaxis.field[3] : undefined
          ],
        };

        return {
          x: data.x,
          y: [data.y1, data.y2, data.y3, data.y4],
        };
      });

      const dataFormat = {
        name: item.name ? item.name : '',
        type: item.type ? item.type : undefined,
        color: item.color ? item.color : undefined,
        data: candleStickCoordinate,
      };
      candleStickChartData.type = options?.chart?.type ? options?.chart?.type : item.type;
      candleStickCoordinateData.push(dataFormat);
    });

    // call common factor function
    const commonFactor = this.commonFactor(options, candleStickChartData);

    // Make Chart Whole object with coordinates and common factors
    const chartConfig = {
      series: candleStickCoordinateData,
      chart: {
        ...chartCommonFactor,
        ...candleStickChartData,
        ...this.chartThemeData,
        ...options.chart,
      },
      labels: { ...options.labels },
      ...commonFactor,
    };
    return chartConfig;
  }

  // rangeBar ~ Range Bar : [ start, end ]
  rangeBarChartFormat(options: any) {
    // Iterate series array object and form x and y coordinates
    const rangeBarCoordinateData: any = [];
    const rangeBarChartData: any = {
      type: '',
    };
    const rangeBarTooltip: any = {
      enabled: true,
    };
    options.series.map((item: any) => {
      const rangeBarCoordinate = item.data.map((coordinate: any) => {
        const data: any = {
          x: coordinate[options.xaxis.field],
          y1: coordinate[
            options.yaxis.field ? options.yaxis.field[0] : undefined
          ],
          y2: coordinate[
            options.yaxis.field ? options.yaxis.field[1] : undefined
          ],
        };

        return {
          x: data.x,
          y: [data.y1, data.y2],
        };
      });

      const dataFormat = {
        name: item.name ? item.name : '',
        type: item.type ? item.type : undefined,
        color: item.color ? item.color : undefined,
        data: rangeBarCoordinate,
      };
      rangeBarChartData.type = options?.chart?.type ? options?.chart?.type : item.type;
      rangeBarCoordinateData.push(dataFormat);

      if (item.type == 'rangeBar') {
        rangeBarTooltip.enabled = false;
      } else {
        rangeBarTooltip.enabled = true;
      }
    });

    // call common factor function
    const commonFactor = this.commonFactor(options, rangeBarChartData);

    // Make Chart Whole object with coordinates and common factors
    const chartConfig = {
      series: rangeBarCoordinateData,
      chart: {
        ...chartCommonFactor,
        ...rangeBarChartData,
        ...this.chartThemeData,
        ...options.chart,
      },
      labels: { ...options.labels },
      ...commonFactor,
      tooltip: { ...rangeBarTooltip },
    };
    return chartConfig;
  }

  // scatter
  scatterChartFormat(options: any) {
    // Iterate series array object and form x and y coordinates

    const chartCoordinateData: any = [];
    let labels: any;
    const chartData: any = {
      type: '',
    };
    options.series.map((item: any) => {
      const chartCoordinate = item.data.map((coordinate: any) => {
        return [
          coordinate[options.xaxis.field],
          coordinate[options.yaxis.field],
        ];
      });

      const dataFormat = {
        name: item.name ? item.name : '',
        color: item.color ? item.color : undefined,
        data: chartCoordinate,
      };

      chartCoordinateData.push(dataFormat);
      chartData.type = options?.chart?.type ? options?.chart?.type : item.type;
    });

    // call common factor function
    const commonFactor = this.commonFactor(options, chartData);

    // Make Chart Whole object with coordinates and common factors
    const chartConfig = {
      series: chartCoordinateData,
      chart: {
        ...chartCommonFactor,
        ...chartData,
        ...this.chartThemeData,
        ...options.chart,
      },
      labels: options.labels ? options.labels : '',
      ...commonFactor,
    };
    return chartConfig;
  }

  // bubble
  bubbleChartFormat(options: any) {
    // Iterate series array object and form x and y coordinates

    const chartCoordinateData: any = [];
    const chartData: any = {
      type: '',
    };
    options.series.map((item: any) => {
      const chartCoordinate = item.data.map((coordinate: any) => {
        return [coordinate.map((item: any) => item.y)];
      });

      chartData.type = options?.chart?.type ? options?.chart?.type : item.type;

      const dataFormat = {
        name: item.name ? item.name : '',
        type: item.type ? item.type : undefined,
        color: item.color ? item.color : undefined,
        data: chartCoordinate,
      };

      chartCoordinateData.push(dataFormat);
    });

    // call common factor function
    const commonFactor = this.commonFactor(options, chartData);

    // Make Chart Whole object with coordinates and common factors
    const chartConfig = {
      series: chartCoordinateData,
      chart: {
        ...chartCommonFactor,
        ...chartData,
        ...this.chartThemeData,
        ...options.chart,
      },
      labels: [options.labels ? options.labels : ''],
      ...commonFactor,
    };
    return chartConfig;
  }

  // sparkline
  sparklineChartFormat(options: any) {
    let chartFormat: any = '';
    switch (options.chart.type) {
      case 'line':
      case 'bar':
      case 'area':
        chartFormat = this.chartFormat(options);
        break;
      case 'pie':
      case 'donut':
      case 'polarArea':
      case 'radialBar':
        chartFormat = this.circleChartFormat(options);
        break;
      case 'treemap':
      case 'heatmap':
      case 'radar':
        chartFormat = this.tree_HeatMap_Radar_ChartFormat(options);
        console.log(chartFormat);
        break;
      case 'boxplot':
        chartFormat = this.boxPlotChartFormat(options);
        break;
      case 'candlestick':
        chartFormat = this.candleStickChartFormat(options);
        break;
      case 'rangeBar':
      case 'rangeArea':
        chartFormat = this.rangeBarChartFormat(options);
        break;
      case 'scatter':
        chartFormat = this.scatterChartFormat(options);
        break;
      case 'bubble':
        chartFormat = this.bubbleChartFormat(options);
        break;
      default:
        console.log('Empty spakline chart.');
    }
    return chartFormat;
  }

  // Chart Common Factors
  commonFactor(options: any, chartType: any) {

    const dataLabelsEnable: any = {
      enabled: true,
    };

    if (
      chartType.type == 'radar' ||
      chartType.type == 'polarArea' ||
      chartType.type == 'boxPlot' ||
      chartType.type == 'candlestick' ||
      chartType.type == 'scatter'
    ) {
      dataLabelsEnable.enabled = false;
    } else {
      dataLabelsEnable.enabled = true;
    }

    const commonFactor = {
      markers: { ...markersCommonFactor, ...options.markers },
      legend: { ...legendCommonFactor, ...options.legend },
      stroke: { ...strokeCommonFactor, ...options.stroke },
      dataLabels: {
        ...dataLabelsCommonFactor,
        ...dataLabelsEnable,
        ...options.dataLabels,
      },
      theme: { ...themeCommonFactor, ...this.chartThemeData, ...options.theme },
      states: { ...statesCommonFactor, ...options.dataLabels },
      annotations: { ...annotationsCommonFactor, ...options.dataLabels },
      tooltip: { ...tooltipCommonFactor, ...options.tooltip },
      plotOptions: { ...plotOptionsCommonFactor, ...options.plotOptions },
      fill: { ...fillCommonFactor, ...options.tooltip },
      responsive: responsiveCommonFactor,
      grid: { ...gridCommonFactor, ...options.grid },
      title: { ...titleCommonFactor, ...options.title },
      subtitle: { ...subtitleCommonFactor, ...options.tooltip },
      noData: { ...noDataCommonFactor, ...options.noData },
      forecastDataPoints: { ...forecastCommonFactor, ...options.forecast },
      xaxis: { ...xaxisCommonFactor, ...options.xaxis },
      yaxis: { ...yaxisCommonFactor, ...options.yaxis },
      colors: [
        ...new Set([
          ...(options.colors ? options.colors : ''),
          ...colorsCommonFactor,
        ]),
      ],
    };
    return commonFactor;
  }

  chartClickEvent(event: any, config: any, chartOptions: any, chartType: any) {
    const seriesIndex = config.seriesIndex;
    const dataPointIndex = config.dataPointIndex;
    const seriesData = chartOptions.series;

    const obj = {
      xaxisField: chartOptions.xaxis?.field
        ? chartOptions.xaxis.field
        : 'xaxis',
      yaxisField: chartOptions.yaxis?.field
        ? chartOptions.yaxis.field
        : 'yaxis',
      xaxisDataIdentifier: chartOptions.xaxis?.dataIdentifier
        ? chartOptions.xaxis.dataIdentifier
        : 'xaxisDataId',
      yaxisDataIdentifier: chartOptions.yaxis?.dataIdentifier
        ? chartOptions.yaxis.dataIdentifier
        : 'yaxisDataId',
      segmentIdentifier: chartOptions.xaxis?.segmentIdentifier
        ? chartOptions.xaxis.segmentIdentifier
        : 'segmentId',
      startLevelField: chartOptions.yaxis?.field
        ? chartOptions.yaxis.field[0]
        : 'startLevelField',
      endLevelField: chartOptions.yaxis?.field
        ? chartOptions.yaxis.field[1]
        : 'endLevelField',
    };

    const fieldData = {
      [obj.xaxisDataIdentifier]:
        seriesData[seriesIndex].data[dataPointIndex][obj.xaxisDataIdentifier],
      [obj.yaxisDataIdentifier]:
        seriesData[seriesIndex].data[dataPointIndex][obj.yaxisDataIdentifier],
      [obj.segmentIdentifier]:
        seriesData[seriesIndex].data[dataPointIndex][obj.segmentIdentifier],
    };

    switch (chartType) {
      case 'line':
      case 'area':
      case 'bar':
      case 'radar':
      case 'treemap':
      case 'heatmap': {
        const data = {
          type: config.w.config.series[config.seriesIndex].type,
          segmentName: config.w.config.series[config.seriesIndex].name,
          [obj.yaxisField]:
            config.w.config.series[config.seriesIndex].data[
              config.dataPointIndex
            ].y,
          [obj.xaxisField]:
            config.w.config.series[config.seriesIndex].data[
              config.dataPointIndex
            ].x,
          ...fieldData,
        };
        return data;
      }
      case 'pie':
      case 'donut':
      case 'polarArea':
      case 'radialBar': {
        const data = {
          type: chartType,
          segmentName: '',
          [obj.yaxisField]: config.w.config.series[config.dataPointIndex],
          [obj.xaxisField]: config.w.config.labels[config.dataPointIndex],
          ...fieldData,
        };
        return data;
      }
      case 'boxPlot': {
        const data = {
          type: chartType,
          segmentName: config.w.config.series[config.seriesIndex].name,
          mimimumLevelField:
            config.w.config.series[config.seriesIndex].data[
              config.dataPointIndex
            ].y[0],
          firstQuartileField:
            config.w.config.series[config.seriesIndex].data[
              config.dataPointIndex
            ].y[1],
          secondQuartileField:
            config.w.config.series[config.seriesIndex].data[
              config.dataPointIndex
            ].y[2],
          thirdQuartileField:
            config.w.config.series[config.seriesIndex].data[
              config.dataPointIndex
            ].y[3],
          maximumValueField:
            config.w.config.series[config.seriesIndex].data[
              config.dataPointIndex
            ].y[4],
          [obj.xaxisField]:
            config.w.config.series[config.seriesIndex].data[
              config.dataPointIndex
            ].x,
          ...fieldData,
        };
        return data;
      }
      case 'candlestick': {
        const data = {
          type: chartType,
          segmentName: config.w.config.series[config.seriesIndex].name,
          openLevelField:
            config.w.config.series[config.seriesIndex].data[
              config.dataPointIndex
            ].y[0],
          highLevelField:
            config.w.config.series[config.seriesIndex].data[
              config.dataPointIndex
            ].y[1],
          lowlevelField:
            config.w.config.series[config.seriesIndex].data[
              config.dataPointIndex
            ].y[2],
          closeLevelField:
            config.w.config.series[config.seriesIndex].data[
              config.dataPointIndex
            ].y[3],
          [obj.xaxisField]:
            config.w.config.series[config.seriesIndex].data[
              config.dataPointIndex
            ].x,
          ...fieldData,
        };
        return data;
      }
      case 'rangeBar':
      case 'rangeArea': {
        const data = {
          type: chartType,
          segmentName: config.w.config.series[config.seriesIndex].name,
          [obj.startLevelField]:
            config.w.config.series[config.seriesIndex].data[
              config.dataPointIndex
            ].y[0],
          [obj.endLevelField]:
            config.w.config.series[config.seriesIndex].data[
              config.dataPointIndex
            ].y[1],
          [obj.xaxisField]:
            config.w.config.series[config.seriesIndex].data[
              config.dataPointIndex
            ].x,
          ...fieldData,
        };
        return data;
      }
      case 'scatter': {
        const data = {
          type: chartType,
          segmentName: config.w.config.series[config.seriesIndex].name,
          [obj.xaxisField]:
            config.w.config.series[config.seriesIndex].data[
            config.dataPointIndex
            ][0],
          [obj.yaxisField]:
            config.w.config.series[config.seriesIndex].data[
            config.dataPointIndex
            ][1],
          ...fieldData,
        };
        return data;
      }
      default:
        return false;
    }
  }
}
