import { Injectable } from '@angular/core';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { EMPTY, of } from 'rxjs';
import { catchError, map, mergeMap, startWith } from 'rxjs/operators';
import * as ReportGridActions from './report-grid.actions';
import { ReportGridService } from './services/report-grid.service';
import { ToastService } from '@hash8io/core-web';

@Injectable()
export class ReportGridEffects {
  loadReportGridConfiguration$ = createEffect(() =>
    this.actions$.pipe(
      ofType(ReportGridActions.GET_REPORT_CONFIG),
      mergeMap((action) => {
        return this.reportGridService
          .getReportGridConfiguration(action.apiUrl)
          .pipe(
            map((resp) => resp.data), // Transformation for our api structure, It wont work for general common API
            map(
              (data) => {
                const { columns, name, ...config } = data?.configurations || {};
                const gridObj = {
                  name: action.gridName || name,
                  apiUrl: action.apiUrl,
                  columns: columns || [],
                  rowData: data?.rowData || [],
                  rowDataIds: data?.rowData.map((r: any) => r.uuid),
                  // configurations: config || {}, // FINAL
                  // TEMP
                  configurations:
                    {
                      actions: action.actions,
                      breadcrumbs: action.breadcrumbs,
                      style: action.style,
                      settings: action.settings,
                      ...config,
                    } || {},
                  totalCount: data?.totalCount || 0,
                  loading: false,
                  refreshGrid: false,
                  selectedRows: [],
                  selectedRowIds: [],
                };
                // console.log(
                //   '%creport-grid.effects.ts line:33 gridObj',
                //   'color: #007acc;',
                //   gridObj
                // );
                return ReportGridActions.ADD_DATA_REPORT({
                  grid: { ...gridObj },
                });
              },
              catchError((error) =>
                of(ReportGridActions.REFRESH_REPORT_END({ gridName: '' }))
              )
            )
            // startWith(
            //   ReportGridActions.SET_LOADING({ gridName: action.gridName })
            // )
          );
      })
    )
  );

  updateReportGridConfiguration$ = createEffect(() =>
    this.actions$.pipe(
      ofType(ReportGridActions.RELOAD_REPORT),
      mergeMap((action) => {
        return this.reportGridService
          .getReportGridConfiguration(action.apiUrl)
          .pipe(
            map((resp) => resp.data),
            map(
              (data) => {
                const { columns, name, ...config } = data?.configurations || {};
                const gridObj = {
                  name: action.gridName || name,
                  apiUrl: action.apiUrl,
                  columns: columns || [],
                  rowData: data?.rowData || [],
                  rowDataIds: data?.rowData.map((r: any) => r.uuid),
                  // configurations: config || {}, // FINAL
                  // TEMP
                  configurations:
                    {
                      actions: data?.configurations?.actions,
                      breadcrumbs: data?.configurations?.breadcrumbs,
                      style: data?.configurations?.style,
                      settings: data?.configurations?.settings,
                      ...config,
                    } || {},
                  totalCount: data?.totalCount || 0,
                  loading: true,
                  refreshGrid: false,
                  selectedRows: [],
                  selectedRowIds: [],
                };

                return ReportGridActions.ADD_DATA_REPORT({
                  grid: { ...gridObj },
                });
              },
              catchError((error) =>
                of(ReportGridActions.REFRESH_REPORT_END({ gridName: '' }))
              )
            )
            // startWith(
            //   ReportGridActions.SET_LOADING({ gridName: action.gridName })
            // )
          );
      })
    )
  );

  assignTags$ = createEffect(
    () =>
      this.actions$.pipe(
        ofType(ReportGridActions.ASSIGN_UNASSIGN_TAGS),
        mergeMap((action) =>
          this.reportGridService.assignTags(action.data).pipe(
            map((response: any) => response), // Transform data based on our response
            mergeMap((response: any) => {
              this.toastService.message(response.message);
              if (response.status) {
                return of(
                  ReportGridActions.ASSIGN_TAGS_SUCCESS({
                    gridName: action.gridName,
                    data: response.data,
                  })
                );
              } else {
                return EMPTY;
              }
            }),
            // map((tags: any) => {
            //   return tags;
            // }),
            // map((tags) =>
            //   ReportGridActions.ASSIGN_TAGS_SUCCESS({
            //     gridName: action.gridName,
            //     data: tags,
            //   })
            // ),
            // map((tags) => ReportGridActions.replaceTags({ tags: tags })),
            catchError((error) => EMPTY)
          )
        )
      ),
    {
      dispatch: true,
    }
  );
  // loadReportGrid$ = createEffect(() =>
  //   this.actions$.pipe(
  //     ofType(ReportGridActions.ADD_DATA_REPORT),
  //     mergeMap(() =>
  //       this.reportGridService.getReportConfig('').pipe(
  //         map(
  //           (tags) => ReportGridActions.REFRESH_REPORT({ gridName: '' }),
  //           catchError((error) =>
  //             of(ReportGridActions.REFRESH_REPORT_END({ gridName: '' }))
  //           )
  //         )
  //       )
  //     )
  //   )
  // );

  // refreshTags$ = createEffect(() =>
  //   this.actions$.pipe(
  //     ofType(ReportGridActions.refreshTags),
  //     mergeMap(() =>
  //       this.reportGridService.getTags().pipe(
  //         map((tags) => ReportGridActions.replaceTags({ tags: tags })),
  //         catchError((error) =>
  //           of(ReportGridActions.loadTagsFailure({ error }))
  //         )
  //       )
  //     )
  //   )
  // );

  constructor(
    private actions$: Actions,
    private toastService: ToastService,
    private reportGridService: ReportGridService<any>
  ) {}
}
