import { createReducer, on } from '@ngrx/store';
import * as ReportStoreActions from './report-grid.actions';
import { initialState } from './report-grid.state';
import { produce } from 'immer';

export const dataReportReducer = createReducer(
  initialState,

  /** Get report data grid */
  // on(ReportStoreActions.GET_REPORT_CONFIG, (state, { apiUrl, gridName }) => {
  //   return {
  //     ...state,
  //     [gridName || 'defaultGrid']: {
  //       ...state[gridName || 'defaultGrid'],
  //       name: gridName || 'defaultGrid',
  //       apiUrl,
  //     },
  //   };
  // }),

  /** Add report data grid */
  on(ReportStoreActions.ADD_DATA_REPORT, (state, { grid }) => {
    // console.log(grid, 'Grid');
    // return {
    //   ...state,
    //   [grid?.name || 'defaultGrid']: {
    //     ...state[grid?.name || 'defaultGrid'],
    //     ...grid,
    //     loading: false,
    //   },
    // };
    return {
      ...state,
      [grid?.name || 'defaultGrid']: grid,
    };
  }),

  /** Remove report data grid */
  on(ReportStoreActions.REMOVE_REPORT_CONFIG, (state, { gridName }) => {
    const newState: any = { ...state };
    delete newState[gridName];

    return newState;
  }),

  /** Refresh report data grid */
  on(ReportStoreActions.REFRESH_REPORT, (state: any, { gridName }) => {
    return {
      ...state,
      [gridName]: {
        ...state[gridName],
        refreshGrid: true,
      },
    };
  }),

  /** Refresh report data grid */
  on(ReportStoreActions.REFRESH_REPORT_END, (state: any, { gridName }) => ({
    ...state,
    [gridName]: {
      ...state[gridName],
      refreshGrid: false,
    },
  })),

  /** Update report data grid configuration */
  on(
    ReportStoreActions.UPDATE_REPORT_CONFIG,
    (state: any, { gridName, updates }) => ({
      ...state,
      [gridName]: {
        ...state[gridName],
        ...updates,
      },
    })
  ),

  /** Replace tags */
  on(
    ReportStoreActions.REPLACE_TAGS,
    (state: any, { gridName, selectedRowIds, tagIds }) => {
      const updatedRowObjects = state[gridName]?.rowData?.map((obj: any) => {
        if (selectedRowIds.includes(obj.assetUUID)) {
          return { ...obj, tags: tagIds };
        }
        return obj;
      });

      return {
        ...state,
        [gridName]: {
          ...state[gridName],
          rowData: updatedRowObjects,
        },
      };
    }
  ),

  /** Assign tags */
  on(
    ReportStoreActions.ASSIGN_TAGS_SUCCESS,
    (state: any, { gridName, data }) => {
      return produce(state, (draftState: any) => {
        const updatedRowObjects = draftState[gridName]?.rowData?.map(
          (obj: any) => {
            const resourceUuid = obj.assetUUID;
            const matchingResource = data.resources.find((resource: any) => {
              return resource.uuid === resourceUuid;
            });

            if (matchingResource) {
              let finalTags = obj.tags;
              data.tags.map((tg: any) => {
                if (tg.action === 'Assign') {
                  const eTagExist = finalTags?.findIndex((eTag: any) => {
                    return eTag.uuid === tg.uuid;
                  });
                  if (eTagExist === -1) {
                    finalTags.push(tg);
                  }
                }

                if (tg.action === 'Unassign') {
                  finalTags = finalTags?.filter((eTag: any) => {
                    return eTag.uuid !== tg.uuid;
                  });
                }
              });

              return {
                ...obj,
                tags: [...finalTags],
              };
            }
            return obj;
          }
        );
        draftState[gridName].rowData = updatedRowObjects;
      });
    }
  ),

  on(
    ReportStoreActions.ASSIGN_TAGS,
    (state: any, { gridName, selectedRowIds, tagIds }) => {
      const updatedRowObjects = state[gridName]?.rowData?.map((obj: any) => {
        if (selectedRowIds.includes(obj.assetUUID)) {
          const _rowTagIds = obj.tags?.map((t: any) => t.uuid);
          const updatedTags = tagIds.filter(
            (tag: any) => !_rowTagIds?.includes(tag.uuid)
          );
          return { ...obj, tags: [...(obj.tags || []), ...updatedTags] };
        }
        return obj;
      });

      return {
        ...state,
        [gridName]: {
          ...state[gridName],
          rowData: updatedRowObjects,
        },
      };
    }
  ),

  /** Unassign tags */
  on(
    ReportStoreActions.UNASSIGN_TAGS,
    (state: any, { gridName, selectedRowIds, tagIds }) => {
      const _tagIds = tagIds.map((t) => t.uuid);
      const updatedRowObjects = state[gridName]?.rowData?.map((obj: any) => {
        if (selectedRowIds.includes(obj.assetUUID)) {
          const updatedTags = obj.tags.filter(
            (tag: any) => !_tagIds.includes(tag.uuid)
          );
          console.log(obj.tags, updatedTags);
          return { ...obj, tags: updatedTags };
        }
        return obj;
      });

      return {
        ...state,
        [gridName]: {
          ...state[gridName],
          rowData: updatedRowObjects,
        },
      };
    }
  ),

  on(
    ReportStoreActions.UPDATE_SINGLE_ROW,
    (state: any, { gridName, identifier, data }) => {
      const rowIdField = identifier || 'uuid';
      const gridRowData = state[gridName]?.rowData;

      if (!gridRowData || gridRowData.length === 0 || !data) {
        // If gridRowData is empty or data is not provided, return the original state
        return state;
      }

      const updatedState = produce(state, (draftState: any) => {
        const index = draftState[gridName].rowData.findIndex(
          (rowData: any) => rowData[rowIdField] === data[rowIdField]
        );
        if (index !== -1) {
          draftState[gridName].rowData[index] = {
            ...draftState[gridName].rowData[index],
            ...data,
          };
        }
      });

      return updatedState;
    }
  ),

  /** Add new row data */
  on(ReportStoreActions.ADD_ROW_DATA, (state: any, { gridName, data }) => {
    const gridRowData = state[gridName]?.rowData;
    const finalRowData = [...data, ...gridRowData];

    return {
      ...state,
      [gridName]: {
        ...state[gridName],
        rowData: finalRowData,
      },
    };
  }),

  /** Update exisiting row data */
  on(ReportStoreActions.UPDATE_ROW_DATA, (state: any, { gridName, data }) => {
    const rowIdField = state[gridName]?.configurations?.rowIdField || 'uuid';
    let gridRowData = state[gridName]?.rowData;
    let finalUpdatedRowData!: any;

    data = [
      {
        assetUUID: '3d0ec281-d8c9-432b-af0e-84f2841fb2bb',
        accountName: 'South',
      },
    ];

    data?.forEach((updateData: any) => {
      finalUpdatedRowData = gridRowData?.map((rowData: any, index: number) => {
        if (rowData[rowIdField] === updateData?.[rowIdField]) {
          return Object.assign({}, rowData, updateData);
        } else {
          return rowData;
        }
      });
      gridRowData = finalUpdatedRowData;
      return gridRowData;
    });

    return {
      ...state,
      [gridName]: {
        ...state[gridName],
        // data: {
        // ...state[gridName]?.data,
        rowData: finalUpdatedRowData,
        // },
      },
    };
  }),

  /** Remove exisiting row data */
  on(ReportStoreActions.REMOVE_ROW_DATA, (state: any, { gridName, data }) => {
    const rowIdField = state[gridName]?.configurations?.rowIdField || 'uuid';

    // let gridRowData = state[gridName]?.data?.rowData;
    let gridRowData = state[gridName]?.rowData;

    let finalRowData!: any;

    data?.forEach((removeData: any): any => {
      gridRowData = gridRowData?.filter((rowData: any) => {
        if (rowData[rowIdField] !== removeData?.[rowIdField]) {
          return data;
        }
      });
      finalRowData = gridRowData;
      return gridRowData;
    });

    return {
      ...state,
      [gridName]: {
        ...state[gridName],
        // data: {
        // ...state[gridName]?.data,
        rowData: finalRowData,
        // },
      },
    };
  }),

  // on(DataGridStoreActions.SET_GRIDAPI, (state, { gridAPI }) => ({
  //   ...state,
  //   data: {
  //     gridAPI: gridAPI
  //   }
  // })),

  // on(DataGridStoreActions.SET_COLUMNS, (state, { columns }) => ({
  //   ...state,
  //   data: {
  //     columns: columns
  //   }
  // })),

  // on(DataGridStoreActions.SET_ROWDATA, (state, { rowData }) => ({
  //   ...state,
  //   data: {
  //     rowData: rowData
  //   }
  // })),

  // on(DataGridStoreActions.SET_OPTIONS, (state, { options }) => ({
  //   ...state,
  //   data: {
  //     options: options
  //   }
  // })),

  on(ReportStoreActions.PROCESS_ROWIDS, (state, { gridName }) => {
    const rowIdField = state[gridName]?.configurations?.rowIdField || 'uuid';
    const gridRowData = state[gridName]?.rowData;
    const rowIds: any[] = gridRowData?.map((r: any) => r[rowIdField]) || [];

    return {
      ...state,
      [gridName]: {
        ...state[gridName],
        rowDataIds: rowIds,
      },
    };
  })
);
