import { HttpClient, HttpHeaders, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { ToastService } from '@hash8io/core-web';
import { BehaviorSubject, catchError, EMPTY, map, Observable, of } from 'rxjs';
import {
  REPORTS_CONFIG_LOOKUP,
  REPORT_DATA_LOOKUP,
} from '../mock-data/report-lookup.mock';
import { EventsWebService } from '@hash8io/events-web';
import { HttpUrlEncodingCodec } from '@angular/common/http';

@Injectable({
  providedIn: 'root',
})
export class ReportGridService<T> {
  private dataSource: BehaviorSubject<any> = new BehaviorSubject<any>({});
  private apiUrl = '';
  private assignEndPoint = 'api/v1/tags/assign';
  // private assignEndPoint = 'assigntags';

  data: Observable<string> = this.dataSource.asObservable();
  authenticationToken = localStorage.getItem('access_token');

  headers = new HttpHeaders({
    'Content-Type': 'application/json',
    Authorization: 'Bearer ' + this.authenticationToken,
  });

  constructor(
    private http: HttpClient,
    private route: Router,
    private toast: ToastService,
    private eventService: EventsWebService
  ) { }

  getReportGridConfiguration(apiEndPoint: string): Observable<any> {
    // const params = new HttpParams({
    //   encoder: new HttpUrlEncodingCodec(),
    //   fromObject: {
    //     organizationUUID: ['e520d1b4-437c-4d48-a0ed-13489ab52074']
    //   }
    // })

    // const codec = new HttpUrlEncodingCodec()
    // const encodeUrl = codec.encodeKey(`/api/v1/assets?userUUID=7a6d577d-aa42-42fc-8d73-0c6d440a21a3&organizationUUID=['e520d1b4-437c-4d48-a0ed-13489ab52074']`)



    return this.http.get<T[]>(apiEndPoint, { headers: this.headers }).pipe(
      catchError((error) => {
        if (error?.status === 401) {
          this.route.navigateByUrl('/login');
          this.toast.message('Session Expired');
        }
        return of([]);
      })
    );
  }

  getReportData(apiEndPoint: string, uuid?: any) {
    if (apiEndPoint) {
      return this.http.get<T[]>(apiEndPoint, { headers: this.headers }).pipe(
        catchError((error) => {
          if (error?.status === 401) {
            this.route.navigateByUrl('/login');
            this.toast.message('Session Expired');
          }
          return of([]);
        }),
        map((res: any) => {
          return res.data;
        })
      );
    }
    // TO GET MOCK DATA
    else {
      return of((REPORT_DATA_LOOKUP as any)[uuid]);
    }
  }

  getReportConfig(apiEndPoint: string, uuid?: any) {
    return this.http.get<T[]>(apiEndPoint).pipe((data): any => {
      return of((REPORTS_CONFIG_LOOKUP as any)[uuid || '987']);
    });
  }

  searchReport() {
    //
  }

  assignTags(bodyParams: any) {
    return this.http.post(`${this.apiUrl}/${this.assignEndPoint}`, bodyParams, {
      headers: this.headers,
    });
  }
}
