import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Observable } from 'rxjs';
import { Tag } from '../tag.model';
import { EventsWebService, TemplateService } from '@hash8io/events-web';

@Injectable({
  providedIn: 'root',
})
export class TagService {
  private apiUrl = '';
  private endPoint = '';
  // 'api/v1/tags?organizationUUID={{data.localStorage.organizationUUID}}';

  constructor(
    private http: HttpClient,
    public templateService: TemplateService,
    public eventService: EventsWebService
  ) {
    // this.endPoint = this.templateService.processTemplate(this.endPoint);
  }

  setEndpoint(endpoint: string, data?: any) {
    this.endPoint = this.templateService.processTemplate(endpoint);
  }

  getTags(): Observable<Tag[]> {
    const authenticationToken = localStorage.getItem('access_token');
    const headers = new HttpHeaders({
      'Content-Type': 'application/json',
      Authorization: 'Bearer ' + authenticationToken,
    });

    const options = {
      headers: headers,
    };
    return this.http.get<Tag[]>(`${this.apiUrl}/${this.endPoint}`, options);
  }

  addTag(tag: Tag): Observable<Tag> {
    return this.http.post<Tag>(`${this.apiUrl}/${this.endPoint}`, tag);
  }

  updateTag(tag: Tag): Observable<Tag> {
    const url = `${this.apiUrl}/${this.endPoint}/${tag.id}`;
    return this.http.put<Tag>(url, tag);
  }

  deleteTag(tagId: string): Observable<void> {
    const url = `${this.apiUrl}/${this.endPoint}/${tagId}`;
    return this.http.delete<void>(url);
  }

  generateAPIRequestObject(
    existingTags: any[],
    selectedTags: any[],
    selectedResourceUUIDs: any[],
    organizationUUID: string,
    entityType: string
  ) {
    const output: any = {
      organizationUUID,
      applicationUUID: localStorage.getItem('applicationUUID'),
      entityType,
      resources: selectedResourceUUIDs.map((uuid) => ({ uuid })),
      tags: [],
    };

    // Exclude existing tags from selected tags
    const newSelectedTags = selectedTags.filter(
      (selectedTag) =>
        !existingTags.some(
          (existingTag) => existingTag.uuid === selectedTag.uuid
        )
    );

    // Add action: "assign" to new selected tags
    output.tags.push(
      ...newSelectedTags.map((tag) => {
        const { tagName, uuid, tagColor, ...t } = tag;
        return { uuid, tagName, tagColor, action: 'Assign' };
      })
    );

    // Find missing tags in selected tags compared to existing tags and set action: "Unassign"
    const missingTags = existingTags.filter(
      (existingTag) =>
        !selectedTags.some(
          (selectedTag) => selectedTag.uuid === existingTag.uuid
        )
    );

    output.tags.push(
      ...missingTags.map((tag) => {
        const { tagName, uuid, tagColor, ...t } = tag;
        return { uuid, tagName, tagColor, action: 'Unassign' };
      })
    );

    return output;
  }
}
