import { Injectable } from '@angular/core';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { of } from 'rxjs';
import { catchError, endWith, map, mergeMap, startWith } from 'rxjs/operators';
import * as TagActions from './tag.actions';
import { TagService } from './services/tag.service';

@Injectable()
export class TagEffects {
  loadTags$ = createEffect(() =>
    this.actions$.pipe(
      ofType(TagActions.loadTags),
      mergeMap(() =>
        this.tagService.getTags().pipe(
          map((tags: any) => tags?.data?.rowData),
          map((tags) => TagActions.replaceTags({ tags: tags })),
          catchError((error) => of(TagActions.loadTagsFailure(error))),
          startWith(TagActions.setLoadingFlag()),
          endWith(TagActions.loadTagsSuccess())
        )
      )
    )
  );

  refreshTags$ = createEffect(() =>
    this.actions$.pipe(
      ofType(TagActions.refreshTags),
      mergeMap(() =>
        this.tagService.getTags().pipe(
          map((tags: any) => tags?.data?.rowData),
          map((tags) => TagActions.replaceTags({ tags: tags })),
          catchError((error) => of(TagActions.loadTagsFailure(error))),
          startWith(TagActions.setLoadingFlag()),
          endWith(TagActions.loadTagsSuccess())
        )
      )
    )
  );

  constructor(private actions$: Actions, private tagService: TagService) {}
}
