import { Action, createReducer, on } from '@ngrx/store';
import { TagState, initialState, tagAdapter } from './tag.state';
import * as TagActions from './tag.actions';

export const tagReducer = createReducer(
  initialState,
  on(TagActions.loadTags, (state) => state),
  on(TagActions.loadTagsSuccess, (state) => ({
    ...state,
    loading: false,
  })),
  on(TagActions.setLoadingFlag, (state) => ({
    ...state,
    loading: true,
  })),
  on(TagActions.addTags, (state, { tags }) => tagAdapter.addMany(tags, state)),
  on(TagActions.addTag, (state, { tag }) => tagAdapter.addOne(tag, state)),
  on(TagActions.replaceTags, (state, { tags }) =>
    tagAdapter.setAll(tags, state)
  ),
  on(TagActions.updateTag, (state, { id, changes }) =>
    tagAdapter.updateOne({ id, changes }, state)
  ),
  on(TagActions.deleteTag, (state, { id }) => tagAdapter.removeOne(id, state))
);

export function reducer(state: TagState | undefined, action: Action) {
  return tagReducer(state, action);
}
