/* eslint-disable @typescript-eslint/no-inferrable-types */
/* eslint-disable @typescript-eslint/no-empty-function */
/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
import { SelectionModel } from '@angular/cdk/collections';
import { FlatTreeControl } from '@angular/cdk/tree';
import {
  AfterViewInit,
  Component,
  Input,
  OnInit,
  Output,
  ViewChild,
  ViewContainerRef,
} from '@angular/core';
import {
  MatTreeFlattener,
  MatTreeFlatDataSource,
} from '@angular/material/tree';
import { Subject } from 'rxjs';

@Component({
  selector: 'hash8-platform-hierarchy',
  templateUrl: './hierarchy.component.html',
  styleUrls: ['./hierarchy.component.scss'],
})
export class HierarchyComponent implements OnInit, AfterViewInit {
  @Input() data: any;
  @Input() searchValue: string = '';

  @Input() searchConf!: any;

  treeControl = new FlatTreeControl<any>(
    (node) => node.level,
    (node) => node.expandable
  );

  treeFlattener: any;
  dataSource: any;
  hasNoContent: any;

  @Input() selectionMode: 'single' | 'multiple' = 'single';
  @ViewChild('dropdownContainer', { read: ViewContainerRef })
  dropdownContainer!: ViewContainerRef;

  checklistSelection!: any;
  @Output() selection: Subject<any> = new Subject();

  @Input() selectedValue!: any;

  getLevel = (node: any) => node.level;

  private _transformer = (node: any, level: number) => {
    return {
      expandable: !!node.children && node.children.length > 0,
      label: node.label,
      level: level,
      hasSelection: node?.hasSelection || false,
      isGroupName: node?.isGroupName || false,
      iconUrl: node?.iconUrl || null,
      uuid: node?.uuid || null,
      type: node?._type || null,
    };
  };

  constructor() {}

  ngAfterViewInit() {
    if (this.selectedValue) {
      const selectedUUIDs = this.selectedValue?.map((data: any) => data.uuid);
      this.treeControl.dataNodes.map((nodeData: any) => {
        if (selectedUUIDs.includes(nodeData?.uuid)) {
          this.subItemSelectionToggle(nodeData);
          this.treeControl.expand(nodeData);
        }
      });
    }
  }

  ngOnInit(): void {
    this.treeFlattener = new MatTreeFlattener(
      this._transformer,
      (node) => node.level,
      (node) => node.expandable,
      (node) => node.children
    );
    this.dataSource = new MatTreeFlatDataSource(
      this.treeControl,
      this.treeFlattener
    );
    this.dataSource.data = this.data;

    /** The selection for checklist */
    /**Single Selection - false selection model; Multiple Selection - true selection model */
    this.checklistSelection = new SelectionModel<any>(
      this.selectionMode === 'single' ? false : true /* single */
    );

    this.hasNoContent = (_: number, _nodeData: any) => _nodeData.item === '';
  }

  hasChild = (_: number, node: any) => node.expandable;

  itemSelectionToggle(node: any): void {
    if (node.hasSelection && !node.isGroupName) {
      this.checklistSelection.toggle(node);
      this.checkAllParentsSelection(node);

      // Emit event
      this.selection.next(this.checklistSelection.selected);
      // console.log(
      //   '%cSelected items',
      //   'color: #007acc;',
      //   this.checklistSelection.selected
      // );
    }
  }

  /** Select/deselect all the descendants node */
  subItemSelectionToggle(node: any): void {
    if (this.selectionMode === 'single') {
      //Single selection
      this.checklistSelection.toggle(node);
      // Emit event
      this.selection.next(this.checklistSelection.selected);
      // console.log(
      //   '%cSelected items',
      //   'color: #007acc;',
      //   this.checklistSelection.selected
      // );
    } else {
      //Multiple selection
      if (node.hasSelection && !node.isGroupName) {
        this.checklistSelection.toggle(node);
        const descendants = this.treeControl
          .getDescendants(node)
          .filter((n) => n.hasSelection && !node.isGroupName);
        this.checklistSelection.isSelected(node)
          ? this.checklistSelection.select(...descendants)
          : this.checklistSelection.deselect(...descendants);

        const _descendants = descendants.filter((n) => n.hasSelection) || [];

        // Force update for the parent
        _descendants.forEach((child) =>
          this.checklistSelection.isSelected(child)
        );
        this.checkAllParentsSelection(node);

        // Emit event
        this.selection.next(this.checklistSelection.selected);
        // console.log(
        //   '%cSelected items sub',
        //   'color: #007acc;',
        //   this.checklistSelection.selected
        // );
      }
    }
  }

  /* Checks all the parents when a leaf node is selected/unselected */
  checkAllParentsSelection(node: any): void {
    //Single selection
    if (this.selectionMode != 'single') {
      let parent: any | null = this.getParentNode(node);
      while (parent) {
        this.checkRootNodeSelection(parent);
        parent = this.getParentNode(parent);
      }
    }
  }

  /** Check root node checked state and change it accordingly */
  checkRootNodeSelection(node: any): void {
    const nodeSelected = this.checklistSelection.isSelected(node);
    const descendants = this.treeControl.getDescendants(node);
    const descAllSelected =
      descendants.length > 0 &&
      descendants.every((child) => {
        return (
          child.hasSelection &&
          !node.isGroupName &&
          this.checklistSelection.isSelected(child)
        );
      });
    if (nodeSelected && !descAllSelected) {
      this.checklistSelection.deselect(node);
    } else if (!nodeSelected && descAllSelected) {
      this.checklistSelection.select(node);
    }
  }

  /* Get the parent node of a node */
  getParentNode(node: any): any | null {
    const currentLevel = this.getLevel(node);
    if (currentLevel < 1) {
      return null;
    }

    const startIndex = this.treeControl.dataNodes.indexOf(node) - 1;

    for (let i = startIndex; i >= 0; i--) {
      const currentNode = this.treeControl.dataNodes[i];

      if (this.getLevel(currentNode) < currentLevel) {
        // console.log(
        //   '%chierarchy.component.ts line:245 currentNode',
        //   'color: #007acc;',
        //   currentNode
        // );
        return currentNode;
      }
    }
    return null;
  }

  /** Whether all the descendants of the node are selected. */
  descendantsAllSelected(node: any): boolean {
    //Single selection
    if (this.selectionMode === 'single') {
      return this.checklistSelection.isSelected(node);
    }
    //Multiple selection
    else {
      const descendants = this.treeControl
        .getDescendants(node)
        .filter((n) => n.hasSelection);
      // const descendants = this.treeControl.getDescendants(node);
      const descAllSelected =
        descendants.length > 0 &&
        descendants.every((child) => {
          // console.log(
          //   '%chierarchy.component.ts line:280 descAllSelected',
          //   'color: #007acc;',
          //   child
          // );
          return (
            child.hasSelection &&
            !node.isGroupName &&
            this.checklistSelection.isSelected(child)
          );
        });

      return descAllSelected;
    }
  }

  /** Whether part of the descendants are selected */
  descendantsPartiallySelected(node: any): boolean {
    if (this.selectionMode === 'single') {
      return false;
    } else {
      const descendants = this.treeControl
        .getDescendants(node)
        .filter((n) => n.hasSelection);

      const result = descendants.some(
        (child) =>
          child.hasSelection &&
          !node.isGroupName &&
          this.checklistSelection.isSelected(child)
      );
      return result && !this.descendantsAllSelected(node);
    }
  }

  filterTreeStructure(array: any, text: any) {
    const getChildren = (result: any, object: any) => {
      const re = new RegExp(text, 'gi');
      if (object?.label?.match(re)) {
        result.push(object);
        return result;
      }
      if (Array.isArray(object.children)) {
        const children = object.children.reduce(getChildren, []);
        if (children.length) result.push({ ...object, children });
      }
      return result;
    };

    return array.reduce(getChildren, []);
  }

  filterTreeText(filterText: string) {
    let filteredTreeData;
    if (filterText) {
      // Filter the tree
      filteredTreeData = this.filterTreeStructure(this.data, filterText);
    } else {
      // Return the initial tree
      filteredTreeData = this.data;
    }
    return filteredTreeData;
  }

  // Search tree

  filterTree() {
    // const filterValue = this.searchValue.trim().toLowerCase();
    // const filteredData = this.getFilteredTreeData(this.data, filterValue);
    // this.dataSource.data = filteredData;
    // this.treeControl.dataNodes.forEach((node) => {
    //   this.treeControl.expand(node);
    // });

    const filterValue = this.searchValue.trim().toLowerCase();

    this.dataSource.data = this.filterTreeText(filterValue);
    this.treeControl.dataNodes.forEach((node) => {
      const re = new RegExp(filterValue, 'gi');
      if (node?.label?.match(re)) {
        this.treeControl.collapse(node);
      } else {
        this.treeControl.expand(node);
      }
    });
    if (!filterValue) {
      this.treeControl.dataNodes.forEach((node) => {
        this.treeControl.collapseAll();
      });
    }
  }

  getFilteredTreeData(data: any[], filterValue: string): any[] {
    return data
      .map((node) => {
        const filteredNode: any = { ...node };
        if (node?.children) {
          filteredNode.children = this.getFilteredTreeData(
            filteredNode.children,
            filterValue
          );
        }
        return filteredNode;
      })
      .filter((node) => this.nodeMatchesSearch(node, filterValue));
  }

  nodeMatchesSearch(node: any, filterValue: string): boolean {
    const nodelabel = node.label.toLowerCase();
    const childrenMatch =
      node.children &&
      node.children.some((child: any) =>
        this.nodeMatchesSearch(child, filterValue)
      );
    const matchesSearch = nodelabel.includes(filterValue) || childrenMatch;

    if (matchesSearch) {
      this.expandNodeAndAncestors(node);
    }

    return matchesSearch;
  }

  expandNodeAndAncestors(node: any) {
    node.expanded = true;
    if (node.parent) {
      this.expandNodeAndAncestors(node.parent);
    }
  }

  clearSearch() {
    this.searchValue = '';
    this.filterTree();
  }
}
