/* eslint-disable @typescript-eslint/no-explicit-any */
/* eslint-disable @typescript-eslint/no-inferrable-types */
/* eslint-disable @angular-eslint/no-input-rename */
/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
import {
  Component,
  Input,
  OnInit,
  TemplateRef,
  ViewContainerRef,
  ViewEncapsulation,
} from '@angular/core';
import {
  AppStoreSelectors,
  BreakpointService,
  ThemeService,
} from '@hash8io/core-web';
import { APP_LAYOUT_GRID_CONFIG } from '../../mock-data/app-layout.config';
import { TemplatePortal } from '@angular/cdk/portal';
import { Store } from '@ngrx/store';

@Component({
  selector: 'hash8-platform-app-layout',
  templateUrl: './app-layout.component.html',
  styleUrls: ['./app-layout.component.scss'],
  encapsulation: ViewEncapsulation.None,
})
export class AppLayoutComponent implements OnInit {
  @Input('layout') _grid: any = APP_LAYOUT_GRID_CONFIG.default;
  grid: any;
  breakpoint!: string;
  sideNavMode: 'over' | 'push' | 'side' = 'side';

  @Input() type: 1 | 2 = 1; // Later remove this once we not using the matsidenav anymore

  @Input('header') headerTemplate!: TemplateRef<string>;
  @Input('footer') footerTemplate!: TemplateRef<string>;
  @Input('content') contentTemplate!: TemplateRef<string>;
  @Input('sidebar') sidebarTemplate!: TemplateRef<string>;
  @Input('toolbar') toolbarTemplate!: TemplateRef<string>;
  @Input('banner') bannerTemplate!: TemplateRef<string>;

  // Template Portal
  headerTemplatePortal!: TemplatePortal<any>;
  sidebarTemplatePortal!: TemplatePortal<any>;
  contentTemplatePortal!: TemplatePortal<any>;
  footerTemplatePortal!: TemplatePortal<any>;
  toolbarTemplatePortal!: TemplatePortal<any>;
  bannerTemplatePortal!: TemplatePortal<any>;

  constructor(
    private breakpointService: BreakpointService,
    private _viewContainerRef: ViewContainerRef,
    private store: Store,
    private themeService: ThemeService
  ) {}

  ngOnInit(): void {
    this.breakpointService.breakpoints.subscribe((breakpoint) => {
      this.grid = this._grid[breakpoint];
      this.breakpoint = breakpoint;

      switch (breakpoint) {
        case 'lg':
          this.sideNavMode = 'side';
          break;
        case 'md':
          this.sideNavMode = 'side';
          break;
        case 'sm':
        case 'xs':
          this.sideNavMode = 'over';
          break;
        default:
          break;
      }
    });

    // Theming
    this.store.select(AppStoreSelectors.selectTheme).subscribe((theme) => {
      this.themeService.setTheme(theme);
    });

    //Header
    if (this.headerTemplate)
      this.headerTemplatePortal = new TemplatePortal(
        this.headerTemplate,
        this._viewContainerRef
      );

    //Footer
    if (this.footerTemplate)
      this.footerTemplatePortal = new TemplatePortal(
        this.footerTemplate,
        this._viewContainerRef
      );

    //Page content
    if (this.contentTemplate)
      this.contentTemplatePortal = new TemplatePortal(
        this.contentTemplate,
        this._viewContainerRef
      );

    //Sidebar
    if (this.sidebarTemplate)
      this.sidebarTemplatePortal = new TemplatePortal(
        this.sidebarTemplate,
        this._viewContainerRef
      );

    //Toolbar
    if (this.toolbarTemplate)
      this.toolbarTemplatePortal = new TemplatePortal(
        this.toolbarTemplate,
        this._viewContainerRef
      );

    //Banner
    if (this.bannerTemplate)
      this.bannerTemplatePortal = new TemplatePortal(
        this.bannerTemplate,
        this._viewContainerRef
      );
  }
}
