/* eslint-disable @angular-eslint/no-input-rename */
/* eslint-disable @typescript-eslint/no-empty-function */
/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
import { TemplatePortal } from '@angular/cdk/portal';
import {
  Component,
  Input,
  OnInit,
  TemplateRef,
  ViewContainerRef,
  ViewEncapsulation,
} from '@angular/core';
import { Router } from '@angular/router';
import {
  AppStoreSelectors,
  BreakpointService,
  ThemeService,
} from '@hash8io/core-web';
import { Store } from '@ngrx/store';
import { Observable } from 'rxjs';
import { LOGIN_LAYOUT_GRID_CONFIG } from '../../mock-data/login-layout.config';

@Component({
  selector: 'hash8-platform-login-layout',
  templateUrl: './login-layout.component.html',
  styleUrls: ['./login-layout.component.scss'],
  encapsulation: ViewEncapsulation.None,
})
export class LoginLayoutComponent implements OnInit {
  loadingBarRef = "http"
  color = "red"
  @Input('layout') _grid: any = LOGIN_LAYOUT_GRID_CONFIG;
  @Input() template: any = 'default';
  grid: any;
  breakpoint!: string;
  branding!: Observable<any>;

  @Input('header') headerTemplate!: TemplateRef<string>;
  @Input('footer') footerTemplate!: TemplateRef<string>;
  @Input('content') contentTemplate!: TemplateRef<string>;
  @Input('hero') heroTemplate!: TemplateRef<string>;
  @Input('banner') bannerTemplate!: TemplateRef<string>;

  heroSectionColor !: Observable<any>;

  // Template Portal
  headerTemplatePortal!: TemplatePortal<any>;
  heroTemplatePortal!: TemplatePortal<any>;
  contentTemplatePortal!: TemplatePortal<any>;
  footerTemplatePortal!: TemplatePortal<any>;
  bannerTemplatePortal!: TemplatePortal<any>;

  constructor(
    private breakpointService: BreakpointService,
    private _viewContainerRef: ViewContainerRef,
    public router: Router,
    private store: Store,
    private themeService: ThemeService
  ) { }

  ngOnInit(): void {
    //Breakpoints
    this.breakpointService.breakpoints.subscribe((breakpoint) => {
      this.grid = this._grid[this.template][breakpoint];
      this.breakpoint = breakpoint;
    });

    // Theming
    this.store.select(AppStoreSelectors.selectTheme).subscribe((theme) => {
      this.themeService.setTheme(theme);
    });

    this.branding = this.store.select(AppStoreSelectors.selectBranding);

    this.heroSectionColor = this.store.select(AppStoreSelectors.selectBranding)

    //Header
    if (this.headerTemplate)
      this.headerTemplatePortal = new TemplatePortal(
        this.headerTemplate,
        this._viewContainerRef
      );

    //Footer
    if (this.footerTemplate)
      this.footerTemplatePortal = new TemplatePortal(
        this.footerTemplate,
        this._viewContainerRef
      );

    //Page content
    if (this.contentTemplate)
      this.contentTemplatePortal = new TemplatePortal(
        this.contentTemplate,
        this._viewContainerRef
      );

    //Hero
    if (this.heroTemplate)
      this.heroTemplatePortal = new TemplatePortal(
        this.heroTemplate,
        this._viewContainerRef
      );

    //Banner
    if (this.bannerTemplate)
      this.bannerTemplatePortal = new TemplatePortal(
        this.bannerTemplate,
        this._viewContainerRef
      );
  }

  get classNames() {
    return `login-layout login-layout-theme-${this.template} layout--${this.breakpoint}`;
  }
}
