/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
import { Component, Input, OnInit } from '@angular/core';
import { BreakpointService, Model } from '@hash8io/core-web';

@Component({
  selector: 'hash8-platform-tile-layout',
  templateUrl: './tile-layout.component.html',
  styleUrls: ['./tile-layout.component.scss'],
})
export class TileLayoutComponent implements OnInit {
  @Input() layout: Model.TileLayout = {
    rows: 'auto',
    columns: 12,
    gap: '0px',
    autoFlowColumns: false,
    tiles: {
      lg: [{ column: 3, row: 1 }],
    },
  };
  @Input() columns!: number;
  @Input() rows!: number;
  @Input() gap!: string;
  breakpoint!: string;
  tiles!: Model.Tile[] | undefined;

  constructor(private _breakpoint: BreakpointService) {}

  ngOnInit(): void {
    this._breakpoint.active$.subscribe((breakpoint) => {
      this.breakpoint = breakpoint;
      const _breakpoint = breakpoint as keyof Model.Breakpoint;
      if (this.layout?.tiles[_breakpoint]) {
        this.tiles = this.layout?.tiles[_breakpoint];
      }
    });

    if (this.columns) this.layout.columns = this.columns;
    if (this.rows) this.layout.rows = this.rows;
    if (this.gap) this.layout.gap = this.gap;
  }
}
