/* eslint-disable @typescript-eslint/no-inferrable-types */
import { Component, ElementRef, Inject, Input, OnDestroy, OnInit } from '@angular/core';
import { AppStoreSelectors } from '@hash8io/core-web';
import { Store } from '@ngrx/store';
import { Observable } from 'rxjs';
import { Environment, ENVIRONMENT } from '@hash8io/environment';
@Component({
  selector: 'hash8-platform-notification-panel',
  templateUrl: './notification-panel.component.html',
  styleUrls: ['./notification-panel.component.scss'],
})
export class NotificationPanelComponent implements OnInit {

  myScriptElement!: HTMLScriptElement;
  brand!: any;
  user!: Observable<any>;
  userUUID!: string;

  clientKey !: string;


  constructor(private store: Store, private elementRef: ElementRef, @Inject(ENVIRONMENT) private env: Environment) {



    this.user = this.store.select(AppStoreSelectors.selectUserProfile);
    this.user.subscribe((user) => {
      this.userUUID = user.uuid;
    });

    // this.store.select(AppStoreSelectors.selectTheme).subscribe((theme) => {
    //   this.themePrimaryColor = theme.primaryColor;
    // })


    this.store.select(AppStoreSelectors.selectTheme).subscribe((theme) => {
      const primaryColor = theme.primaryColor;
      const accentColor = theme.accentColor;
      const warningColor = theme.warningColor;

      (<any>window).courierConfig = {
        // clientKey: 'OTE2NmE3NWItNjBhZS00M2JhLWFkNDEtZDcyODkxYjQzMGQxL3Rlc3Q=',//Test Key
        // clientKey: 'OTE2NmE3NWItNjBhZS00M2JhLWFkNDEtZDcyODkxYjQzMGQx',//Production Key
        clientKey: this.env?.clientKey,
        userId: localStorage.getItem('userUUID'),
        components: {
          toast: {
            autoClose: 5000,
            boxShadow: '0px 1px 10px 2px rgba(0,0,0,0.60)',
            theme: {
              dismiss: {
                color: primaryColor
              },
              message: {
                icon: {
                  display: "none"
                },
              },
              progressBar: {
                background: primaryColor
              }
            },
          },
          inbox: {
            title: 'Alerts',
            trigger: 'hover',
            openLinksInNewTab: false,
            className: 'h8-notification-panel',
            // formatDate: (date: string) => date,
            // renderIcon: (open: any, data: any) => {
            //   if (open) {
            //     console.log("Open", open?.unreadMessageCount)
            //   } else {
            //     console.log("OPEN DATA", open?.unreadMessageCount)
            //   }
            // },
            // renderMessage: (message: any): any => {
            //   console.log("Message", message)
            //   return message
            // },

            brand: {
              colors: {
                primary: '#566074',
                // secondary: 'white',
              },
            },
            tooltip: {
              color: "rgba(0, 63, 111, 0.5)",
              "background-color": "rgba(97, 97, 97, 1.0)",
            },
            labels: {
              emptyState: "No Messages",
              archiveMessage: "Archive Message"
            },
            theme: {
              root: {
                'box-shadow':
                  '0px 2px 4px -1px rgb(0 0 0 / 20%), 0px 4px 5px 0px rgb(0 0 0 / 14%), 0px 1px 10px 0px rgb(0 0 0 / 12%)',
                'border-radius': '6px',
              },
              container: {
                padding: 'none',
              },
              messageList: {
                container: {
                  "overflow-x": "hidden",
                  "overflow-y": "scroll",
                }
              },
              emptyState: {
                // color: 'rgba(0,0,0,0.3)',
                fontSize: '1rem',
                fontWeight: '500',
              },
              footer: {
                // display: "none"
                height: "0px",
                visibility: 'hidden'
              },
              header: {
                // background: primaryColor,
                color: 'rgb(28, 39, 58)',
                height: '40px',
                fontWeight: '700',
                padding: '0px 24px 0px 24px',
                'font-size': '16px'

              },
              tabList: {
                container: {
                  height: '48px',
                },
                tab: {
                  borderWidth: '1px 0px 2px',
                  '&.active': {
                    color: primaryColor,
                    borderBottomColor: primaryColor,
                  },
                },
              },
              message: {
                icon: {
                  display: "none"
                },

                title: {
                  "font-size": "14px",

                },
                textElement: {
                },

                container: {
                  marginTop: '1px',
                  padding: '16px 12px 16px 24px',
                  "&.read:hover": {
                    "background-color": "rgba(0,0,0,0.1)"
                  },
                  "&.read": {
                    "background-color": "rgba(222,222,222,0.3)"
                  },
                  '&:hover': { 'background-color': 'rgba(0,0,0,0.05)' },
                  '&:not(.read):hover': {
                    'background-color': 'rgba(0,0,0,0.1)',
                  },


                },
                actionBlock: {
                  '& a': {
                    background: 'transparent',
                    color: primaryColor,
                    boxShadow: 'none',
                    border: 'rgba(0,0,0,0.1)',
                    padding: 0,
                    fontWeight: 600,
                    textDecoration: 'underline',
                  },
                  '& a:hover': {
                    background: 'transparent',
                    color: primaryColor,
                    textDecoration: 'underline',
                  },
                },
                unreadIndicator: {
                  background: "red",
                  height: "8px",
                  width: "8px",
                  borderRadius: "50%",
                  top: "50%",
                  left: "17px"
                  // display: "flex",
                  // justifyContent: "center"

                },
              },
            },
          },
        },
      };
    });
  }

  ngOnInit() {


    if (!document.getElementById('courierScript')) {
      this.myScriptElement = document.createElement('script');
      this.myScriptElement.src =
        'https://courier-components-xvdza5.s3.amazonaws.com/v2.0.1.js';
      this.myScriptElement.id = 'courierScript';
      document.body.appendChild(this.myScriptElement);
    }
  }
}


// 'https://courier-components-xvdza5.s3.amazonaws.com/v2.0.1.js';

// 'https://courier-components-xvdza5.s3.amazonaws.com/v3.6.2.js';
