/**
 * Debounce a method
 */
export function debounce(ms: number) {
  return function (target: any, key: any, descriptor: any) {
    const oldFunc = descriptor.value;
    const newFunc = _debounce(oldFunc, ms);
    descriptor.value = function () {
      return newFunc.apply(this, arguments);
    };
  };
}

function _debounce(fn: any, ms: number) {
  _debounce.inProgress === undefined ? (_debounce.inProgress = false) : null;
  if (!_debounce.inProgress) {
    _debounce.inProgress = true;
    fn.apply();
    const timeout = setTimeout(() => {
      _debounce.inProgress = false;
      clearTimeout(timeout);
    }, ms);
  }
}
