import { HttpClient } from '@angular/common/http';
import { catchError, retry } from 'rxjs/operators';
import { throwError } from 'rxjs';

export function RetryOnFailure(maxRetries: number) {
  return function (
    target: HttpClient,
    key: string,
    descriptor: PropertyDescriptor
  ) {
    const originalMethod = descriptor.value;

    descriptor.value = function (...args: any[]) {
      return originalMethod.apply(this, args).pipe(
        retry(maxRetries), // Retry failed request up to maxRetries times
        catchError((error) => {
          console.error(`Request failed after ${maxRetries} retries.`);
          return throwError(error);
        })
      );
    };

    return descriptor;
  };
}
