export interface SnapshotModel {
  layout?: TileLayout;
  config?: SnapshotConfig;
  data?: string;

  title: Title;
  toolbar: Toolbar;
  reports: Menu[];
  notes?: Notes;
}

/** Title */
export interface Title {
  label: string;
  class?: any;
  iconUrl?: string;
  hidelabel?: boolean;
  hideIcon?: boolean;
  description?: string;
  showBorder?: boolean;
}

/**
 * Breadcrumb
 */
export interface Breadcrumb {
  label: string;
  iconUrl?: string;
  key?: string;
  value?: string;
  targetUrl: string;
  linkColor?: string;
  clickable?: boolean;
}

/**
 * Menu Model
 */
export interface Menu {
  label: string;
  iconUrl?: string;
  color?: string;
  type?: MenuType;
  visible?: boolean;
  clickable?: boolean;
  targetUrl?: string;
  params?: string;
  configApiUrl?: string;
  apiUrl?: string;
  showIcon?: boolean;
  hasSubMenu?: boolean;
  disabled?: boolean;
  events?: any;
  subMenus?: Menu[];
  id?: string;
  tooltip?: string;
  isSubMenuOpen?: boolean;
}

export enum MenuType {
  Button = 'button',
  Link = 'link',
  Dropdown = 'dropdown',
  Heading = 'heading',
}

/**
 * Action Menu Model
 */
export interface Action extends Menu {
  isPrimaryAction?: boolean;
  isLink?: boolean;
  iconPosition?: string;
}

/**
 * Toolbar Model
 */
export interface Toolbar {
  title?: Title;
  subTitle?: Title;
  breadcrumbs?: Breadcrumb[];
  settings?: Action[];
  actions?: Action[];
  content?: Content[];
  options?: any;
  actionOptions?: any;
}

/**
 * Notes Model
 */
export interface Notes {
  content: Content[];
  position?: 'outside' | 'inside';
}

/**
 * Card Model
 */
export interface Card {
  header?: Toolbar;
  content: Content[];
  footer?: CardFooter;
  options?: any;
  notes?: Notes;
  type?: CardType;
  componentType?: string; // eg. LeftRightComponent, GaugeComponent
}

export enum CardType {
  LeftRight = 'left-right',
  TopBottom = 'top-bottom',
  KPI = 'kpi',
}

export interface CardHeader extends Toolbar {
  title: Title;
  subTitle: Title;
  actions: Action[];
}

export interface CardFooter {
  actions: Action[];
}

/** SnapshotLayout Model [DEPRECATED] */
/**
 *  - Use TileLayout Model instead
 */
export interface SnapshotLayout {
  rows: number | string;
  columns: number | string;
  gap: number | string;
  autoFlowColumns: boolean;
  board: any;
}

export interface TileLayout {
  rows: number | string;
  columns: number | string;
  gap: number | string;
  autoFlowColumns?: boolean;
  tiles: Breakpoint;
}

export interface Tile {
  row: number | string;
  column: number | string;
  area?: string;
  type?: any; // previously component
  props?: any;
  outputs?: any;
}

export interface Breakpoint {
  lg: Tile[];
  md?: Tile[];
  sm?: Tile[];
  xs?: Tile[];
}

export interface Content {
  type?: string; // Previously it was component
  label?: string;
  key?: string;
  value?: any;
  options?: any;
  props?: any | Content[]; // Previously it was inputs
  outputs?: any;
}

/** Report Model [DEPRECATED] */
/**
 *  - Use Menu Model instead
 */
export interface ReportMenu extends Menu {
  params: string;
  subMenus?: ReportMenu[];
}

/** Snapshot Config */
export interface SnapshotConfig {
  cards: Record<string, Card>;
}
