import { Pipe, PipeTransform } from '@angular/core';
import { COLORS } from '../../shared/utils/colors';

@Pipe({
  name: 'color'
})
export class ColorPipe implements PipeTransform {

  transform(value: any, args?: any): any {
    const colorName = this.colorNameToHex(value)
    if (colorName) {
      return this.addAlpha(colorName, args);
    }
    return this.addAlpha(value, args)
  }

  /**
   *
   * @param bgColor
   * @param lightColor
   * @param darkColor
   * @returns
   */
  getTextColorBasedOnBgColor(bgColor: any, lightColor: any, darkColor: any) {

    const color = (bgColor.charAt(0) === '#') ? bgColor.substring(1, 7) : bgColor;
    const r = parseInt(color.substring(0, 2), 16); // hexToR
    const g = parseInt(color.substring(2, 4), 16); // hexToG
    const b = parseInt(color.substring(4, 6), 16); // hexToB
    const uicolors = [r / 255, g / 255, b / 255];
    const c = uicolors.map((col) => {
      if (col <= 0.03928) {
        return col / 12.92;
      }
      return Math.pow((col + 0.055) / 1.055, 2.4);
    });
    const L = (0.2126 * c[0]) + (0.7152 * c[1]) + (0.0722 * c[2]);
    return (L > 0.179) ? darkColor : lightColor;
  }

  /**
   *
   * @param color
   * @param opacity
   * @returns
   */
  addAlpha(color: string, opacity: number): string {

    // coerce values so ti is between 0 and 1.
    const _opacity = Math.round(Math.min(Math.max(opacity || 1, 0), 1) * 255);
    return color + _opacity.toString(16).toUpperCase();
  }
  // addAlpha('FF0000', 1); // returns 'FF0000FF'
  // addAlpha('FF0000', 0.5); // returns 'FF000080'


  /**
   *
   * @param colour // red, green
   * @returns
   */
  colorNameToHex(colour: any) {

    const colors = COLORS;

    if (typeof colors[colour?.toLowerCase()] != 'undefined')
      return colors[colour.toLowerCase()];

    // return false;
  }
}
