import { Injectable } from '@angular/core';
import { MediaChange, MediaObserver } from '@angular/flex-layout';
import { BehaviorSubject, map, Observable } from 'rxjs';

@Injectable({
  providedIn: 'root',
})
export class BreakpointService {
  private _lg: BehaviorSubject<boolean> = new BehaviorSubject(false);
  private _md: BehaviorSubject<boolean> = new BehaviorSubject(false);
  private _sm: BehaviorSubject<boolean> = new BehaviorSubject(false);
  private _xs: BehaviorSubject<boolean> = new BehaviorSubject(false);
  private _isActive: BehaviorSubject<any> = new BehaviorSubject({});

  lg$ = this._lg.asObservable();
  md$ = this._md.asObservable();
  sm$ = this._sm.asObservable();
  xs$ = this._xs.asObservable();
  active$ = this._isActive.asObservable();

  get isLg(): BehaviorSubject<boolean> {
    return this._lg;
  }

  get isMd(): BehaviorSubject<boolean> {
    return this._md;
  }

  get isSm(): BehaviorSubject<boolean> {
    return this._sm;
  }

  get isXs(): BehaviorSubject<boolean> {
    return this._xs;
  }

  get breakpoints(): BehaviorSubject<any> {
    return this._isActive;
  }

  get isDesktop(): BehaviorSubject<boolean> {
    return this._lg;
  }

  get isTablet(): BehaviorSubject<boolean> {
    return this._md;
  }

  get isMobile(): BehaviorSubject<boolean> {
    return this._xs;
  }

  breakpoint$!: Observable<MediaChange>;

  constructor(public mediaObserver: MediaObserver) {
    this.breakpoint$ = mediaObserver.asObservable().pipe(
      map((change: MediaChange[]) => {
        return change[0];
      })
    );
    this.init();
  }

  init() {
    this.mediaObserver.asObservable().subscribe(() => {
      this._lg.next(this.mediaObserver.isActive('lg'));
      this._md.next(this.mediaObserver.isActive('md'));
      this._sm.next(this.mediaObserver.isActive('sm'));
      this._xs.next(this.mediaObserver.isActive('xs'));
    });

    this.breakpoint$
      .pipe(
        map((change: MediaChange) => {
          this._isActive.next(change.mqAlias);
          // TODO: Later advanced usage
          // switch (change.mqAlias) {
          //   case 'lg':
          //     // this.isLg.next(true);
          //     break;
          //   case 'md':
          //     // this.isMd.next(true);
          //     break;
          //   case 'sm':
          //     // this.isSm.next(true);
          //     break;
          //   default:
          //     break;
          // }
        })
      )
      .subscribe();
  }
}
