import { Injectable } from '@angular/core';
import { Store } from '@ngrx/store';
import { ClearTokenService } from '../../services/clear-token/clear-token.service';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { map, exhaustMap, catchError } from 'rxjs/operators';
import { of } from 'rxjs';
import { AppStoreActions } from '.';

@Injectable()
export class ClearTokenStoreEffects {
  constructor(
    private actions$: Actions,
    private clearTokenService: ClearTokenService,
    private store: Store
  ) {}

  clearAuthTokens$ = createEffect(() => {
    return this.actions$.pipe(
      ofType(AppStoreActions.CLEAR_AUTH_TOKEN),
      exhaustMap(() => {
        // this.clearTokenService.clearAuthToken();
        this.clearTokenService.clearLocalStorage();

        return of(AppStoreActions.CLEAR_AUTH_TOKEN_SUCCESS());
      }),
      catchError((error) => {
        return of(AppStoreActions.CLEAR_AUTH_TOKEN_FAILED());
      })
    );
  });

  endAuthSession$ = createEffect(() => {
    return this.actions$.pipe(
      ofType(AppStoreActions.END_AUTH_SESSION),
      exhaustMap((events) => {
        this.store.dispatch({
          type: '[Events] Process events',
          events: events.events,
        });
        this.store.dispatch(AppStoreActions.END_AUTH_SESSION_SUCCESS()); //Initiate LOGOUT API

        return of(AppStoreActions.CLEAR_AUTH_TOKEN());
      }),
      catchError((error) => {
        this.store.dispatch(AppStoreActions.CLEAR_AUTH_TOKEN_FAILED());
        return of(AppStoreActions.END_AUTH_SESSION_FAILED());
      })
    );
  });
}
