import { createReducer, on } from '@ngrx/store';
import * as AppStoreActions from './actions';
import { initialState } from './state';

// Dummy Data
const newData = {
  columns: [{ field: 'make' }, { field: 'model' }, { field: 'price' }],
  data: [
    { make: 'Toyota New', model: 'Celica', price: 35000 },
    { make: 'Toyota New', model: 'Celica', price: 35000 },
    { make: 'Ford', model: 'Mondeo', price: 32000 },
    { make: 'Toyota', model: 'Celica', price: 35000 },
    { make: 'Ford', model: 'Mondeo', price: 32000 },
    { make: 'Porsche', model: 'Boxster', price: 72000 },
    { make: 'Porsche', model: 'Boxster', price: 72000 },
    { make: 'Ford', model: 'Mondeo', price: 32000 },
    { make: 'Porsche', model: 'Boxster', price: 72000 },
  ],
  loading: false,
  error: undefined,
};

export const applicationReducer = createReducer(
  initialState,

  // Set Dark Mode
  on(AppStoreActions.SET_DARK_MODE, (state, { darkMode }) => ({
    ...state,
    userSettings: {
      enableDarkMode: darkMode,
    },
  })),

  // Set Light Mode
  on(AppStoreActions.SET_LIGHT_MODE, (state, { darkMode }) => ({
    ...state,
    userSettings: {
      enableDarkMode: darkMode,
    },
  })),

  // Load Devices
  on(AppStoreActions.LOAD_DEVICES, (state) => ({
    ...state,
    devices: newData,
    // devices: {
    //   ...state.devices,
    //   data: newData.data,
    // },
  })),

  // Load Devices Success
  on(AppStoreActions.LOAD_DEVICES_SUCCESS, (state, { devices }) => ({
    ...state,
    devices: {
      ...state.devices,
      devices: devices,
      loading: false,
    },
  })),

  // Load Devices Failure
  on(AppStoreActions.LOAD_DEVICES_FAILURE, (state, { error }) => ({
    ...state,
    devices: {
      ...state.devices,
      error: error,
      loading: false,
    },
  })),

  on(AppStoreActions.SET_APP_VERSION, (state, { version }) => ({
    ...state,
    appVersion: version,
  })),

  on(AppStoreActions.SET_APP_BRANDING, (state, { branding }) => ({
    ...state,
    branding: branding,
  })),

  on(AppStoreActions.SET_APP_NAVIGATION, (state, { navigationMenu }) => ({
    ...state,
    navigationMenu: navigationMenu,
  })),

  on(AppStoreActions.SET_APP_THEME, (state, { theme }) => ({
    ...state,
    theme: theme,
  })),

  on(
    AppStoreActions.SET_APP_APPROVED_APPLICATION,
    (state, { approvedApplications }) => ({
      ...state,
      approvedApplications: approvedApplications,
    })
  ),

  on(
    AppStoreActions.SET_APP_APPLICATION_SEGMENT,
    (state, { applicationSegments }) => ({
      ...state,
      applicationSegments: applicationSegments,
    })
  ),

  on(
    AppStoreActions.SET_APPROVED_APPLICATION_Configuration,
    (state, { approvedApplicationsConfiguration }) => ({
      ...state,
      approvedApplicationsConfiguration: approvedApplicationsConfiguration,
    })
  ),

  on(AppStoreActions.SET_INITIAL_STORE_OBJ, (state, { storeObj }) => ({
    ...state,
    ...storeObj,
  })),

  on(AppStoreActions.SET_HEADER_STORE_OBJ, (state, { header }) => ({
    ...state,
    ...header,
  })),

  on(
    AppStoreActions.SET_USER_DISPLAY_SETTINGS,
    (state, { userDisplaySettings }) => ({
      ...state,
      userDisplaySettings: {
        ...state.userDisplaySettings,
        ...userDisplaySettings,
      },
    })
  ),
  on(AppStoreActions.SET_USER_DETAILS, (state, { userDetails }) => ({
    ...state,
    userDetails: {
      ...state.userDetails,
      ...userDetails,
    },
  }))
);
