/* eslint-disable @typescript-eslint/no-inferrable-types */
/* eslint-disable @typescript-eslint/no-empty-function */
import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { Model } from '@hash8io/core-web';
import { ToastService } from '@hash8io/core-web';
import { HierarchyComponent } from '@hash8io/core-web/hierarchy';
import { Store } from '@ngrx/store';
import { Subject } from 'rxjs';

@Component({
  selector: 'hash8-platform-toolbar',
  templateUrl: './toolbar.component.html',
  styleUrls: ['./toolbar.component.scss'],
})
export class ToolbarComponent implements OnInit {
  @Input() title!: Model.Title | undefined;
  @Input() breadcrumbs!: Model.Breadcrumb[] | undefined;
  @Input() search!: any;
  @Input() settings: Model.Action[] = [];
  @Input() actions: Model.Action[] | undefined = [];
  @Input() grid!: any;
  @Input() count!: any;

  @Input() showTitle: boolean = false;
  @Input() showBreadcrumb: boolean = true;
  @Input() showActions: boolean = true;
  @Input() showSearch: boolean = true;
  @Input() showSettings: boolean = true;
  @Input() showRefresh!: boolean;

  @Input() options!: any;
  @Input() actionOptions!: any;

  @Input() filterConfiguration: any;

  @Input() filterData!: any;

  filterType!: any;

  selectedData!: any;

  @Output() eventAction = new EventEmitter();
  @Output() refresh: Subject<any> = new Subject();

  defaultOptions = {
    showTitle: this.showTitle,
    showBreadcrumb: this.showBreadcrumb,
    showActions: this.showActions,
    showSearch: this.showSearch,
    showSettings: this.showSettings,
  };

  defaultActionOptions = {
    showLabel: true,
    showIcon: true,
    iconUrl: 'more_horiz',
    label: 'Actions',
    stroked: true,
  };

  constructor(private toast: ToastService, private store: Store) {}

  ngOnInit(): void {
    // console.log("Count", this.count)
    // console.log("options", this.options)
    // console.log("Settings", this.actions)

    if (this.options) {
      const options = { ...this.defaultOptions, ...this.options };
      this.showTitle = options.showTitle;
      this.showBreadcrumb = options.showBreadcrumb;
      this.showActions = options.showActions;
      this.showSearch = options.showSearch;
      this.showSettings = options.showSettings;
    }

    this.actionOptions = {
      ...this.defaultActionOptions,
      ...this.actionOptions,
    };

    this.breadcrumbs = this.breadcrumbs || [];

    // console.log(this.breadcrumbs);
    // this.showTitle = false;
    // this.showBreadcrumb = true;
  }

  getMenuAction(events: any) {
    this.actions?.forEach((data: any) => {
      if (data?.isPrimaryAction) {
        this.eventAction.emit(events);
      }
    });

    // if(!this.grid?.gridApi){
    // }

    //Data grid export
    if (this.grid?.gridApi) {
      // console.log("this.grid", this.grid)

      const exportField: any = [];

      this.grid?.columns.forEach((columnField: any) => {
        // eslint-disable-next-line no-prototype-builtins
        if (columnField.hasOwnProperty('children')) {
          columnField.children.map((o: any) => {
            if (o.export + ''.toLowerCase() !== 'false') {
              exportField.push(o?.field);
            }
          });
        } else {
          if (columnField.export + ''.toLowerCase() !== 'false') {
            exportField.push(columnField?.field);
          }
        }
        //Push column fields
      });

      const finalExportFields = exportField.filter((x: any) => x != 'action');

      const params = {
        // allColumns: true,
        columnKeys: finalExportFields,
        // fileName: ''
      };

      const renameHeaders = () => ({
        processGroupHeaderCallback(params: any): string {
          return '';
        },
        processHeaderCallback(params: any): string {
          const groupHeaderName =
            params?.column?.originalParent?.colGroupDef?.headerName;
          const columnHeaderName = params.columnApi.getDisplayNameForColumn(
            params.column,
            null
          );

          const renamedColumn = groupHeaderName
            ? `${groupHeaderName} ${columnHeaderName}`
            : columnHeaderName;

          return `${renamedColumn}`;
        },
      });

      events?.forEach((event: any) => {
        if (event?.method === 'exportToCSV') {
          // this.grid?.gridApi.exportDataAsCsv({ ...params, ...event?.params });
          if (event?.params?.groupExport + ''.toLowerCase() == 'true') {
            this.grid?.gridApi.exportDataAsCsv({
              ...params,
              ...event?.params,
            });
          } else {
            this.grid?.gridApi.exportDataAsCsv({
              ...params,
              ...renameHeaders(),
              ...event?.params,
              skipColumnGroupHeaders: true,
            });
          }
          // this.grid?.gridApi.exportDataAsCsv({ ...event?.params });

          // this.showToast("Exported Successfully")

          event?.events?.forEach((action: any) => {
            if (action.type === 'success') {
              this.showToast(action?.params?.message);
            }
          });
        }

        if (event?.method === 'exportToExcel') {
          if (event?.params?.groupExport + ''.toLowerCase() === 'true') {
            this.grid?.gridApi.exportDataAsExcel({
              ...params,
              ...event?.params,
            });
          } else {
            this.grid?.gridApi.exportDataAsExcel({
              ...params,
              ...renameHeaders(),
              ...event?.params,
              skipColumnGroupHeaders: true,
            });
          }
          // this.grid?.gridApi.exportDataAsExcel({ ...event?.params });

          event?.events?.forEach((action: any) => {
            if (action.type === 'success') {
              this.showToast(action?.params?.message);
            }
          });
        }
      });
    }
  }

  onRefresh() {
    this.refresh.next(true);
  }

  showToast(message: string) {
    this.toast.message(message);
  }
}
