import { Component, OnInit, ViewChild } from '@angular/core';
import { Hash8ChartOptions } from '@hash8io/chart-web';
import { TemplateService } from '@hash8io/events-web';
import { Store } from '@ngrx/store';
import { ICellRendererAngularComp } from 'ag-grid-angular';
import { AgChartOptions, ICellRendererParams } from 'ag-grid-community';

@Component({
  selector: 'hash8-platform-app-grid-chart',
  templateUrl: './grid-chart.component.html',
  styleUrls: ['./grid-chart.component.scss']
})
export class GridChartComponent implements ICellRendererAngularComp {
  label !: any;

  public options!: any;

  sparklineChartOptions !: Hash8ChartOptions;

  rowData !: any;

  seriesData !: any

  deviceStateChart !: any;

  iconUrl !: any

  iconColor !: any;

  dataFields !: any;

  events !: any;

  constructor(private store: Store, private templateService: TemplateService) { }

  ngOnInit() {
  }

  agInit(params: ICellRendererParams<any, any>): void {

    this.label = params?.value
    const cellRendererData = params?.colDef?.cellRendererParams;
    const groupRendererData =
      params?.colDef?.cellRendererParams?.innerRendererParams;
    const singleGroupRendererData =
      params?.node?.rowGroupColumn?.getColDef()?.cellRendererParams;

    this.dataFields = cellRendererData?.dataFields || groupRendererData?.dataFields || singleGroupRendererData?.dataFields

    this.iconUrl = cellRendererData?.defaultIconUrl || groupRendererData?.defaultIconUrl || singleGroupRendererData?.defaultIconUrl
    this.iconColor = cellRendererData?.defaultIconClass || groupRendererData?.defaultIconClass || singleGroupRendererData?.defaultIconClass

    this.events = cellRendererData?.events || groupRendererData?.events || singleGroupRendererData?.events

    this.rowData = {
      setPointTemperature: {
        // xaxis: 'x',
        // yaxis: 'setPointTemperature',
        // seriesName: 'Set Point Temperature',
        data: [
          { time: '12 AM', setPointTemperature: 20 },
          { time: '3 AM', setPointTemperature: 20 },
          { time: '6 AM', setPointTemperature: 21 },
          { time: '9 AM', setPointTemperature: 20 },
          { time: '12 PM', setPointTemperature: 20 },
          { time: '3 PM', setPointTemperature: 20 },]
      },
      supplyTemperature: {
        // seriesName: 'Supply Temperature',
        // xaxis: 'x',
        // yaxis: 'supplyTemperature',
        data: [
          { time: '12 AM', supplyTemperature: 23 },
          { time: '3 AM', supplyTemperature: 20 },
          { time: '6 AM', supplyTemperature: 21 },
          { time: '9 AM', supplyTemperature: 24 },
          { time: '12 PM', supplyTemperature: 22 },
          { time: '3 PM', supplyTemperature: 20 },
        ]
      },
      returnTemperature: {
        // seriesName: 'Return Temperature',
        // xaxis: 'x',
        // yaxis: 'supplyTemperature',
        data: [
          { time: '12 AM', returnTemperature: 20 },
          { time: '3 AM', returnTemperature: 23 },
          { time: '6 AM', returnTemperature: 25 },
          { time: '9 AM', returnTemperature: 22 },
          { time: '12 PM', returnTemperature: 21 },
          { time: '3 PM', returnTemperature: 20 },
        ]
      }
    }


    this.seriesData = this.dataFields?.map((dataField: any) => {

      if (this.rowData[dataField?.field]) {
        this.rowData[dataField?.field]['xaxis'] = dataField?.xaxis
        this.rowData[dataField?.field]['yaxis'] = dataField?.yaxis
        this.rowData[dataField?.field]['name'] = dataField?.name
        return this.rowData[dataField?.field]
      }
    })

    this.templateService.setData({ seriesData: this.seriesData })

    this.sparklineChartOptions = {
      chart: {
        type: 'line',
        width: 100,
        height: 35,
      },
      series: this.seriesData
    }


    this.deviceStateChart = {
      chart: {
        type: 'line',
        height: '400px',
        width: '800px',
      },
      title: {
        label: 'Set Point Temperature vs Supply Temperature vs Return Temperature',
      },
      series: this.seriesData
    }

  }

  refresh(params: ICellRendererParams<any, any>): boolean {
    return true;
  }

  openChartOverlay() {


    const events = [
      {
        event: 'click',
        method: 'openDialog',
        params: {
          maxWidth: '900px',
          width: '850px',
          data: {
            type: 'chart',
            title:
            {
              label: '(S1) Temp.Trend',
              iconUrl: '',
              showBorder: true,
            },
            content: [
              {
                type: 'chart',
                props: {
                  type: 'line',
                  chartStyle: "padding:10px 20px",
                  chartOptions: this.deviceStateChart,
                  title: this.deviceStateChart.title,
                  subtitle: this.deviceStateChart?.subtitle

                }
              }
            ]
          }
        }
      }
    ]

    console.log("This.events", this.templateService.getData())

    this.store.dispatch({
      type: '[Events] Process events',
      events: events
    });
  }
}
