import { Component, Input, OnInit } from '@angular/core';
import { AppStoreSelectors, Model } from '@hash8io/core-web';
import { ICellRendererAngularComp } from 'ag-grid-angular';
import { ICellRendererParams } from 'ag-grid-community';
import { DataGridWebService } from '../../services/data-grid-web.service';
import { Store } from '@ngrx/store';
import { formatInTimeZone } from 'date-fns-tz';

// import moment from 'moment';

export interface progress {
  percentage: number;
  color: string;
}

@Component({
  selector: 'hash8-platform-grid-date-time',
  templateUrl: './grid-date-time.component.html',
  styleUrls: ['./grid-date-time.component.scss'],
})
export class GridDateTimeComponent implements ICellRendererAngularComp {
  @Input() label!: any | undefined;
  @Input() ISOString!: any;
  @Input() iconUrl!: string | undefined;
  @Input() iconColor!: string;
  @Input() color!: string;
  @Input() iconSize: 'default' | 'xl' = 'default';

  userPreference: any;
  timeFormat!: any;
  timeZone!: any;
  dateFormat!: any;
  formattedTime!: any;

  tooltip = '';

  constructor(
    public dataGridWebService: DataGridWebService,
    private store: Store
  ) {
    //
  }

  ngOnInit() {
    if (this.ISOString) {
      this.changeTimeFormat(this.ISOString);
    }
  }
  agInit(params: ICellRendererParams<any, any>): void {
    const cellRendererData = params?.colDef?.cellRendererParams;
    const groupRendererData =
      params?.colDef?.cellRendererParams?.innerRendererParams;
    const singleGroupRendererData =
      params?.node?.rowGroupColumn?.getColDef()?.cellRendererParams;

    const fieldOptions =
      cellRendererData?.options ||
      groupRendererData?.options ||
      singleGroupRendererData?.options;

    const colorExpression =
      cellRendererData?.colorExpression ||
      groupRendererData?.colorExpression ||
      singleGroupRendererData?.colorExpression;

    //-------------------------Display Preference---------------------------
    this.changeTimeFormat(params?.value);

    //-----------------------------------------------------------------------

    if (this.label) {
      this.iconUrl =
        cellRendererData?.defaultIconUrl ||
        groupRendererData?.defaultIconUrl ||
        singleGroupRendererData?.defaultIconUrl;
      this.iconColor =
        cellRendererData?.defaultIconColor ||
        groupRendererData?.defaultIconColor ||
        singleGroupRendererData?.defaultIconColor;
      this.color =
        cellRendererData?.defaultColor ||
        groupRendererData?.defaultColor ||
        singleGroupRendererData?.defaultColor;
      this.iconSize =
        cellRendererData?.defaultIconSize ||
        'default' ||
        groupRendererData?.defaultIconSize ||
        singleGroupRendererData?.defaultIconSize;
      this.tooltip =
        cellRendererData?.tooltip ||
        groupRendererData?.tooltip ||
        singleGroupRendererData?.tooltip;
    }

    fieldOptions?.forEach((x: any) => {
      if (typeof params?.value === 'string') {
        if (
          params?.value?.toLowerCase() ===
          (x?.value?.toLowerCase() || x?.text?.toLowerCase())
        ) {
          this.label = x.label;
          this.color = x.color;
          this.iconUrl =
            x.iconUrl ||
            cellRendererData?.defaultIconUrl ||
            groupRendererData?.defaultIconUrl ||
            singleGroupRendererData?.defaultIconUrl;
          this.iconColor =
            x.iconColor ||
            cellRendererData?.defaultIconColor ||
            groupRendererData?.defaultIconColor ||
            singleGroupRendererData?.defaultIconColor;

          if (x?.colorExpression) {
            this.color =
              this.dataGridWebService.evaluate(x?.colorExpression, params) ||
              x.color;
          }
        }
      }

      if (typeof params?.value === 'number') {
        this.label = params.value;

        if (x?.colorExpression) {
          if (this.dataGridWebService.evaluate(x?.colorExpression, params)) {
            this.iconUrl = x.iconUrl;

            this.iconColor = this.dataGridWebService.evaluate(
              x?.colorExpression,
              params
            );
            if (params?.valueFormatted && params?.value) {
              this.label = params?.valueFormatted;
            }
          }
        }
      }
    });

    //This if condition only for number with expression & value formatted
    if (typeof params?.value === 'number') {
      //Value format with expression
      if (colorExpression) {
        if (this.dataGridWebService.evaluate(colorExpression, params)) {
          this.color = this.dataGridWebService.evaluate(
            colorExpression,
            params
          );
          if (params?.valueFormatted) {
            this.label = params?.valueFormatted;
          }
        }
      }

      //Value formatted either with icon added or not

      if (params?.valueFormatted) {
        this.label = params?.valueFormatted;
      }
    }

    //To Change icon Color

    if (cellRendererData?.colorExpressionOption === 'iconUrl') {
      if (this.dataGridWebService.evaluate(colorExpression, params)) {
        this.iconColor = this.dataGridWebService.evaluate(
          colorExpression,
          params
        );
      }
    }

    //To Change Text Color

    if (cellRendererData?.colorExpressionOption === 'text') {
      if (this.dataGridWebService.evaluate(colorExpression, params)) {
        this.iconColor = this.dataGridWebService.evaluate(
          colorExpression,
          params
        );
      }
    }
  }
  changeTimeFormat(timeData: any) {
    this.userPreference = this.store
      .select(AppStoreSelectors.selectDisplaySettings)
      .subscribe((data: any) => {
        this.timeFormat = data.timeFormat || 'hh:mm a';
        // this.timeFormat = 'MM/dd/yyyy hh:mm a';
        this.timeZone = data.timeZone || 'America/New_York';
        // this.timeZone = 'EDT';
        this.dateFormat = data?.dateFormat || 'MM/dd/yyyy';

        const dateTime = `${this.dateFormat} ${this.timeFormat}`;

        if ((timeData && timeData != '-') || this.ISOString) {
          this.formattedTime = formatInTimeZone(
            timeData ? timeData : this.ISOString,
            this.timeZone,
            dateTime
          );
        } else {
          this.formattedTime = '-';
        }

        // this.formattedTime = formatInTimeZone(
        //   this.ISOString || '2023-09-26T08:39:43.387Z',
        //   'America/New_York',
        //   'MM/dd/yy hh:mm a'
        // );
      });
    // this.label = params.value;
    this.label = this.formattedTime;
  }

  refresh(params: ICellRendererParams<any, any>): boolean {
    return true;
  }
}
