/* eslint-disable @typescript-eslint/no-empty-function */
/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
import { ChangeDetectionStrategy, Component, ViewChild } from '@angular/core';
import { MatMenuTrigger } from '@angular/material/menu';
import { TemplateService } from '@hash8io/events-web';
import { ICellRendererAngularComp } from 'ag-grid-angular';
import { ICellRendererParams, IStatusPanelParams } from 'ag-grid-community';

@Component({
  selector: 'hash8-platform-grid-indicator',
  templateUrl: './grid-indicator.component.html',
  styleUrls: ['./grid-indicator.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class GridIndicatorComponent implements ICellRendererAngularComp {
  @ViewChild(MatMenuTrigger) menuTrigger!: MatMenuTrigger;

  params!: any;
  data!: any;
  rowData!: any;
  indicatorLabel = '';
  indicatorOptions: any = [];
  indicatorValues: any = [];
  indicatorValueProp: any;
  indicatorExpression: any;
  popupTemplate!: any;

  constructor(private templateService: TemplateService) {}

  agInit(params: ICellRendererParams<any, any>): void {
    this.params = params;
    this.data = params.value;
    this.indicatorOptions = this.params.options;
    this.indicatorValueProp = this.params.valueProp || null;
    this.indicatorExpression = this.params.valueExpression;
    this.popupTemplate = this.params.popupTemplate;

    this.indicatorLabel =
      this.data?.[this.params?.labelProp] || this.params?.labelNullValue || '';

    this.indicatorValues = this.getIndicatorValues();

    this.rowData = params.data;
    // console.log(
    //   '%cgrid-indicator.component.ts line:21 params',
    //   'color: #007acc;',
    //   params
    // );
  }

  refresh(params: ICellRendererParams<any, any>): boolean {
    return true;
    // throw new Error('Method not implemented.');
  }

  getIndicatorValues(): any {
    const _data = this.data?.[this.params?.fieldProp] || [];
    const indicators = _data.map((d: any) => {
      const matchingOption = this.indicatorOptions.find((obj: any) => {
        // return obj.value === d.status;
        if (this.indicatorValueProp) {
          return obj.value === d?.[this.indicatorValueProp];
        } else {
          return (
            obj.value ===
            this.templateService.processTemplate(this.indicatorExpression, d)
          );
        }
      });

      if (matchingOption) {
        return {
          ...d,
          ...matchingOption,
          popupTemplate: this.templateService.processTemplate(
            this.popupTemplate,
            { item: { ...d, ...matchingOption } }
          ),
        };
      } else {
        return d;
      }
    });

    // console.log(
    //   '%cgrid-indicator.component.ts line:27 indicators',
    //   'color: #007acc;',
    //   indicators
    // );

    return indicators;
  }

  openMenu(menu: MatMenuTrigger) {
    if (menu) {
      menu.openMenu();
    }
  }

  closeMenu(menu: MatMenuTrigger) {
    if (menu) {
      menu.closeMenu();
    }
  }
}
