import { Component, Input, OnInit } from '@angular/core';
import { TemplateService } from '@hash8io/events-web';
import { ICellRendererAngularComp } from 'ag-grid-angular';
import { ICellRendererParams } from 'ag-grid-community';

@Component({
  selector: 'hash8-platform-grid-link',
  templateUrl: './grid-link.component.html',
  styleUrls: ['./grid-link.component.scss'],
})
export class GridLinkComponent implements ICellRendererAngularComp {
  @Input() url!: string;
  @Input() style!: string;
  @Input() disabled = false;
  @Input() hasUnderline = true;
  @Input() iconUrl: any;
  @Input() iconColor: any;
  @Input() iconSize: any = 'text-sm';
  @Input() linkColor: 'default' | 'muted' = 'default';
  @Input() tooltip: any = '';
  @Input() label!: string;

  selectedRowData!: any;

  constructor(private templateService: TemplateService) {}

  agInit(params: ICellRendererParams<any, any>): void {
    this.label = params.value;
    this.selectedRowData = params?.data;
    this.templateService.setData({ item: this.selectedRowData });

    const cellRendererData = params?.colDef?.cellRendererParams;
    const groupRendererData =
      params?.colDef?.cellRendererParams?.innerRendererParams;
    const singleGroupRendererData =
      params?.node?.rowGroupColumn?.getColDef()?.cellRendererParams;

    this.url = this.templateService.processTemplate(
      cellRendererData?.url ||
        groupRendererData?.url ||
        singleGroupRendererData?.url
    );

    this.label =
      cellRendererData?.label ||
      groupRendererData?.label ||
      singleGroupRendererData?.label;
    this.style =
      cellRendererData?.style ||
      groupRendererData?.style ||
      singleGroupRendererData?.style;

    this.iconColor = cellRendererData?.iconColor;
    this.iconUrl = cellRendererData?.iconUrl;
    this.iconSize = cellRendererData?.iconSize || '';
    this.tooltip = cellRendererData?.tooltip || '';
    this.hasUnderline = cellRendererData?.underlined;

    this.templateService.removeData('item');
  }

  refresh(params: ICellRendererParams<any, any>): boolean {
    return true;
  }
}
