/* eslint-disable @typescript-eslint/no-empty-function */
/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
import { Component, OnInit } from '@angular/core';
import { TemplateService } from '@hash8io/events-web';
import { Store } from '@ngrx/store';
import { ICellRendererAngularComp } from 'ag-grid-angular';
import { ICellRendererParams } from 'ag-grid-community';

@Component({
  selector: 'hash8-platform-grid-location',
  templateUrl: './grid-location.component.html',
  styleUrls: ['./grid-location.component.scss'],
})
export class GridLocationComponent implements ICellRendererAngularComp {
  selectedRowData!: any;

  label!: any;
  labelObj: any = {};
  iconUrl!: any;
  iconColor!: any;
  labelClass!: any;
  locationEvents!: any;
  _options: any = {
    checkGeofence: false,
    geofenceKeyField: 'geofence',
    geofenceLabelField: 'geofenceName',
    geofenceUUIDField: 'geofenceUUID',
    alwaysShowLocation: true,
  };

  constructor(private templateService: TemplateService, private store: Store) {}

  ngOnInit(): void {}

  agInit(params: ICellRendererParams<any, any>): void {
    this.label = params?.value;
    this.selectedRowData = params?.data;

    const cellRendererData = params?.colDef?.cellRendererParams;
    const groupRendererData =
      params?.colDef?.cellRendererParams?.innerRendererParams;
    const singleGroupRendererData =
      params?.node?.rowGroupColumn?.getColDef()?.cellRendererParams;

    this._options = Object.assign(
      this._options,
      cellRendererData?.options || {}
    );

    this.locationEvents = cellRendererData?.events;

    this.iconUrl =
      cellRendererData?.defaultIconUrl ||
      groupRendererData?.defaultIconUrl ||
      singleGroupRendererData?.defaultIconUrl;
    this.iconColor =
      cellRendererData?.defaultIconColor ||
      groupRendererData?.defaultIconColor ||
      singleGroupRendererData?.defaultIconColor;
    this.labelClass =
      cellRendererData?.labelClass ||
      'no-underline hover:underline cursor-pointer';

    this.labelObj = this.getLabelValue();
  }

  refresh(params: ICellRendererParams<any, any>): boolean {
    return true;
  }

  emitLocationAction() {
    this.templateService.setData({ _selectedRow: this.selectedRowData });

    this.store.dispatch({
      type: '[Events] Process events',
      events: this.locationEvents,
    });
  }

  getLabelValue() {
    const label = {
      isInGeofence: false,
      label: this.label,
    };

    if (this._options.checkGeofence) {
      if (this.selectedRowData[this._options.geofenceKeyField]) {
        label.isInGeofence = true;
        label.label =
          this.selectedRowData[this._options.geofenceKeyField]?.[
            this._options.geofenceLabelField
          ];
      }
    }

    return label;
  }
}
