import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { MatSlideToggleChange } from '@angular/material/slide-toggle';
import { AppStoreSelectors } from '@hash8io/core-web';
import { Store } from '@ngrx/store';
import { ICellRendererAngularComp } from 'ag-grid-angular';
import { ICellRendererParams } from 'ag-grid-community';
import { DataGridWebService } from '../../services/data-grid-web.service';
import * as expressions from 'angular-expressions';
import { TemplateService } from '@hash8io/events-web';

@Component({
  selector: 'hash8-platform-grid-slide-toggle',
  templateUrl: './grid-slide-toggle.component.html',
  styleUrls: ['./grid-slide-toggle.component.scss'],
})
export class GridSlideToggleComponent implements ICellRendererAngularComp {
  @Input() options: any = {}; //All Combined options (Not in use)

  @Input() label!: any;
  @Input() labelPosition!: 'before' | 'after' | undefined;
  @Input() hideLabel: any = true;

  //Theme
  @Input() barColor!: any;
  @Input() thumbColor!: any;
  @Input() color!: any;

  @Input() iconUrl!: any;

  //Actions
  @Input() toggle = false;
  @Input() clickable: any = true;
  @Input() isDisabled: any = false;

  @Input() tooltip!: any;
  // @Input() events: any;
  public events: any;
  value!: any;

  rowData!: any;

  @Output() toggleChange = new EventEmitter();

  constructor(
    private store: Store,
    public dataGridWebService: DataGridWebService,
    public templateService: TemplateService
  ) {}
  agInit(params: ICellRendererParams<any, any>): void {
    this.rowData = params.data;
    this.value = params.value;

    const cellRendererData = params?.colDef?.cellRendererParams;
    const groupRendererData =
      params?.colDef?.cellRendererParams?.innerRendererParams;
    const singleGroupRendererData =
      params?.node?.rowGroupColumn?.getColDef()?.cellRendererParams;

    //------------------------------------Collect and Assign Values--------------------------------------

    const toggleOptions =
      cellRendererData || groupRendererData || singleGroupRendererData;

    // const toggleOptions: any = {
    //   // field: 'smsEnabled',
    //   fieldType: 'side-toggle',
    //   disableToggle: `phoneNumberVerified === false`,
    //   iconUrl: 'error',
    //   iconUrlExpression: 'phoneNumberVerified === false',
    //   tooltip: 'Phone Number Not Verified',
    //   tooltipExpression: 'phoneNumberVerified === false',
    //   events: [
    //     { event: 'click', enabled: 'userCategory === true' },
    //     {
    //       event: 'click',
    //       enabled: "userCategory === true && phoneNumber ==='' ",
    //     },
    //     { event: 'click', enabled: '' },
    //     { event: 'toggleOn', method: 'callAPI', enabled: '' },
    //     { event: 'toggleOff', method: 'callAPI', enabled: '' },
    //   ],
    //   options: [
    //     { value: 'kljl', toggle: true, tooltip: '' },
    //     { value: 'lhl', toggle: false, tooltip: '' },
    //     { value: 'Deactivated', toggle: false, tooltip: '' },
    //   ],
    // };

    if (toggleOptions?.disableToggle) {
      this.isDisabled = this.evaluateExpression(
        toggleOptions?.disableToggle,
        this.rowData
      );
    }

    if (toggleOptions?.tooltip) {
      this.tooltip = this.templateService.apply(toggleOptions.tooltip, {
        ...this.templateService.data,
        ...this.rowData,
      });
    }
    if (
      toggleOptions?.iconUrlExpression &&
      this.evaluateExpression(toggleOptions.iconUrlExpression, this.rowData)
    ) {
      this.iconUrl = toggleOptions?.iconUrl;
    }

    if (toggleOptions?.options?.value) {
      this.options = this.dataGridWebService.computeToggleExpression(
        toggleOptions.options,
        this.value,
        this.rowData
      );
    }
    if (!toggleOptions?.options?.value || this.options === null) {
      //Toggle Default Values [0, 'true', true]
      this.options = {};
      this.options['toggle'] =
        this.value + '' === 'true' || this.value === 0
          ? true
          : this.value + '' === 'false' || this.value === 1
          ? false
          : null;
    }
    this.events = toggleOptions.events;

    //-----------------------------------------------------------------------------------------------------
  }

  public onToggleChange(toggleEvent?: MatSlideToggleChange, events?: any) {
    // this.toggleChange.emit(); //EMIT EVENT
    this.templateService.setData({ toggleData: toggleEvent?.checked });

    this.onToggleClick(this.events);
  }

  public onToggleClick(events?: any) {
    this.templateService.setData({ _selectedRow: this.rowData });

    const parsedEvent = events;

    this.store.dispatch({
      type: '[Events] Process events',
      events: parsedEvent,
    });
  }

  private evaluateExpression(expression: any, data: any): boolean {
    const evaluate = expressions.compile(expression);
    if (evaluate(data)) {
      return true;
    } else {
      return false;
    }
  }

  refresh(params: ICellRendererParams<any, any>): boolean {
    return true;
  }
}
