import {
  Component,
  Input,
  OnChanges,
  OnInit,
  SimpleChanges,
} from '@angular/core';
import { ICellRendererAngularComp } from 'ag-grid-angular';
import { ICellRendererParams, IStatusPanelParams } from 'ag-grid-community';
import { DataGridWebService } from '../../services/data-grid-web.service';
import { TemplateService } from '@hash8io/events-web';
@Component({
  selector: 'hash8-platform-grid-status-bar',
  templateUrl: './grid-status-bar.component.html',
  styleUrls: ['./grid-status-bar.component.scss'],
})
export class GridStatusBarComponent
  implements ICellRendererAngularComp, OnInit, OnChanges {
  private params!: IStatusPanelParams;
  components!: any;
  rowData!: any;
  fields!: any;
  fieldKeys!: any;

  @Input() statusObject: any = [];

  @Input() statusFields!: any;

  @Input() statusRowData!: any;

  tooltip!: any;

  constructor(
    public dataGridWebService: DataGridWebService,
    private templateService: TemplateService
  ) {
    //
  }

  ngOnInit() {
    const selectedRowData = this.templateService.getData()._selectedRow;

    if (selectedRowData) {
      this.processStatusFields(this.statusFields, selectedRowData);
    }
  }

  ngOnChanges(changes: SimpleChanges) {

    const dataChanges = changes['statusRowData']?.currentValue;
    // this.statusObject = [];
    if (dataChanges) {
      this.processStatusFields(this.statusFields, dataChanges);
    }
  }

  agInit(params: ICellRendererParams<any, any>): void {
    this.params = params;

    const cellRendererData = params?.colDef?.cellRendererParams;
    const singleGroupRendererData =
      params?.node?.rowGroupColumn?.getColDef()?.cellRendererParams;

    this.rowData = params.data;

    // this.components = cellRendererData.status;

    this.fields = cellRendererData.fields;
    this.processStatusFields(this.fields, this.rowData);
    // this.fieldKeys = Object.keys(this.fields);

    // for (const f in this.fields) {
    //   const field = this.fields[f];
    //   const labelObj = this.dataGridWebService.computeLabelExpression(
    //     field,
    //     this.rowData[field.field],
    //     this.rowData
    //   );
    //   this.statusObject.push(labelObj);
    // }
  }

  processStatusFields(fields: any, data: any) {
    this.fields = fields;
    this.fieldKeys = Object.keys(fields);
    this.statusObject = [];

    for (const f in this.fields) {
      const field = this.fields[f];
      const labelObj = this.dataGridWebService.computeLabelExpression(
        field,
        data[field.field],
        data
      );

      //To use data in Status Fields
      this.templateService.setData({ item: data });

      this.statusObject.push(this.templateService.processTemplate(labelObj));
    }
  }

  refresh(params: ICellRendererParams<any, any>): boolean {
    return true;
  }
}
