/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
import { CommonModule } from '@angular/common';
import { Component, Input, OnInit, Output, ViewChild } from '@angular/core';
import { MatMenuTrigger } from '@angular/material/menu';
import { ReportGridActions } from '@hash8io/common-store-web/report-grid-store';
import {
  TagActions,
  TagSelectors,
  TagService,
  TagStoreModule,
} from '@hash8io/common-store-web/tag-store';
import {
  BackgroundColorPipe,
  ColorService,
  SharedModule,
  TextColorPipe,
  ToastService,
} from '@hash8io/core-web';
import { SearchModule } from '@hash8io/core-web/search';
import { ToolbarModule } from '@hash8io/core-web/toolbar';
import { TemplateService } from '@hash8io/events-web';
import { Store } from '@ngrx/store';
// import { AssignTagListComponent } from '@hash8io/tags-web';
import {
  AgGridAngular,
  AgGridModule,
  ICellRendererAngularComp,
} from 'ag-grid-angular';
import { ColDef, GridApi, ICellRendererParams } from 'ag-grid-community';
import { Observable, Subject, switchMap, take, tap } from 'rxjs';
@Component({
  selector: 'hash8-platform-grid-tag',
  templateUrl: './grid-tag.component.html',
  styleUrls: ['./grid-tag.component.scss'],
  standalone: true,
  imports: [
    CommonModule,
    SharedModule,
    ToolbarModule,
    AgGridModule,
    SearchModule,
    TagStoreModule,
    TextColorPipe,
    BackgroundColorPipe,
  ],
})
export class GridTagComponent implements OnInit, ICellRendererAngularComp {
  @Input() disabled = false;
  @Input() buttonType: 'default' | 'icon' = 'icon';
  @Input() showAssignButton = true;
  @Input() showUnassignButton = false;
  @Input() hasAssign = false;
  valueProp: any = 'value';
  labelProp: any = 'label';

  @ViewChild(MatMenuTrigger) menuTrigger!: MatMenuTrigger;
  @ViewChild('tagGrid') tagGrid!: AgGridAngular;

  @Output() assign = new Subject();
  @Output() unassign = new Subject();
  @Output() cancel = new Subject();

  @Input() targetGrid!: any;

  // _config = {
  //   apiUrl:
  //     'api/v1/tags?organizationUUID={{data.localStorage.organizationUUID}}',
  // };

  _selectedTagRows: any = [];
  _exisitingTags: any = [];
  _disableApplyButton: any = true;
  _tags: any = [];
  _selectedEntityRow: any = {};
  _params: any;

  private gridApi!: GridApi;

  columnDefs: ColDef[] = [
    {
      headerName: 'Tag Name',
      field: 'tagName',
      sortable: true,
      pinned: 'left',
      suppressMovable: false,
      headerCheckboxSelection: true,
      checkboxSelection: true,
      cellRendererParams: {
        fieldType: 'text',
        defaultIconUrl: 'local_offer',
        iconColor: '#001ADD',
      },
      width: 280,
      filter: 'agTextColumnFilter',
      filterParams: {
        suppressAndOrCondition: true,
        buttons: ['reset', 'apply'],
      },
    },
    {
      headerName: 'Tag Key',
      field: 'tagKey',
      width: 100,
      resizable: false,
    },
  ];
  rowData: any = [];
  loading$!: Observable<any>;
  loadingSwitched = false;

  constructor(
    private store: Store,
    private colorService: ColorService,
    private templateService: TemplateService,
    private tagService: TagService,
    private toastService: ToastService
  ) {
    this.store.select(TagSelectors.selectAllTags).subscribe((d: any) => {
      this.rowData = d;
    });
  }

  ngOnInit(): void {
    // console.log('ngOnInit', this.targetGrid);
    this.loading$ = this.store.select(TagSelectors.selectTagLoading);
  }

  OnGridReady(params: any) {
    this.gridApi = params.api;
  }

  agInit(params: ICellRendererParams<any, any>) {
    this._params = params;
    this._tags = params?.value || [];
    this._selectedEntityRow = params.data || {};

    // this.templateService.setData({
    //   _selectedRow: this._selectedEntityRow,
    // });

    if (params) {
      // this.buttonType = params?.colDef?.cellRendererParams?.buttonType;
      const keys = Object.keys(params?.colDef?.cellRendererParams);
      if (keys.includes('showAssignButton')) {
        this.showAssignButton = params?.colDef?.cellRendererParams
          ?.showAssignButton
          ? true
          : false;
      }

      if (keys.includes('labelProp')) {
        this.labelProp = params?.colDef?.cellRendererParams?.labelProp;
      }

      if (keys.includes('valueProp')) {
        this.valueProp = params?.colDef?.cellRendererParams?.valueProp;
      }
    }

    // this.store
    //   .select(TagSelectors.selectTagLoading)
    //   .pipe(take(2)) // Use the take(1) operator to take only the first emitted value
    //   .subscribe((loading: boolean) => {
    //     if (!loading) {
    //       console.log(
    //         '%cgrid-tag.component.ts line:176 !loading',
    //         'color: #007acc;',
    //         !loading
    //       );
    //       setTimeout(() => {
    //         this.markAssignedTagSelected();
    //       }, 500);
    //     }
    //   });
  }

  refresh(params: ICellRendererParams<any, any>): boolean {
    return true;
  }

  refreshTags() {
    this.store.dispatch(TagActions.refreshTags());
    this.loading$
      .pipe(take(2)) // Take only the first two emitted value (true)
      .subscribe((loading: boolean) => {
        // if (!loading && !this.loadingSwitched) { //Disabled => Due to mark not made after cancelling model
        if (!loading) {
          setTimeout(() => {
            this.loadingSwitched = true;
            this.markAssignedTagSelected();
          }, 100);
        }
      });
  }

  handleOverlayClick(event: MouseEvent) {
    event.stopPropagation(); // Prevent the event from propagating to the button
  }

  assignButtonClick(event: MouseEvent) {
    event.stopPropagation();

    if (!this.hasAssign) {
      this.toastService.message('Select Assets to Assign/Unassign Tags');
      this.closeOverlay();
      return;
    }

    if (!this.disabled && this.hasAssign) {
      this.handleButtonClick(event);
    }
  }

  handleButtonClick(event: MouseEvent) {
    event.stopPropagation();

    if (!this.disabled) {
      this.openOverlay();

      if (this.buttonType === 'default') {
        // Later we can change to mode
        // Common assign - while changing the dropdown set the value in localstorage and use the name here
        this.tagService.setEndpoint(
          `api/v1/tags?organizationUUID=[%22{{data.selectedOrgFilterUUIDs[0]}}%22]&applicationUUID={{data.localStorage.applicationUUID}}`
        );
      } else {
        // Single assign
        this.tagService.setEndpoint(
          `api/v1/tags?organizationUUID=[%22${this._selectedEntityRow.organizationUUID}%22]&applicationUUID={{data.localStorage.applicationUUID}}`
        );
      }

      this.store.dispatch(TagActions.loadTags());

      this.loading$
        .pipe(take(2)) // Take only the first two emitted value (true)
        .subscribe((loading: boolean) => {
          // if (!loading && !this.loadingSwitched) { //Disabled => Due to mark not made after cancelling model
          if (!loading) {
            setTimeout(() => {
              this.loadingSwitched = true;
              this.markAssignedTagSelected();
            }, 100);
          }
        });
    }
  }

  markAssignedTagSelected() {
    if (this._params) {
      const selectedTagUUID = (this._params.data?.tags || []).map(
        (t: any) => t.uuid
      );

      this.gridApi?.forEachNode((node: any) => {
        if (node.data && selectedTagUUID.includes(node.data.uuid)) {
          node.setSelected(true);
        }
      });
    }
  }

  openOverlay() {
    this.menuTrigger.openMenu(); // open the menu
    this.reloadTagList();
  }

  closeOverlay() {
    this.menuTrigger.closeMenu(); // close the menu
  }

  reloadTagList() {
    this._selectedTagRows = [];
    this.enableDisableButton();

    this.tagGrid?.api?.deselectAll();
    this.tagGrid?.api?.refreshCells();
  }

  onRowSelect(event: any) {
    this._selectedTagRows = event.api.getSelectedRows();
    this.enableDisableButton();
  }

  enableDisableButton() {
    this._disableApplyButton = this.rowData.length > 0 ? false : true;
    // this._disableApplyButton = this._selectedTagRows.length > 0 ? false : true;
  }

  assignChanges() {
    this.assign.next(this._selectedTagRows);
    this.closeOverlay();

    if (!this._selectedEntityRow.assetUUID) {
      return;
    }

    if (this._params?.gridName) {
      this.store.dispatch(
        ReportGridActions.ASSIGN_UNASSIGN_TAGS({
          gridName: this._params?.gridName,
          data: this.tagService.generateAPIRequestObject(
            this._tags,
            this._selectedTagRows,
            [this._selectedEntityRow.assetUUID],
            this._selectedEntityRow.organizationUUID,
            'Asset'
          ),
        })
      );
      // this.store.dispatch(
      //   ReportGridActions.REPLACE_TAGS({
      //     gridName: this._params.gridName,
      //     selectedRowIds: [this._selectedEntityRow.assetUUID],
      //     tagIds: this._selectedTagRows,
      //   })
      // );
    }
  }

  // Not used as of now. Have to change each tag has (x) mark on click this will be used.
  unassignChanges() {
    this.unassign.next(this._selectedTagRows);
    console.log(
      'Selected item for unassign:',
      this._selectedEntityRow.assetUUID,
      this._selectedTagRows
    );
    this.closeOverlay();

    if (this._params?.gridName) {
      this.store.dispatch(
        ReportGridActions.UNASSIGN_TAGS({
          gridName: this._params.gridName,
          selectedRowIds: [this._selectedEntityRow.assetUUID],
          tagIds: this._selectedTagRows,
        })
      );
    }
  }

  cancelChanges() {
    this.cancel.next(null);
    this.closeOverlay();
  }
}
