import { Component, Input, OnInit } from '@angular/core';
import { Model } from '@hash8io/core-web';
import { ICellRendererAngularComp } from 'ag-grid-angular';
import { ICellRendererParams } from 'ag-grid-community';
import { DataGridWebService } from '../../services/data-grid-web.service';


export interface progress {
  percentage: number,
  color: string
}

@Component({
  selector: 'hash8-platform-grid-text',
  templateUrl: './grid-text.component.html',
  styleUrls: ['./grid-text.component.scss'],
})
export class GridTextComponent implements ICellRendererAngularComp {

  @Input() label!: any | undefined;
  @Input() iconUrl!: string | undefined;
  @Input() iconColor!: string
  @Input() color!: string;
  @Input() iconSize: 'default' | 'xl' = 'default';
  @Input() iconStyle !: any;


  tooltip = '';


  constructor(public dataGridWebService: DataGridWebService) {
    //
  }
  agInit(params: ICellRendererParams<any, any>): void {


    const cellRendererData = params?.colDef?.cellRendererParams;
    const groupRendererData = params?.colDef?.cellRendererParams?.innerRendererParams
    const singleGroupRendererData = params?.node?.rowGroupColumn?.getColDef()?.cellRendererParams


    const fieldOptions = cellRendererData?.options || groupRendererData?.options || singleGroupRendererData?.options

    const colorExpression = cellRendererData?.colorExpression || groupRendererData?.colorExpression || singleGroupRendererData?.colorExpression

    this.label = params.value
    if (this.label) {
      this.iconUrl = cellRendererData?.defaultIconUrl || groupRendererData?.defaultIconUrl || singleGroupRendererData?.defaultIconUrl
      this.iconColor = cellRendererData?.defaultIconColor || groupRendererData?.defaultIconColor || singleGroupRendererData?.defaultIconColor
      this.color = cellRendererData?.defaultColor || groupRendererData?.defaultColor || singleGroupRendererData?.defaultColor
      this.iconSize = cellRendererData?.defaultIconSize || 'default' || groupRendererData?.defaultIconSize || singleGroupRendererData?.defaultIconSize
      this.iconStyle = cellRendererData?.defaultIconStyle || 'default' || groupRendererData?.defaultIconStyle || singleGroupRendererData?.defaultIconStyle

      this.tooltip = cellRendererData?.tooltip || groupRendererData?.tooltip || singleGroupRendererData?.tooltip
    }


    fieldOptions?.forEach((x: any) => {

      if (typeof params?.value === 'string') {
        if (params?.value?.toLowerCase() === (x?.value?.toLowerCase() || x?.text?.toLowerCase())) {
          this.label = x.label
          this.color = x.color
          this.iconUrl = x.iconUrl || cellRendererData?.defaultIconUrl || groupRendererData?.defaultIconUrl || singleGroupRendererData?.defaultIconUrl
          this.iconColor = x.iconColor || cellRendererData?.defaultIconColor || groupRendererData?.defaultIconColor || singleGroupRendererData?.defaultIconColor

          if (x?.colorExpression) {
            this.color = this.dataGridWebService.evaluate(x?.colorExpression, params) || x.color
          }

        }
        if (x?.colorExpression) {
          if (this.dataGridWebService.evaluate(x?.colorExpression, params)) {
            this.iconUrl = x.iconUrl

            this.iconColor = this.dataGridWebService.evaluate(x?.colorExpression, params)
            if (params?.valueFormatted && params?.value) {
              this.label = params?.valueFormatted
            }
          }
        }
      }

      if (typeof params?.value === 'number') {

        this.label = params.value

        if (x?.colorExpression) {
          if (this.dataGridWebService.evaluate(x?.colorExpression, params)) {
            this.iconUrl = x.iconUrl

            this.iconColor = this.dataGridWebService.evaluate(x?.colorExpression, params)
            if (params?.valueFormatted && params?.value) {
              this.label = params?.valueFormatted
            }
          }
        }
      }
    })

    //This if condition only for number with expression & value formatted
    if (typeof params?.value === 'number') {

      //Value format with expression
      if (colorExpression) {

        if (this.dataGridWebService.evaluate(colorExpression, params)) {
          this.color = this.dataGridWebService.evaluate(colorExpression, params)
          if (params?.valueFormatted) {
            this.label = params?.valueFormatted
          }
        }
      }

      //Value formatted either with icon added or not

      if (params?.valueFormatted) {
        this.label = params?.valueFormatted
      }

    }

    //To Change icon Color

    if (cellRendererData?.colorExpressionOption === 'iconUrl') {

      if (this.dataGridWebService.evaluate(colorExpression, params)) {
        this.iconColor = this.dataGridWebService.evaluate(colorExpression, params)
      }
    }

    //To Change Text Color

    if (cellRendererData?.colorExpressionOption === 'text') {

      if (this.dataGridWebService.evaluate(colorExpression, params)) {
        this.iconColor = this.dataGridWebService.evaluate(colorExpression, params)
      }
    }





  }







  refresh(params: ICellRendererParams<any, any>): boolean {
    return true;
  }

}

