import { Component, Input, OnInit } from '@angular/core';
import { Store } from '@ngrx/store';
import { ICellRendererAngularComp } from 'ag-grid-angular';
import { ICellRendererParams } from 'ag-grid-community';

@Component({
  selector: 'hash8-platform-grid-title',
  templateUrl: './grid-title.component.html',
  styleUrls: ['./grid-title.component.scss'],
})
export class GridTitleComponent implements ICellRendererAngularComp {
  @Input() label!: any | undefined;
  @Input() iconUrlType1!: string | undefined;
  @Input() iconUrlType2!: string | undefined;

  @Input() iconColor!: string;
  @Input() color!: string;
  @Input() iconSize: 'default' | 'xl' = 'default';

  @Input() isLink!: boolean;
  @Input() link!: any;
  @Input() linkUrl!: any;
  rowData: any;
  dialogEvent: any;
  count: any;
  countField: any;
  labelBGColor: any = '#c3cfd9';
  labelColor: any = 'black';
  status: any;
  icon1Link: any;
  icon1Url: any;

  constructor(private store: Store) {
    //
  }
  agInit(params: ICellRendererParams<any, any>): void {
    const cellRendererData = params?.colDef?.cellRendererParams;
    const groupRendererData =
      params?.colDef?.cellRendererParams?.innerRendererParams;
    const singleGroupRendererData =
      params?.node?.rowGroupColumn?.getColDef()?.cellRendererParams;

    if (params.value instanceof Array) {
      this.label = params.value[0];
    } else {
      this.label = params.value;
    }
    this.rowData = params.data;

    if (this.label) {
      this.iconUrlType1 =
        cellRendererData?.defaultIconType1 ||
        groupRendererData?.defaultIconType1 ||
        singleGroupRendererData?.defaultIconType1;
      this.iconUrlType2 =
        cellRendererData?.defaultIconType2 ||
        groupRendererData?.defaultIconType2 ||
        singleGroupRendererData?.defaultIconType2;
      this.iconColor =
        cellRendererData?.iconColor ||
        groupRendererData?.iconColor ||
        singleGroupRendererData?.iconColor;
      this.color =
        cellRendererData?.color ||
        groupRendererData?.color ||
        singleGroupRendererData?.color;

      //TITLE AS LINK
      this.isLink =
        cellRendererData?.isLink ||
        groupRendererData?.isLink ||
        singleGroupRendererData?.isLink;
      this.link =
        cellRendererData?.link ||
        groupRendererData?.link ||
        singleGroupRendererData?.link;
      this.status =
        cellRendererData?.fields ||
        groupRendererData?.fields ||
        singleGroupRendererData?.fields;

      //icon 1 as link
      this.icon1Link =
        cellRendererData?.iconType1link ||
        groupRendererData?.iconType1link ||
        singleGroupRendererData?.iconType1link;

      if (this.icon1Link) {
        if (!this.icon1Link?.url2) {
          this.icon1Url = `${this.icon1Link?.url}/${this.rowData[this.icon1Link?.field] || ''
            }`;
        } else {
          this.icon1Url = `${this.icon1Link?.url}/${this.rowData[this.icon1Link?.field] || ''
            }/${this.icon1Link?.url2}`;
        }

      }

      //Title Link for Page

      if (this.isLink === true || this.link) {
        switch (this.link.type) {
          case 'page':
            this.linkUrl = `${this.link?.url}/${this.rowData[this.link.field]}`;
            break;
          case 'dialog':
            this.dialogEvent = this.link.events;

            this.count = this.rowData[this.status?.counts?.field];
            // if (this.labelBGColor !== '') {
            //   this.labelBGColor = this.status?.color;
            // }
            // if (this.labelColor !== '') {
            //   this.labelColor = this.status?.labelColor;
            // }

            break;
        }
      }
    }
  }

  refresh(params: ICellRendererParams<any, any>): boolean {
    return true;
  }

  executeEvents() {
    // Change Event Data as per User Actions
    //----------------------------------------------------------------------------
    const eventData = JSON.stringify(this.dialogEvent);
    const parsedEvent = JSON.parse(eventData);
    let apiParams = '?';
    console.log('ParsedEvent', parsedEvent);
    parsedEvent.forEach((event: any) => {
      if (event.event === 'click' && event.method === 'openDialog') {
        let title = event?.params?.data?.title;
        let rowDataTitle = this.rowData?.eventGroupName;
        let rowDataDescription = this.rowData?.description;
        title['label'] = rowDataTitle;
        title['description'] = rowDataDescription;
        console.log('TITLE', title);

        // let description = this.rowData?.descriptions;
        // title.label = this.rowData?.eventName;
        // title['description'] = this.rowData?.descriptions;

        event?.params?.data.content.forEach((e: any) => {
          if (e.type === 'report') {
            e.props?.config?.bodyParams.forEach((x: any) => {
              if (x.value) {
                apiParams = `${apiParams}${x.key}=${x.value}&`;
              } else if (!x?.value && this.rowData[x.key]) {
                apiParams = `${apiParams}${x.key}=${this.rowData[x.key]}&`;
              } else {
                apiParams = `${apiParams}${x.key}=${localStorage.getItem(
                  x.key
                )}&`;
              }
            });
            e.props.config.apiUrl =
              `${e.props.config.apiUrl}${apiParams}`.replace(/.$/, '');
            console.log('API', e.props.config.apiUrl);
          }
          // else if (e.event === 'failure' && e.method === 'showErrorToast') {
          //   e.params.message = `${this.colDef.headerName} ${e.params.message}`;
          // } else if (e.event === 'success' && e.method === 'showToast') {
          //   e.params.message = `${this.colDef.headerName} ${e.params.message}`;
          // } else if (e.event === 'submit' && e.method === 'postAPI') {
          //   //logic
          // }
        });
      }
    });

    //----------------------------------------------------------------------------
    console.log('UPDATED parsed', parsedEvent);
    this.store.dispatch({
      type: '[Events] Process events',
      events: parsedEvent,
    });
    // }
  }
}
