import { Component, Input, OnInit } from '@angular/core';
import { AppStoreSelectors } from '@hash8io/core-web';
import { Store } from '@ngrx/store';
import { ICellRendererAngularComp } from 'ag-grid-angular';
import { ICellRendererParams } from 'ag-grid-community';
import { DataGridWebService } from '../../services/data-grid-web.service';

@Component({
  selector: 'hash8-platform-grid-unit-conversion',
  templateUrl: './grid-unit-conversion.component.html',
  styleUrls: ['./grid-unit-conversion.component.scss'],
})
export class GridUnitConversionComponent implements ICellRendererAngularComp {
  @Input() value!: any;
  @Input() baseUnit!: any;
  @Input() unit!: any;
  @Input() label!: any;

  userPreference: any;
  timeFormat!: any;
  timeZone!: any;
  formattedTime!: any;

  tooltip = '';

  constructor(
    public dataGridWebService: DataGridWebService,
    private store: Store
  ) {
    //
  }
  agInit(params: ICellRendererParams<any, any>): void {
    const cellRendererData = params?.colDef?.cellRendererParams;
    const groupRendererData =
      params?.colDef?.cellRendererParams?.innerRendererParams;
    const singleGroupRendererData =
      params?.node?.rowGroupColumn?.getColDef()?.cellRendererParams;

    const fieldOptions =
      cellRendererData?.options ||
      groupRendererData?.options ||
      singleGroupRendererData?.options;

    const colorExpression =
      cellRendererData?.colorExpression ||
      groupRendererData?.colorExpression ||
      singleGroupRendererData?.colorExpression;

    //-------------------------Display Preference---------------------------

    this.userPreference = this.store
      .select(AppStoreSelectors.selectDisplaySettings)
      .subscribe((data: any) => {
        if (params.value) {
          if (data.temperature.toLowerCase() === 'celsius') {
            this.label = params.value;
          } else if (data.temperature.toLowerCase() === 'fahrenheit') {
            this.label =
              params.value != '-'
                ? (params.value * (9 / 5) + 32).toFixed(2)
                : '-';
          } else if (data.temperature.toLowerCase() === 'kelvin') {
            this.label =
              params.value != '-' ? (params.value + 273.15).toFixed(2) : '-';
          } else {
            this.label = params.value;
          }
        } else {
          this.label = '-';
        }
      });

    //-----------------------------------------------------------------------
  }

  refresh(params: ICellRendererParams<any, any>): boolean {
    return true;
  }
}
