/* eslint-disable no-case-declarations */
/* eslint-disable prefer-const */
/* eslint-disable @typescript-eslint/no-inferrable-types */
/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { NewColumnDef } from './models/newColumnDefinition.interface';
import {
  CheckboxSelectionCallbackParams,
  DisplayedColumnsChangedEvent,
  ExcelStyle,
  FilterChangedEvent,
  GetDataPath,
  GridOptions,
  GridReadyEvent,
  HeaderCheckboxSelectionCallbackParams,
  IColumnToolPanel,
} from 'ag-grid-community';
import 'ag-grid-enterprise';
import { ColDef, GridApi } from 'ag-grid-enterprise';
import { Observable, Subject } from 'rxjs';
import { DataGridOptionsModel } from './models/data-grid';
import { GridLabelComponent } from './components/grid-label/grid-label.component';
import { GridTextComponent } from './components/grid-text/grid-text.component';
import { GridIconComponent } from './components/grid-icon/grid-icon.component';
import { GridProgressComponent } from './components/grid-progress/grid-progress.component';
import { GridActionComponent } from './components/grid-action/grid-action.component';
import { GridTitleComponent } from './components/grid-title/grid-title.component';
import { GridStatusBarComponent } from './components/grid-status-bar/grid-status-bar.component';
import { GridLinkComponent } from './components/grid-link/grid-link.component';
import { Store } from '@ngrx/store';

import { GridDateTimeComponent } from './components/grid-date-time/grid-date-time.component';
import { GridSlideToggleComponent } from './components/grid-slide-toggle/grid-slide-toggle.component';
import { GridCheckboxComponent } from './components/grid-checkbox/grid-checkbox.component';
import { GridTagComponent } from './components/grid-tag/grid-tag.component';
import { TemplateService } from '@hash8io/events-web';
import { DataGridWebService } from './services/data-grid-web.service';
import { GridIndicatorComponent } from './components/grid-indicator/grid-indicator.component';
import { GridLocationComponent } from './components/grid-location/grid-location.component';
import { GridChartComponent } from './components/grid-chart/grid-chart.component';
import { GridUnitConversionComponent } from './components/grid-unit-conversion/grid-unit-conversion.component';
import { GridTooltipComponent } from './components/grid-tooltip/grid-tooltip.component';
@Component({
  selector: 'hash8-platform-data-grid-web',
  templateUrl: './data-grid-web.component.html',
  styleUrls: ['./data-grid-web.component.scss'],
})
export class DataGridWebComponent implements OnInit {
  @Input() name!: string;
  @Input() columns!: NewColumnDef[];
  @Input() rowData: any[] = [];
  @Input() rowIdField!: any;
  columns$!: Observable<any>;
  newColumns!: any[];

  @Input() style: any = {
    width: '100%',
    height: '600px',
  };
  private heightMode!: any;

  @Input() options!: DataGridOptionsModel;
  _options!: DataGridOptionsModel;
  sideBarToolbar = {
    toolPanels: [
      {
        id: 'columns',
        labelDefault: 'Columns',
        labelKey: 'columns',
        iconKey: 'menu',
        toolPanel: 'agColumnsToolPanel',
        toolPanelParams: {
          suppressPivotMode: false,
        },
      },
      {
        id: 'filters',
        labelDefault: 'Filters',
        labelKey: 'filters',
        iconKey: 'filter',
        toolPanel: 'agFiltersToolPanel',
      },
    ],
    defaultToolPanel: '',
  };

  defaultGridOptions: DataGridOptionsModel = {
    theme: 'ag-theme-hash8-theme',
    hasPagination: true,
    paginationSize: 15,
    showSideBar: true,
    hasPivot: true,
    hasRowGroupPanel: true,
    hasFilters: true,
    hasGrouping: true,
    hasSideBarValues: true,
    hasColumns: true,
    hasRowAction: false,
    hasColumnVisibility: true,
    rowSelection: 'multiple',
    hasRangeSelection: false,
    columnSizing: 'auto',
    hasColumnPinning: true,
    hasRightClick: true,
    showColumnOptions: false,
    // statusBar: {},
    heightMode: 'normal',
    height: '100%',
    // sideBar: SideBarDef | string | string[];
    sideBarPanel: this.sideBarToolbar,
    sortable: true,
    filter: false,
    resizable: true,
    editable: true,
    allowContextMenuWithControlKey: false,
    enablePivot: true,
    enableValue: true,
    enableRowGroup: true,
    suppressMenu: true,
    useValueFormatterForExport: true,
  };

  public excelStyles: ExcelStyle[] = [
    {
      id: 'numberType',
      numberFormat: {
        format: '0',
      },
    },
    {
      id: 'currencyFormat',
      numberFormat: {
        format: '#,##0.00 €',
      },
    },
    {
      id: 'negativeInBrackets',
      numberFormat: {
        format: '$[blue] #,##0;$ [red](#,##0)',
      },
    },
    {
      id: 'booleanType',
      dataType: 'Boolean',
    },
    {
      id: 'stringType',
      dataType: 'String',
    },
    {
      id: 'dateType',
      dataType: 'DateTime',
    },
  ];

  get rowGroupPanel() {
    if (this._options?.hasRowGroupPanel) {
      return 'onlyWhenGrouping';
    }
    return 'never';
  }
  public autoGroupColumnDef: ColDef = {
    width: 250,
  };

  updateRowSelect!: any;
  // OLD FIELDS

  @Input() localeText!: { [key: string]: string }; // TODO: Later

  @Input()
  statusBar!: any;

  @Output()
  gridReady: EventEmitter<GridReadyEvent> = new EventEmitter();

  @Output()
  selectionChanged = new EventEmitter();

  @Output()
  rowSelected = new EventEmitter();

  @Output() rowSelect: Subject<any> = new Subject();

  @Output()
  totalRowCount: EventEmitter<number> = new EventEmitter<number>();

  gridApi!: GridApi;

  gridOptions!: GridOptions;

  isRowSelectable: any;

  public components: any;

  columnApi!: any;
  columnIds: any[] = [];

  noRowsTemplate = '<span>No Data Found</span>';

  // public rowModelType: RowModelType = 'clientSide';

  rowDataValue!: any;

  // public getRowId: GetRowIdFunc = (params: GetRowIdParams) => {
  //   return params?.data[this.rowIdField] || params?.data?.uuid
  // };

  constructor(
    private store: Store,
    public templateService: TemplateService,
    private dataGridService: DataGridWebService
  ) {}

  // FOr Tree Structure View
  public getDataPath: GetDataPath = (data: any) => {
    return data[this._options.treeHierarchyColumnName];
  };

  columnDataChanged(params: any) {
    if (this._options.columnSizing + ''?.toUpperCase() === 'AUTO') {
      this.columnApi = params.columnApi;
      this.columnApi = this.gridOptions.columnApi;
      this.columnApi?.getColumns().forEach((column: any) => {
        this.columnIds.push(column.colId);
      });
      this.columnApi.autoSizeColumns(this.columnIds);
    } else if (this._options.columnSizing + ''?.toUpperCase() === 'FIT') {
      //If you want to resize to fit grid width
      params.api.sizeColumnsToFit();
    }
  }

  ngOnChanges(changes: any) {
    let newColumns: any = changes.columns.currentValue?.map((column: any) => {
      let obj = { ...column };
      if (obj?.cellRendererParams?.innerRenderer) {
        obj.innerRenderer = GridTextComponent;
      }

      if (obj.children) {
        obj = this.getColumnGroupCustomCellRenderer(obj);
      } else {
        obj = this.getCustomCellRenderer(obj);
      }

      // switch (obj?.cellRendererParams?.fieldType) {
      //   case 'label':
      //     obj.cellRenderer = GridLabelComponent;
      //     break;
      //   case 'text':
      //     obj.cellRenderer = GridTextComponent;
      //     break;
      //   case 'dateTime':
      //     obj.cellRenderer = GridDateTimeComponent;
      //     break;
      //   case 'title':
      //     obj.cellRenderer = GridTitleComponent;
      //     break;
      //   case 'icon':
      //     obj.cellRenderer = GridIconComponent;
      //     break;
      //   case 'progress':
      //     obj.cellRenderer = GridProgressComponent;
      //     break;
      //   case 'status':
      //     obj.cellRenderer = GridStatusBarComponent;
      //     break;
      //   case 'indicator':
      //     obj.cellRenderer = GridIndicatorComponent;
      //     break;
      //   case 'action':
      //     obj.cellRenderer = GridActionComponent;
      //     // obj.cellRenderer = CallToActionWebComponent;
      //     break;
      //   case 'slide-toggle':
      //     obj.cellRenderer = GridSlideToggleComponent;
      //     break;
      //   case 'link':
      //     obj.cellRenderer = GridLinkComponent;
      //     break;

      //   case 'checkbox':
      //     obj.cellRenderer = GridCheckboxComponent;
      //     break;

      //   case 'tag':
      //     obj.cellRenderer = GridTagComponent;
      //     // obj.cellRenderer = GridCustomCellComponent;
      //     break;

      //   case 'location':
      //     obj.cellRenderer = GridLocationComponent;
      //     break;

      //   case 'chart':
      //     obj.cellRenderer = GridChartComponent;
      //     break;
      // }

      if (obj?.expression) {
        obj.valueGetter = obj?.expression;
      }

      if (obj.keyCreator) {
        const keyCreatorField = obj.keyCreator;
        obj['keyCreator'] = (params: any) => {
          return params.value[keyCreatorField];
        };
      }

      return {
        ...column,
        ...obj,
      };
    });
    const newValue = changes?.options?.currentValue
      ? changes?.options?.currentValue
      : {};
    this._options = { ...this._options, ...newValue };
    this.newColumns = newColumns;
  }

  ngOnInit(): void {
    this._options = {
      ...this.defaultGridOptions,
      ...this.options,
    };

    this.gridOptions = {
      columnDefs: this.columns,
      defaultColDef: {
        suppressMenu: this._options.suppressMenu,
        sortable: this._options.sortable,
        filter: this._options.filter,
        resizable: this._options.resizable,
        editable: this._options.editable,
        enablePivot: this._options.enablePivot,
        enableValue: this._options.enableValue,
        enableRowGroup: this._options.enableRowGroup,
        // headerCheckboxSelection: this.isFirstColumn,
        // checkboxSelection: this.isFirstColumn,
      },
      pagination: this._options?.hasPagination || false,
      localeText: {
        dateFormatOoo: 'yyyy-mm-dd',
      },

      onCellClicked: (event) => {
        if (event?.colDef?.field === 'action') {
          event.node.setSelected(false);
        }
      },
      onRowSelected(params) {
        // Kowsi: Why we added this line
        // if (params.node.isSelected()) params.node.setSelected(true, true);
        // else params.node.setSelected(false, true);
      },
    };

    this.heightMode = this.style.height;
  }

  getCustomCellRenderer(obj: any) {
    const o = { ...obj };

    //Custom Tooltip Component
    o.tooltipComponent = GridTooltipComponent;

    switch (o?.cellRendererParams?.fieldType) {
      case 'label':
        o.cellRenderer = GridLabelComponent;
        break;
      case 'text':
        o.cellRenderer = GridTextComponent;
        break;
      case 'dateTime':
        o.cellRenderer = GridDateTimeComponent;
        break;
      case 'title':
        o.cellRenderer = GridTitleComponent;
        break;
      case 'icon':
        o.cellRenderer = GridIconComponent;
        break;
      case 'progress':
        o.cellRenderer = GridProgressComponent;
        break;
      case 'status':
        o.cellRenderer = GridStatusBarComponent;
        break;
      case 'indicator':
        o.cellRenderer = GridIndicatorComponent;
        break;
      case 'action':
        o.cellRenderer = GridActionComponent;
        // obj.cellRenderer = CallToActionWebComponent;
        break;
      case 'slide-toggle':
        o.cellRenderer = GridSlideToggleComponent;
        break;
      case 'link':
        o.cellRenderer = GridLinkComponent;
        break;

      case 'checkbox':
        o.cellRenderer = GridCheckboxComponent;
        break;

      case 'tag':
        o.cellRenderer = GridTagComponent;
        // obj.cellRenderer = GridCustomCellComponent;
        break;

      case 'location':
        o.cellRenderer = GridLocationComponent;
        break;

      case 'chart':
        o.cellRenderer = GridChartComponent;
        break;
      case 'unit-conversion': //Need to rethink
        o.valueGetter = (params: any) =>
          this.dataGridService.dataUnitConversion(
            params.data[o.field],
            obj?.cellRendererParams
          );

        o.valueFormatter = (params: any) =>
          this.dataGridService.dataUnitConversion(
            params.data[o.field],
            obj?.cellRendererParams
          );

        break;
    }
    return o;
  }

  getColumnGroupCustomCellRenderer(obj: any) {
    const gObj = { ...obj };
    let children: any[] = [];
    gObj.children.map((o: any) => {
      children.push({
        ...this.getCustomCellRenderer(o),
      });
    });
    return { ...gObj, children };
  }

  isFirstColumn(
    params:
      | CheckboxSelectionCallbackParams
      | HeaderCheckboxSelectionCallbackParams
  ) {
    const displayedColumns = params.columnApi.getAllDisplayedColumns();
    const thisIsFirstColumn = displayedColumns[0] === params.column;
    return thisIsFirstColumn;
  }

  getContextMenuItems(): string[] {
    // const result: string[] = ['export'];
    // return result;
    return [];
  }

  onDisplayedColumnsChanged(event: DisplayedColumnsChangedEvent) {
    let totalRowCount = 0;

    const rowCount = this.gridApi?.paginationGetRowCount();

    if (this._options?.isTreeData) {
      for (let i = 0; i < rowCount; i++) {
        totalRowCount =
          1 +
          totalRowCount +
          (this.gridApi?.getDisplayedRowAtIndex(i)?.allChildrenCount || 0);
      }
    } else {
      totalRowCount = rowCount;
    }

    this.totalRowCount.emit(totalRowCount);

    // this.totalRowCount.emit(this.gridApi?.paginationGetRowCount());
  }

  onFilterChanged(event: FilterChangedEvent) {
    let totalRowCount = 0;

    let rowCount = this.gridApi?.paginationGetRowCount();

    if (this._options?.isTreeData) {
      for (let i = 0; i < rowCount; i++) {
        if (this.gridApi?.getDisplayedRowAtIndex(i)?.level == 0) {
          totalRowCount =
            1 + // Current Row
            totalRowCount +
            (this.gridApi?.getDisplayedRowAtIndex(i)?.allChildrenCount || 0); // Children of Row
        } else {
          ++rowCount;
        }
      }
    } else {
      totalRowCount = rowCount;
    }

    this.totalRowCount.emit(totalRowCount);
  }

  OnGridReady(params: any) {
    this.gridApi = params.api;
    this.gridOptions.api?.closeToolPanel();
    this.gridApi.setSideBarVisible(this._options?.showSideBar || false);

    if (this._options.hasRowAction) {
      // this.rowData.map((x) => {
      //   if (x.actions) {
      //     this.columns.push(colDefActions)
      //     x.actions = rowAction
      //   }
      // })
    }

    let newColumns: any = this.columns?.map((column: any) => {
      let obj = { ...column };
      if (obj?.cellRendererParams?.innerRenderer) {
        obj.innerRenderer = GridTextComponent;
      }

      if (obj.children) {
        obj = this.getColumnGroupCustomCellRenderer(obj);
      } else {
        obj = this.getCustomCellRenderer(obj);
      }

      // switch (obj?.cellRendererParams?.fieldType) {
      //   case 'label':
      //     obj.cellRenderer = GridLabelComponent;
      //     break;
      //   case 'text':
      //     obj.cellRenderer = GridTextComponent;
      //     break;
      //   case 'dateTime':
      //     obj.cellRenderer = GridDateTimeComponent;
      //     break;
      //   case 'title':
      //     obj.cellRenderer = GridTitleComponent;
      //     break;
      //   case 'icon':
      //     obj.cellRenderer = GridIconComponent;
      //     break;
      //   case 'progress':
      //     obj.cellRenderer = GridProgressComponent;
      //     break;
      //   case 'status':
      //     obj.cellRenderer = GridStatusBarComponent;
      //     break;
      //   case 'indicator':
      //     obj.cellRenderer = GridIndicatorComponent;
      //     break;
      //   case 'action':
      //     obj.cellRenderer = GridActionComponent;
      //     // obj.cellRenderer = CallToActionWebComponent;
      //     break;
      //   case 'slide-toggle':
      //     obj.cellRenderer = GridSlideToggleComponent;
      //     break;
      //   case 'link':
      //     obj.cellRenderer = GridLinkComponent;
      //     break;

      //   case 'checkbox':
      //     obj.cellRenderer = GridCheckboxComponent;
      //     break;

      //   case 'tag':
      //     obj.cellRenderer = GridTagComponent;
      //     // obj.cellRenderer = GridCustomCellComponent;
      //     break;

      // case 'location':
      //   obj.cellRenderer = GridLocationComponent;
      //   break;

      // case 'chart':
      //   obj.cellRenderer = GridChartComponent;
      //   break;
      // }

      if (obj?.expression) {
        obj.valueGetter = obj?.expression;
      }

      if (obj.keyCreator) {
        const keyCreatorField = obj.keyCreator;
        obj['keyCreator'] = (params: any) => {
          // return params.value[keyCreatorField];
          if (params.value) return params.value[keyCreatorField];
          else return '';
        };
      }

      return {
        ...column,
        ...obj,
      };
    });

    this.newColumns = newColumns;

    const columnToolPanel = params.api.getToolPanelInstance(
      'columns'
    ) as unknown as IColumnToolPanel;
    columnToolPanel.setPivotModeSectionVisible(
      this._options?.hasPivot || false
    );

    //Set height
    // this.style.height = Number(totalCount) * 40 + 100 + 'px';
    const height = this.dataGridService.getGridHeight(
      this.gridApi,
      this.rowData.length || 0,
      this.style.height,
      this.style.negativeSpace
    );
    this.style = { ...this.style, height };

    //Tryout Server side Data source
    // const fakeServer = this.createFakeServer(this.rowData);
    // const dataSource = this.createServerSideDatasource(fakeServer);
    // params.api.setServerSideDatasource(dataSource);

    ///Autosize
    // If you need to resize All columns

    // if (this._options.columnSizing?.toUpperCase() === 'AUTO') {
    //   this.columnApi = params.columnApi;
    //   this.columnApi = this.gridOptions.columnApi;
    //   this.columnApi?.getColumns().forEach((column: any) => {
    //     this.columnIds.push(column.colId);
    //   });
    //   this.columnApi.autoSizeColumns(this.columnIds);
    // } else if (this._options.columnSizing?.toUpperCase() === 'FIT') {
    //   //If you want to resize to fit grid width
    //   params.api.sizeColumnsToFit();
    // }

    // this.store.dispatch(
    //   DataGridStoreAction.ADD_DATA_GRID({ grid: gridStoreObj })
    // );

    // THIS WONT WORK
    // this.columns$ = of(this.columns);
    // this.columns$ = this.store.select(
    //   DataGridStoreSelectors.selectGridColumns(this.name)
    // );

    // this.store
    //   .select(DataGridStoreSelectors.selectRefreshFlag(this.name))
    //   .subscribe((data) => {
    //     if (data) {
    //       console.log('Refreshing data grid....');
    //       this.store.dispatch(
    //         DataGridStoreAction.REFRESH_GRID_END({ gridName: this.name })
    //       );

    //       this.gridApi.refreshCells({
    //         force: true,
    //         suppressFlash: true,
    //       });
    //     }
    //   });
  }

  onColumnResized(event: any) {
    //
  }

  onFirstDataRendered(params: any) {
    if (this._options.columnSizing + ''?.toUpperCase() === 'AUTO') {
      this.columnApi = params.columnApi;
      this.columnApi = this.gridOptions.columnApi;
      this.columnApi?.getColumns().forEach((column: any) => {
        this.columnIds.push(column.colId);
      });
      this.columnApi.autoSizeColumns(this.columnIds);
    } else if (this._options.columnSizing + ''?.toUpperCase() === 'FIT') {
      //If you want to resize to fit grid width
      params.api.sizeColumnsToFit();
    }
  }

  // createServerSideDatasource(server: any): IServerSideDatasource {
  //   return {
  //     getRows: (params) => {
  //       // get data for request from our fake server
  //       const response = server.getData(params.request);
  //       // simulating real server call with a 500ms delay
  //       setTimeout(function () {
  //         if (response.success) {
  //           // supply rows for requested block to grid
  //           params.success({ rowData: response.rows });
  //         } else {
  //           params.fail();
  //         }
  //       }, 500);
  //     },
  //   };
  // }

  // createFakeServer(allData: any[]) {
  //   return {
  //     getData: (request: IServerSideGetRowsRequest) => {
  //       // in this simplified fake server all rows are contained in an array
  //       const requestedRows = allData.slice(request.startRow, request.endRow);
  //       return {
  //         success: true,
  //         rows: requestedRows,
  //       };
  //     },
  //   };
  // }

  onSelectionChanged(params: any) {
    // this.selectionChanged.emit(params);
    this.templateService.setData({ [this.name]: params.selectedRows });
    this.rowSelect.next(params);
  }

  onRowSelected(params: GridReadyEvent) {
    this.rowSelected.emit(params);
  }

  onPageSizeChanged(event: any) {
    // const value = (document.getElementById('page-size') as HTMLInputElement)
    //   .value;  // This wont work. It always take the first element if we used two grids in one page

    const value = event.target.value;
    this.gridApi.paginationSetPageSize(Number(value));

    //Set height
    if (this.heightMode === 'auto') {
      const height = this.dataGridService.getGridHeight(
        this.gridApi,
        this.rowData.length || 0,
        'auto'
      );
      this.style = { ...this.style, height };
    }
  }

  exportToExcel() {
    this.gridApi.exportDataAsExcel();
  }
  exportToCsv() {
    this.gridApi.exportDataAsCsv();
  }

  // getGridHeight(
  //   gridApi: any,
  //   rowsCount: number,
  //   heightType = 'fixed',
  //   customHeight = '500px',
  //   autoRowsLimit = 13
  // ) {
  //   if (heightType === 'fixed') {
  //     gridApi.setDomLayout('normal');
  //     return customHeight;
  //   } else if (heightType === 'auto') {
  //     console.log(
  //       rowsCount,
  //       gridApi.paginationGetPageSize(),
  //       rowsCount > gridApi.paginationGetPageSize()
  //     );
  //     if (rowsCount < gridApi.paginationGetPageSize()) {
  //       gridApi.setDomLayout('autoHeight');
  //       return '';
  //     } else if (gridApi.paginationGetPageSize() >= autoRowsLimit) {
  //       gridApi.setDomLayout('normal');
  //       // const height = Number(autoRowsLimit) * 40 + 100 + 'px';
  //       // return height;
  //       return 'calc(80vh - 64px)';
  //     } else {
  //       gridApi.setDomLayout('autoHeight');
  //       return '';
  //     }
  //   } else if (heightType === 'fill') {
  //     gridApi.setDomLayout('normal');
  //     return 'calc(80vh - 64px)';
  //   } else if (typeof heightType === 'string') {
  //     return heightType;
  //   } else {
  //     return customHeight; // Default to '250px'
  //   }
  // }
}
