/* eslint-disable @typescript-eslint/no-inferrable-types */
/* eslint-disable @angular-eslint/no-input-rename */
/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
/* eslint-disable @typescript-eslint/no-empty-function */
import {
  Component,
  EventEmitter,
  Input,
  OnChanges,
  OnDestroy,
  OnInit,
  Output,
  SimpleChanges,
} from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import {
  ReportGridActions,
  ReportGridSelectors,
  ReportGridService,
} from '@hash8io/common-store-web/report-grid-store';
import { TagService } from '@hash8io/common-store-web/tag-store';
import { Model } from '@hash8io/core-web';
import { TemplateService } from '@hash8io/events-web';
import { Store, select } from '@ngrx/store';
import { BehaviorSubject, Observable, Subject, of, takeUntil } from 'rxjs';
import { TreeviewConfig } from '@hash8io/tree-view-web';
import { statusBar } from './mock-data/statusbar-mock';

// import { DataReportStoreActions, DataReportStoreSelectors } from './store';

@Component({
  selector: 'hash8-platform-data-report',
  templateUrl: './data-report-web.component.html',
  styleUrls: ['./data-report-web.component.scss'],
})
export class DataReportWebComponent implements OnInit, OnDestroy, OnChanges {
  // Toolbar configurations
  @Input() showTitle!: boolean;
  @Input() showBreadcrumb!: boolean;
  @Input() showActions!: boolean;
  @Input() showSearch!: boolean;
  @Input() showSettings!: boolean;
  @Input() showToolbar: boolean = true;
  // @Input() showAssignTagAction: boolean = false;

  // Mode 1 - Give API url and get all configuration from api response
  @Input() apiUrl!: string;

  // Mode 2 - Also without API we can provide individual configuration
  @Input() name: string = 'defaultGrid';
  @Input() title!: Model.Title;
  @Input() breadcrumbs: Model.Breadcrumb[] = [];
  @Input() settings: Model.Action[] = [];
  @Input() actions: Model.Action[] = [];

  @Input() reportUUID!: any; // Is that needed? - For mock purpose
  @Input() config!: any; // Overall inputs ah config la um send pani use panalam
  @Input() options!: any; // Need to analyse
  @Input() rowSelection: 'single' | 'multiple' = 'single'; // use in code // cascade may not work

  @Input() filterConfiguration!: any;
  @Input() externalReportFilter!: any;

  @Output() externalFilterValueChanged = new EventEmitter();

  // @Output() addedRows = new EventEmitter();

  // @Output() removedRows = new EventEmitter();

  @Output() masterRowData: Subject<any> = new Subject();

  statusBar!: any;

  gridStyle = {
    width: '100%',
    height: '600px',
  };
  @Input() style: any = this.gridStyle; // context puriyala

  @Input() mode: 'static' | 'api' = 'api'; // Remove this if not needed

  @Output() rowSelect: Subject<any> = new Subject();

  configurations: any;
  columns: any;
  data: any;
  rowData$!: Observable<any>; // Store
  count: any;

  noRowsTemplate: any = 'No Data Found';
  _selectedRows: any = [];
  _noRowsSelected: boolean = true;
  routeSubscription: any;
  rowTotalCount!: number;
  loaded: boolean = false;

  hasTagButton: boolean = false;
  _disableAssignButton: boolean = true;

  rowIdField!: any;

  private selectRefresh$!: Observable<any>;
  private selectRefreshSubscription$!: any;
  private reportServiceData$: any;
  private readonly destroySubscription$ = new Subject<void>();

  private dataSource$: BehaviorSubject<any> = new BehaviorSubject<any>({});
  dataSource: Observable<string> = this.dataSource$.asObservable();

  private _defaultConfig: any = {};
  private _config: any = {};

  public reportFilterData: any;

  orgValues!: any[];
  orgFilterConfig!: any;

  constructor(
    private reportService: ReportGridService<any>,
    private route: ActivatedRoute,
    private store: Store,
    private templateService: TemplateService,
    private tagService: TagService
  ) {}

  ngOnInit(): void {
    this.loaded = false;
    this.routeSubscription = this.route.params.subscribe((params) => {
      // if (params['uuid']) this.reportUUID = params['uuid'];

      this._defaultConfig = {
        name: this.name,
        apiUrl: this.config?.apiUrl || '',
        columns: this.columns || [],
        rowData: [],
        configurations: {
          breadcrumbs: this.config?.breadcrumbs || [],
          statusBar: this.config?.statusBar || [],
          actions: this.config?.actions || [],
          settings: this.config?.settings || [],
        },
        totalCount: 0,
        loading: true,
        refreshGrid: false,
      };

      this._config = Object.assign({ ...this._defaultConfig, ...this.config });

      // Organization Filter
      this.orgFilterConfig = TreeviewConfig.create(
        this._config?.organizationFilterConfig || {}
      );

      this.count = { label: '', value: 0 };
      // this.fetchReportDetails();
      // first entry point
      this.initReportConfig();
    });

    // Only for refresh it used
    this.reportServiceData$ = this.dataSource.subscribe((d: any) => {
      if (d?.name) {
        this.selectRefresh$ = this.store.pipe(
          select(ReportGridSelectors.selectRefreshFlag(d?.name))
        );

        this.selectRefreshSubscription$ = this.selectRefresh$.subscribe(
          (data) => {
            if (data) {
              //Unsubscribe Existing Report Before Refresh
              this.selectRefreshSubscription$?.unsubscribe();

              // this.getReportData(this.apiUrl, this.reportUUID);
              // this.store.dispatch(
              //   ReportGridActions.GET_REPORT_CONFIG({
              //     apiUrl: this._config.apiUrl,
              //     gridName: this._config.name,
              //   })
              // );
              this.store.dispatch(
                ReportGridActions.GET_REPORT_CONFIG({
                  apiUrl: this._config.apiUrl,
                  gridName: this._config.name,
                  actions: this._config.actions,
                  settings: this._config.settings,
                  style: this._config.style,
                  breadcrumbs: this._config.breadcrumbs,
                  statusBar: this._config?.statusBar,
                })
              );

              this.store.dispatch(
                ReportGridActions.REFRESH_REPORT_END({
                  gridName: d?.name,
                })
              );
            }
          }
        );
      }
    });
  }

  ngOnDestroy() {
    this.destroySubscription$.next();
    this.destroySubscription$.complete();

    this.routeSubscription.unsubscribe();
    // this.reportServiceData$.unsubscribe();
    this.selectRefreshSubscription$?.unsubscribe();

    /** Destroy from store */
    this.store.dispatch(
      ReportGridActions.REMOVE_REPORT_CONFIG({
        gridName: this._config.name,
      })
    );

    this.templateService.removeData('_selectedRow');
    this.templateService.removeData(this._config.name);
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (!changes?.['config']?.firstChange) {
      this.config = changes?.['config']?.currentValue;
      this._config = Object.assign({ ...this._defaultConfig, ...this.config });
      this.count = { label: '', value: 0 };

      this.initReportConfig();
    }
  }

  initReportConfig() {
    // Get report configuration
    // this.store.dispatch(
    //   ReportGridActions.GET_REPORT_CONFIG({
    //     apiUrl: this._config.apiUrl,
    //     gridName: this._config.name,
    //   })
    // );

    this._config = this.templateService.processTemplate(this._config);
    this.store.dispatch(
      ReportGridActions.GET_REPORT_CONFIG({
        apiUrl: this._config.apiUrl,
        gridName: this._config.name,
        actions: this._config.actions,
        settings: this._config.settings,
        style: this._config.style,
        breadcrumbs: this._config.breadcrumbs,
        statusBar: this._config.statusBar,
      })
    );

    // Report Grid Columns
    this.store
      .select(ReportGridSelectors.selectReportGrid(this._config.name))
      .pipe(takeUntil(this.destroySubscription$))
      .subscribe((config: any) => {
        if (config) {
          this.apiUrl = config.apiUrl;
          // console.log(
          //   '%cdata-report-web.component.ts line:395 columns',
          //   'color: #007acc;',
          //   config
          // );
          // this.columns = columns;
        }
      });

    // Report Grid Columns
    this.store
      .select(ReportGridSelectors.selectReportColumns(this._config.name))
      .pipe(takeUntil(this.destroySubscription$))
      .subscribe((columns: any) => {
        if (columns) {
          this.loaded = true;
          this.columns = columns;
        }
      });

    // Report Grid Row Data
    this.rowData$ = this.store.select(
      ReportGridSelectors.selectReportRowData(this._config.name)
    );

    this.rowData$.subscribe((d) => {
      this._disableAssignButton = false;
      // this.store.dispatch({
      //   type: '[Data Report] Process Row IDS',
      //   gridName: this._config.name,
      // });
      this.masterRowData.next(d);
    });

    // this.store
    //   .select(ReportGridSelectors.selectAddedRows(this._config.name))
    //   .pipe(takeUntil(this.destroySubscription$))
    //   .subscribe((d) => {
    //     this.addedRows.emit(d);
    //   });

    // this.store
    //   .select(ReportGridSelectors.selectRemovedRows(this._config.name))
    //   .pipe(takeUntil(this.destroySubscription$))
    //   .subscribe((d) => {
    //     this.removedRows.emit(d);
    //   });

    // Report Grid Total Count
    this.store
      .select(ReportGridSelectors.selectReportTotalCount(this._config.name))
      .pipe(takeUntil(this.destroySubscription$))
      .subscribe((totalCount) => {
        this.count['value'] = totalCount;
      });

    // Report Grid Configurations
    this.store
      .select(ReportGridSelectors.selectReportConfigurations(this._config.name))
      .pipe(takeUntil(this.destroySubscription$))
      .subscribe((config: any) => {
        if (config) {
          config = this.templateService.processTemplate(config);

          this.name = config?.name;
          // console.log(config);
          this.options = config?.options;
          this.title = config?.title;
          this.settings = config?.settings;
          this.breadcrumbs = config?.breadcrumbs;
          this.statusBar = config?.statusBar;
          this.actions = config?.actions;
          this.style = config?.style || this.gridStyle;
          this.count['label'] = config?.label;
        }
      });

    // Refresh Data grid
    this.selectRefresh$ = this.store.pipe(
      select(ReportGridSelectors.selectRefreshFlag(this._config.name))
    );

    this.selectRefresh$
      .pipe(takeUntil(this.destroySubscription$))
      .subscribe((flag) => {
        if (flag) {
          this.dataSource$.next(this._config);
        }
      });

    this.dataSource$.next(this._config);
  }

  rowCount(params: any) {
    this.count['value'] = params;
  }

  onRowSelected(params: any) {
    this.templateService.setData({
      [this._config.name]: params.api.getSelectedRows(),
    });
    this.rowSelect.next({
      configuration: this.configurations,
      params: params,
    });
    // this._disableAssignButton =
    //   params.api.getSelectedRows().length > 0 ? false : true;
    if (params.api.getSelectedRows().length > 0) {
      this._noRowsSelected = false;
    } else {
      this._noRowsSelected = true;
    }
    this.updateSelectedRowsStoreObject(params);
  }

  updateSelectedRowsStoreObject(selectedRows: any) {
    const _selected = selectedRows.api.getSelectedRows();
    this._selectedRows = _selected;

    this.store.dispatch(
      ReportGridActions.UPDATE_REPORT_CONFIG({
        gridName: this._config.name,
        updates: {
          selectedRows: _selected,
          // selectedRowIds: _selected?.map((r: any) => r.assetUUID),
        },
      })
    );
  }

  onAssignTags(tags: any) {
    const _selectedRowIds = this._selectedRows?.map((r: any) => r.assetUUID);
    const _tagIds = tags.map((t: any) => t.uuid);

    // Common assign - while changing the organization dropdown set the value in localstorage and use the name here
    this.store.dispatch(
      ReportGridActions.ASSIGN_UNASSIGN_TAGS({
        gridName: this._config.name,
        data: this.tagService.generateAPIRequestObject(
          [],
          tags,
          _selectedRowIds,
          this.templateService.getData('selectedOrgFilterUUIDs')[0],
          'Asset'
        ),
      })
    );
  }

  onUnassignTags(tags: any) {
    const _selectedRowIds = this._selectedRows?.map((r: any) => r.assetUUID);
    const _tagIds = tags.map((t: any) => t.uuid);

    // Common unassign - while changing the organization dropdown set the value in localstorage and use the name here
    this.store.dispatch(
      ReportGridActions.ASSIGN_UNASSIGN_TAGS({
        gridName: this._config.name,
        data: this.tagService.generateAPIRequestObject(
          tags,
          [],
          _selectedRowIds,
          this.templateService.getData('selectedOrgFilterUUIDs')[0],
          'Asset'
        ),
      })
    );
  }

  onRefresh(event: any) {
    const refreshEvent = [
      {
        event: 'click',
        method: 'reloadReport',
        params: {
          gridName: this._config.name,
          apiUrl: this.apiUrl,
        },
      },
    ];

    this.store.dispatch({
      type: '[Events] Process events',
      events: refreshEvent,
    });

    // this.store.dispatch({
    //   type: '[Data Report] Refresh data report',
    //   gridName: this._config.name,
    // });
  }

  externalReportFilterValueChange(event: any) {
    this.externalFilterValueChanged.emit(event);
  }
}
