/* eslint-disable prefer-const */
import { Injectable } from '@angular/core';
import { MatDialog, MatDialogRef } from '@angular/material/dialog';
import { Observable } from 'rxjs';
import { DialogWebComponent } from '../dialog-web.component';
import { DialogModel } from '../models/dialog.model';
import { DataReportWebComponent } from '@hash8io/data-report-web';
import { CascadeReportWebComponent } from '@hash8io/cascade-report-web';
import { Store } from '@ngrx/store';

@Injectable({
  providedIn: 'root',
})
export class DialogWebService {
  //Temp
  dialogRef!: MatDialogRef<DialogWebComponent>;
  componentLookup: any = {
    cascade: CascadeReportWebComponent,
    report: DataReportWebComponent,
  };
  constructor(public dialog: MatDialog, private store: Store) {}

  openDialog(modal: DialogModel): Observable<boolean> {
    // let dialogRef: MatDialogRef<DialogWebComponent>;

    //Change Model Data As per the Component
    if (modal?.data?.customComponents) {
      for (const [key] of Object.entries(modal?.data?.customComponents)) {
        modal.data = JSON.parse(JSON.stringify(modal.data)); //For Write Access
        // Object.assign(modal.data, modal.data);
        if (this.componentLookup[key])
          modal.data.customComponents[key] = this.componentLookup[key];
      }
    }

    this.dialogRef = this.dialog.open(DialogWebComponent, {
      ...modal,
      minWidth: modal.minWidth || 500,
      maxWidth: modal.maxWidth || 600,
      minHeight: modal?.minHeight,
      maxHeight: modal?.maxHeight,
      height: modal?.height,
      autoFocus: modal.autoFocus || false,
      data: modal.data,
    });

    return this.dialogRef.afterClosed();
  }

  closeDialog() {
    this.dialogRef?.close();

    const events = [
      {
        event: 'success',
        method: 'removeData',
        params: {
          keys: ['formData', '_selectedRow', 'selectedOrgObject'],
        },
      },
    ];
    this.store.dispatch({
      type: '[Events] Process events',
      events: events,
    });
  }
}
