/* eslint-disable @nrwl/nx/enforce-module-boundaries */
import { Injectable } from '@angular/core';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { map } from 'rxjs/operators';
import { Store } from '@ngrx/store';
import * as DialogStoreActions from './actions';
import { DialogWebService } from '../services';
import { TemplateService } from '@hash8io/events-web';

@Injectable()
export class DialogStoreEffects {
  constructor(
    private actions$: Actions,
    private dialogService: DialogWebService,
    private templateService: TemplateService,
    private store: Store
  ) { }

  openDialog$ = createEffect(() => {
    return this.actions$.pipe(
      ofType(DialogStoreActions.OPEN_DAILOG),
      map((d) => {
        this.dialogService.openDialog(d.dialog);
        return DialogStoreActions.ADD_DIALOG({});
      })
    );
  });

  closeDialog$ = createEffect(() => {
    return this.actions$.pipe(
      ofType(DialogStoreActions.CLOSE_DIALOG),
      map((d) => {
        this.templateService.removeData(d?.dialog?.clearData);
        this.dialogService.closeDialog();
        return DialogStoreActions.REMOVE_DIALOG({});
      })
    );
  });
}
