/* eslint-disable prefer-const */
/* eslint-disable no-case-declarations */
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { ToastService } from '@hash8io/core-web';
import { Store } from '@ngrx/store';

import * as expressions from 'angular-expressions';

@Injectable({
  providedIn: 'root',
})
export class EventsWebService {
  constructor(
    private http: HttpClient,
    private toast: ToastService,
    private store: Store,
    private router: Router,
    private route: ActivatedRoute
  ) { }

  fetchAPI(event: any, service: any) {
    const apiEndpoint = event?.params?.apiUrl;

    // HTTP Headers
    let headers = new HttpHeaders();

    headers = headers.set('Content-Type', 'application/json');
    if (event?.params?.headers) {
      event?.params?.headers.forEach((p: any) => {
        headers = headers.set(p.key, p.value);
      });
    }

    if (!event?.params?.noAccessToken) {
      let authenticationToken = localStorage.getItem('access_token');
      headers = headers.set('Authorization', 'Bearer ' + authenticationToken);
    }

    let queryParams = event?.data || {};

    if (event?.params?.getParams) {
      event?.params?.getParams?.forEach((data: any) => {
        if (data?.value) {
          queryParams[data.key] = data.value;
        } else {
          if (event?.data) {
            queryParams[data.key] = event?.data[data?.key];
          } else {
            queryParams[data.key] = localStorage.getItem(data.key);
          }
        }
      });
    }

    let options = {
      params: queryParams,
      headers: headers,
    };

    service.http.get(apiEndpoint, options, { headers: headers }).subscribe({
      next(response: any) {
        if (response) {
          const events = event?.events?.filter(
            (e: any) => e.event !== 'failure'
          );

          const _events = events
            .filter((e: any) => {
              if (e?.enabled) {
                const evaluate = expressions.compile(e.enabled);

                if (evaluate(response)) {
                  return true;
                }
                return false;
              }
              return true;
            })
            .map((e: any) => {
              return Object.assign({}, e, { data: response.data });
            });

          service?.processEvents(_events, service, response?.data || response);
        }
      },
      error(error: any) {
        const events = event?.events.filter((e: any) => e.event !== 'success');
        service?.processEvents(events, service);
      },
    });
  }

  postAPI(event: any, service: any) {
    const apiEndpoint = event?.params?.apiUrl;

    // HTTP Headers
    let headers = new HttpHeaders();

    headers = headers.set('Content-Type', 'application/json');
    if (event?.params?.headers) {
      event?.params?.headers.forEach((p: any) => {
        headers = headers.set(p.key, p.value);
      });
    }

    if (!event?.params?.noAccessToken) {
      let authenticationToken = localStorage.getItem('access_token');
      headers = headers.set('Authorization', 'Bearer ' + authenticationToken);
    }

    let body = event?.formData || {};

    if (event?.params?.bodyParams) {
      event?.params?.bodyParams?.forEach((data: any) => {
        if (data?.value) {
          body[data.key] = data.value;
        } else if (data.key && !data.value) {
          body[data?.key] = localStorage.getItem(data.key);
        } else {
          if (event?.data) {
            body[data.key] = event?.data[data?.key];
          } else {
            body[data.key] = localStorage.getItem(data.key);
          }
        }
      });
    }

    service.http.post(apiEndpoint, body, { headers: headers }).subscribe({
      next(response: any) {
        if (response) {
          const events = event?.events?.filter(
            (e: any) => e.event !== 'failure'
          );
          service?.processEvents(events, service, response?.data || response);
        }
      },
      error(error: any) {
        const events = event?.events.filter((e: any) => e.event !== 'success');
        service?.processEvents(events, service);
      },
    });
  }

  putAPI(event: any, service: any) {
    const apiEndpoint = event?.params?.apiUrl;

    // HTTP Headers
    let headers = new HttpHeaders();

    headers = headers.set('Content-Type', 'application/json');
    if (event?.params?.headers) {
      event?.params?.headers.forEach((p: any) => {
        headers = headers.set(p.key, p.value);
      });
    }

    if (!event?.params?.noAccessToken) {
      let authenticationToken = localStorage.getItem('access_token');
      headers = headers.set('Authorization', 'Bearer ' + authenticationToken);
    }

    let body = event?.formData || event?.moveEntityData || {};

    if (event?.params?.bodyParams) {

      event?.params?.bodyParams?.forEach((data: any) => {
        if (data?.value) {
          body[data.key] = data.value;
        }
        else if (data.key && !data.value && event?.moveEntityData) {
          body[data.key] = event?.moveEntityData[data?.key];
        }
        else if (data.key && !data.value) {
          body[data?.key] = localStorage.getItem(data.key);
        }
        else {
          if (event?.data) {
            body[data.key] = event?.data[data?.key];
          } else {
            body[data.key] = localStorage.getItem(data.key);
          }
        }
      });
    }

    service.http.put(apiEndpoint, body, { headers: headers }).subscribe({
      next(response: any) {
        if (response) {
          const events = event?.events?.filter(
            (e: any) => e.
              event !== 'failure'
          );
          service?.processEvents(events, service, response?.data || response);
        }
      },
      error(error: any) {
        const events = event?.events.filter((e: any) => e.event !== 'success');
        service?.processEvents(events, service);
      },
    });
  }

  allStorage() {
    let archive: any = {}, // Notice change here
      keys = Object.keys(localStorage),
      i = keys.length;

    while (i--) {
      archive[keys[i]] = localStorage.getItem(keys[i]);
    }

    return archive;
  }

  processEvents(events: any, service?: any, data?: any) {

    const _events = events
      .filter((e: any) => {
        if (e?.enabled) {
          const evaluate = expressions.compile(e.enabled);

          if (evaluate({ data: data })) {
            console.log("TRUE")
            return true;
          }
          console.log("FALSE")
          return false;
        }
        return true;
      })
      .map((e: any) => {
        return Object.assign({}, e, { data: data });
      });


    _events?.forEach((e: any) => {
      switch (e.event) {
        case 'success':
          const successMethod = service.getMethod(e, service);
          successMethod?.(e, service);
          break;

        case 'failure':
          const failureMethod = service.getMethod(e, service);
          failureMethod?.(e, service);
          break;

        case 'click':
          const clickMethod = service.getMethod(e, service);
          clickMethod?.(e, service);
          break;

        case 'submit':
          const submitMethod = service.getMethod(e, service);
          submitMethod?.(e, service);
          break;

        case 'getQuery':
          const getQueryMethod = service.getMethod(e, service);
          getQueryMethod?.(e, service);
          break;

        case 'pageLoad':
          const pageLoadMethod = service.getMethod(e, service);
          pageLoadMethod?.(e, service);
          break;

        case 'queryParams':
          const getQueryParamsMethod = service.getMethod(e, service);
          getQueryParamsMethod(e, service);
          break;
        case 'toggle':
          const autoMethod = service.getMethod(e, service);
          autoMethod(e, service);
          break;
      }
    });
  }

  getMethod(event: any, service: any) {

    switch (event?.method) {
      case 'showToast':
        return service.showToast;
        break;

      case 'showErrorToast':
        return service.showErrorToast;
        break;

      case 'refreshDataGrid':
        return service.refreshDataGrid;
        break;

      case 'refreshTreeData':
        return service.refreshTreeData;
        break;

      case 'openDialog':
        return service.openDialog;
        break;

      case 'moveEntity':
        return service.moveEntity;
        break;

      case 'closeDialog':
        return service.closeDialog;
        break;

      case 'closeMoveEntityDialog':
        return service.closeMoveEntityDialog;
        break;

      case 'postAPI':
        return service.postAPI;
        break;

      case 'putAPI':
        return service.putAPI;
        break;

      case 'fetchAPI':
        return service.fetchAPI;

      case 'setLocalStorage':
        return service.setLocalStorage;
        break;

      case 'getLocalStorage':
        return service.getLocalStorage;
        break;

      case 'reloadPage':
        return service.reloadPage;
        break;

      case 'redirectUrl':
        return service.redirectUrl;
        break;

      case 'updateStore':
        return service.updateStoreObject;
        break;

      case 'goToPage':
        return service.goToPage;
        break;

      case 'getQuery':
        return service.getQuery;
        break;

      case 'saveQueryParams':
        return service.saveQueryParams;
        break;

      default:
        return null;
        break;
    }
  }

  setLocalStorage(event: any, service?: any) {
    event?.params?.storeParams?.map((x: any) => {
      if (event?.data && event?.data[x?.key]) {
        localStorage.setItem(x?.key, event?.data[x?.key]);
      }
    });
  }
  getLocalStorage(event: any, service?: any) {
    let localStorageData: any = {} || event?.data;

    event?.params?.storeParams?.map((x: any) => {
      if (x?.key) {
        localStorageData[x?.key] = localStorage.getItem(x?.key);
      }
    });

    let dataValue = {
      ...localStorageData,
      ...event.data,
    };

    // service.appendDataToEvent(event, service, dataValue)
    if (event?.events) {
      // service.processEvents(event?.events, service)
    }
  }

  redirectUrl(event: any, service?: any) {
    let urlParams = '';
    if (event?.params?.urlParams) {
      urlParams = '?' + service.encodeQueryData(event?.params?.urlParams);
    }
    window.location.href = event.params.urlPath + urlParams;
  }

  goToPage(event: any, service?: any) {
    let urlParams = '';
    if (event?.params?.urlParams) {
      urlParams = '?' + service.encodeQueryData(event?.params?.urlParams);
    }
    service.router.navigateByUrl(event.params.urlPath + urlParams);
  }

  encodeQueryData(urlParams: any) {
    const ret: any = [];
    urlParams?.forEach((p: any) => {
      ret.push(encodeURIComponent(p[0]) + '=' + encodeURIComponent(p[1]));
    });
    return ret.join('&');
  }

  showToast(event: any, service?: any) {
    if (event?.params?.type) {
      switch (event?.params?.type) {
        case 'information':
          service.toast.information(event?.params?.message);
      }
    } else {
      service.toast.message(event?.params?.message);
    }
  }

  showErrorToast(event: any, service?: any) {
    service.toast.error(event?.params?.message);
  }

  reloadPage(event: any, service?: any) {
    console.log('Refresh page');
    window.location.reload();
  }

  refreshDataGrid(event: any, service?: any) {
    // service.store.dispatch({
    //   type: '[Data Grid] Refresh data grid',
    //   gridName: event?.params?.gridName,
    // });

    service.store.dispatch({
      type: '[Data Report] Refresh data report',
      gridName: event?.params?.gridName,
    });
  }

  refreshTreeData(event: any, service?: any) {
    //***TO-DO*** Need to Plan for Tree data refresh.
    const self = true; //Get From Event Data
    const url = self ? service.router.url : 'administration/accounts';
    service.router.navigateByUrl('/', { skipLocationChange: true }).then(() => {
      service.router.navigate([`/${url}`]);
    });
  }

  updateStoreObject(event: any, service?: any) {
    console.log('Update store object');
  }

  getStoreObject(event: any, service?: any) {
    console.log('Get store object');
  }

  openDialog(event: any, service?: any) {
    service.store.dispatch({
      type: '[Dialog] Open dialog',
      dialog: event.params,
    });
  }

  moveEntity(event: any, service?: any) {
    service.store.dispatch({
      type: '[Dialog] Open move entity dialog',
      moveEntity: event.params,
    });
  }

  closeDialog(event: any, service?: any) {
    service.store.dispatch({
      type: '[Dialog] Close dialog',
      dialog: event.params,
    });
  }

  closeMoveEntityDialog(event: any, service?: any) {
    console.log("CLOSE ****")
    service.store.dispatch({
      type: '[Dialog] Close move entity dialog',
      dialog: event.params,
    });
  }

  returnProcessEvents(responseData: any, service: any) {
    service.store.dispatch({
      type: 'PROCESS_RESPONSE_DATA',
      responseData: responseData,
    });
  }

  getQuery(event: any, service?: any) {
    let localStorageData: any = {};
    service.route?.queryParams?.subscribe((params: any) => {
      event?.params?.queryParams.forEach((x: any) => {
        localStorageData[x.key] = params[x.key];
      });
    });
    service.processNestedEvents(event, service, localStorageData);
  }

  appendDataToEvent(event: any, service: any, data: any) {
    // const eventData = event?.events?.map((e: any) => {
    //   let eventValue = { ...e }

    return Object.assign({}, { ...event, data: data });
    // })

    // service?.processEvents(eventData, service);
  }

  saveQueryParams(event: any, service: any) {
    //Get query params
    //Save in local storage
    let queryParamsValue: any = {};
    service.route?.queryParams
      ?.subscribe((params: any) => {
        event?.params?.queryParams.forEach((x: any) => {
          queryParamsValue[x.key] = params[x.key];

          if (x?.store) {
            localStorage.setItem(x?.storeName || x?.key, params[x?.key]);
          }
        });
      })
      .unsubscribe();

    if (event?.events) {
      service?.processEvents(event?.events, service, service?.allStorage());
    }
  }
}
