/* eslint-disable no-prototype-builtins */
/* eslint-disable @typescript-eslint/no-empty-function */
import { Injectable } from '@angular/core';
import * as _ from 'lodash';
import * as expressions from 'angular-expressions';

@Injectable({
  providedIn: 'root',
})
export class TemplateService {
  data: any = {};

  constructor() {
    this.setData({ localStorage: this.getLocalStorage() });
  }

  apply(template: string, data: any, isSingleObject: boolean = false): string {
    const compiled = _.template(template, {
      interpolate: /{{([\s\S]+?)}}/g,
      variable: 'data',
    });

    if (isSingleObject) {
      const evaluate = expressions.compile(template);
      // console.log(template, evaluate({ data: data }));
      return evaluate({ data: data }) || template;
    }

    if (this.isTemplateString(template)) {
      try {
        const _compiled = compiled(data);
        if (_compiled !== '') {
          return _compiled;
        } else {
          return template;
        }
      } catch (e) {
        return template;
      }
    }

    return template;
  }

  processTemplate(json: any, data?: any): any {
    data = data || this.data;

    this.setData({ localStorage: this.getLocalStorage() });

    if (typeof json === 'string') {
      const stringStartsWith = json.trim().search(/^({%+)/g);
      const stringList = json.trim().match(/({%+)/g);
      const isSingleObject = stringList?.length === 1 && stringStartsWith === 0;

      if (isSingleObject) {
        const templateRegx = /{%(.*?)%}/gm;
        const match: RegExpExecArray | null = templateRegx.exec(json);
        if (match) {
          json = match[1];
        }
      }

      return this.apply(json, data, isSingleObject);
    } else if (typeof json === 'object' && json !== null) {
      if (Array.isArray(json)) {
        // console.log("Array Process")
        return json.map((item) => this.processTemplate(item, data));
      } else {
        const processedObj: any = {};
        for (const key in json) {
          if (json.hasOwnProperty(key)) {
            processedObj[key] = this.processTemplate(json[key], data);
          }
        }
        return processedObj;
      }
    } else {
      return json;
    }
  }

  isTemplateString(template: string): boolean {
    // Check if the template contains the interpolation pattern {{ ... }}
    return template.includes('{{') && template.includes('}}');
  }

  getData(key?: string): any {
    return key ? this.data[key] : this.data;
  }

  setData(data: any): void {
    this.data = Object.assign({}, this.data, data);
  }

  removeData(key: string | object): void {
    if (typeof key === 'string' && this.data.hasOwnProperty(key)) {
      delete this.data[key];
    }
    if (Array.isArray(key)) {
      for (const prop of key) {
        if (typeof prop === 'string' && this.data.hasOwnProperty(prop)) {
          delete this.data[prop];
        }
      }
    }
  }

  clearData(): void {
    this.data = {};
  }

  getLocalStorage() {
    const archive: any = {},
      keys = Object.keys(localStorage);
    let i = keys.length;

    while (i--) {
      archive[keys[i]] = localStorage.getItem(keys[i]);
    }

    return archive;
  }
}
