/* eslint-disable @nrwl/nx/enforce-module-boundaries */
import { Injectable } from '@angular/core';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { map, tap } from 'rxjs/operators';
import { Store } from '@ngrx/store';
import { EventsWebService } from '../service/events-web.service';
import * as EventsStoreActions from './actions';
import { TemplateService } from '../service/template.service';

@Injectable()
export class EventsStoreEffects {
  constructor(
    private actions$: Actions,
    private eventsService: EventsWebService,
    private store: Store,
    private templateService: TemplateService
  ) {}

  processEvents$ = createEffect(() => {
    return this.actions$.pipe(
      ofType(EventsStoreActions.PROCESS_EVENTS),
      map((e) => {
        this.eventsService.processEvents(
          e.events,
          this.eventsService,
          this.eventsService?.allStorage()
        );
        return EventsStoreActions.ADD_EVENT({});
      })
    );
  });

  setData$ = createEffect(
    () => {
      return this.actions$.pipe(
        ofType(EventsStoreActions.SET_DATA),
        tap((action) => {
          this.templateService.setData(action.data);
        })
      );
    },
    { dispatch: false }
  );

  removeData$ = createEffect(
    () => {
      return this.actions$.pipe(
        ofType(EventsStoreActions.REMOVE_DATA),
        tap((action) => {
          this.templateService.removeData(action.keys);
        })
      );
    },
    { dispatch: false }
  );

  clearData$ = createEffect(
    () => {
      return this.actions$.pipe(
        ofType(EventsStoreActions.CLEAR_DATA),
        tap((action) => {
          this.templateService.clearData();
        })
      );
    },
    { dispatch: false }
  );
}
