import { FormFieldTypeConfig } from '../../../models/forms.model';
import {
  Component,
  ChangeDetectionStrategy,
  OnInit,
  OnDestroy,
} from '@angular/core';
import { FieldType } from '@ngx-formly/material/form-field';

import { TemplateService } from '@hash8io/events-web';
import { ReportGridSelectors } from '@hash8io/common-store-web/report-grid-store';
import { Store } from '@ngrx/store';
import { Subject, takeUntil } from 'rxjs';

@Component({
  selector: 'hash8-platform-dropdown-form',
  templateUrl: './report.type.html',
  styleUrls: ['./report.type.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
// eslint-disable-next-line @angular-eslint/component-class-suffix
export class ReportType
  extends FieldType<FormFieldTypeConfig<any>>
  implements OnInit, OnDestroy
{
  constructor(private templateService: TemplateService, private store: Store) {
    super();
  }

  isFirstTimeLoaded = true;
  masterData: any;
  finalValue: any[] = [];
  addedData!: any;
  removedData!: any;
  rowDataIds!: any;
  unassignedRowIdsCount!: any;
  assignedRowDataIdsCount!: any;
  private readonly destroySubscription$ = new Subject<void>();

  ADMIN_FLEET_REPORT_CONFIG = {
    name: 'administrationFleetDataGrid',
    apiUrl: `/api/mock/fleets`,
    // apiUrl: `/api/v1/users?organizationUUID={{data.orgFilterUUIDs}}`,
    // organizationFilterConfig: this.orgConfig,
  };

  ADMIN_FLEET_REPORT_CONFIG2 = {
    name: 'administrationFleetDataGrid2',
    apiUrl: `/api/mock/fleets`,
    // apiUrl: `/api/v1/users?organizationUUID={{data.orgFilterUUIDs}}`,
    // organizationFilterConfig: this.orgConfig,
  };
  // eslint-disable-next-line @angular-eslint/no-empty-lifecycle-method
  ngOnInit(): void {
    this.store
      .select(
        ReportGridSelectors.selectReportRowData(
          this.field?.props?.selectedEntitiesConfig?.reportConfig?.name
        )
      )
      .pipe(takeUntil(this.destroySubscription$))

      .subscribe((data: any) => {
        this.assignedRowDataIdsCount = data?.length || 0;

        if (data?.length > 0) {
          this.rowDataIds = data.map((x: any) => x.uuid); //Existing Data

          this.finalValue = [];

          const added = this.rowDataIds?.filter((x: any) => {
            return this.masterData?.indexOf(x) == -1;
          });
          added?.map((x: any) => {
            this.finalValue.push({ uuid: x, action: 'add' });
          });

          const removed = this.masterData?.filter((x: any) => {
            return this.rowDataIds?.indexOf(x) == -1;
          });

          removed?.map((x: any) => {
            this.finalValue.push({ uuid: x, action: 'remove' });
          });
        } else {
          //FOr Empty DATA
          this.finalValue = [];

          this.masterData?.map((x: any) => {
            this.finalValue = this.finalValue?.filter((y) => y.uuid !== x);
            this.finalValue.push({ uuid: x, action: 'remove' });
          });
        }

        this.formControl.setValue(this.finalValue, { onlySelf: true });
        if (this.model?.resources?.length > 0) this.formControl.markAsDirty();
        if (this.model?.resources?.length === 0)
          this.formControl.setErrors([`No ${this.props.resources} Updated`]);
      });

    this.store
      .select(
        ReportGridSelectors.selectReportRowData(
          this.field?.props?.selectionEntitiesConfig?.reportConfig?.name
        )
      )
      .pipe(takeUntil(this.destroySubscription$))
      .subscribe((data: any) => {
        this.unassignedRowIdsCount = data?.length || 0;
      });
  }

  override ngOnDestroy() {
    this.destroySubscription$.next();
    this.destroySubscription$.complete();
  }

  getMasterData(event: any) {
    if (event) {
      if (this.isFirstTimeLoaded) {
        this.isFirstTimeLoaded = false;
        this.masterData = event.map((d: any) => d.uuid);
      }
    }
  }

  removed(event: any) {}

  get type() {
    return this.props.type ?? 'text';
  }

  get mask() {
    return this.props['mask'] ?? '';
  }

  get prefix() {
    return this.props['prefix'] ?? '';
  }
  get separator() {
    return this.props['separator'] ?? '';
  }

  get suffix() {
    return this.props['suffix'] ?? ' mi';
  }
}
