/* eslint-disable @typescript-eslint/no-empty-function */
/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
/* eslint-disable @angular-eslint/component-class-suffix */

import { FormFieldTypeConfig } from '../../../models/forms.model';
import { Component, ChangeDetectionStrategy } from '@angular/core';
import { FieldType } from '@ngx-formly/material/form-field';
import { FlatTreeControl } from '@angular/cdk/tree';
import {
  MatTreeFlatDataSource,
  MatTreeFlattener,
} from '@angular/material/tree';
import { BehaviorSubject, isObservable } from 'rxjs';
import { TemplateService } from '@hash8io/events-web';
import { Store } from '@ngrx/store';

@Component({
  selector: 'hash8-platform-dropdown-form',
  templateUrl: './dropdown.type.html',
  styleUrls: ['./dropdown.type.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DropdownType extends FieldType<FormFieldTypeConfig<any>> {
  dropdownLabel = '';
  selectedValue = '';
  selectedLabel = '';
  loading = false;
  loading$ = new BehaviorSubject<boolean>(false);
  treeControl = new FlatTreeControl<any>(
    (node) => node.level,
    (node) => node.expandable
  );

  treeFlattener: any;
  dataSource: any;
  selectionMode: 'single' | 'multiple' = 'single';
  checklistSelection!: any;

  override defaultOptions = {
    props: {
      compareWith(o1: any, o2: any) {
        return o1 === o2;
      },
    },
  };
  getLevel = (node: any) => node.level;

  private _transformer = (node: any, level: number) => {
    const treeNode = {
      expandable: !!node.children && node.children.length > 0,
      label: node[this.props.labelProp],
      level: level,
      hasSelection: node?.hasSelection || false,
      isGroupName: node?.isGroupName || false,
      iconUrl: node?.iconUrl || null,
      uuid: node?.[this.props.valueProp] || null,
    };
    if (treeNode.uuid === this.props.defaultValue) {
      this.selectedLabel = treeNode.label;
      this.selectedValue = treeNode.uuid;
      this.formControl.setValue(treeNode.uuid);
      this.setData(treeNode);
      // this.getParentNode(treeNode);
    }
    return treeNode;
  };

  constructor(private templateService: TemplateService, private store: Store) {
    super();
  }

  ngOnInit(): void {
    this.selectedLabel = this.props.placeholder;
    this.treeFlattener = new MatTreeFlattener(
      this._transformer,
      (node) => node.level,
      (node) => node.expandable,
      (node) => node.children
    );
    this.dataSource = new MatTreeFlatDataSource(
      this.treeControl,
      this.treeFlattener
    );

    // this.dataSource.data = this.props.options;
    if (isObservable(this.props.options)) {
      this.loading = true;
      this.loading$.next(true);
      this.props.options.subscribe((d: any) => {
        this.loading = false;
        this.loading$.next(false);
        this.dataSource.data = d;
        this.treeControl.expand(this.treeControl.dataNodes[0]);
        // this.expand(this.dataSource.data, this.props.defaultValue);
      });
    } else {
      this.dataSource.data = this.isJsonString(this.props.options)
        ? JSON.parse(this.props.options)
        : this.props.options;
      this.treeControl.expand(this.treeControl.dataNodes[0]);
    }
  }

  hasChild = (_: number, node: any) => node.expandable;

  get type() {
    return this.props.type ?? 'text';
  }

  get mask() {
    return this.props['mask'] ?? '';
  }

  get prefix() {
    return this.props['prefix'] ?? '';
  }
  get separator() {
    return this.props['separator'] ?? '';
  }

  get suffix() {
    return this.props['suffix'] ?? ' mi';
  }

  _getAriaLabelledby() {
    if (this.props.attributes?.['aria-labelledby']) {
      return this.props.attributes['aria-labelledby'] as string;
    }

    return this.formField?._labelId;
  }

  _getAriaLabel() {
    return this.props.attributes?.['aria-label'] as string;
  }

  getSelectedValue(event: any) {
    this.dropdownLabel = event.label;
    this.selectedLabel = event.label;
    this.selectedValue = event.uuid;
    this.formControl.setValue(event.uuid);
    this.setData(event);

    //TEMPORARY
    if (this.props?.events) {
      this.store.dispatch({
        type: '[Events] Process events',
        events: this.templateService.processTemplate(
          this?.props?.events,
          this.templateService.getData()
        ),
      });
    }
  }

  setData(obj: any) {
    if (this.props?.dataKey) {
      this.templateService.setData({ [this.props?.dataKey]: obj });
      localStorage.setItem(this.props?.dataKey, obj.uuid);
    }
  }

  /* Checks all the parents when a leaf node is selected/unselected */
  checkAllParentsSelection(node: any): void {
    let parent: any = this.getParentNode(node);
    while (parent) {
      // this.checkRootNodeSelection(parent);
      parent = this.getParentNode(parent);
    }
    console.log(
      '%cdropdown.type.ts line:152 parent',
      'color: #007acc;',
      parent
    );
  }

  /** Check root node checked state and change it accordingly */
  checkRootNodeSelection(node: any): void {
    const nodeSelected = this.checklistSelection.isSelected(node);
    const descendants = this.treeControl.getDescendants(node);
    const descAllSelected = descendants.every((child) =>
      this.checklistSelection.isSelected(child)
    );
    if (nodeSelected && !descAllSelected) {
      this.checklistSelection.deselect(node);
    } else if (!nodeSelected && descAllSelected) {
      this.checklistSelection.select(node);
    }
  }

  /* Get the parent node of a node */
  getParentNode(node: any): any {
    const currentLevel = this.getLevel(node);
    if (currentLevel < 1) {
      return null;
    }

    const startIndex = this.treeControl.dataNodes.indexOf(node) - 1;
    for (let i = startIndex; i >= 0; i--) {
      const currentNode = this.treeControl.dataNodes[i];
      if (this.getLevel(currentNode) < currentLevel) {
        return currentNode;
      }
    }
    return null;
  }

  expand(data: any[], uniqueId: string): any {
    data.forEach((node) => {
      console.log('%cdropdown.type.ts line:147 node', 'color: #007acc;', node);
      if (
        node.children &&
        node.children.find(
          (c: any) => c?.[this.props.valueProp] === this.props.defaultValue
        )
      ) {
        this.treeControl.expand(node);
        this.expand(this.dataSource.data, node?.[this.props.valueProp]);
      } else if (node.children && node.children.find((c: any) => c.children)) {
        this.expand(node.children, uniqueId);
      }
    });
  }

  isJsonString(str: any) {
    try {
      JSON.parse(str);
    } catch (e) {
      return false;
    }
    return true;
  }
}
