/* eslint-disable @typescript-eslint/no-inferrable-types */
import { Component, Input, OnInit, Output } from '@angular/core';
import { FormGroup } from '@angular/forms';
import { Store } from '@ngrx/store';
import { Subject } from 'rxjs';
import { FormFields, FormOptions } from '../../models/forms.model';
import { FormsWebService } from '../../services/forms-web.service';

@Component({
  selector: 'hash8-platform-multi-step-form',
  templateUrl: './multi-step-form.component.html',
  styleUrls: ['./multi-step-form.component.scss'],
})
export class MultiStepFormComponent implements OnInit {
  @Input() form = new FormGroup({});
  @Input() model: any = {};
  @Input() name: any = 'formData';
  @Input() fields: FormFields[] = [];
  @Input() options: FormOptions = {};
  @Input() submitLabel: string = 'Registration';
  @Input() submitLabelColor: string = 'accent';
  @Input() cancelLabel: string = 'reset';

  @Output() modelChange: Subject<any> = new Subject<any>();
  @Output() formSubmit: Subject<any> = new Subject<any>();

  _fields: FormFields[] = [];
  hasReset: boolean = true;
  submitLabelClass: string | string[] = '';
  resetLabelClass: string | string[] = '';
  resetLabelColor: string = '';

  submitCloseDialog!: boolean;
  methodName!: string;
  _eventsWebService!: any;
  obj!: any;

  HttpRef = 'http';

  constructor(private formService: FormsWebService, private store: Store) {}

  ngOnInit(): void {
    console.log('this.optionssss', this.options);
    this.formService
      .processFormFields(this.fields)
      .pipe()
      .subscribe((fields) => {
        this._fields = fields;
        this.updateFormView();
      });
  }

  updateFormView() {
    if (this.options) {
      // console.log('Options:', this.options);
      this.submitLabel = this.options.submitLabel
        ? this.options.submitLabel
        : 'Submit';
      this.submitLabelClass = this.options.submitLabelClass
        ? this.options.submitLabelClass
        : '';
      this.submitLabelColor = this.options.submitLabelColor
        ? this.options.submitLabelColor
        : 'accent';

      if (this.options.submit) {
        const submitOptions = this.options.submit;
        this.submitLabel = submitOptions?.label
          ? submitOptions?.label
          : 'Submit';
        this.submitLabelClass = submitOptions?.labelClass
          ? submitOptions?.labelClass
          : '';
        this.submitLabelColor = submitOptions?.labelColor
          ? submitOptions?.labelColor
          : 'accent';
      }

      this.resetLabelClass = this.options.resetLabelClass
        ? this.options.resetLabelClass
        : '';

      if (this.options.cancel) {
        const cancelOptions = this.options.cancel;
        this.cancelLabel = cancelOptions?.label
          ? cancelOptions?.label
          : 'Reset';
        this.resetLabelClass = cancelOptions?.labelClass
          ? cancelOptions?.labelClass
          : '';
        this.resetLabelColor = cancelOptions?.labelColor
          ? cancelOptions?.labelColor
          : 'accent';
      }

      this.hasReset = this.options.hasReset == false ? false : true;
    }
  }
  onModelChange(event: any) {
    this.model = event;
    this.modelChange.next(event);
  }

  onSubmit() {
    console.log('Stepper form', this.model);
    this.formSubmit.next(this.model);

    if (this.options?.submit) {
      if (this.options?.submit?.events) {
        this.store.dispatch({
          type: '[Events] Process events',
          events: this.options?.submit?.events.map((e: any) => ({
            ...e,
            formData: this.model,
          })),
        });
      }
    }
  }
}
