/* eslint-disable prefer-const */
/* eslint-disable no-case-declarations */
/* eslint-disable @typescript-eslint/no-inferrable-types */
/* eslint-disable @typescript-eslint/no-empty-function */
/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
import {
  Component,
  Input,
  OnChanges,
  OnInit,
  Output,
  SimpleChanges,
} from '@angular/core';
import { FormGroup } from '@angular/forms';
import { Subject } from 'rxjs';
import { FormFields, FormOptions } from '../../models/forms.model';
import { FormsWebService } from '../../services/forms-web.service';
import { ToastService } from '@hash8io/core-web';
import { Store } from '@ngrx/store';
import { TemplateService } from '@hash8io/events-web';

@Component({
  selector: 'hash8-platform-single-step-form',
  templateUrl: './single-step-form.component.html',
  styleUrls: ['./single-step-form.component.scss'],
})
export class SingleStepFormComponent implements OnInit, OnChanges {
  @Input() form = new FormGroup({});
  @Input() model: any = {};
  @Input() name: any = 'formData';
  @Input() fields: FormFields[] = [];
  @Input() options: FormOptions = {};
  @Input() submitLabel: string = 'Submit';
  @Input() submitLabelColor: string = 'accent';
  @Input() cancelLabel: string = 'Reset';
  @Input() closeDialog!: boolean;
  @Input() selectedRowData!: any;
  @Input() editState: boolean = true;
  @Input() resetForm: boolean = false;

  @Output() modelChange: Subject<any> = new Subject<any>();
  @Output() formSubmit: Subject<any> = new Subject<any>();

  _fields: FormFields[] = [];
  hasReset: boolean = true;
  submitLabelClass: string | string[] = '';
  resetLabelClass: string | string[] = '';
  resetLabelColor: string = '';

  submitCloseDialog!: boolean;
  methodName!: string;
  _eventsWebService!: any;
  obj!: any;

  actionClass!: any;
  HttpRef = 'http';

  submitDisabled!: any;
  submitExpression!: any;
  constructor(
    private formService: FormsWebService,
    private toast: ToastService,
    private store: Store,
    private templateService: TemplateService
  ) {}

  ngOnInit(): void {
    this.updateFormView();
  }

  ngOnChanges(changes: SimpleChanges) {
    this.fields = this.templateService.processTemplate(this.fields);

    this.formService
      .processFormFields(this.fields)
      .pipe()
      .subscribe((fields) => {
        this._fields = fields;
        this.updateFormView();
      });
  }

  updateFormView() {
    this.options = this.templateService.processTemplate(this.options);

    if (this.options) {
      this.actionClass = this.options?.actionClass;
      // console.log('Options:', this.options);
      this.submitLabel = this.options.submitLabel
        ? this.options.submitLabel
        : 'Submit';
      this.submitLabelClass = this.options.submitLabelClass
        ? this.options.submitLabelClass
        : '';
      this.submitLabelColor = this.options.submitLabelColor
        ? this.options.submitLabelColor
        : 'accent';

      if (this.options.submit) {
        const submitOptions = this.options.submit;
        this.submitLabel = submitOptions?.label
          ? submitOptions?.label
          : 'Submit';
        this.submitLabelClass = submitOptions?.labelClass
          ? submitOptions?.labelClass
          : '';
        this.submitLabelColor = submitOptions?.labelColor
          ? submitOptions?.labelColor
          : 'accent';

        this.submitExpression = submitOptions?.expression
          ? submitOptions?.expression
          : '';

        //Validation even when no Model Data
        if (this.submitExpression && !this.model) {
          this.submitDisabled = this.templateService.processTemplate(
            this.submitExpression
          );
        }
      }

      this.resetLabelClass = this.options.resetLabelClass
        ? this.options.resetLabelClass
        : '';

      if (this.options.cancel) {
        const cancelOptions = this.options.cancel;
        this.cancelLabel = cancelOptions?.label
          ? cancelOptions?.label
          : 'Reset';
        this.resetLabelClass = cancelOptions?.labelClass
          ? cancelOptions?.labelClass
          : '';
        this.resetLabelColor = cancelOptions?.labelColor
          ? cancelOptions?.labelColor
          : 'accent';
      }

      this.hasReset = this.options.hasReset == false ? false : true;
      this.resetForm = this.options?.hasReset == false ? false : true;
      // this.editState = this.options?.editState == false ? false : true;
    }
  }

  onModelChange(event: any) {
    this.model = event;
    // console.log(
    //   '%csingle-step-form.component.ts line:131 this.model',
    //   'color: #007acc;',
    //   this.model
    // );

    this.templateService.setData({ [this.name]: this.model });
    if (this.submitExpression) {
      this.submitDisabled = this.templateService.processTemplate(
        this.submitExpression
      );
    }
    this.modelChange.next(event);
  }

  onSubmit() {
    this.templateService.setData({ [this.name]: this.model });

    // console.log(this.model);
    // return false;

    let formData: any = {};

    // TODO: Need to verify where it used and where it cause error
    if (this.model?.users?.length === 1) {
      formData = this.model.users[0];
    } else {
      formData = this.model;
    }

    this.formSubmit.next(this.model);
    if (this.options?.submit) {
      if (this.options?.submit?.events) {
        this.store.dispatch({
          type: '[Events] Process events',
          events: this.options?.submit?.events.map((e: any) => ({
            ...e,
            formData: formData,
          })),
        });
      }
    }
  }

  cancelAction() {
    if (this.options?.cancel) {
      if (this.options?.cancel?.events) {
        this.store.dispatch({
          type: '[Events] Process events',
          events: this.options?.cancel?.events,
        });
      }
    }
    if (this.resetForm) {
      this.form.reset();
    }
  }
}
