import { Component, Input, OnInit, Output, SimpleChanges } from '@angular/core';
import { FormArray, FormGroup } from '@angular/forms';
import { FormFields, FormOptions } from '../../models/forms.model';
import { Store } from '@ngrx/store';
import { FormlyFormOptions } from '@ngx-formly/core';
import { Subject } from 'rxjs';
import { FormsWebService } from '../../services/forms-web.service';

@Component({
  selector: 'hash8-platform-stepper-form',
  templateUrl: './stepper-form.component.html',
  styleUrls: ['./stepper-form.component.scss'],
})
export class StepperFormComponent implements OnInit {
  @Input() options!: FormOptions;
  @Input() fields: FormFields[] = [];

  @Input() steps!: any;

  submitLabel!: any;
  @Input() submitLabelColor!: string;
  submitLabelClass: string | string[] = '';

  @Output() formSubmit: Subject<any> = new Subject<any>();
  @Output() modelChange: Subject<any> = new Subject<any>();

  optionsForm!: any;

  HttpRef = 'http';

  activedStep = 0;

  model = {};

  form: any;

  _fields: FormFields[] = [];

  ngOnInit(): void {
    this.form = new FormArray(this.steps?.map(() => new FormGroup({})));

    this.updateFormView();
  }

  ngOnChanges(changes: SimpleChanges) {
    this.steps.map((x: any) => {
      this.formService
        .processFormFields(x?.fields)
        .pipe()
        .subscribe((fields) => {
          this._fields = fields;
          this.updateFormView();
        });
    });
  }
  constructor(private store: Store, private formService: FormsWebService) {}

  updateFormView() {
    if (this.options) {
      this.submitLabel = this.options?.submitLabel
        ? this.options.submitLabel
        : 'submit';
      this.submitLabelClass = this.options?.submitLabelClass
        ? this.options.submitLabelClass
        : '';
      this.submitLabelColor = this.options.submitLabelColor
        ? this.options.submitLabelColor
        : 'accent';
    }
  }

  onModelChange(event: any) {
    this.model = event;

    this.modelChange.next(event);
  }

  prevStep(step: any) {
    this.activedStep = step - 1;
  }

  nextStep(step: any) {
    if (this.options?.next) {
      if (this.options?.next?.events) {
        this.store.dispatch({
          type: '[Events] Process events',
          events: this.options?.next?.events.map((e: any) => ({
            ...e,
            data: this.model,
          })),
        });
      }
      this.activedStep = step + 1;
    }
  }

  submit() {
    console.log('This.model', this.model);
    this.formSubmit.next(this.model);
    if (this.options?.submit) {
      if (this.options?.submit?.events) {
        this.store.dispatch({
          type: '[Events] Process events',
          events: this.options?.submit?.events.map((e: any) => ({
            ...e,
            formData: this.model,
          })),
        });
      }
    }
  }
}
