/* eslint-disable @typescript-eslint/no-empty-interface */
import { Type } from '@angular/core';
import { FormControl } from '@angular/forms';
import { Model } from '@hash8io/core-web';
import {
  FieldType,
  FormlyFieldConfig,
  FormlyFormOptions,
} from '@ngx-formly/core';
import { FormlyAttributeEvent } from '@ngx-formly/core/lib/models';
import { Observable } from 'rxjs';

export interface FormModel {
  type?: 'simple' | 'steps';
  title?: Model.Title | undefined;
  model?: any;
  fields: FormFields[];
  options?: FormOptions;
}

export type MakeOptional<Type, Key extends keyof Type> = Omit<Type, Key> &
  Partial<Pick<Type, Key>>;

export interface FormFields extends FormlyFieldConfig {
  type?:
    | '#text'
    | '#number'
    | '#phoneNumber'
    | '#template'
    | '#checkbox'
    | '#decimal'
    | '#password'
    | '#color'
    | '#date'
    | '#email'
    | '#input'
    | '#ip-address'
    | '#label'
    | '#checkbox-multiple'
    | '#select-multiple'
    | '#select'
    | '#odometer'
    | '#dropdown'
    | '#select'
    | '#radio'
    | '#repeat'
    | '#repeat-grid'
    | '#repeat-vertical'
    | '#select-async'
    | '#textbox-async'
    | '#slider'
    | '#textarea'
    | '#toggle'
    | '#group'
    | 'stepper'
    | 'datepicker'
    | 'array'
    | 'addEntity'
    | 'select'
    | Type<FieldType>;
  fieldGroup?: FormFields[];
  fieldArray?: FormFields;
  props?: FieldProps;
  extras?: {
    labelOverInput?: boolean;
    hintBelowInput?: boolean;
  };
}

export interface FormOptions extends FormlyFormOptions {
  submit?: {
    label?: string;
    events?: any;
    labelClass?: string;
    labelColor?: string;
    expression?: any;
  };
  cancel?: {
    label?: string;
    events?: any;
    labelClass?: string;
    labelColor?: string;
  };

  next?: {
    label?: string;
    events?: any;
    labelClass?: string;
    labelColor?: string;
  };

  hasReset?: boolean;

  // later remove this
  submitLabel?: string;
  submitEvent?: any;
  events?: any;
  submitLabelClass?: string | string[];
  submitLabelColor?: string;
  resetLabelClass?: string | string[];
  actionClass?: any;
  dirtyCheck?: any;
}

export interface FormFieldTypeConfig<T = FormFields['props']>
  extends FormlyFieldConfig<T> {
  formControl: FormControl;
  props: NonNullable<T>;
  extras?: {
    labelOverInput?: boolean;
    hintBelowInput?: boolean;
  };
}

export interface FormAttributes {
  title: Model.Title;
  formType: 'simple' | 'steps' | 'tab';
  form: any;
  model: any;
  fields: FormFields[];
  options: FormOptions;
  submitText?: string;
  showReset?: boolean;
  showTitle?: boolean;
}

export interface FieldProps
  extends FieldMask,
    FieldTextarea,
    FieldCheckbox,
    FieldMultiCheckbox,
    FieldDate,
    FieldNumber,
    FieldRadio,
    FieldSelection,
    FieldSlider,
    FieldAsync {
  label?: string;
  type?: string;
  align?: 'column' | 'row';
  options?: any[] | Observable<any[]>;
  appearance?: 'standard' | 'legacy' | 'fill' | 'outline';
  readonly?: boolean;
  disabled?: boolean;
  placeholder?: string;
  description?: string;
  required?: boolean;
  hideLabel?: boolean;
  minLength?: number;
  maxLength?: number;
  hideRequiredMarker?: boolean;
  color?: string;
  floatLabel?: any;
  [additionalProperties: string]: any;
  pattern?: string | RegExp;
  tabindex?: number;
  attributes?: {
    [key: string]: string | number;
  };
  class?: any;
  step?: number;
  focus?: FormlyAttributeEvent;
  blur?: FormlyAttributeEvent;
  keyup?: FormlyAttributeEvent;
  keydown?: FormlyAttributeEvent;
  click?: FormlyAttributeEvent;
  change?: FormlyAttributeEvent;
  keypress?: FormlyAttributeEvent;
}

// Mask
export interface FieldMask {
  prefix?: string;
  suffix?: string;
  separator?: string;
  mask?: string;
}

// Number
export interface FieldNumber {
  min?: number;
  max?: number;
}

// Textarea
export interface FieldTextarea {
  autosize?: boolean;
  autosizeMaxRows?: number;
  autosizeMinRows?: number;
  cols?: 4;
  rows?: 4;
}

// Checkbox
export interface FieldCheckbox {
  labelPosition?: 'after' | 'before';
}

export interface FieldMultiCheckbox {
  options?: any[] | Observable<any[]>;
}

// Select
export interface FieldSelection {
  multiple?: boolean;
  options?: any[] | Observable<any[]>;
  selectAllOption?: string;
  valueProp?: string;
  labelProp?: string;
}

// Radio
export interface FieldRadio {
  thumbLabel?: boolean;
}

// Date
export interface FieldDate {
  datepickerOptions?: any;
}

// Slider
export interface FieldSlider {
  vertical?: boolean;
  invert?: boolean;
  tickInterval?: number;
}

// Async
export interface FieldAsync {
  async?: boolean;
  apiEndpoint?: string;
  apiKey?: string;
  successMessage?: string;
  errorMessage?: string;
  dependentField?: string;
  dependentFieldKey?: string;
}
