import { HttpClient, HttpHeaders, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { AbstractControl } from '@angular/forms';
import { FormlyHookConfig } from '@ngx-formly/core/lib/models';
import {
  distinctUntilChanged,
  map,
  Observable,
  of,
  startWith,
  switchMap,
} from 'rxjs';
import { FormFields } from '../models/forms.model';
import { TemplateService } from '@hash8io/events-web';
import { Store } from '@ngrx/store';

@Injectable({
  providedIn: 'root',
})
export class FormsWebService {
  constructor(
    private http: HttpClient,
    private templateService: TemplateService,
    private store: Store
  ) {}
  authenticationToken = localStorage.getItem('access_token');

  headers = new HttpHeaders({
    'Content-Type': 'application/json',
    Authorization: 'Bearer ' + this.authenticationToken,
  });

  processFormFields(fields: FormFields[]): Observable<any> {
    return of(this.populateFormFields(fields));
  }

  getAPIData(
    apiEndpoint: string,
    filterKey?: string,
    filterId?: string | number
  ): Observable<any[]> {
    if (filterId) {
      if (filterKey) apiEndpoint = `${apiEndpoint}/${filterId}/${filterKey}`;
      else apiEndpoint = `${apiEndpoint}=${filterId}`;
    }
    return this.http.get(apiEndpoint, { headers: this.headers }).pipe(
      map((response: any) => {
        // console.log("Response", response)
        //   // TODO: Temporary filtering, need to send in API itself
        // if (filterId) {

        //   const data = response.filter(
        //     (d: any) => d[filterKey || ''] === filterId
        //   );
        //   // if (!data.find((option) => sportId === option.id)) {
        //   //   field.formControl?.setValue(null);
        //   // }
        //   return data;
        // }

        if (response.data) {
          return response.data;
        } else {
          return response;
        }
      })
    );
  }

  getAPIDataFromEvent(props: any, events?: any): Observable<any[]> {
    const queryParams: any = {};
    // console.log(
    //   '%cforms-web.service.ts line:59 events',
    //   'color: #007acc;',
    //   events
    // );

    // if (events.bodyParams) {
    //   events.bodyParams.map((x: any) => {
    //     if (x?.key && x?.value) {
    //       console.log(this.templateService.getData());
    //       console.log(x?.value, this.templateService.processTemplate(x?.value));
    //       queryParams[x?.key] = this.templateService.processTemplate(x?.value);
    //     }
    //   });
    // }

    // events?.map((event: any) => {
    //   event?.params?.storeParams?.map((x: any) => {
    //     if (x?.key) {
    //       queryParams[x?.key] = localStorage.getItem(x?.key);
    //     }
    //   });
    // });

    const options = {
      params: queryParams,
      headers: this.headers,
    };

    return this.http.get(props['apiEndpoint'], options).pipe(
      map((response: any) => {
        if (response.data) {
          return response.data;
        } else {
          return response;
        }
      })
    );
  }

  postAPIData(apiEndpoint: string, params: any): Observable<any[]> {
    return this.http.post(apiEndpoint, params, { headers: this.headers }).pipe(
      map((response: any) => {
        //   // TODO: Temporary filtering, need to send in API itself
        // if (filterId) {

        //   const data = response.filter(
        //     (d: any) => d[filterKey || ''] === filterId
        //   );
        //   // if (!data.find((option) => sportId === option.id)) {
        //   //   field.formControl?.setValue(null);
        //   // }
        //   return data;
        // }
        return response;
      })
    );
  }
  checkValueExist(
    apiEndpoint: string,
    value: any | string,
    key: string
  ): Observable<boolean> {
    return this.http.get(apiEndpoint, { headers: this.headers }).pipe(
      map((response: any) => {
        const options = response.filter((data: any) => data[key] === value);
        return options.length === 0;
      })
    );
  }

  checkExist(
    apiEndpoint: string,
    apiKey: any,
    value: any | string,
    params: any
  ): Observable<boolean> {
    const queryParams: any = {};

    params?.bodyParams?.map((x: any) => {
      if (x?.value) {
        queryParams[x.key] = x.value;
      } else {
        queryParams[x?.key] = localStorage.getItem(x.dataKey || x.key);
      }
    });

    if (apiKey) {
      // queryParams.append(new HttpParams().append(apiKey, value.trim()));
      queryParams[apiKey] = value.trim();
    }

    const options = {
      params: queryParams,
      headers: this.headers,
    };
    return this.http.get(apiEndpoint, options).pipe(
      map((response: any) => {
        // const options = response.filter((data: any) => data[key] === value);
        return !response.status;
      })
    );
  }

  populateFormFields(fields: FormFields[]) {
    const newFields = fields.map((field) => {
      // if (field?.props)
      //   field.props.change = (field, event) => console.log(event);

      switch (field.type) {
        case '#textbox-async':
          field.asyncValidators = this.addAsyncValidators(field);
          if (field?.props?.dependentField) {
            field.hooks = this.addAsyncOptions(field, true, true);
          }
          return field;
          break;

        case 'select':
        case '#select-async':
          if (field.props?.async) {
            field.hooks = this.addAsyncOptions(field);
          }
          if (field.props?.dependentField) {
            field.hooks = this.addAsyncOptions(field, true);
          }
          // if (field?.props)
          //   field.props.change = (field, event) => console.log(event);

          return field;
          break;

        case '#checkbox-multiple':
          if (field.props?.async && field.props?.dependentField === undefined) {
            field.hooks = this.addAsyncOptions(field);
          }

          if (field.props?.async && field.props?.dependentField) {
            // field.hooks = this.addDependentOptions(field);
            field.hooks = this.addAsyncOptions(field, true);
          }

          // if (field?.props)
          //   field.props.change = (field, event) => console.log(event);

          return field;
          break;

        case '#dropdown':
          if (field.props?.async) {
            field.hooks = this.addAsyncOptions(field);
          }
          return field;
          break;

        case '#repeat':
        case '#repeat-vertical':
        case '#repeat-grid':
          if (field.fieldArray) {
            this.populateFormFields(field.fieldArray['fieldGroup'] || []);
          }
          return field;
          break;

        case 'stepper':
          if (field.fieldGroup) {
            field.fieldGroup.map((group) => {
              this.populateFormFields(group.fieldGroup || []);
            });
          }
          return field;
          break;

        default:
          return field;
      }
    });
    return newFields;
  }

  addAsyncOptions(
    field: FormFields,
    filter: boolean = false,
    uniqueValueCheck?: boolean
  ): FormlyHookConfig {
    const hook = {
      onInit: (field: FormFields) => {
        // const formControl = field.formControl;
        if (field && field?.props && !filter) {
          const props = field.props;

          if (field.props['params']) {
            field.props.options = this.getAPIDataFromEvent(
              props,
              field.props['params']
            );
          } else {
            field.props.options = this.getAPIData(props['apiEndpoint'] || '');
          }
        }

        if (field && field?.props && filter) {
          const props = field.props;
          const parentField = field.props.dependentField || '';
          const parentFieldKey = field.props.dependentFieldKey || '';
          const parentFormControl = field.parent?.formControl?.get(parentField);

          parentFormControl?.valueChanges
            .pipe(
              startWith(parentFormControl.value),
              distinctUntilChanged(),
              map((filterId) => {
                if (field && field?.props && filterId) {
                  if (filterId) {
                    field.props.options = this.getAPIData(
                      props['apiEndpoint'] || '',
                      parentFieldKey,
                      filterId
                    );
                    if (uniqueValueCheck) {
                      setTimeout(() => {
                        field.formControl?.updateValueAndValidity({
                          onlySelf: true,
                        });
                      }, 1000);
                    }
                  } else {
                    field.props.options = [];
                  }
                }
              })
            )
            .subscribe();
        }
        // const sportControl = field.parent?.formControl?.get('sport');
        // if (field && field?.props) {
        //   sportControl?.valueChanges
        //     .pipe(
        //       startWith(sportControl.value),
        //       distinctUntilChanged(),
        //       map((sportId) => {
        //         console.log(sportId);
        //         if (field && field?.props) {
        //           field.props.options = this.getTodos(sportId);
        //         }
        //       })
        //     )
        //     .subscribe();
        // }
      },
    };
    return hook as FormlyHookConfig;
  }

  addAsyncValidators(field: FormFields) {
    const asyncValidators = {
      uniqueUsername: {
        expression: (control: AbstractControl, field: FormFields) => {
          if (field.props) {
            const props = field.props;
            if (props['apiEndpoint']) {
              if (props['valueCheck'] == true)
                return this.checkExist(
                  props['apiEndpoint'],
                  props['apiKey'],
                  control?.value,
                  props['params']
                );
              else
                return this.checkValueExist(
                  props['apiEndpoint'],
                  control?.value,
                  props['apiKey'] || ''
                );
            } else {
              return of(0);
            }
          } else {
            return of(0);
          }
        },
        message: (error: any, field: FormFields) => {
          if (field?.props) {
            return `"${field?.formControl?.value}" ${
              field?.props['errorMessage'] || 'already exist'
            }`;
          }
          return `"${field?.formControl?.value}" already exist`;
        },
      },
    };
    return asyncValidators;
  }
}
