import { Component, OnInit } from '@angular/core';
import { ElementRef, Input, ViewChild } from '@angular/core';
import { OrgChart } from 'd3-org-chart';
import { LEVELS, ORGANIZATION_MOCK } from './mock-data/organization-mock';
import { HierarchicalTreeService } from './service/hierarchical-tree.service';
import * as d3 from 'd3';

@Component({
  selector: 'hash8-platform-hierarchical-tree',
  templateUrl: './hierarchical-tree.component.html',
  styleUrls: ['./hierarchical-tree.component.scss'],
})
export class HierarchicalTreeComponent implements OnInit {
  @ViewChild('chartContainer') chartContainer!: any;
  @ViewChild('treeLevel') treeLevel!: any;

  @Input() treeEvents!: any;
  data!: any;
  chart: any;
  levels!: any;

  width!: any;
  configuration!: any;
  loading!: boolean;

  // configuration = {
  //   options: {
  //     showBreadcrumb: false,
  //     showSearch: false,
  //     showSettings: true,
  //     showActions: true,
  //   },

  //   settings: [{
  //     label: 'Export Image',
  //     iconUrl: '',
  //     urlPath: '',
  //     isActive: false
  //   },
  //   {
  //     label: 'Export SVG',
  //     iconUrl: '',
  //     urlPath: '',
  //     isActive: false
  //   }],
  //   count: {
  //     label: 'Accounts',
  //     value: 8
  //   },
  //   actions: [
  //     {
  //       label: "Create Account",
  //       iconUrl: "mail",
  //       urlPath: "",
  //       isPrimaryAction: true,
  //       events: [{}]
  //     }
  //   ],
  //   events: [{
  //     event: "GET",
  //     method: "fetchAPI",
  //     apiUrl: '',//Tree view data api
  //     params: {
  //       getParams: [{
  //         key: '',//Tree view data api params
  //       },
  //       ]
  //     }
  //   }],

  // }

  constructor(public hierarchicalTreeService: HierarchicalTreeService) {
    // this.data = ORG_MOCK;
    // this.levelConfig = ORGANIZATION_MOCK
  }

  ngAfterViewInit() {
    this.chart = new OrgChart();

    // this.updateChart(this.data);
  }

  ngOnInit(): void {
    this.fetchTreeConfiguration();
  }

  fetchTreeConfiguration() {
    this.loading = true;

    let params: any = {};

    this.treeEvents?.map((e: any) => {
      if (e?.apiUrl) {
        e?.params?.getParams?.forEach((data: any) => {
          if (data?.value) {
            params[data.key] = data.value;
          } else {
            params[data.key] = localStorage.getItem(data.key);
          }
        });
        this.hierarchicalTreeService
          .fetchTreeData(e?.apiUrl, params)
          .subscribe((response: any) => {
            this.configuration = response;
            this.fetchTreeData(response?.events);
          });
      }
    });
  }

  fetchTreeData(events: any) {
    let params: any = {};

    events?.map((e: any) => {
      if (e?.apiUrl) {
        e?.params?.getParams?.forEach((data: any) => {
          if (data?.value) {
            params[data.key] = data.value;
          } else {
            params[data.key] = localStorage.getItem(data.key);
          }
        });
        this.hierarchicalTreeService
          .fetchTreeData(e?.apiUrl, params)
          .subscribe((response: any) => {
            this.loading = false;

            // console.log('Response', response?.data?.data);
            this.data = response?.data?.data;
            this.configuration.count['value'] = response?.data?.data.length - 1; //-1 added because Organization Name added Temporarily as account for Tree View
            this.updateChart(this.data);
            this.levels = LEVELS;
          });
      }
    });
  }

  ngOnChanges() {
    // this.updateChart(this.data);
  }

  updateChart(data: any) {
    setTimeout(() => {
      this.chart
        .container(this.chartContainer.nativeElement)
        .data(data)
        .nodeWidth((d: any) => 130)
        .initialZoom(0.9)
        .zoomBehavior(null)
        .svgWidth((d: any) => {
          // console.log('width', this.width);
        })
        .svgHeight((d: any) => {
          return this.treeLevel.nativeElement.offsetHeight;
        })
        // .buttonContent((node: any, state: any) => {
        //   return `<div style="color:#2CAAE5;border-radius:5px;padding:3px;font-size:10px;margin:auto auto;background-color:#040910;border: 1px solid #2CAAE5"> <i class="material-icons">expand_more</i></div>`
        // })
        // .linkUpdate(function (d: any, i: any, arr: any) {
        //   console.log("D value", d)
        //   console.log("I value", i)
        //   console.log("Arr value", arr)
        //   let childrenLength = d?.parent?.children.length
        //   if (childrenLength === 4) {
        //     d3.select(".link")
        //       .attr("stroke", "#ff0000")
        //   } else {
        //     d3.select(".link")
        //       .attr("stroke", "#0000ff")
        //   }
        // })
        // .zoomBehavior().selection.on("zoom", '')
        .nodeHeight((d: any) => {
          if (d?.data?.organizationName) {
            return 44;
          } else {
            return 84;
          }
        })
        // .zoomBehavior(selection.call(d3.zoom().on("zoom", 'null')))

        .nodeWidth((d: any) => {
          return 150;
        })
        .childrenMargin((d: any) => 40)
        .compactMarginBetween((d: any) => 15)
        .compactMarginPair((d: any) => 80)
        .compact(false)
        .setActiveNodeCentered(false)
        .siblingsMargin((d: any) => 30)
        .neightbourMargin((n1: any, n2: any) => 20)
        .nodeContent(function (d: any, i: any, arr: any, state: any) {
          if (d?.data?.organizationName) {
            return ` <div class="bg-white w-34 h-31" style="border: 1px solid #D3D3D3" >
            <div class="text-center font-bold p-4" >
              ${d?.data?.organizationName}
          </div>
            </div>
              `;
          } else if (d?.data?.accountName === 'Unassigned') {
            return `<div class="bg-white w-34 h-22"  style="border: 1px solid #D3D3D3">
            <div class="flex flex-col space-y-3">
              <div>
                <div class="text-center font-bold flex-1 px-2 pt-2">
                  ${d?.data?.accountName}
                </div>

                <div class="flex justify-center space-x-3 p-2">
                  <div class="flex flex-col justify-center items-center space-y-1">
                    <div>
                    <svg
                    xmlns="http://www.w3.org/2000/svg"
                    class="icon icon-tabler icon-tabler-user"
                    width="22"
                    height="22"
                    viewBox="0 0 24 24"
                    stroke-width="1"
                    stroke="currentColor"
                    fill="none"
                    stroke-linecap="round"
                    stroke-linejoin="round"
                  >
                    <path stroke="none" d="M0 0h24v24H0z" fill="none"></path>
                    <path d="M8 7a4 4 0 1 0 8 0a4 4 0 0 0 -8 0"></path>
                    <path d="M6 21v-2a4 4 0 0 1 4 -4h4a4 4 0 0 1 4 4v2"></path>
                  </svg>
                    </div>

                    <div>${
                      d.data.totalNumberOfUsers ? d.data.totalNumberOfUsers : 0
                    }</div>
                  </div>

                  <!--Asset Count-->
                  <div class="flex flex-col justify-center items-center space-y-1">
                    <div>
                    <svg
                    xmlns="http://www.w3.org/2000/svg"
                    class="icon icon-tabler icon-tabler-box"
                    width="22"
                    height="22"
                    viewBox="0 0 24 24"
                    stroke-width="1"
                    stroke="currentColor"
                    fill="none"
                    stroke-linecap="round"
                    stroke-linejoin="round"
                  >
                    <path stroke="none" d="M0 0h24v24H0z" fill="none"></path>
                    <path d="M12 3l8 4.5l0 9l-8 4.5l-8 -4.5l0 -9l8 -4.5"></path>
                    <path d="M12 12l8 -4.5"></path>
                    <path d="M12 12l0 9"></path>
                    <path d="M12 12l-8 -4.5"></path>
                  </svg>
                    </div>

                    <div>${
                      d.data.totalNumberOfAssets
                        ? d.data.totalNumberOfAssets
                        : 0
                    }</div>
                  </div>

                  <!--Device Count-->
                  <div class="flex flex-col justify-center items-center space-y-1">
                    <div>
                    <svg
                    xmlns="http://www.w3.org/2000/svg"
                    class="icon icon-tabler icon-tabler-cpu-2"
                    width="24"
                    height="24"
                    viewBox="0 0 24 24"
                    stroke-width="1"
                    stroke="currentColor"
                    fill="none"
                    stroke-linecap="round"
                    stroke-linejoin="round"
                  >
                    <path stroke="none" d="M0 0h24v24H0z" fill="none"></path>
                    <path
                      d="M5 5m0 1a1 1 0 0 1 1 -1h12a1 1 0 0 1 1 1v12a1 1 0 0 1 -1 1h-12a1 1 0 0 1 -1 -1z"
                    ></path>
                    <path d="M8 10v-2h2m6 6v2h-2m-4 0h-2v-2m8 -4v-2h-2"></path>
                    <path d="M3 10h2"></path>
                    <path d="M3 14h2"></path>
                    <path d="M10 3v2"></path>
                    <path d="M14 3v2"></path>
                    <path d="M21 10h-2"></path>
                    <path d="M21 14h-2"></path>
                    <path d="M14 21v-2"></path>
                    <path d="M10 21v-2"></path>
                  </svg>
                    </div>

                    <div>${
                      d.data.totalNumberOfDevices
                        ? d.data.totalNumberOfDevices
                        : 0
                    }</div>
                  </div>
                </div>
              </div>
            </div>
          </div>
          `;
          } else {
            return `
            <div class="bg-white w-34 h-22 "  style="border: 1px solid #D3D3D3">
            <div class="flex flex-col space-y-3 ">
              <div class="flex justify-between">
                <div>
                  <div class="text-center font-bold flex-1 px-2 pt-2">${
                    d?.data?.accountName ? d?.data?.accountName : '-'
                  }</div>

                  <div class="flex justify-center space-x-3 p-2">
                    <!--User Count-->
                    <div class="flex flex-col justify-center items-center space-y-1">
                      <div>
                        <svg
                          xmlns="http://www.w3.org/2000/svg"
                          class="icon icon-tabler icon-tabler-user"
                          width="22"
                          height="22"
                          viewBox="0 0 24 24"
                          stroke-width="1"
                          stroke="currentColor"
                          fill="none"
                          stroke-linecap="round"
                          stroke-linejoin="round"
                        >
                          <path stroke="none" d="M0 0h24v24H0z" fill="none"></path>
                          <path d="M8 7a4 4 0 1 0 8 0a4 4 0 0 0 -8 0"></path>
                          <path d="M6 21v-2a4 4 0 0 1 4 -4h4a4 4 0 0 1 4 4v2"></path>
                        </svg>
                      </div>

                      <div>${
                        d.data.totalNumberOfUsers
                          ? d.data.totalNumberOfUsers
                          : 0
                      }</div>
                    </div>

                    <!--Asset Count-->
                    <div class="flex flex-col justify-center items-center space-y-1">
                      <div>
                        <svg
                          xmlns="http://www.w3.org/2000/svg"
                          class="icon icon-tabler icon-tabler-box"
                          width="22"
                          height="22"
                          viewBox="0 0 24 24"
                          stroke-width="1"
                          stroke="currentColor"
                          fill="none"
                          stroke-linecap="round"
                          stroke-linejoin="round"
                        >
                          <path stroke="none" d="M0 0h24v24H0z" fill="none"></path>
                          <path d="M12 3l8 4.5l0 9l-8 4.5l-8 -4.5l0 -9l8 -4.5"></path>
                          <path d="M12 12l8 -4.5"></path>
                          <path d="M12 12l0 9"></path>
                          <path d="M12 12l-8 -4.5"></path>
                        </svg>
                      </div>

                      <div>${
                        d.data.totalNumberOfAssets
                          ? d.data.totalNumberOfAssets
                          : 0
                      }</div>
                    </div>

                    <!--Device Count-->
                    <div class="flex flex-col justify-center items-center space-y-1">
                      <div>
                        <svg
                          xmlns="http://www.w3.org/2000/svg"
                          class="icon icon-tabler icon-tabler-cpu-2"
                          width="24"
                          height="24"
                          viewBox="0 0 24 24"
                          stroke-width="1"
                          stroke="currentColor"
                          fill="none"
                          stroke-linecap="round"
                          stroke-linejoin="round"
                        >
                          <path stroke="none" d="M0 0h24v24H0z" fill="none"></path>
                          <path
                            d="M5 5m0 1a1 1 0 0 1 1 -1h12a1 1 0 0 1 1 1v12a1 1 0 0 1 -1 1h-12a1 1 0 0 1 -1 -1z"
                          ></path>
                          <path d="M8 10v-2h2m6 6v2h-2m-4 0h-2v-2m8 -4v-2h-2"></path>
                          <path d="M3 10h2"></path>
                          <path d="M3 14h2"></path>
                          <path d="M10 3v2"></path>
                          <path d="M14 3v2"></path>
                          <path d="M21 10h-2"></path>
                          <path d="M21 14h-2"></path>
                          <path d="M14 21v-2"></path>
                          <path d="M10 21v-2"></path>
                        </svg>
                      </div>

                      <div>${
                        d.data.totalNumberOfDevices
                          ? d.data.totalNumberOfDevices
                          : 0
                      }</div>
                    </div>
                  </div>
                </div>

                <div class="flex flex-col place-content-evenly place-items-center bg-gray-300 p-2">
                  <div>
                  <!--Add Icon-->
                  <svg xmlns="http://www.w3.org/2000/svg" width="20" height="20" fill="#808080" class="bi bi-plus" viewBox="0 0 16 16"> <path d="M8 4a.5.5 0 0 1 .5.5v3h3a.5.5 0 0 1 0 1h-3v3a.5.5 0 0 1-1 0v-3h-3a.5.5 0 0 1 0-1h3v-3A.5.5 0 0 1 8 4z"/> </svg></div>

                  <div>
                  <!--Menu Icon-->
                  <svg
                    xmlns="http://www.w3.org/2000/svg"
                    width="12"
                    height="12"
                    fill="#808080"
                    class="bi bi-three-dots"
                    viewBox="0 0 16 16"
                  >
                    <path
                      d="M3 9.5a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3zm5 0a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3zm5 0a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3z"
                    />
                  </svg>
                  </div>
                </div>
              </div>
            </div>
          </div>`;
          }
        })
        .render();
      // const selection = d3.select(".svg-chart-container");
      // selection.on('.zoom', null)
    }, 10);
  }
}
