/* eslint-disable @typescript-eslint/no-this-alias */
/* eslint-disable @typescript-eslint/no-empty-function */
/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
import {
  AfterViewInit,
  Component,
  ElementRef,
  Input,
  OnChanges,
  OnInit,
  Output,
  QueryList,
  SimpleChanges,
  ViewChildren,
} from '@angular/core';
import { Subject } from 'rxjs';
import { MapCircle as MapCircleModel } from '../../models/map-circle.model';
import { MapDrawingService } from '../../services/map-drawing.service';
import { MapCircleService } from '../../services/map-circle.service';
import { GoogleMap, MapCircle, MapMarker } from '@angular/google-maps';

@Component({
  selector: 'hash8-platform-map-circle',
  templateUrl: './map-circle.component.html',
  styleUrls: ['./map-circle.component.scss'],
})
export class MapCircleComponent implements OnInit, OnChanges, AfterViewInit {
  @Input() map!: GoogleMap;
  @Input() mapOptions!: any;

  @Input() circles: any;
  @Input() circleOptions: MapCircleModel;

  @Input() fitBounds: any = false;

  @Output() circleData: Subject<any> = new Subject();

  @ViewChildren(MapCircle) map_circles!: QueryList<MapCircle>;
  @ViewChildren(MapMarker) map_markers!: QueryList<MapMarker>;

  // private _circles: any = {};
  _circlesList: any[] = [];
  _showAllFences = false;

  constructor(
    private mapDrawingService: MapDrawingService,
    public mapCircleService: MapCircleService
  ) {
    this.circleOptions = this.mapCircleService._defaultOptions;
  }

  ngOnInit(): void {
    this.mapCircleService.circles$.subscribe((circles) => {
      this._circlesList = circles;
    });
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes['circles']) {
      const circles = this.mapCircleService.formatCircleFences(
        changes['circles'].currentValue,
        'geofenceDetails'
      );
      this.mapCircleService._circles.next(circles);
      // this._circles = this.processCircleGeofenceData(
      //   changes['circles'].currentValue,
      //   'uuid'
      // );
    }

    if (changes['circleOptions']) {
      this.circleOptions = changes['circleOptions'].currentValue;
    }
  }

  ngAfterViewInit(): void {
    if (this.fitBounds) {
      this.map_circles?.map((c: any) => {
        this.mapDrawingService.fitBoundsCircle(c.circle);
      });
    }

    this.map.zoomChanged.subscribe(() => {
      if (this.map) {
        if (
          (this.map.getZoom() as any) >
          (this.mapOptions.geofenceLabelVisibilityZoomLevel || 14)
        ) {
          this._showAllFences = true;
          this.showAllCirclesLabels();
        } else {
          this._showAllFences = false;
          this.hideAllCirclesLabels();
        }
      }
    });
  }

  // drawCircle(event: any, circle: any, circleComp: any) {}

  circleClick(event: any, index: any, circle: any, circleComp: any) {
    this.circleData.next(circle);

    circle.options = {
      fillColor: '#ff0000',
      fillOpacity: 0.25,
      strokeWeight: 2,
      strokeColor: '#ff0000',
      clickable: true,
      editable: true,
      zIndex: 1,
    };
  }

  editCircle(event: any, index: any, circle: any) {
    return false;
    this.map_circles?.map((c: any) => {
      if (c.circle.geofenceId === circle.geofenceId) {
        if (!c.circle) {
          return;
        }

        if (c.circle.editable) {
          c.circle.setOptions(this.mapCircleService._defaultOptions);
          c.circle.setEditable(false);
        } else {
          c.circle.setOptions(this.mapDrawingService.circleConfig.options);
          c.circle.setEditable(true);
        }
        this.mapDrawingService.fitBoundsCircle(c.circle);
      }
    });
  }

  mouseOverCircle(c: any, circle: any, m: any) {
    m.marker.setOptions({ zIndex: 999 });
    if (this._showAllFences) return;
    this.toggleFenceLabel(m.marker, m.marker.title || '', true);
  }

  mouseOutCircle(c: any, circle: any, m: any) {
    m.marker.setOptions({ zIndex: 1 });
    if (this._showAllFences) return;
    this.toggleFenceLabel(m.marker);
  }

  toggleFenceLabel(marker: any, label: any = '', show: boolean = false) {
    if (show) {
      marker.setLabel({
        text: 'Geofence: ' + label,
        className: 'geofence-label',
      });
    } else {
      marker.setLabel(null);
    }
  }

  showAllCirclesLabels() {
    this.map_markers.forEach((m: any) => {
      this.toggleFenceLabel(m.marker, m.marker.title, true);
    });
  }

  hideAllCirclesLabels() {
    this.map_markers.forEach((m: any) => {
      this.toggleFenceLabel(m.marker);
    });
  }
}
