/* eslint-disable no-prototype-builtins */
/* eslint-disable @typescript-eslint/no-empty-function */
/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
import {
  AfterViewInit,
  Component,
  Input,
  OnChanges,
  OnDestroy,
  OnInit,
  Output,
  SimpleChanges,
} from '@angular/core';
import { MapDrawingService } from '../../services/map-drawing.service';
import { Subject, catchError, map, of, take, takeUntil, takeWhile } from 'rxjs';
import { FormGroup } from '@angular/forms';
import { EventsWebService } from '@hash8io/events-web';
import { GeofenceService } from './map-geofence-form.service';
import { HttpClient } from '@angular/common/http';
import { MapCircleService } from '../../services/map-circle.service';
import { MapPolygonService } from '../../services/map-polygon.service';

@Component({
  selector: 'hash8-platform-map-geofence-form',
  templateUrl: './map-geofence-form.component.html',
  styleUrls: ['./map-geofence-form.component.scss'],
})
export class MapGeofenceFormComponent
  implements OnInit, AfterViewInit, OnChanges, OnDestroy
{
  form = new FormGroup({});
  @Input() map!: any;
  @Input() mapOptions!: any;
  @Input() drawer!: any;
  @Input() hideClose: any = false;
  @Input() mode: 'create' | 'update' = 'create';

  @Output() cancelForm: Subject<any> = new Subject();
  @Output() geofenceFormSubmit: Subject<any> = new Subject();

  @Input() model: any = {
    geofenceDetails: null,
  };
  geofenceForm: any;
  createEventConfig: any;
  updateEventConfig: any;
  selectedMode: any;
  geofence: any;
  _keyValueArray: any = [];

  private _editFenceStore: any = {};
  private _geofence: any = {};
  private _geofenceShape: any = {};
  private destroy$: Subject<void> = new Subject<void>();

  constructor(
    public mapDrawingService: MapDrawingService,
    private eventService: EventsWebService,
    public geofenceService: GeofenceService,
    private circleService: MapCircleService,
    private polygonService: MapPolygonService,
    public http: HttpClient
  ) {
    // this.geofenceForm = GEOFENCE_CREATE_FORM;
    // this.createEventConfig = GEOFENCE_CREATE_SUBMIT_EVENTS;
    this.geofenceForm = this.geofenceService.formConfig;
    this.createEventConfig = this.geofenceService.createSubmitConfig;
    this.updateEventConfig = this.geofenceService.updateSubmitConfig;
  }

  ngOnInit(): void {
    // if (this.mode === 'update') {
    //   this.geofenceForm = GEOFENCE_UPDATE_FORM;
    //   this.model = {
    //     geofenceName: 'My Home',
    //     category: 'shipyard',
    //   };
    // } else {
    //   this.geofenceForm = GEOFENCE_CREATE_FORM;
    // }

    this.mapDrawingService.circle$.subscribe((circle) => {
      this.geofence = circle;
      this.validateForm();
    });

    this.mapDrawingService.polygon$.subscribe((polygon) => {
      this.geofence = polygon;
      this.validateForm();
    });

    // this.geofenceService.formModel$.subscribe((formModel) => {
    //   this.model = formModel;
    //   this.geofenceForm = this.geofenceService.formConfig;
    //   console.log(
    //     '%cmap-geofence-form.component.ts line:76 object',
    //     'color: #007acc;',
    //     this.model,
    //     this.geofenceForm
    //   );
    // });

    this.geofenceService.modeChanged.subscribe((obj) => {
      if (obj) {
        this.mode = obj.type;
        this.model = obj.data;
        this.geofenceForm = this.geofenceService.formConfig;
      }
    });

    this.geofenceService.formModelChanged
      .pipe(takeWhile((data) => !data, true))
      .pipe(takeUntil(this.destroy$))
      .subscribe((data) => {
        if (data) {
          this._editFenceStore = Object.assign({}, data);
        }
      });

    this.geofenceService.formModelChanged
      // .pipe(takeWhile((data) => !data, true))
      .pipe(takeUntil(this.destroy$))
      .subscribe((data) => {
        if (data) console.log('data', data, data.geofenceType);
        if (data) {
          this.model = data;
          // this._editFenceStore = Object.assign({}, data);
          if (this.mapOptions.geofenceMode === 'edit') {
            if (data.geofenceType === 'polygon') {
              this.mapDrawingService.setDrawingMode('polygon');
              const shape: any = this.polygonService.formatPolygonFences([
                data,
              ]);
              this._geofenceShape = shape[0];
              const polygon: any = (this._geofence =
                new window.google.maps.Polygon(this._geofenceShape));
              this._geofence.setMap(this.map.googleMap);
              this.mapDrawingService.onPolygonDrawingCompleted(this._geofence);
            } else if (data.geofenceType === 'circle') {
              this.mapDrawingService.setDrawingMode('circle');
              const shape: any = this.circleService.formatCircleFences([data]);
              this._geofenceShape = shape[0];
              const circle: any = (this._geofence =
                new window.google.maps.Circle(shape[0]));
              this._geofence.setMap(this.map.googleMap);
              // this.circleService._circles.next(shape);
              this.mapDrawingService.onCircleDrawingCompleted(this._geofence);
            }
          }
        }
      });
  }

  ngAfterViewInit(): void {
    // alert(this.mode);
    // setTimeout(() => {
    //   if (this.mode === 'update') {
    //     const btn = document.getElementById('editGeofenceBtn');
    //     btn?.click();
    //     this.geofenceService.setCurrentMode('update', this.model);
    //   }
    // }, 100);
  }

  ngOnChanges(changes: SimpleChanges): void {
    console.log(changes);
  }

  onDrawingModeChange() {
    this._keyValueArray = [];
    this.mapDrawingService.changeDrawingMode(
      this.mapDrawingService.selectedDrawingMode
    );
    // this.model.geofenceDetails = null;
  }

  closeDrawer() {
    // this.drawer.close();
    this.model = {};
    this.cancelForm.next(null);
  }

  clearGeofence() {
    this.mapDrawingService.redraw();
    this.model.geofenceDetails = null;
  }

  resetGeofence() {
    this.model = this._editFenceStore;
    this._editFenceStore = Object.assign({}, this.model);
    this.resetGeofenceShape();
  }

  resetGeofenceShape() {
    if (this.mapOptions.geofenceMode === 'edit') {
      if (this.selectedMode === 'polygon') {
        this._geofence.setPaths(this._geofenceShape.paths);
        this.mapDrawingService.onPolygonDrawingCompleted(this._geofence);
      } else if (this.selectedMode === 'circle') {
        this._geofence.setRadius(this._geofenceShape.radius);
        this._geofence.setCenter(this._geofenceShape.center);
      }
    }
  }

  modelChange(event: any) {
    this.validateForm();
  }

  validateForm() {
    // Validate selected geofence type
    this.selectedMode = this.mapDrawingService.selectedDrawingMode;
    if (this.selectedMode === null || this.selectedMode === undefined) {
      this.form.setErrors({ geofenceTypeInvalid: true });
      return;
    }

    // Validate geofence object
    if (this.geofence === null || this.geofence === undefined) {
      this.form.setErrors({ geofenceDetailsInvalid: true });
      return;
    }

    // Validate geofence area
    if (!this.geofence?.validation?.valid) {
      this.form.setErrors({ geofenceAreaInvalid: true });
      return;
    }

    this.form.updateValueAndValidity();
  }

  formatFormData(formData: any) {
    this._keyValueArray = [];

    for (const key in formData) {
      if (formData.hasOwnProperty(key)) {
        const value = formData[key];
        if (
          key === 'geofenceDetails' &&
          typeof value === 'object' &&
          value !== null
        ) {
          // Handle the 'geofence' property and exclude the 'validation' object
          const geofenceWithoutValidation = { ...value };
          delete geofenceWithoutValidation.validation;
          this._keyValueArray.push({ key, value: geofenceWithoutValidation });
        } else {
          this._keyValueArray.push({ key, value });
        }
      }
    }

    return this._keyValueArray;
  }

  onSubmit(model: any) {
    this.validateForm();

    const formData = {
      ...this.form.value,
      geofenceDetails: this.geofence,
      geofenceType: this.selectedMode,
    };

    if (this.mode === 'create') {
      this.createEventConfig.params.bodyParams = [
        ...this.createEventConfig.params.bodyParams,
        ...this.formatFormData(formData),
      ];
      this.eventService.processEvents(
        [this.createEventConfig],
        this.eventService
      );
    } else {
      this.updateEventConfig.params.bodyParams = [
        ...this.updateEventConfig.params.bodyParams,
        ...this.formatFormData(formData),
      ];
      this.eventService.processEvents(
        [this.updateEventConfig],
        this.eventService,
        { geofence: this.model }
      );
    }
  }

  ngOnDestroy(): void {
    this.geofence = null;
    this._geofence = null;
    this.geofenceService.clearFormModel();
    this.destroy$.next();
    this.destroy$.complete();
  }
}
