/* eslint-disable @typescript-eslint/no-empty-function */
import { Injectable } from '@angular/core';
import { BehaviorSubject, Observable, catchError, map, of } from 'rxjs';
import {
  GEOFENCE_CREATE_FORM,
  GEOFENCE_CREATE_SUBMIT_EVENTS,
  GEOFENCE_EDIT_SUBMIT_EVENTS,
  GEOFENCE_UPDATE_FORM,
} from './geofence-form-config';
import { TemplateService } from '@hash8io/events-web';

import { InjectionToken } from '@angular/core';
import { HttpClient } from '@angular/common/http';

export const GEOFENCE_FORM_CONFIG = new InjectionToken<any>(
  'GeofenceFormConfig'
);

@Injectable({
  providedIn: 'root',
})
export class GeofenceService {
  currentGeofence: any;
  curentGeofenceId: any;

  currentMode: 'create' | 'update' = 'create';
  selectedDrawingMode: 'circle' | 'polygon' = 'circle';
  formModel: any = {};

  _modeChanged = new BehaviorSubject<any>(null);
  modeChanged = this._modeChanged.asObservable();

  _formModelChanged = new BehaviorSubject<any>(null);
  formModelChanged = this._formModelChanged.asObservable();

  private config: any = {
    // Your default configuration settings here
    createConfig: GEOFENCE_CREATE_FORM,
    updateConfig: GEOFENCE_UPDATE_FORM,
    createSubmitConfig: GEOFENCE_CREATE_SUBMIT_EVENTS,
    updateSubmitConfig: GEOFENCE_EDIT_SUBMIT_EVENTS,
  };
  get createConfig(): any {
    return this.config.createConfig;
  }

  set createConfig(value: any) {
    this.config.createConfig = value;
  }

  get updateConfig(): any {
    return this.config.updateConfig;
  }

  set updateConfig(value: any) {
    this.config.updateConfig = value;
  }

  get createSubmitConfig(): any {
    return this.config.createSubmitConfig;
  }

  set createSubmitConfig(value: any) {
    this.config.createSubmitConfig = value;
  }

  get updateSubmitConfig(): any {
    return this.config.updateSubmitConfig;
  }

  set updateSubmitConfig(value: any) {
    this.config.updateSubmitConfig = value;
  }

  get formConfig() {
    if (this.currentMode === 'create') {
      return this.templateService.processTemplate(this.config.createConfig, {
        geofence: this.formModel,
      });
    } else {
      return this.templateService.processTemplate(this.config.updateConfig, {
        geofence: this.formModel,
      });
    }
  }

  constructor(
    private templateService: TemplateService,
    private http: HttpClient
  ) {}

  switchToCreateMode(data: any) {
    this.currentMode = 'create';
    this.formModel = data;
    this._modeChanged.next({
      type: 'create',
      data: data,
    });
  }

  switchToUpdateMode(data: any) {
    this.currentMode = 'update';
    this.formModel = data;
    this._modeChanged.next({
      type: 'update',
      data: data,
    });
  }

  clearFormModel() {
    this.formModel = null;
  }

  setCurrentMode(mode: 'create' | 'update' = 'create', data: any = null) {
    this.currentMode = mode;
    if (mode === 'create') {
      this.switchToCreateMode(data);
    } else {
      this.switchToUpdateMode(data);
    }
  }

  readGeofence(geofenceUUID: any): Observable<any> {
    return (
      this.http
        .get(`http://localhost:3000/geofencesList?uuid=${geofenceUUID}`)
        // .get(`/api/v1/geofences/${this.geofenceUUID}`)
        .pipe(map((resp: any) => resp[0]))
        // .pipe(
        //   map((data: any) => {
        //     this.formModel = data;
        //     //     // this._formModelChanged.next(data);
        //     return data;
        //   })
        // )
        .pipe(
          catchError(() => {
            return of({});
          })
        )
    );
  }
}
