/* eslint-disable @typescript-eslint/no-this-alias */
/* eslint-disable @typescript-eslint/no-empty-function */
/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
import {
  AfterViewInit,
  Component,
  Input,
  OnChanges,
  OnInit,
  Output,
  QueryList,
  SimpleChanges,
  ViewChildren,
} from '@angular/core';
import { Subject } from 'rxjs';
import { MapPolygon as MapPolygonModel } from '../../models/map-polygon.model';
import { MapPolygonService } from '../../services/map-polygon.service';
import { GoogleMap, MapMarker, MapPolygon } from '@angular/google-maps';
import { MapDrawingService } from '../../services/map-drawing.service';

@Component({
  selector: 'hash8-platform-map-polygon',
  templateUrl: './map-polygon.component.html',
  styleUrls: ['./map-polygon.component.scss'],
})
export class MapPolygonComponent implements OnInit, OnChanges, AfterViewInit {
  @Input() map!: GoogleMap;
  @Input() mapOptions!: any;

  @Input() polygons: any;
  @Input() polygonOptions!: MapPolygonModel;

  @Input() fitBounds: any = false;

  @Output() polygonData: Subject<any> = new Subject();

  @ViewChildren(MapPolygon) map_polygons!: QueryList<MapPolygon>;
  @ViewChildren(MapMarker) map_markers!: QueryList<MapMarker>;

  // private _polygons: any = {};
  _showAllFences = false;

  constructor(
    public mapPolygonService: MapPolygonService,
    public mapDrawingService: MapDrawingService
  ) {
    this.polygonOptions = this.mapPolygonService._defaultOptions;
  }

  ngOnInit(): void {}

  ngOnChanges(changes: SimpleChanges): void {
    if (changes['polygons']) {
      const polygons = this.mapPolygonService.formatPolygonFences(
        changes['polygons'].currentValue,
        'geofenceDetails'
      );
      this.mapPolygonService._polygons.next(polygons);
      // this._polygons = this.mapPolygonService.processPolygonGeofenceData(
      //   changes['polygons'].currentValue,
      //   'uuid'
      // );
    }

    if (changes['polygonOptions']) {
      this.polygonOptions = changes['polygonOptions'].currentValue;
    }
  }

  ngAfterViewInit(): void {
    if (this.fitBounds) {
      this.map_polygons?.map((p: any) => {
        this.mapDrawingService.fitBoundsPolygon(p.polygon);
      });
    }

    this.map.zoomChanged.subscribe(() => {
      if (this.map) {
        if (
          (this.map.getZoom() as any) >
          (this.mapOptions.geofenceLabelVisibilityZoomLevel || 14)
        ) {
          this._showAllFences = true;
          this.showAllPolygonsLabels();
        } else {
          this._showAllFences = false;
          this.hideAllPolygonsLabels();
        }
      }
    });
  }

  // drawPolygon(event: any, polygon: any, polygonComp: any) {}

  polygonClick(event: any, polygon: any, polygonComp: any) {
    this.polygonData.next(polygon);

    polygon.options = {
      fillColor: '#ff0000',
      fillOpacity: 0.25,
      strokeWeight: 2,
      strokeColor: '#ff0000',
      clickable: true,
      editable: true,
      zIndex: 1,
    };
  }

  mouseOverPolygon(p: any, polygon: any, m: any) {
    m.marker.setOptions({ zIndex: 999 });
    if (this._showAllFences) return;
    this.toggleFenceLabel(m.marker, m.marker.title || '', true);
  }

  mouseOutPolygon(p: any, polygon: any, m: any) {
    m.marker.setOptions({ zIndex: 1 });
    if (this._showAllFences) return;
    this.toggleFenceLabel(m.marker);
  }

  toggleFenceLabel(marker: any, label: any = '', show: boolean = false) {
    if (show) {
      marker.setLabel({
        text: 'Geofence: ' + label,
        className: 'geofence-label',
      });
    } else {
      marker.setLabel(null);
    }
  }

  showAllPolygonsLabels() {
    this.map_markers.forEach((m: any) => {
      this.toggleFenceLabel(m.marker, m.marker.title, true);
    });
  }

  hideAllPolygonsLabels() {
    this.map_markers.forEach((m: any) => {
      this.toggleFenceLabel(m.marker);
    });
  }
}
