/* eslint-disable @typescript-eslint/no-this-alias */
/* eslint-disable @angular-eslint/no-input-rename */
/* eslint-disable @typescript-eslint/no-empty-function */
/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
import {
  AfterViewInit,
  Component,
  ContentChild,
  Input,
  OnChanges,
  OnInit,
  Output,
  SimpleChanges,
  TemplateRef,
  ViewChild,
} from '@angular/core';
import { GoogleMap, MapCircle, MapInfoWindow } from '@angular/google-maps';
import { BehaviorSubject, map, Observable, Subject } from 'rxjs';

import { MapCircleService } from './services/map-circle.service';
import { MapMarkerService } from './services/map-marker.service';
import { MapPolygonService } from './services/map-polygon.service';
import { MapPolylineService } from './services/map-polyline.service';
import { MapService } from './services/map.service';
import { MapCircle as MapCircleModel } from './models/map-circle.model';
import { MapMarker } from './models/map-marker.model';
import { MapPolygon } from './models/map-polygon.model';
import { MapPolyline } from './models/map-polyline.model';
import { TemplateService } from '@hash8io/events-web';

import { MarkerClusterer } from '@googlemaps/markerclusterer';
import { MapControlService } from './services/map-control.service';
import { MapDrawingService } from './services/map-drawing.service';
import { MatDrawer } from '@angular/material/sidenav';
import { HttpClient } from '@angular/common/http';

@Component({
  selector: 'hash8-platform-map-web',
  templateUrl: './map-web.component.html',
  styleUrls: ['./map-web.component.scss'],
  providers: [MapService, MapDrawingService],
})
export class MapWebComponent implements OnInit, AfterViewInit, OnChanges {
  @Input() name = 'defaultMap';
  map!: any;

  @Input() mapOptions: any; // Map Options
  defaultMapOptions: any;

  // Marker
  @Input() markers: any[] = []; // Raw API Marker data
  @Input() markerOptions: MapMarker = {};
  // @Output() markerData: Subject<any> = new Subject();

  markers$ = new BehaviorSubject(this.markers);
  markersObject$!: Observable<any>;

  // Info Window
  @ViewChild(MapInfoWindow) infoWindow: any;
  infoWindowObj: any; // Info Window object

  // Polyline
  @Input() polylines: any = []; // Raw API Polyline data
  @Input() polylineOptions: MapPolyline = {};
  @Output() polylineData: Subject<any> = new Subject();

  polylines$ = new BehaviorSubject(this.polylines);
  polylinesObject$!: Observable<any>;

  // Polygon
  @Input() polygons: any = []; // Raw API Polygon data
  polygons$ = new BehaviorSubject(this.polygons);
  get polygonsObject$(): Observable<any> {
    return this.polygons$.asObservable();
  }
  @Input() polygonOptions: MapPolygon = {};
  @Output() polygonData: Subject<any> = new Subject();

  // Circle
  @Input() circles: any = []; // Raw API Circle data
  circles$ = new BehaviorSubject(this.circles);
  get circlesObject$(): Observable<any> {
    return this.circles$.asObservable();
  }
  @Input() circleOptions: MapCircleModel = {};
  @Output() circleData: Subject<any> = new Subject();

  @Input() geofences: any = [];

  @Input() infoWindowFields!: any;
  @Input() mapTypeId!: any;

  @ViewChild('googleMap', { static: true }) googleMap!: GoogleMap;
  @ViewChild('drawer', { static: true }) drawer!: MatDrawer;
  @ViewChild(MapCircle, { static: true }) mapCircle!: GoogleMap;
  markerCluster?: MarkerClusterer;

  @ContentChild(TemplateRef) templateRef!: TemplateRef<any>;

  searchValue!: any;

  _zoom = 18;
  markersData!: any;
  options = {
    zoom: 18,
    imagePath:
      'https://developers.google.com/maps/documentation/javascript/examples/markerclusterer/m',
  };
  isMapLoaded = false;
  shapeBounds: any;

  _enableFitBounds: any = false;

  constructor(
    private mapService: MapService,
    private markerService: MapMarkerService,
    private polylineService: MapPolylineService,
    private polygonService: MapPolygonService,
    private circleService: MapCircleService,
    private templateService: TemplateService,
    private mapControlService: MapControlService,
    public mapDrawingService: MapDrawingService
  ) {}

  ngOnInit(): void {
    const { center, ...options } = this.mapOptions;
    this.defaultMapOptions = this.mapService.getMapOptions(options);
    // this.shapeBounds = new google.maps.LatLngBounds();
    // this.loadMap();
    // this.loadMarker();
    // this.loadPolyline();
    // this.loadPolygon();
    // this.loadCircle();
  }

  ngOnChanges(changes: SimpleChanges) {
    if (changes['markers']) {
      this.markers$.next(changes['markers'].currentValue);
    }

    if (changes['polylines']) {
      this.polylines$.next(changes['polylines'].currentValue);
    }

    if (changes['polygons']) {
      const polygons = this.polygonService.formatPolygonFences(
        changes['polygons'].currentValue,
        'geofenceDetails'
      );

      this.polygons$.next(polygons);
    }

    if (changes['circles']) {
      const circles = this.circleService.formatCircleFences(
        changes['circles'].currentValue,
        'geofenceDetails'
      );
      this.circles$.next(circles);
    }

    if (changes['geofences']) {
      this.formatGeofences(changes['geofences'].currentValue);
    }
  }

  ngAfterViewInit() {
    // this.map = this.googleMap?.googleMap;
    // this.mapService.map = this.googleMap?.googleMap;
    // const markers = this.markersData;
    // if (markers) {
    //   this.markerCluster = new MarkerClusterer({
    //     map: this.map,
    //     markers,
    //   });
    // }
    // this.map.addListener('idle', () => {
    //   this.mapService.triggerEvent(this.map);
    // });
  }

  onMapInitialized(map: any) {
    this.map = map;
    this.mapService.map = map;
    this.isMapLoaded = true;

    this.loadMap();
    if (this.defaultMapOptions.preLoadAllGeofences) {
      this.mapService
        .loadAllGeofences(this.defaultMapOptions.geofenceListApiUrl)
        .subscribe((res: any) => {
          this.formatGeofences(res);
        });
    }
  }

  loadMap() {
    if (typeof this.mapOptions?.center == 'string') {
      this.mapOptions.center = JSON.parse(this.mapOptions?.center);
    }

    const center: any = { ...this.mapOptions?.center };

    if (this.mapOptions?.center?.lat || this.mapOptions?.center?.latitude) {
      center['lat'] =
        (this.mapOptions?.center['lat'] ||
          this.mapOptions?.center['latitude']) * 1;
    }
    if (this.mapOptions?.center?.lng || this.mapOptions?.center?.longitude) {
      center['lng'] =
        (this.mapOptions?.center['lng'] ||
          this.mapOptions?.center['longitude']) * 1;
    }
    // if (this.mapOptions?.center?.latitude) {
    //   center['lat'] = this.mapOptions?.center['latitude'] * 1;
    // }

    // if (this.mapOptions?.center?.longitude) {
    //   center['lng'] = 1 * this.mapOptions?.center['longitude'];
    // }
    this.defaultMapOptions = this.mapService.getMapOptions(
      Object.assign(this.mapOptions, center)
    );
    this._zoom = this.defaultMapOptions.zoom * 1;
  }

  // formatCircleFences(data: any, circleField: string = 'geofenceDetails') {
  //   const shape = data?.map((item: any) => {
  //     // this.shapeBounds.union(this.mapCircle.getBounds());
  //     const options = {
  //       ...this.mapDrawingService.circleConfig.options,
  //       editable: false,
  //     };

  //     return {
  //       center: {
  //         lat: item[circleField]['center']['lat'] * 1,
  //         lng: item[circleField]['center']['lng'] * 1,
  //       },
  //       radius: item[circleField]['radius'] * 1,
  //       geofenceId: item.uuid,
  //       ...options,
  //     };
  //   });
  //   return shape;
  // }

  // formatPolygonFences(data: any, polygonField: string = 'geofenceDetails') {
  //   const shape = data?.map((item: any) => {
  //     // this.shapeBounds.union(this.mapCircle.getBounds());
  //     const options = {
  //       ...this.mapDrawingService.polygonConfig.options,
  //       editable: false,
  //     };

  //     return {
  //       paths: item[polygonField]['path'],
  //       geofenceId: item.uuid,
  //       ...options,
  //     };
  //   });
  //   return shape;
  // }

  formatGeofences(data: any) {
    if (!data) {
      return;
    }

    if (typeof data == 'string') {
      data = JSON.parse(data);
    }

    const circleGeofences = data.filter(
      (item: any) => item.geofenceType === 'circle'
    );
    const polygonGeofences = data.filter(
      (item: any) => item.geofenceType === 'polygon'
    );
    const circles = this.circleService.formatCircleFences(
      circleGeofences,
      'geofenceDetails'
    );
    this.circles$.next(circles);
    this.circles = circleGeofences;

    const polygons = this.polygonService.formatPolygonFences(
      polygonGeofences,
      'geofenceDetails'
    );
    this.polygons$.next(polygons);
    this.polygons = polygonGeofences;

    const geofences = [...circleGeofences, ...polygonGeofences];
    this._enableFitBounds = false;

    if (geofences.length === 1) {
      this._enableFitBounds = true;
    }

    return geofences;
  }

  exitDrawingMode() {
    this.defaultMapOptions.drawingMode = false;
  }

  drawingModeChange(mode: any) {
    this.defaultMapOptions.drawingMode = mode;
  }

  searchAddress(place: any) {
    if (!place.geometry || !place.geometry.location) {
      return;
    }
    const bounds = new google.maps.LatLngBounds();

    if (place.geometry.viewport) {
      bounds.union(place.geometry.viewport);
    } else {
      bounds.extend(place.geometry.location);
    }
    this.map.fitBounds(bounds);

    // this.defaultMapOptions.drawingMode = true;
    // this.drawer.open();
    // this.mapDrawingService.changeDrawingMode('circle');
  }

  mapClick(event: any) {
    // console.log(
    //   '%cmap-drawing-control.component.ts line:189 event',
    //   'color: #007acc;',
    //   event
    // );
  }

  loadPolyline() {
    this.polylinesObject$ = this.polylines$.pipe(
      map((polylines) => {
        return this.polylineService.getPolyline(
          polylines,
          this.polylineOptions
        );
      })
    );
  }

  loadPolygon() {
    // this.polygonsObject$ = this.polygons$.pipe(
    //   map((polygons) => {
    //     return this.polygonService.getPolygon(polygons, this.polygonOptions);
    //   })
    // );
  }

  loadCircle() {
    // this.circlesObject$ = this.circles$.pipe(
    //   map((circles) => {
    //     return this.circleService.getCircle(circles, this.circleOptions);
    //   })
    // );
  }

  // openMarkerInfoWindow(marker: any, markerObj: any) {
  //   this.infoWindow.open(marker);
  //   this.infoWindowObj = marker._options.position;

  //   this.infoWindowObj = {
  //     infoWindowFields: this.infoWindowFields,
  //     // uuid: marker._options.position.uuid,
  //     // geofenceId: marker._options.geofenceId,
  //     // accountId: marker._options.position.accountId,
  //     // accountName: marker._options.position.accountName,
  //     // status: marker._options.position.status,
  //     data: marker?._options?.position,
  //   };
  //   console.log('this.infwindow', this.infoWindowObj);

  //   this.markerData.next(this.infoWindowObj);
  // }

  polylineClick(event: any, polyline: any) {
    this.polylineData.next(polyline);
  }

  polygonClick(event: any, polygon: any) {
    this.polylineData.next(polygon);
  }

  // circleClick(event: any, circle: any) {
  //   this.circleData.next(circle);
  // }

  onSearch() {}

  ngOnDestroy() {}
}
