/* eslint-disable @typescript-eslint/no-empty-function */
import { Injectable } from '@angular/core';
import { MapCircle } from '../models/map-circle.model';
import { MapExpressionService } from './map-expression.service';
import { BehaviorSubject, Observable } from 'rxjs';
import { MapDrawingService } from './map-drawing.service';

@Injectable({
  providedIn: 'root',
})
export class MapCircleService {
  _defaultOptions: MapCircle = {
    fillColor: '#ffff00',
    fillOpacity: 0.25,
    strokeWeight: 2.5,
    strokeColor: '#ffff00',
    strokeOpacity: 1,
    clickable: true,
    editable: false,
    suppressUndo: true,
    draggable: false,
    visible: true,
  };

  _circles: any = new BehaviorSubject([]);
  get circles$(): Observable<any> {
    return this._circles.asObservable();
  }

  constructor(private expressionService: MapExpressionService) {}

  getCircle(data: any, customCircleOptions?: any) {
    // console.log('typeof data :: ',typeof data);
    const circleOption = this.getCircleOptions(customCircleOptions);
    const circles = this.getCircleData(data, circleOption);

    return circles;
  }

  getCircleMapObject(geofenceId: any) {
    console.log('geofenceId :: ', geofenceId);
  }

  // Circle Options
  getCircleOptions(customCircleOptions?: any) {
    const circleOption = {
      ...this._defaultOptions,
      fillColor: customCircleOptions?.strokeColor
        ? customCircleOptions?.strokeColor
        : this._defaultOptions.strokeColor,
      ...customCircleOptions,
    };
    return circleOption;
  }

  formatCircleFences(
    data: any,
    circleField: string = 'geofenceDetails',
    circleOption: any = this._defaultOptions
  ) {
    const shape = data?.map((item: any) => {
      const circleObj = {
        center: {
          lat: item[circleField]['center']['lat'] * 1,
          lng: item[circleField]['center']['lng'] * 1,
        },
        radius: item[circleField]['radius'] * 1,
        geofenceId: item.uuid,
        data: item,
      };

      const options = {
        ...this._defaultOptions,
        editable: false,
      };

      // Circle Color Expression
      if (typeof circleOption?.strokeColor !== 'string') {
        options.strokeColor = this.expressionService.evaluate(
          circleOption.strokeColor.expression,
          item
        );
        options.fillColor = this.expressionService.evaluate(
          circleOption.strokeColor.expression,
          item
        );
      }

      return {
        ...circleObj,
        ...options,
      };
    });
    return shape;
  }

  // Circle Data Format
  getCircleData(data: any, circleOption?: any) {
    const circles = data.map((item: any) => {
      const circleObj = {
        center: {
          lat: item[circleOption.latitudeField],
          lng: item[circleOption.longitudeField],
          ...item,
        },
        radius: item.radius,
        ...circleOption,
      };

      // Circle Color Expression
      if (typeof circleOption?.strokeColor !== 'string') {
        circleObj.strokeColor = this.expressionService.evaluate(
          circleOption.strokeColor.expression,
          item
        );
        circleObj.fillColor = this.expressionService.evaluate(
          circleOption.strokeColor.expression,
          item
        );
      }

      // Circle Visibility Expression

      return circleObj;
    });

    return circles || [];
  }
}
