/* eslint-disable @typescript-eslint/no-this-alias */
/* eslint-disable @typescript-eslint/no-empty-function */
import { Injectable } from '@angular/core';
import { MapService } from './map.service';
import { ToastService } from '@hash8io/core-web';
import { BehaviorSubject, Observable } from 'rxjs';
import { MapCircleService } from './map-circle.service';

@Injectable({ providedIn: 'root' })
export class MapDrawingService {
  map!: any;
  selectedDrawingMode: 'marker' | 'polyline' | 'polygon' | 'circle' = 'circle';
  drawingManager: any;
  isDrawingModeEnabled = false;
  isDrawingCompleted = true;

  circlesObject!: any;
  circleMinRadius = 0;
  circleMaxRadius = 0;
  private radiusChangeListener: google.maps.MapsEventListener | null = null;
  isProgrammaticRadiusChange = false; // Flag to track programmatic changes

  markersObject!: any;
  polylinesObject!: any;

  polygonsObject!: any;
  polygonArea: any = 0;

  _polygon$: any = new BehaviorSubject({
    address: '',
    path: [],
    center: {},
    area: 0,
  });
  get polygon$(): Observable<any> {
    return this._polygon$.asObservable();
  }

  _circle$: BehaviorSubject<any> = new BehaviorSubject({
    address: '',
    center: {},
    radius: 0,
    area: 0,
  });
  // circle$: Observable<any> = this._circle$.asObservable() as any;
  get circle$(): Observable<any> {
    return this._circle$.asObservable();
  }

  circleConfig: any = {
    circle: {
      address: '',
      center: {},
      radius: 0,
      area: 0,
    },
    options: {
      fillColor: '#80FF00',
      fillOpacity: 0.15,
      strokeWeight: 3,
      strokeColor: '#80FF00',
      strokeOpacity: 1,
      clickable: true,
      editable: true,
      zIndex: 1,
      suppressUndo: true,
    },
    isEnabled: true,
    minArea: 750,
    maxArea: 12000000,
    defaultRadius: 75,
    unit: 'ft²',
  };
  polygonConfig: any = {
    polygon: {
      address: '',
      path: [],
      center: {},
      area: 0,
    },
    options: {
      fillColor: '#80FF00',
      fillOpacity: 0.15,
      strokeWeight: 3,
      strokeColor: '#80FF00',
      strokeOpacity: 1,
      clickable: true,
      editable: true,
      zIndex: 1,
    },
    isEnabled: false,
    minArea: 750,
    maxArea: 120000000,
    unit: 'ft²',
  };

  shapeErrorConfig = {
    fillColor: '#ff0000',
    fillOpacity: 0.25,
    strokeWeight: 2,
    strokeColor: '#ff0000',
    clickable: true,
    editable: true,
    zIndex: 1,
  };

  circles$ = new BehaviorSubject([]);
  circlesObject$!: Observable<any>;

  /** */
  selectedShape: any;
  colors = ['#1E90FF', '#FF1493', '#32CD32', '#FF8C00', '#4B0082'];
  selectedColor: any;
  colorButtons: any = {};

  constructor(
    private mapService: MapService,
    private toastService: ToastService,
    private circleService: MapCircleService
  ) {
    this.circleMinRadius = this.calculateCircleRadiusFromAreaInSquareFeet(
      this.circleConfig.minArea
    );
    this.circleMaxRadius = this.calculateCircleRadiusFromAreaInSquareFeet(
      this.circleConfig.maxArea
    );
  }

  changeDrawingMode(mode: 'marker' | 'polyline' | 'polygon' | 'circle' | null) {
    this.exitDrawingMode();
    if (mode) {
      this.initDrawingMode(mode);
    } else {
      this.exitDrawingMode();
    }
  }

  exitDrawingMode() {
    this.drawingManager?.setMap(null);
    this.clearAllDrawing();
  }

  initDrawingMode(mode: 'circle' | 'polyline' | 'polygon' | 'marker') {
    this.selectedDrawingMode = mode;
    this.isDrawingCompleted = false;
    const drawingOptions = {
      drawingMode: mode as google.maps.drawing.OverlayType,
      drawingControl: false,
      drawingControlOptions: {
        position: google.maps.ControlPosition.TOP_CENTER,
        drawingModes: [
          google.maps.drawing.OverlayType.MARKER,
          google.maps.drawing.OverlayType.CIRCLE,
          google.maps.drawing.OverlayType.POLYGON,
          google.maps.drawing.OverlayType.POLYLINE,
        ],
      },
      polygonOptions: this.polygonConfig.options,
      circleOptions: this.circleConfig.options,
    };
    this.drawingManager = new google.maps.drawing.DrawingManager(
      drawingOptions
    );
    // alert(this.mapService.map);
    this.drawingManager.setMap(this.mapService.map);

    const _this = this;
    google.maps.event.addListener(
      this.drawingManager,
      'overlaycomplete',
      function (event: any) {
        _this.drawingManager.setMap(null);
        _this.isDrawingCompleted = true;
        if (event.type == 'circle') {
          _this.onCircleDrawingCompleted(event.overlay);
        }
        if (event.type == 'polygon') {
          _this.onPolygonDrawingCompleted(event.overlay);
        }
      }
    );

    // this.bindCustomEvents();
  }

  clearAllDrawing() {
    this.clearCircleDrawing();
    this.clearPolygonDrawing();
    this.clearPolylineDrawing();
    this.drawingManager?.setMap(null);
    this.isDrawingCompleted = true;
  }

  clearCircleDrawing() {
    this.circlesObject?.setMap(null);
    this.circleConfig.circle = {
      address: '',
      center: {},
      radius: 0,
      area: 0,
    };
    this._circle$.next(null);
  }

  clearPolygonDrawing() {
    this.polygonsObject?.setMap(null);
    this.polygonConfig.polygon = {
      address: '',
      path: [],
      center: {},
      area: 0,
    };
    this._polygon$.next(null);
  }

  clearPolylineDrawing() {
    this.polylinesObject?.setMap(null);
  }

  drawCircle(circle: any) {
    this.circleConfig.circle = circle;
  }

  onCircleDrawingCompleted(circle: any) {
    this.circlesObject = circle;
    this.circleService._circles.next(circle);

    this.keepCircleRadiusInRange(this.circlesObject);
    setTimeout(() => this.fitBoundsCircle(), 100);
    this._circle$.next(this.circleConfig.circle);

    const radiusChangeHandler = () => {
      if (!this.isProgrammaticRadiusChange) {
        this.keepCircleRadiusInRange(this.circlesObject);
        this.fitBoundsCircle();
        // this._circle$.next(this.circleConfig.circle);
      }
      // this._circle$.next(this.circleConfig.circle);
    };

    google.maps.event.addListener(
      this.circlesObject,
      'radius_changed',
      radiusChangeHandler
    );

    google.maps.event.addListener(
      this.circlesObject,
      'center_changed',
      radiusChangeHandler
    );
  }

  calculateRadiusFromArea(area: any) {
    return Math.sqrt(area / Math.PI);
  }

  calculateCircleAreaInSquareFeet(circle: any) {
    const radius = circle.getRadius();
    const areaInSquareMeters = Math.PI * Math.pow(radius, 2);
    const areaInSquareFeet =
      Math.round(areaInSquareMeters * 10.7639 * 100) / 100;

    return areaInSquareFeet;
  }

  calculateCircleRadiusFromAreaInSquareFeet(areaInSquareFeet: any) {
    // Convert square feet to square meters
    // 1 square meter ≈ 10.7639 square feet (aprox)
    const areaInSquareMeters = areaInSquareFeet / 10.7639;

    // Calculate the radius from the area in square meters
    const radius = Math.sqrt(areaInSquareMeters / Math.PI);

    return radius;
  }

  validateCircleArea(circle: any, minArea: any, maxArea: any) {
    const area = this.calculateCircleAreaInSquareFeet(circle);

    if (area < minArea) {
      return {
        valid: false,
        radius: this.circleMinRadius,
        message: 'Circle area is too small.',
      };
    }

    if (area > maxArea) {
      return {
        valid: false,
        radius: this.circleMaxRadius,
        message: 'Circle area is too large.',
      };
    }

    return {
      valid: true,
      radius: circle.getRadius(),
      message: 'Circle area is within the allowed range.',
    };
  }

  keepCircleRadiusInRange(circle: any) {
    const validation = this.validateCircleArea(
      circle,
      this.circleConfig.minArea,
      this.circleConfig.maxArea
    );

    if (validation.valid) {
      this.circlesObject.setOptions(this.circleConfig.options);
      this.isProgrammaticRadiusChange = false;
      console.log(validation);
    } else {
      console.error(validation); // Circle area is too small or too large.
      this.isProgrammaticRadiusChange = true;
      // circle.setRadius(validation.radius);
      this.circlesObject.setOptions(this.shapeErrorConfig);
      this.isProgrammaticRadiusChange = false;
    }

    this.circleConfig.circle['center'] = circle.getCenter().toJSON();
    this.circleConfig.circle['radius'] = circle.getRadius();
    this.circleConfig.circle['area'] =
      this.calculateCircleAreaInSquareFeet(circle);
    this.circleConfig.circle['validation'] = validation;
    this._circle$.next(this.circleConfig.circle);
  }

  fitBoundsCircle(circle: any = this.circlesObject) {
    this.mapService.map.fitBounds(circle.getBounds());
    // this._circle$.next(this.circleConfig.circle);
  }

  // Polygon
  onPolygonDrawingCompleted(polygon: any) {
    this.polygonsObject = polygon;
    this.fitBoundsPolygon(this.polygonsObject);
    this.updatePolygonShapeDetails(polygon);
    this.bindCustomPolygonEvents(polygon);
  }

  fitBoundsPolygon(polygon: any) {
    const bounds = new google.maps.LatLngBounds();
    polygon.getPath().forEach((element: any, index: any) => {
      bounds.extend(element);
    });
    this.mapService.map.fitBounds(bounds);

    // Polygon center
    this.polygonConfig.polygon['center'] = bounds.getCenter().toJSON();
  }

  validatePolygonArea(polygon: any, minArea: any, maxArea: any) {
    const polygonArea =
      Math.round(
        google.maps.geometry.spherical.computeArea(polygon.getPath()) *
          10.7639 *
          100
      ) / 100;

    if (polygonArea > maxArea) {
      return {
        valid: false,
        area: polygonArea,
        message: 'Polygon area is too large.',
      };
    }
    if (polygonArea < minArea) {
      return {
        valid: false,
        area: polygonArea,
        message: 'Polygon area is too small.',
      };
    }

    return {
      valid: true,
      area: polygonArea,
      message: 'Polygon area is within the allowed range.',
    };
  }

  updatePolygonShapeDetails(polygon: any) {
    const validation = this.validatePolygonArea(
      polygon,
      this.polygonConfig.minArea,
      this.polygonConfig.maxArea
    );

    this.fitBoundsPolygon(polygon);
    this.polygonConfig.polygon['area'] = validation.area;
    this.polygonConfig.polygon['validation'] = validation;

    if (!validation.valid) {
      console.error(validation);
      this.polygonsObject.setOptions(this.shapeErrorConfig);
    } else {
      this.polygonsObject.setOptions(this.polygonConfig.options);
    }

    // Polygon path array
    this.polygonConfig.polygon['path'] = [];
    polygon.getPath().forEach((element: any, index: any) => {
      this.polygonConfig.polygon['path'].push(
        polygon.getPath().getAt(index).toJSON()
      );
    });

    //TODO: Polygon location

    // this.fitBoundsPolygon(polygon);
    this._polygon$.next(this.polygonConfig.polygon);
  }

  bindCustomPolygonEvents(polygon: any) {
    const self = this;
    const paths = polygon.getPaths();
    paths.forEach(function (path: any, i: any) {
      google.maps.event.addListener(path, 'insert_at', function () {
        self.updatePolygonShapeDetails(polygon);
      });
      google.maps.event.addListener(path, 'set_at', function () {
        self.updatePolygonShapeDetails(polygon);
      });
      google.maps.event.addListener(path, 'remove_at', function () {
        self.updatePolygonShapeDetails(polygon);
      });
    });
  }

  redraw() {
    this.clearAllDrawing();
    this.changeDrawingMode(this.selectedDrawingMode);
  }

  setDrawingMode(mode: 'circle' | 'polyline' | 'polygon' = 'circle') {
    this.selectedDrawingMode = mode;
  }

  getDrawingMode() {}

  setDrawerSize() {}

  getDrawerSize() {}

  setDrawerPosition() {}

  getDrawerPosition() {}

  // load fence creation form
  loadFenceCreationForm() {}

  clearSelection() {
    if (this.selectedShape) {
      this.selectedShape.setEditable(false);
      this.selectedShape = null;
    }
  }

  setSelection(shape: any) {
    this.clearSelection();

    // getting shape coordinates
    const v = shape.getPath();
    for (let i = 0; i < v.getLength(); i++) {
      const xy = v.getAt(i);
      console.log('Cordinate lat: ' + xy.lat() + ' and lng: ' + xy.lng());
    }

    this.selectedShape = shape;
    shape.setEditable(true);
    this.selectColor(shape.get('fillColor') || shape.get('strokeColor'));
  }

  deleteSelectedShape() {
    if (this.selectedShape) {
      this.selectedShape.setMap(null);
    }
  }

  selectColor(color: any) {
    this.selectedColor = color;
    for (let i = 0; i < this.colors.length; ++i) {
      const currColor = this.colors[i];
      this.colorButtons[currColor].style.border =
        currColor == color ? '2px solid #789' : '2px solid #fff';
    }

    // Retrieves the current options from the drawing manager and replaces the
    // stroke or fill color as appropriate.
    const polylineOptions = this.drawingManager.get('polylineOptions');
    polylineOptions.strokeColor = color;
    this.drawingManager.set('polylineOptions', polylineOptions);

    const rectangleOptions = this.drawingManager.get('rectangleOptions');
    rectangleOptions.fillColor = color;
    this.drawingManager.set('rectangleOptions', rectangleOptions);

    const circleOptions = this.drawingManager.get('circleOptions');
    circleOptions.fillColor = color;
    this.drawingManager.set('circleOptions', circleOptions);

    const polygonOptions = this.drawingManager.get('polygonOptions');
    polygonOptions.fillColor = color;
    this.drawingManager.set('polygonOptions', polygonOptions);
  }

  setSelectedShapeColor(color: any) {
    console.log('fn setSelectedShapeColor()');
    console.log(this.selectedShape);
    if (this.selectedShape) {
      if (this.selectedShape.type == google.maps.drawing.OverlayType.POLYLINE) {
        this.selectedShape.set('strokeColor', color);
      } else {
        this.selectedShape.set('fillColor', color);
      }
    }
  }
}
