/* eslint-disable @typescript-eslint/no-empty-function */
import { Injectable } from '@angular/core';

@Injectable({
  providedIn: 'root',
})
export class MapExpressionService {
  constructor() {}

  evaluate(expressionOrFunc: any, params: any) {
    if (typeof expressionOrFunc === 'function') {
      // valueGetter is a function, so just call it
      const func = expressionOrFunc;
      return func(params);
    } else if (typeof expressionOrFunc === 'string') {
      // valueGetter is an expression, so execute the expression
      const expression = expressionOrFunc;
      return this.evaluateExpression(expression, params);
    } else {
      console.error(
        'H8 Expression: value should be either a string or a function',
        expressionOrFunc
      );
    }
  }

  evaluateExpression(expression: any, params: any) {
    // console.log(params);
    try {
      const javaScriptFunction = this.createExpressionFunction(expression);
      // the params don't have all these values, rather we add every possible
      // value a params can have, which makes whatever is in the params available.
      const result = javaScriptFunction(params);
      return result;
    } catch (e) {
      // the expression failed, which can happen, as it's the client that
      // provides the expression. so print a nice message
      // tslint:disable-next-line
      console.log('Processing of the expression failed');
      // tslint:disable-next-line
      console.log('Expression = ' + expression);
      // tslint:disable-next-line
      console.log('Params =', params);
      // tslint:disable-next-line
      console.log('Exception = ' + e);
      return null;
    }
  }

  createExpressionFunction(expression: any) {
    // if not found in cache, return the function
    const functionBody = this.createFunctionBody(expression);
    const theFunction = new Function('data', functionBody);

    return theFunction;
  }

  createFunctionBody(expression: any) {
    // if the expression has the 'return' word in it, then use as is,
    // if not, then wrap it with return and ';' to make a function
    if (expression.indexOf('return') >= 0) {
      return expression;
    } else {
      return 'return ' + expression + ';';
    }
  }
}
