/* eslint-disable @typescript-eslint/no-empty-function */
import { Injectable } from '@angular/core';
import { MapMarker } from '../models/map-marker.model';
import { MapExpressionService } from './map-expression.service';

@Injectable({
  providedIn: 'root',
})
export class MapMarkerService {
  _defaultMarkerOptions: MapMarker = {
    draggable: false,
    animation: false, // google.maps.Animation.DROP,
    clickable: true,
    visible: true,
    // icon: {
    //   url: 'assets/Reefer_marker.svg',
    // },
    latitudeField: 'lat',
    longitudeField: 'lng',
  };

  constructor(private expressionService: MapExpressionService) {}

  getMarker(data: any, customMarkerOptions?: any) {
    const markerOption = this.getMarkerOptions(customMarkerOptions);
    const markers = this.getMarkerData(data, markerOption);

    return markers;
  }

  // Marker Options
  getMarkerOptions(customMarkerOptions?: any) {
    const markerOption = {
      ...this._defaultMarkerOptions,
      ...customMarkerOptions,
    };
    return markerOption;
  }

  // Marker Data Format
  getMarkerData(data: any, markerOption: any) {
    const markers = data.map((item: any) => {
      const markerObj = {
        position: {
          lat: item[markerOption.latitudeField] * 1,
          lng: item[markerOption.longitudeField] * 1,
          // ...item,
        },
        data: item,
        ...markerOption,
      };

      // Marker Icon Expression
      if (
        typeof markerOption?.icon !== 'string' &&
        markerOption.icon?.expression
      ) {
        markerObj.icon = this.expressionService.evaluate(
          markerOption.icon.expression,
          item
        );
      }

      // Marker Visibility Expression
      return markerObj;
    });

    return markers || [];
  }
}
