/* eslint-disable @typescript-eslint/no-empty-function */
import { Injectable } from '@angular/core';
import { MapPolygon } from '../models/map-polygon.model';
import { MapExpressionService } from './map-expression.service';
import { BehaviorSubject, Observable } from 'rxjs';

@Injectable({
  providedIn: 'root',
})
export class MapPolygonService {
  _defaultOptions: MapPolygon = {
    fillColor: '#ffff00',
    fillOpacity: 0.25,
    strokeWeight: 2.5,
    strokeColor: '#ffff00',
    strokeOpacity: 1,
    clickable: true,
    editable: false,
    draggable: false,
    suppressUndo: true,
    visible: true,
  };

  _polygons: any = new BehaviorSubject([]);
  get polygons$(): Observable<any> {
    return this._polygons.asObservable();
  }

  constructor(private expressionService: MapExpressionService) {}

  processPolygonGeofenceData(polygonArray: any, keyName: any = 'uuid') {
    if (polygonArray) {
      const result = polygonArray.reduce((obj: any, item: any) => {
        // console.log(obj);
        obj[item[keyName]] = item;
        return obj;
      }, {});
      return result;
    }
    return {};
  }

  getPolygon(data: any, customPolygonOptions?: any) {
    const polygonOption = this.getPolygonOptions(customPolygonOptions);
    const polygons = this.getPolygonData(data, polygonOption);

    return polygons;
  }

  formatPolygonFences(
    data: any,
    polygonField: string = 'geofenceDetails',
    polygonOption: any = this._defaultOptions
  ) {
    const shape = data?.map((item: any) => {
      // this.shapeBounds.union(this.mapPolygon.getBounds());
      const options = {
        ...this._defaultOptions,
        editable: false,
      };

      // Polygon Color Expression
      if (typeof polygonOption?.strokeColor !== 'string') {
        options.strokeColor = this.expressionService.evaluate(
          polygonOption.strokeColor.expression,
          item
        );
        options.fillColor = this.expressionService.evaluate(
          polygonOption.strokeColor.expression,
          item
        );
      }

      return {
        data: item,
        geofenceId: item.uuid,
        paths: item[polygonField]['path'],
        center: item[polygonField]['center'],
        ...options,
      };
    });
    return shape;
  }

  // Polygon Options
  getPolygonOptions(customPolygonOptions?: any) {
    const polygonOption = {
      ...this._defaultOptions,
      fillColor: customPolygonOptions?.strokeColor
        ? customPolygonOptions?.strokeColor
        : this._defaultOptions.strokeColor,
      ...customPolygonOptions,
    };
    return polygonOption;
  }

  // Polygon Data Format
  getPolygonData(data: any, polygonOption: any) {
    const polygons: any[] = [];

    data.map((item: any) => {
      const polygonObj = item.map((item: any) => {
        return {
          lat: item[polygonOption.latitudeField],
          lng: item[polygonOption.longitudeField],
          ...item,
        };
      });

      const dataFormat = {
        ...polygonOption,
        paths: polygonObj,
      };

      polygons.push(dataFormat);
    });

    return polygons || [];
  }
}
