/* eslint-disable @typescript-eslint/no-empty-function */
import { Injectable } from '@angular/core';
import { MapPolyline } from '../models/map-polyline.model';
import { MapExpressionService } from './map-expression.service';

@Injectable({
  providedIn: 'root',
})
export class MapPolylineService {
  _defaultPolylineOptions: MapPolyline = {
    draggable: false,
    clickable: true,
    visible: true,
    editable: false,
    strokeColor: 'blue',
    strokeOpacity: 1.0,
    strokeWeight: 2,
    latitudeField: 'lat',
    longitudeField: 'lng',
    icons: [
      {
        icon: {
          path: google.maps.SymbolPath.FORWARD_OPEN_ARROW,
        },
        offset: '20%',
        repeat: '200px',
      },
    ],
  };

  _dashPolylineStyle = {
    strokeOpacity: 0,
    icons: [
      {
        icon: {
          path: 'M 0,-1 0,1',
          strokeOpacity: 1,
          scale: 3,
        },
        offset: '0',
        repeat: '20px',
      },
    ],
  };

  _linePolylineStyle = {
    strokeOpacity: 1.0,
    icons: [
      {
        icon: {},
      },
    ],
  };

  _arrowPolylineStyle = {
    strokeOpacity: 1.0,
    icons: [
      {
        icon: {
          path: google.maps.SymbolPath.FORWARD_OPEN_ARROW,
        },
        offset: '20%',
        repeat: '200px',
      },
    ],
  };

  constructor(private expressionService: MapExpressionService) {}

  getPolyline(data: any, customPolylineOptions?: any) {
    const polylineOption = this.getPolylineOptions(customPolylineOptions);
    const polylines = this.getPolylineData(data, polylineOption);

    return polylines;
  };

  // Polyline Options
  getPolylineOptions(customPolylineOptions?: any) {
    const polylineOption = {
      ...this._defaultPolylineOptions,
      ...(customPolylineOptions?.lineStyle == 'line' ? this._linePolylineStyle 
          : customPolylineOptions?.lineStyle == 'dash' ? this._dashPolylineStyle : this._arrowPolylineStyle),
      ...customPolylineOptions,
    };

    return polylineOption;
  };

  // Polyline Data Format
  getPolylineData(data: any, polylineOption: any) {
    const polylines: any[] = [];

    data.map((item: any) => {
      const polylineObj = item.map((item: any) => {
        return {
          lat: item[polylineOption.latitudeField],
          lng: item[polylineOption.longitudeField],
          ...item,
        };
      });

      const dataFormat = {
        ...polylineOption,
        path: polylineObj,
      };

      polylines.push(dataFormat);
    });

    return polylines || [];
  };
}
