/* eslint-disable @typescript-eslint/no-empty-function */
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { map } from 'rxjs';

@Injectable({ providedIn: 'root' })
export class MapService {
  private _map: any;
  set map(value: any) {
    this._map = value;
  }

  get map() {
    return this._map;
  }

  _defaultMapOptions = {
    zoom: 4,
    center: { lat: 37.6, lng: -95.665 },
    mapTypeControl: false,
    zoomControl: false,
    scaleControl: false,
    streetViewControl: false,
    fullscreenControl: false,
    panControl: false,
    height: '100%',
    mapMode: 'hybrid',
    disableDefaultUI: true,
    minZoom: 1,
    maxZoom: 20,
    tilt: 0,
    hideMapLayerControl: false,
    hideZoomControl: false,
    hideDrawingControl: false,
    hideSearchControl: false,
    iconButton: false,
    drawingMode: false,
    drawerSize: 'w-80',
    drawerHideClose: false,
    drawerClass: 'pl-0',
    drawerInnerClass: 'border-none rounded-none bg-white',
    hasGeofenceCreate: true,
    hasGeofenceEdit: true,
    hasGeofenceDelete: true,
    geofenceMode: 'create',
    disableMarkerClustering: false,
    maxZoomMarkerClustering: 15,
    preLoadAllGeofences: false,
    geofenceListApiUrl: '/api/v1/geofences',
    geofenceReadApiUrl: '/api/v1/geofences/{{data.geofence.uuid}}',
    geofenceUpdateApiUrl: '/api/v1/geofences/{{data.geofence.uuid}}',
    geofenceLabelField: 'geofenceName',
    assetLabelField: 'reeferName',
    geofenceLabelVisibilityZoomLevel: 14,
  };

  constructor(public http: HttpClient) {}

  getMapOptions(customMapOptions?: any) {
    const mapOptions = {
      ...this._defaultMapOptions,
      ...customMapOptions,
    };

    mapOptions['mapTypeId'] = mapOptions.mapMode;

    return mapOptions || {};
  }

  loadAllGeofences(apiUrl: any, data: any = {}) {
    // HTTP Headers
    const authenticationToken = localStorage.getItem('access_token');
    const headers = new HttpHeaders({
      'Content-Type': 'application/json',
      Authorization: 'Bearer ' + authenticationToken,
    });

    const options = {
      headers: headers,
      ...data,
    };

    return this.http
      .get(apiUrl, options)
      .pipe(map((res: any) => res.data?.rowData || []));
  }
}
