/* eslint-disable prefer-const */
import { Injectable } from '@angular/core';
import { MatDialog, MatDialogRef } from '@angular/material/dialog';
import { MoveEntityWebComponent } from '../move-entity-web.component';
import { Observable, catchError, map } from 'rxjs';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Router } from '@angular/router';
import { ToastService } from '@hash8io/core-web';


@Injectable({
  providedIn: 'root',
})
export class MoveEntityWebService {

  dialogRef!: MatDialogRef<MoveEntityWebComponent>;

  authenticationToken = localStorage.getItem('access_token');


  headers = new HttpHeaders({
    'Content-Type': 'application/json',
    Authorization: 'Bearer ' + this.authenticationToken,
  });

  constructor(public dialog: MatDialog, public http: HttpClient, public route: Router, public toast: ToastService) {

  }

  openDialog(modal: any): Observable<boolean> {
    this.dialogRef = this.dialog.open(MoveEntityWebComponent, {
      ...modal,
      minWidth: modal.minWidth || 500,
      maxWidth: modal.maxWidth || 600,
      autoFocus: modal.autoFocus || false,
      height: modal?.height,
      data: modal.data,
    });

    return this.dialogRef.afterClosed();
  }

  getHierarchyData(apiEndpoint: any, queryParams: any): any {
    let params: any = {}

    queryParams?.forEach((data: any) => {
      params[data?.key] = data?.value
    })

    const api = {
      params: params,
      headers: this.headers
    }

    return this.http.get(apiEndpoint, api).pipe(
      catchError(error => {
        if (error?.status === 401) {
          this.route.navigateByUrl('/login')
          this.toast.message('Session Expired')
        }
        return ([]);
      }), map((res: any) => {
        return res;
      }));
  }

  closeDialog() {
    this.dialogRef.close();
  }


}
