import { Component, Input, OnDestroy, OnInit } from '@angular/core';
import { UntypedFormGroup } from '@angular/forms';
import { Router } from '@angular/router';
import { AppStoreSelectors } from '@hash8io/core-web';
import { Store } from '@ngrx/store';
import { LoadingBarService } from '@ngx-loading-bar/core';
import { Observable, take, timer } from 'rxjs';
import { RegisterService } from '../../services/register.service';
import { verification } from '../../verification/verification-mock';
import { Location } from '@angular/common';

@Component({
  selector: 'hash8-platform-otp-verification',
  templateUrl: './otp-verification.component.html',
  styleUrls: ['./otp-verification.component.scss'],
})
export class OtpVerificationComponent implements OnInit, OnDestroy {
  branding!: Observable<any>;
  appSegments!: Observable<any>;

  emailOtp!: any;
  phoneNumberOtp!: any;

  email!: any;
  @Input() phoneNumber!: any;

  model!: any;
  HttpRef = 'http';

  @Input() verification!: any;
  @Input() verifyFrom!: any;

  registerForm!: any;
  validSmsOtp = false;
  resendTimer!: any;
  counter = 30;

  config = {
    allowNumbersOnly: false,
    length: 6,
    isPasswordInput: false,
    disableAutoFocus: false,
    placeholder: '',
    inputStyles: {
      width: '30px',
      height: '30px',
      'font-size': '14px',
      'margin-right': '5%',
      'border-radius': '0',
    },
  };
  constructor(
    private store: Store,
    public loader: LoadingBarService,
    public location: Location
  ) {
    this.verification = verification;
  }
  ngOnInit(): void {
    this.email = localStorage.getItem('email');
    this.verifyFrom = this.verifyFrom
      ? this.verifyFrom
      : localStorage.getItem('verifyFrom');

    this.phoneNumber = this.phoneNumber
      ? this.phoneNumber
      : localStorage.getItem('phoneNumber');
    localStorage.setItem('phoneNumber', this.phoneNumber);

    this.branding = this.store.select(AppStoreSelectors.selectBranding);

    this.branding.subscribe((data: any) => {
      this.registerForm = data?.registerForm;
    });

    this.resendTimer = timer(0, 1000)
      .pipe(take(this.counter))
      .subscribe(() => {
        --this.counter;
        if (this.counter == 0) {
          this.resendTimer.unsubscribe();
        }
      });
  }

  ngOnDestroy() {
    if (this.verifyFrom === 'registration') {
      //No Operation
    } else {
      localStorage.removeItem('phoneNumber');
    }
  }

  transform(value: number): string {
    const minutes: number = Math.floor(value / 60);
    return (
      ('00' + minutes).slice(-2) +
      ':' +
      ('00' + Math.floor(value - minutes * 60)).slice(-2)
    );
  }

  emailOtpChange(otp: any) {
    this.emailOtp = otp;
  }

  phoneNumberOtpChange(otp: any) {
    this.phoneNumberOtp = otp;
    otp.length === 6 ? (this.validSmsOtp = true) : (this.validSmsOtp = false);
    (
      document.getElementById('otpErrorMessage') as HTMLInputElement
    ).style.display = 'none';
  }

  modelChange(event: any) {
    //
  }
  onSubmit(verifyFrom: any) {
    let otpSubmitEvent!: any;
    if (verifyFrom === 'profile' || verifyFrom === null) {
      otpSubmitEvent = [
        {
          event: 'click',
          method: 'postAPI',
          params: {
            apiUrl: '/api/v1/user/verify-otp',
            bodyParams: [
              {
                key: 'verificationType',
                value: 'Phone Number',
              },
              {
                key: 'code',
                value: this.phoneNumberOtp,
              },
            ],
          },
          events: [
            {
              event: 'success', //FAILURE CASE
              method: 'show',
              // enabled: "!data.message ===  'Invalid OTP'",
              enabled: 'data.status === false',
              params: {
                id: 'otpErrorMessage',
              },
            },
            {
              event: 'failure',
              method: 'showErrorToast',
              params: {
                message: 'Error Verifying OTP',
              },
            },
            {
              event: 'success',
              method: 'showToast',
              // enabled: "!data.message ===  'Invalid OTP'",
              enabled: 'data.status === true',
              params: {
                type: 'success',
                message: 'Phone Number Verified!',
              },
            },
            {
              event: 'success',
              method: 'goToPage',
              enabled: 'data.status === true',
              params: {
                previousPage: true,
                // urlPath: '/myProfile',
              },
              type: '',
            },
          ],
        },
      ];
    } else if (verifyFrom === 'registration') {
      this.model = {
        // emailOtp: this.emailOtp,
        phoneNumberOtp: this.phoneNumberOtp,
      };

      let registrationAPI = '';
      let otpVerificationAPI = '';
      let bodyParams = [];

      const userRoleName = localStorage.getItem('userRoleName');
      switch (userRoleName) {
        case 'Organization Owner':
          registrationAPI = '/api/v1/customers';
          otpVerificationAPI = '/api/v1/customers/otp-verification';
          bodyParams = [
            {
              key: 'invitationUUID',
            },
            {
              key: 'organizationName',
            },
            {
              key: 'address',
            },
            {
              key: 'city',
            },
            {
              key: 'country',
            },
            {
              key: 'state',
            },
            {
              key: 'zipCode',
            },
            {
              key: 'firstName',
            },
            {
              key: 'lastName',
            },
            {
              key: 'username',
            },
            {
              key: 'email',
            },
            {
              key: 'phoneNumber',
            },
            {
              key: 'confirmPassword',
            },
            {
              key: 'password',
            },
          ];
          break;
        case 'Sales Channel Owner':
          registrationAPI = '/api/v1/sales-channels';
          otpVerificationAPI = '/api/v1/sales/otp-verification';
          bodyParams = [
            {
              key: 'invitationUUID',
            },
            {
              key: 'organizationName',
            },
            {
              key: 'address',
            },
            {
              key: 'city',
            },
            {
              key: 'country',
            },
            {
              key: 'state',
            },
            {
              key: 'zipCode',
            },
            {
              key: 'firstName',
            },
            {
              key: 'lastName',
            },
            {
              key: 'username',
            },
            {
              key: 'email',
            },
            {
              key: 'phoneNumber',
            },
            {
              key: 'confirmPassword',
            },
            {
              key: 'password',
            },
          ];
          break;
        default:
          registrationAPI = '/api/v1/users';
          otpVerificationAPI = '/api/v1/verify-otp';
          bodyParams = [
            {
              key: 'invitationUUID',
            },
            {
              key: 'firstName',
            },
            {
              key: 'lastName',
            },
            {
              key: 'username',
            },
            {
              key: 'email',
            },
            {
              key: 'phoneNumber',
            },
            {
              key: 'confirmPassword',
            },
            {
              key: 'password',
            },
          ];
          break;
      }

      otpSubmitEvent = [
        {
          event: 'click',
          method: 'postAPI',
          params: {
            apiUrl: otpVerificationAPI,
            bodyParams: [
              {
                key: 'invitationUUID',
              },
              {
                key: 'verificationType',
                value: 'Phone Number',
              },
              { key: 'email' },
              {
                key: 'code',
                value: this.phoneNumberOtp,
              },
            ],
          },
          events: [
            {
              event: 'success', //FAILURE CASE
              method: 'show',
              // enabled: "!data.message ===  'Invalid OTP'",
              enabled: 'data.status === false',
              params: {
                id: 'otpErrorMessage',
              },
            },
            {
              event: 'failure',
              method: 'showErrorToast',
              params: {
                message: 'Error Verifying OTP',
              },
            },
            {
              event: 'success',
              method: 'showToast',
              // enabled: "!data.message ===  'Invalid OTP'",
              enabled: 'data.status === true',
              params: {
                type: 'success',
                message: 'Phone Number Verified!',
              },
            },
            {
              event: 'success',
              method: 'postAPI',
              enabled: 'data.status === true',
              params: {
                apiUrl: registrationAPI,
                bodyParams: bodyParams,
                noAccessToken: true,
              },
              events: [
                {
                  event: 'success',
                  method: 'showToast',
                  params: {
                    type: 'information',
                    message:
                      'Registration successful. Please sign in with your credentials.',
                  },
                },
                {
                  event: 'success',
                  method: 'goToPage',
                  params: {
                    urlPath: '/login',
                  },
                  type: '',
                },
                {
                  event: 'failure',
                  method: 'showErrorToast',
                  params: {
                    message: 'Registration not successful. Please try again.',
                  },
                },
              ],
            },
          ],
        },
      ];
    }

    this.store.dispatch({
      type: '[Events] Process events',
      events: otpSubmitEvent,
    });
  }
  changeNumber() {
    localStorage.setItem('changeNumber', 'true');
  }

  resendOtp(phoneNumber: any) {
    let getOtpEvent = [];
    let otpAPI = '';
    let bodyParams = [];
    const userRoleName = localStorage.getItem('userRoleName');

    if (this.verifyFrom === 'registration') {
      switch (userRoleName) {
        case 'Organization Owner':
          otpAPI = '/api/v1/customers/otp';
          bodyParams = [
            {
              key: 'invitationUUID',
            },
            {
              key: 'verificationType',
              value: 'Phone Number',
            },
            {
              key: 'phoneNumber',
            },
            {
              key: 'email',
            },
          ];
          break;
        case 'Sales Channel Owner':
          otpAPI = '/api/v1/sales/otp';
          bodyParams = [
            {
              key: 'invitationUUID',
            },
            {
              key: 'verificationType',
              value: 'Phone Number',
            },
            {
              key: 'phoneNumber',
            },
            {
              key: 'email',
            },
          ];
          break;
        default:
          otpAPI = '/api/v1/otp';
          bodyParams = [
            {
              key: 'invitationUUID',
            },
            {
              key: 'verificationType',
              value: 'Phone Number',
            },
            {
              key: 'phoneNumber',
            },
          ];
          break;
      }
      getOtpEvent = [
        {
          event: 'success',
          method: 'postAPI',
          params: {
            apiUrl: otpAPI,
            bodyParams: bodyParams,
            noAccessToken: true,
          },
          events: [
            {
              event: 'success',
              method: 'showToast',
              params: {
                message: 'Verification code resent to your phone number',
              },
            },

            {
              event: 'failure',
              method: 'showErrorToast',
              params: {
                message: 'Error Generating OTP',
              },
            },
          ],
        },
      ];
    } else {
      getOtpEvent = [
        {
          event: 'success',
          method: 'postAPI',
          params: {
            apiUrl:
              this.verifyFrom === 'profile'
                ? '/api/v1/user/otp'
                : this.verifyFrom === 'registration'
                ? '/api/v1/otp'
                : '/api/v1/user/otp',
            bodyParams: [
              {
                key: 'invitationUUID',
              },
              {
                key: 'verificationType',
                value: 'Phone Number',
              },
              {
                key: 'phoneNumber',
                value: phoneNumber,
              },
            ],
            noAccessToken:
              this.verifyFrom === 'profile'
                ? false
                : this.verifyFrom === 'registration'
                ? true
                : false,
          },
          events: [
            {
              event: 'success',
              method: 'showToast',
              params: {
                message: 'Verification code resent to your Phone number',
              },
            },

            {
              event: 'failure',
              method: 'showErrorToast',
              params: {
                message: 'Error Generating OTP',
              },
            },
          ],
        },
      ];
    }
    this.store.dispatch({
      type: '[Events] Process events',
      events: getOtpEvent,
    });
  }
}
