import { NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { RegistrationComponent } from './registration/registration.component';
import { HeroContentComponent } from './components/hero-content/hero-content.component';
import { RegisterFooterComponent } from './components/register-footer/register-footer.component';
import { RegisterFormComponent } from './components/register-form/register-form.component';
import { RegisterHeaderComponent } from './components/register-header/register-header.component';
import { RegistrationWebRoutingModule } from './registration-web-routing.module';

import { FormsWebModule } from '@hash8io/forms-web';
import { LoadingBarLibraryModule } from '@hash8io/core-web/loading-bar';

import { SharedModule, FormlyLibraryModule } from '@hash8io/core-web';
import { LayoutModule } from '@hash8io/core-web/layout';
import { AppHeaderModule } from '@hash8io/core-web/app-header';
import { EffectsModule } from '@ngrx/effects';

import { StoreModule } from '@ngrx/store';
import {
  RegisterStoreSelectors,
  RegisterStoreReducer,
  RegisterStoreEffects,
} from './store';

import { MatProgressSpinnerModule } from '@angular/material/progress-spinner';
import { InvitationComponent } from './invitation/invitation.component';
import { VerificationComponent } from './verification/verification.component';

import { NgOtpInputModule } from 'ng-otp-input';
import { OtpVerificationComponent } from './components/otp-verification/otp-verification.component';
import { VerificationConfirmationComponent } from './verification/verification-confirmation/verification-confirmation.component';
import { MatIconModule } from '@angular/material/icon';

@NgModule({
  imports: [
    CommonModule,
    SharedModule,
    LayoutModule,
    AppHeaderModule,
    RegistrationWebRoutingModule,
    FormlyLibraryModule,
    StoreModule.forFeature(
      RegisterStoreSelectors.featureKey,
      RegisterStoreReducer.reducer
    ),
    EffectsModule.forFeature([RegisterStoreEffects]),
    LoadingBarLibraryModule,
    FormsWebModule,
    MatProgressSpinnerModule,
    NgOtpInputModule,
    MatIconModule,
  ],
  declarations: [
    RegistrationComponent,
    HeroContentComponent,
    RegisterFooterComponent,
    RegisterFormComponent,
    RegisterHeaderComponent,
    InvitationComponent,
    VerificationComponent,
    OtpVerificationComponent,
    VerificationConfirmationComponent,
  ],
  exports: [
    RegistrationComponent,
    HeroContentComponent,
    RegisterFooterComponent,
    RegisterFormComponent,
    RegisterHeaderComponent,
    InvitationComponent,
    VerificationComponent,
    VerificationConfirmationComponent,
    OtpVerificationComponent,
  ],
})
export class RegistrationWebModule {}
