//REGISTER_MOCK_CONFIG

export const REGISTRATION_FORM_ORGANIZATION = {
  fields: [
    {
      label: 'Organization',
      fields: [
        {
          type: '#template',
          template: `<h4 class="font-semibold text-md pb-1">Create your Organization</h4>
      <p class="text-sm text-gray-600 pb-3">Please provide your organization details.</p>`,
        },

        {
          key: 'organizationName',
          type: '#text',
          props: {
            label: 'Organization Name',
            required: true,
            hideLabel: true,
            disabled: true,
            // align: 'row',
          },
          extras: {
            labelOverInput: true,
          },
        },
        {
          key: 'address',
          type: '#text',
          props: {
            label: 'Address',
            required: false,
            hideLabel: true,
            // align: 'row',
          },
          defaultValue: '',
          extras: {
            labelOverInput: true,
          },
        },
        {
          key: 'country',
          type: '#text',
          props: {
            label: 'Country',
            hideLabel: true,
            required: true,
            // align: 'row',
          },
          extras: {
            labelOverInput: true,
          },
        },
        {
          key: 'state',
          type: '#text',
          props: {
            label: 'State',
            required: true,
            // async: true,
            // labelProp: 'country',
            // valueProp: 'id',
            // apiEndpoint:
            //   'https://63239166bb2321cba91c006d.mockapi.io/country',
            hideLabel: true,
            // align: 'row',
          },
          extras: {
            labelOverInput: true,
          },
        },
        {
          key: 'city',
          type: '#text',
          props: {
            label: 'City',
            required: true,
            // async: true,
            // labelProp: 'state',
            // valueProp: 'id',
            // apiEndpoint:
            //   'https://63239166bb2321cba91c006d.mockapi.io/country',
            hideLabel: true,
            // align: 'row',
          },
          extras: {
            labelOverInput: true,
          },
        },

        {
          key: 'zipCode',
          type: '#number',
          props: {
            label: 'Zip Code',
            mask: '0{5}',
            required: true,
            hideLabel: true,
            // align: 'row',
          },
          extras: {
            labelOverInput: true,
          },
        },
      ],
    },
    {
      label: 'Registration',

      fields: [
        {
          type: '#template',
          template: `<h4 class=" font-semibold text-md pb-1">Sign up into your organization</h4>
            <p class="text-sm text-gray-600 pb-3">Please provide your organization owner details.</p>`,
        },

        {
          type: '#text',
          className: 'w-full',
          key: 'firstName',
          props: {
            label: 'First Name',
            required: true,
            hideLabel: true,
          },
          extras: {
            labelOverInput: true,
          },
        },
        {
          type: '#text',
          className: 'w-full',
          key: 'lastName',
          props: {
            label: 'Last Name',
            required: true,
            hideLabel: true,
          },
          extras: {
            labelOverInput: true,
          },
        },
        {
          type: '#text',
          key: 'email',
          props: {
            label: 'Email',
            type: 'email',
            required: true,
            readonly: true,
            disabled: true,
            hideLabel: true,
          },
          extras: {
            labelOverInput: true,
          },
          defaultValue: 'johnmathew@gmail.com',
          expressions: {
            // 'props.disabled': '!model.username',
          },
        },
        {
          type: '#textbox-async',
          key: 'username',
          parsers: [(value: string) => value?.trim()],

          props: {
            label: 'Username',
            required: true,
            hideLabel: true,
            apiEndpoint: '/api/v1/username-unique-status',
            apiKey: 'username',
            errorMessage: `username already exists`,
            valueCheck: true,
            minLength: 3,
          },
          extras: {
            labelOverInput: true,
          },
        },
        {
          type: '#phoneNumber',
          key: 'phoneNumber',
          props: {
            hideLabel: true,
            // required: true,
            enableSearch: true,
            intlTelInput: true,
            // format: 'national',
          },
          defaultValue: '',

          extras: {
            labelOverInput: true,
          },
        },
        {
          validators: {
            validation: [
              {
                name: 'fieldMatch',
                options: { errorPath: 'confirmPassword' },
              },
            ],
          },
          fieldGroup: [
            {
              type: '#password',
              key: 'password',
              props: {
                hideLabel: true,
                required: true,
              },
              extras: {
                labelOverInput: true,
              },
              validators: {
                validation: [{ name: 'password' }],
              },
            },
            {
              type: '#password',
              key: 'confirmPassword',
              props: {
                label: 'Confirm Password',
                hideLabel: true,
                required: true,
              },
              extras: {
                labelOverInput: true,
              },
            },
          ],
        },
        {
          template: `<p class="text-xs -mt-1 mb-2 opacity-50">Note: Your password should meet all the following conditions.<br>
          * Atleast one upper case letter.<br>
          * Atleast one number.<br>
          * Min.8 characters in length.</p>`,
        },
      ],
    },
  ],
  options: {
    submit: {
      label: 'Next',
      events: [
        {
          event: 'success',
          method: 'postAPI',
          enabled:
            'data.formData.phoneNumber === null || data.formData.phoneNumber === undefined || data.formData.phoneNumber === "" ',
          params: {
            apiUrl: '/api/v1/customers',
            bodyParams: [
              {
                key: 'invitationUUID',
              },
            ],
            noAccessToken: true,
          },
          events: [
            {
              event: 'success',
              method: 'showToast',
              params: {
                type: 'information',
                message:
                  'Registration successful. Please sign in with your credentials.',
              },
            },
            {
              event: 'success',
              method: 'goToPage',
              params: {
                urlPath: '/login',
              },
              type: '',
            },
            {
              event: 'failure',
              method: 'showErrorToast',
              params: {
                message: 'Registration not successful. Please try again.',
              },
            },
          ],
        },
        {
          event: 'click',
          method: 'setLocalStorage',
          enabled:
            'data.formData.phoneNumber !== null && data.formData.phoneNumber !== undefined && data.formData.phoneNumber !== ""',
          params: {
            storeParams: [
              {
                key: 'organizationName',
                store: true,
                formData: true,
                storeName: 'organizationName',
              },
              {
                key: 'address',
                store: true,
                formData: true,
                storeName: 'address',
              },
              {
                key: 'city',
                store: true,
                formData: true,
                storeName: 'city',
              },
              {
                key: 'country',
                store: true,
                formData: true,
                storeName: 'country',
              },
              {
                key: 'state',
                store: true,
                formData: true,
                storeName: 'state',
              },
              {
                key: 'zipCode',
                store: true,
                formData: true,
                storeName: 'zipCode',
              },
              {
                key: 'username',
                store: true,
                formData: true,
                storeName: 'username',
              },
              {
                key: 'firstName',
                store: true,
                formData: true,
                storeName: 'firstName',
              },
              {
                key: 'lastName',
                store: true,
                formData: true,
                storeName: 'lastName',
              },
              {
                key: 'phoneNumber',
                store: true,
                formData: true,
                storeName: 'phoneNumber',
              },
              {
                key: 'password',
                store: true,
                formData: true,
                storeName: 'password',
              },
              {
                key: 'confirmPassword',
                store: true,
                formData: true,
                storeName: 'confirmPassword',
              },
            ],
          },
        },
        {
          event: 'click',
          method: 'goToPage',
          enabled:
            '!data.changeNumber && data.formData.phoneNumber !== null && data.formData.phoneNumber !== "" && data.formData.phoneNumber !== undefined',
          params: {
            urlPath: '/confirm-verification',
          },
          type: '',
        },
        {
          event: 'success',
          method: 'postAPI',
          enabled: 'data.changeNumber',
          params: {
            apiUrl: '/api/v1/otp',
            bodyParams: [
              {
                key: 'invitationUUID',
              },
              {
                key: 'verificationType',
                value: 'Phone Number',
              },
              {
                key: 'phoneNumber',
              },
            ],
          },
          events: [
            {
              event: 'success',
              method: 'showToast',
              params: {
                type: 'success',
                message: 'Verification code sent to your phone number',
              },
            },
            {
              event: 'success',
              method: 'goToPage',
              // enabled: 'data.changeNumber',
              params: {
                urlPath: '/verification',
              },
              type: '',
            },
            {
              event: 'failure',
              method: 'showErrorToast',
              params: {
                message: 'Error Generating Phone Number Verification Code',
              },
            },
          ],
        },
        // {
        //   event: 'success',
        //   method: 'postAPI',
        //   params: {
        //     apiUrl: '/api/v1/customers',
        //     bodyParams: [
        //       {
        //         key: 'invitationUUID',
        //       },
        //     ],
        //     noAccessToken: true,
        //   },
        //   events: [
        //     {
        //       event: 'success',
        //       method: 'showToast',
        //       params: {
        //         type: 'information',
        //         message:
        //           'Registration successful. Please sign in with your credentials.',
        //       },
        //     },
        //     {
        //       event: 'success',
        //       method: 'goToPage',
        //       params: {
        //         urlPath: '/login',
        //       },
        //       type: '',
        //     },
        //   ],
        // },
      ],
    },
    next: {
      label: 'next',
      events: [
        {
          event: 'click',
          method: 'setLocalStorage',
          params: {
            storeParams: [
              {
                key: 'organizationName',
                store: true,
                formData: true,
                storeName: 'organizationName',
              },
            ],
          },
        },
      ],
    },

    submitLabel: 'Next',
    hasReset: false,
    // cancel: {
    //   label: 'Cancel',
    //   events: [
    //     {
    //       event: 'click',
    //       method: 'closeDialog',
    //       params: {},
    //       type: '',
    //     },
    //   ],
    // },
  },
};

export const REGISTRATION_FORM = {
  fields: [
    {
      template: `<h4 class=" font-medium text-md pb-3 ">Sign up into AssetiQ</h4>`,
    },
    {
      type: '#text',
      className: 'w-full',
      key: 'firstName',
      props: {
        label: 'First Name',
        required: true,
        hideLabel: true,
      },
      extras: {
        labelOverInput: true,
      },
    },
    {
      type: '#text',
      className: 'w-full',
      key: 'lastName',
      props: {
        label: 'Last Name',
        required: true,
        hideLabel: true,
      },
      extras: {
        labelOverInput: true,
      },
    },
    {
      type: '#text',
      key: 'email',
      props: {
        label: 'Email',
        type: 'email',
        required: true,
        readonly: true,
        disabled: true,
        hideLabel: true,
      },
      extras: {
        labelOverInput: true,
      },
      defaultValue: 'johnmathew@gmail.com',
      expressions: {
        // 'props.disabled': '!model.username',
      },
    },
    {
      type: '#textbox-async',
      key: 'username',
      parsers: [(value: string) => value?.trim()],
      props: {
        label: 'Username',
        required: true,
        hideLabel: true,
        apiEndpoint: '/api/v1/username-unique-status',
        apiKey: 'username',
        errorMessage: `username already exists`,
        valueCheck: true,
        minLength: 3,
        async: true,
      },
      extras: {
        labelOverInput: true,
      },
    },
    {
      type: '#phoneNumber',
      key: 'phoneNumber',
      props: {
        hideLabel: true,
        // required: true,
        enableSearch: true,
        intlTelInput: true,
        // format: 'national',
      },
      defaultValue: '',

      extras: {
        labelOverInput: true,
      },
    },
    {
      validators: {
        validation: [
          {
            name: 'fieldMatch',
            options: { errorPath: 'confirmPassword' },
          },
        ],
      },
      fieldGroup: [
        {
          type: '#password',
          key: 'password',
          props: {
            hideLabel: true,
            required: true,
          },
          extras: {
            labelOverInput: true,
          },
          validators: {
            validation: [{ name: 'password' }],
          },
        },
        {
          type: '#password',
          key: 'confirmPassword',
          props: {
            label: 'Confirm Password',
            hideLabel: true,
            required: true,
          },
          extras: {
            labelOverInput: true,
          },
        },
      ],
    },
    {
      template: `<p class="text-xs -mt-1 mb-2 opacity-50">Note: Your password should meet all the following conditions.<br>
    * Atleast one upper case letter.<br>
    * Atleast one number.<br>
    * Min.8 characters in length.</p>`,
    },
  ],
  options: {
    submit: {
      label: 'Next',
      events: [
        {
          event: 'success',
          enabled:
            'data.formData.phoneNumber === null || data.formData.phoneNumber === undefined || data.formData.phoneNumber === "" ',
          method: 'postAPI',
          params: {
            apiUrl: '/api/v1/users',
            bodyParams: [
              {
                key: 'invitationUUID',
              },
            ],
          },
          events: [
            {
              event: 'success',
              method: 'showToast',
              params: {
                type: 'information',
                message:
                  'Registration successful. Please sign in with your credentials.',
              },
            },
            {
              event: 'success',
              method: 'goToPage',
              params: {
                urlPath: '/login',
              },
              type: '',
            },
            {
              event: 'failure',
              method: 'showErrorToast',
              params: {
                message: 'Registration not successful. Please try again.',
              },
            },
          ],
        },
        {
          event: 'click',
          method: 'setLocalStorage',
          enabled:
            'data.formData.phoneNumber !== null && data.formData.phoneNumber !== undefined && data.formData.phoneNumber !== ""',
          params: {
            storeParams: [
              {
                key: 'username',
                store: true,
                formData: true,
                storeName: 'username',
              },
              {
                key: 'firstName',
                store: true,
                formData: true,
                storeName: 'firstName',
              },
              {
                key: 'lastName',
                store: true,
                formData: true,
                storeName: 'lastName',
              },
              {
                key: 'phoneNumber',
                store: true,
                formData: true,
                storeName: 'phoneNumber',
              },
              {
                key: 'password',
                store: true,
                formData: true,
                storeName: 'password',
              },
              {
                key: 'confirmPassword',
                store: true,
                formData: true,
                storeName: 'confirmPassword',
              },
            ],
          },
        },
        {
          event: 'click',
          method: 'goToPage',
          enabled:
            '!data.changeNumber && data.formData.phoneNumber !== null && data.formData.phoneNumber !== "" && data.formData.phoneNumber !== undefined',
          params: {
            urlPath: '/confirm-verification',
          },
          type: '',
        },
        {
          event: 'success',
          method: 'postAPI',
          enabled: 'data.changeNumber',
          params: {
            apiUrl: '/api/v1/otp',
            bodyParams: [
              {
                key: 'invitationUUID',
              },
              {
                key: 'verificationType',
                value: 'Phone Number',
              },
              {
                key: 'phoneNumber',
              },
            ],
          },
          events: [
            {
              event: 'success',
              method: 'showToast',
              params: {
                type: 'success',
                message: 'Verification code sent to your phone number',
              },
            },
            {
              event: 'success',
              method: 'goToPage',
              // enabled: 'data.changeNumber',
              params: {
                urlPath: '/verification',
              },
              type: '',
            },
            {
              event: 'failure',
              method: 'showErrorToast',
              params: {
                message: 'Error Generating Phone Number Verification Code',
              },
            },
          ],
        },

        // {
        //   // events: [
        //   //   {
        //   event: 'success',
        //   method: 'postAPI',
        //   params: {
        //     apiUrl: '/api/v1/users',
        //     bodyParams: [
        //       {
        //         key: 'invitationUUID',
        //       },
        //     ],
        //   },
        //   events: [
        //     {
        //       event: 'success',
        //       method: 'showToast',
        //       params: {
        //         type: 'information',
        //         message:
        //           'Registration successful. Please sign in with your credentials.',
        //       },
        //     },
        //     {
        //       event: 'success',
        //       method: 'goToPage',
        //       params: {
        //         urlPath: '/login',
        //       },
        //       type: '',
        //     },
        //   ],
        //   //   },
        //   // ],
        // },
      ],
    },
    hasReset: false,
  },
};

export const REGISTRATION_FORM_SALES_CHANNEL = {
  fields: [
    {
      label: 'Organization',
      fields: [
        {
          type: '#template',
          template: `<h4 class="font-semibold text-md pb-1">Create your Organization</h4>
      <p class="text-sm text-gray-600 pb-3">Please provide your organization details.</p>`,
        },

        {
          key: 'organizationName',
          type: '#text',
          props: {
            label: 'Organization Name',
            required: true,
            hideLabel: true,
            disabled: true,
            // align: 'row',
          },
          extras: {
            labelOverInput: true,
          },
        },
        {
          key: 'address',
          type: '#text',
          props: {
            label: 'Address',
            required: false,
            hideLabel: true,
            // align: 'row',
          },
          defaultValue: '',
          extras: {
            labelOverInput: true,
          },
        },
        {
          key: 'country',
          type: '#text',
          props: {
            label: 'Country',
            hideLabel: true,
            required: true,
            // align: 'row',
          },
          extras: {
            labelOverInput: true,
          },
        },
        {
          key: 'state',
          type: '#text',
          props: {
            label: 'State',
            required: true,
            // async: true,
            // labelProp: 'country',
            // valueProp: 'id',
            // apiEndpoint:
            //   'https://63239166bb2321cba91c006d.mockapi.io/country',
            hideLabel: true,
            // align: 'row',
          },
          extras: {
            labelOverInput: true,
          },
        },
        {
          key: 'city',
          type: '#text',
          props: {
            label: 'City',
            required: true,
            // async: true,
            // labelProp: 'state',
            // valueProp: 'id',
            // apiEndpoint:
            //   'https://63239166bb2321cba91c006d.mockapi.io/country',
            hideLabel: true,
            // align: 'row',
          },
          extras: {
            labelOverInput: true,
          },
        },

        {
          key: 'zipCode',
          type: '#number',
          props: {
            label: 'Zip Code',
            mask: '0{5}',
            required: true,
            hideLabel: true,
            // align: 'row',
          },
          extras: {
            labelOverInput: true,
          },
        },
      ],
    },
    {
      label: 'Registration',

      fields: [
        {
          type: '#template',
          template: `<h4 class=" font-semibold text-md pb-1">Sign up into your organization</h4>
            <p class="text-sm text-gray-600 pb-3">Please provide your organization owner details.</p>`,
        },

        {
          type: '#text',
          className: 'w-full',
          key: 'firstName',
          props: {
            label: 'First Name',
            required: true,
            hideLabel: true,
          },
          extras: {
            labelOverInput: true,
          },
        },
        {
          type: '#text',
          className: 'w-full',
          key: 'lastName',
          props: {
            label: 'Last Name',
            required: true,
            hideLabel: true,
          },
          extras: {
            labelOverInput: true,
          },
        },
        {
          type: '#text',
          key: 'email',
          props: {
            label: 'Email',
            type: 'email',
            required: true,
            readonly: true,
            disabled: true,
            hideLabel: true,
          },
          extras: {
            labelOverInput: true,
          },
          defaultValue: 'johnmathew@gmail.com',
          expressions: {
            // 'props.disabled': '!model.username',
          },
        },
        {
          type: '#textbox-async',
          key: 'username',
          parsers: [(value: string) => value?.trim()],

          props: {
            label: 'Username',
            required: true,
            hideLabel: true,
            apiEndpoint: '/api/v1/username-unique-status',
            apiKey: 'username',
            errorMessage: `username already exists`,
            valueCheck: true,
            minLength: 3,
          },
          extras: {
            labelOverInput: true,
          },
        },
        {
          type: '#phoneNumber',
          key: 'phoneNumber',
          props: {
            hideLabel: true,
            // required: true,
            enableSearch: true,
            intlTelInput: true,
            // format: 'national',
          },
          defaultValue: '',

          extras: {
            labelOverInput: true,
          },
        },
        {
          validators: {
            validation: [
              {
                name: 'fieldMatch',
                options: { errorPath: 'confirmPassword' },
              },
            ],
          },
          fieldGroup: [
            {
              type: '#password',
              key: 'password',
              props: {
                hideLabel: true,
                required: true,
              },
              extras: {
                labelOverInput: true,
              },
              validators: {
                validation: [{ name: 'password' }],
              },
            },
            {
              type: '#password',
              key: 'confirmPassword',
              props: {
                label: 'Confirm Password',
                hideLabel: true,
                required: true,
              },
              extras: {
                labelOverInput: true,
              },
            },
          ],
        },
        {
          template: `<p class="text-xs -mt-1 mb-2 opacity-50">Note: Your password should meet all the following conditions.<br>
          * Atleast one upper case letter.<br>
          * Atleast one number.<br>
          * Min.8 characters in length.</p>`,
        },
      ],
    },
  ],
  options: {
    submit: {
      label: 'Next',
      events: [
        {
          event: 'click',
          method: 'setLocalStorage',
          enabled:
            'data.formData.phoneNumber !== null && data.formData.phoneNumber !== undefined && data.formData.phoneNumber !== ""',
          params: {
            storeParams: [
              {
                key: 'organizationName',
                store: true,
                formData: true,
                storeName: 'organizationName',
              },
              {
                key: 'address',
                store: true,
                formData: true,
                storeName: 'address',
              },
              {
                key: 'city',
                store: true,
                formData: true,
                storeName: 'city',
              },
              {
                key: 'country',
                store: true,
                formData: true,
                storeName: 'country',
              },
              {
                key: 'state',
                store: true,
                formData: true,
                storeName: 'state',
              },
              {
                key: 'zipCode',
                store: true,
                formData: true,
                storeName: 'zipCode',
              },
              {
                key: 'username',
                store: true,
                formData: true,
                storeName: 'username',
              },
              {
                key: 'firstName',
                store: true,
                formData: true,
                storeName: 'firstName',
              },
              {
                key: 'lastName',
                store: true,
                formData: true,
                storeName: 'lastName',
              },
              {
                key: 'phoneNumber',
                store: true,
                formData: true,
                storeName: 'phoneNumber',
              },
              {
                key: 'password',
                store: true,
                formData: true,
                storeName: 'password',
              },
              {
                key: 'confirmPassword',
                store: true,
                formData: true,
                storeName: 'confirmPassword',
              },
            ],
          },
        },
        {
          event: 'success',
          method: 'postAPI',
          enabled:
            'data.formData.phoneNumber === null || data.formData.phoneNumber === undefined || data.formData.phoneNumber === "" ',
          params: {
            apiUrl: '/api/v1/sales-channels',
            bodyParams: [
              {
                key: 'invitationUUID',
              },
            ],
            noAccessToken: true,
          },
          events: [
            {
              event: 'success',
              method: 'showToast',
              params: {
                type: 'information',
                message:
                  'Registration successful. Please sign in with your credentials.',
              },
            },
            {
              event: 'success',
              method: 'goToPage',
              params: {
                urlPath: '/login',
              },
              type: '',
            },
            {
              event: 'failure',
              method: 'showErrorToast',
              params: {
                message: 'Registration not successful. Please try again.',
              },
            },
          ],
        },
        {
          event: 'click',
          method: 'goToPage',
          enabled:
            '!data.changeNumber && data.formData.phoneNumber !== null && data.formData.phoneNumber !== "" && data.formData.phoneNumber !== undefined',
          params: {
            urlPath: '/confirm-verification',
          },
          type: '',
        },
        {
          event: 'success',
          method: 'postAPI',
          enabled: 'data.changeNumber',
          params: {
            apiUrl: '/api/v1/otp',
            bodyParams: [
              {
                key: 'invitationUUID',
              },
              {
                key: 'verificationType',
                value: 'Phone Number',
              },
              {
                key: 'phoneNumber',
              },
            ],
          },
          events: [
            {
              event: 'success',
              method: 'showToast',
              params: {
                type: 'success',
                message: 'Verification code sent to your phone number',
              },
            },
            {
              event: 'success',
              method: 'goToPage',
              // enabled: 'data.changeNumber',
              params: {
                urlPath: '/verification',
              },
              type: '',
            },
            {
              event: 'failure',
              method: 'showErrorToast',
              params: {
                message: 'Error Generating Phone Number Verification Code',
              },
            },
          ],
        },
        // {
        //   event: 'success',
        //   method: 'postAPI',
        //   params: {
        //     apiUrl: '/api/v1/sales-channels',
        //     bodyParams: [
        //       {
        //         key: 'invitationUUID',
        //       },
        //     ],
        //     noAccessToken: true,
        //   },
        //   events: [
        //     {
        //       event: 'success',
        //       method: 'showToast',
        //       params: {
        //         type: 'information',
        //         message:
        //           'Registration successful. Please sign in with your credentials.',
        //       },
        //     },
        //     {
        //       event: 'success',
        //       method: 'goToPage',
        //       params: {
        //         urlPath: '/login',
        //       },
        //       type: '',
        //     },
        //   ],
        // },
      ],
    },
    next: {
      label: 'next',
      events: [
        {
          event: 'click',
          method: 'setLocalStorage',
          params: {
            storeParams: [
              {
                key: 'organizationName',
                store: true,
                storeName: 'organizationName',
              },
            ],
          },
        },
      ],
    },

    submitLabel: 'Next',
    hasReset: false,
    // cancel: {
    //   label: 'Cancel',
    //   events: [
    //     {
    //       event: 'click',
    //       method: 'closeDialog',
    //       params: {},
    //       type: '',
    //     },
    //   ],
    // },
  },
};
