import { HttpClient } from '@angular/common/http';
import { Component, Input, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { AppStoreSelectors } from '@hash8io/core-web';
import { Store } from '@ngrx/store';
import { Observable } from 'rxjs';
import {
  REGISTRATION_FORM,
  REGISTRATION_FORM_ORGANIZATION,
  REGISTRATION_FORM_SALES_CHANNEL,
} from './registration-mock-config';

@Component({
  selector: 'hash8-platform-registration',
  templateUrl: './registration.component.html',
  styleUrls: ['./registration.component.scss'],
})
export class RegistrationComponent implements OnInit {
  @Input() template = 'default';

  branding!: Observable<any>;
  heroContent!: any;

  registerForm!: any;

  page = 'validation';

  registerFormData: any;

  emailValue!: any;

  appSegments!: any;

  validationStoreData!: any;

  registerFormConfig!: string;

  constructor(
    private route: ActivatedRoute,
    private store: Store,
    private http: HttpClient,
    private router: Router
  ) {}

  ngOnInit(): void {
    this.branding = this.store.select(AppStoreSelectors.selectBranding);

    this.branding?.subscribe((data: any) => {
      this.heroContent = data?.heroSection;
    });

    this.appSegments = this.store.select(
      AppStoreSelectors.selectApplicationSegments
    );

    // this.registerForm = REGISTRATION_FORM

    const userRole = localStorage.getItem('userRoleName');
    if (userRole === 'Organization Owner') {
      this.registerFormConfig = 'stepper';

      const formOption = REGISTRATION_FORM_ORGANIZATION?.options;

      const formFields = REGISTRATION_FORM_ORGANIZATION?.fields;

      const updatedFormFields = formFields.map((fields: any) => {
        fields?.fields?.map((obj: any) => {
          if (obj.key === 'email') {
            obj.defaultValue = localStorage.getItem('email');
            return obj;
          }
          if (obj.key === 'organizationName') {
            obj.defaultValue = localStorage.getItem('organizationName');
            return obj;
          }
          if (obj.key === 'address') {
            obj.defaultValue = localStorage.getItem('address');
            return obj;
          }
          if (obj.key === 'city') {
            obj.defaultValue = localStorage.getItem('city');
            return obj;
          }
          if (obj.key === 'country') {
            obj.defaultValue = localStorage.getItem('country');
            return obj;
          }
          if (obj.key === 'state') {
            obj.defaultValue = localStorage.getItem('state');
            return obj;
          }
          if (obj.key === 'zipCode') {
            obj.defaultValue = localStorage.getItem('zipCode');
            return obj;
          }
          if (obj.key === 'firstName') {
            obj.defaultValue = localStorage.getItem('firstName');
            return obj;
          }
          if (obj.key === 'lastName') {
            obj.defaultValue = localStorage.getItem('lastName');
            return obj;
          }
          if (obj.key === 'username') {
            obj.defaultValue = localStorage.getItem('username');
            return obj;
          }
          if (obj.key === 'phoneNumber') {
            obj.defaultValue =
              localStorage.getItem('phoneNumber') === null
                ? ''
                : localStorage.getItem('phoneNumber');
            obj.focus = true;

            return obj;
          } else {
            return obj;
          }
        });
        return fields;
      });

      this.registerForm = {
        fields: updatedFormFields,
        options: formOption,
      };
    } else if (userRole === 'Sales Channel Owner') {
      this.registerFormConfig = 'stepper';

      const formOption = REGISTRATION_FORM_SALES_CHANNEL?.options;

      const formFields = REGISTRATION_FORM_SALES_CHANNEL?.fields;

      const updatedFormFields = formFields.map((fields: any) => {
        fields?.fields?.map((obj: any) => {
          if (obj.key === 'email') {
            obj.defaultValue = localStorage.getItem('email');
            return obj;
          }
          if (obj.key === 'organizationName') {
            obj.defaultValue = localStorage.getItem('organizationName');
            return obj;
          }
          if (obj.key === 'address') {
            obj.defaultValue = localStorage.getItem('address');
            return obj;
          }
          if (obj.key === 'city') {
            obj.defaultValue = localStorage.getItem('city');
            return obj;
          }
          if (obj.key === 'country') {
            obj.defaultValue = localStorage.getItem('country');
            return obj;
          }
          if (obj.key === 'state') {
            obj.defaultValue = localStorage.getItem('state');
            return obj;
          }
          if (obj.key === 'zipCode') {
            obj.defaultValue = localStorage.getItem('zipCode');
            return obj;
          }
          if (obj.key === 'firstName') {
            obj.defaultValue = localStorage.getItem('firstName');
            return obj;
          }
          if (obj.key === 'lastName') {
            obj.defaultValue = localStorage.getItem('lastName');
            return obj;
          }
          if (obj.key === 'username') {
            obj.defaultValue = localStorage.getItem('username');
            return obj;
          }
          if (obj.key === 'phoneNumber') {
            localStorage.getItem('phoneNumber') === null
              ? ''
              : localStorage.getItem('phoneNumber');
            obj.focus = true;
            return obj;
          } else {
            return obj;
          }
        });
        return fields;
      });

      this.registerForm = {
        fields: updatedFormFields,
        options: formOption,
      };
    } else {
      const formOption = REGISTRATION_FORM?.options;

      const formFields = REGISTRATION_FORM?.fields;

      const updatedFormFields = formFields.map((obj: any) => {
        if (obj.key === 'email') {
          obj.defaultValue = localStorage.getItem('email');
          return obj;
        }
        if (obj.key === 'firstName') {
          obj.defaultValue = localStorage.getItem('firstName');
          return obj;
        }
        if (obj.key === 'lastName') {
          obj.defaultValue = localStorage.getItem('lastName');
          return obj;
        }
        if (obj.key === 'username') {
          obj.defaultValue = localStorage.getItem('username');

          return obj;
        }
        if (obj.key === 'phoneNumber') {
          localStorage.getItem('phoneNumber') === null
            ? ''
            : localStorage.getItem('phoneNumber');
          obj.focus = true;

          return obj;
        } else {
          return obj;
        }
      });

      this.registerForm = {
        fields: updatedFormFields,
        options: formOption,
      };
    }

    //Registration form
    // const formOption = REGISTRATION_FORM?.options

    // const formFields = REGISTRATION_FORM?.fields

    // const updatedFormFields = formFields.map((obj: any) => {
    //   if (obj.key === 'email') {
    //     obj.defaultValue = localStorage.getItem('email')
    //     return obj
    //   }
    //   else {
    //     return obj
    //   }
    // })

    // this.registerForm = {
    //   fields: updatedFormFields,
    //   options: formOption
    // }

    this.route.data.subscribe((data) => {
      this.template = data['template'] || this.template;
      this.page = data['page'];
    });

    //
  }
}
