import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { AppStoreActions, AppStoreSelectors } from '@hash8io/core-web'
import * as RegisterStoreAction from '../store/actions'
import * as RegisterStoreSelectors from '../store/selectors'
import { Store } from '@ngrx/store';
import { map, Observable, of } from 'rxjs';

@Injectable({
  providedIn: 'root',
})
export class RegisterService {
  apiUrl = '';
  isAuthenticate = false;


  authenticationToken = localStorage.getItem('access_token')

  headers = new HttpHeaders({
    'Content-Type': 'application/json',
    Authorization: 'Bearer ' + this.authenticationToken
  })

  constructor(
    private http: HttpClient,
    private store: Store,
    private router: Router
  ) {
  }

  register(user: any) {
    let endPoint = '';
    let redirectUrl = '';
    let events: any;
    this.store
      .select(AppStoreSelectors.selectApplicationSegments)
      .pipe(map((segments) => segments.register))
      .subscribe((register) => {
        this.apiUrl = register?.apiUrl
        redirectUrl = register.redirectUrl;
        endPoint = register.apiUrl;
        events = register?.events

      });
    return this.http
      .post(`${this.apiUrl}`, user, { headers: this.headers })
      .pipe(
        map((response: any) => {
          if (response?.data?.access_token) {

            events?.forEach((x: any) => {
              const functionName = x?.method;

              if (functionName === 'clearValue') {
                this.clearValue(x?.params);
              }
              if (functionName === 'redirectUrl') {
                // this.redirectUrl(x?.params)
                window.location.href = redirectUrl

              }


            });

          }
          return response;
        }, (error: any) => {
          return error
          // return of(false)
        })
      );
  }

  clearValue(params: any) {
    localStorage.removeItem(params.key)
  }

  redirectUrl(params: any) {
    this.router.navigateByUrl(params.url)

  }

  setValue(params: any) {
    localStorage.setItem(params?.key, params?.value)
  }



}
