import { HttpClient } from '@angular/common/http';
import { Component, Input, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { AppStoreSelectors } from '@hash8io/core-web';
import { Store } from '@ngrx/store';
import { Observable } from 'rxjs';

@Component({
  selector: 'hash8-platform-verification-confirmation',
  templateUrl: './verification-confirmation.component.html',
  styleUrls: ['./verification-confirmation.component.scss'],
})
export class VerificationConfirmationComponent implements OnInit {
  @Input() template: string = 'default';

  branding!: Observable<any>;
  heroContent!: any;

  emailValue!: any;

  appSegments!: any;

  verification!: Observable<any>;

  registerForm!: any;
  HttpRef = 'http';
  buttonIdentity = '';

  constructor(
    private route: ActivatedRoute,
    private store: Store,
    private http: HttpClient,
    private router: Router
  ) {}

  ngOnInit(): void {
    this.branding = this.store.select(AppStoreSelectors.selectBranding);

    this.branding?.subscribe((data: any) => {
      this.heroContent = data?.heroSection;
    });

    this.appSegments = this.store.select(
      AppStoreSelectors.selectApplicationSegments
    );

    this.appSegments.subscribe((application: any) => {
      this.verification = application?.verification;
    });

    this.branding.subscribe((data: any) => {
      this.registerForm = data?.registerForm;
    });

    this.route.data.subscribe((data) => {
      this.template = data['template'] || this.template;
    });
  }

  onSubmit() {
    this.buttonIdentity = 'otp';
    localStorage.setItem('verifyFrom', 'registration');
    let otpAPI = '';
    let bodyParams = [];
    const userRoleName = localStorage.getItem('userRoleName');
    switch (userRoleName) {
      case 'Organization Owner':
        otpAPI = '/api/v1/customers/otp';
        bodyParams = [
          {
            key: 'invitationUUID',
          },
          {
            key: 'verificationType',
            value: 'Phone Number',
          },
          {
            key: 'phoneNumber',
          },
          {
            key: 'email',
          },
        ];
        break;
      case 'Sales Channel Owner':
        otpAPI = '/api/v1/sales/otp';
        bodyParams = [
          {
            key: 'invitationUUID',
          },
          {
            key: 'verificationType',
            value: 'Phone Number',
          },
          {
            key: 'phoneNumber',
          },
          {
            key: 'email',
          },
        ];
        break;
      default:
        otpAPI = '/api/v1/otp';
        bodyParams = [
          {
            key: 'invitationUUID',
          },
          {
            key: 'verificationType',
            value: 'Phone Number',
          },
          {
            key: 'phoneNumber',
          },
        ];
        break;
    }

    const getOtpEvent = [
      {
        event: 'success',
        method: 'postAPI',
        params: {
          apiUrl: otpAPI,
          bodyParams: bodyParams,
          noAccessToken: true,
        },
        events: [
          {
            event: 'success',
            method: 'showToast',
            params: {
              type: 'success',
              message: 'Verification code sent to your phone number',
            },
          },
          {
            event: 'success',
            method: 'goToPage',
            params: {
              urlPath: '/verification',
            },
            type: '',
          },
          {
            event: 'failure',
            method: 'showErrorToast',
            params: {
              message: 'Error Generating Verification Code',
            },
          },
        ],
      },
    ];
    this.store.dispatch({
      type: '[Events] Process events',
      events: getOtpEvent,
    });
  }

  register() {
    this.buttonIdentity = 'registration';

    let registrationAPI = '';
    let bodyParams = [];
    const userRoleName = localStorage.getItem('userRoleName');
    switch (userRoleName) {
      case 'Organization Owner':
        registrationAPI = '/api/v1/customers';
        bodyParams = [
          {
            key: 'invitationUUID',
          },
          {
            key: 'organizationName',
          },
          {
            key: 'address',
          },
          {
            key: 'city',
          },
          {
            key: 'country',
          },
          {
            key: 'state',
          },
          {
            key: 'zipCode',
          },
          {
            key: 'firstName',
          },
          {
            key: 'lastName',
          },
          {
            key: 'username',
          },
          {
            key: 'email',
          },
          {
            key: 'phoneNumber',
          },
          {
            key: 'confirmPassword',
          },
          {
            key: 'password',
          },
        ];
        break;
      case 'Sales Channel Owner':
        registrationAPI = '/api/v1/sales-channels';
        bodyParams = [
          {
            key: 'invitationUUID',
          },
          {
            key: 'organizationName',
          },
          {
            key: 'address',
          },
          {
            key: 'city',
          },
          {
            key: 'country',
          },
          {
            key: 'state',
          },
          {
            key: 'zipCode',
          },
          {
            key: 'firstName',
          },
          {
            key: 'lastName',
          },
          {
            key: 'username',
          },
          {
            key: 'email',
          },
          {
            key: 'phoneNumber',
          },
          {
            key: 'confirmPassword',
          },
          {
            key: 'password',
          },
        ];
        break;
      default:
        registrationAPI = '/api/v1/users';
        bodyParams = [
          {
            key: 'invitationUUID',
          },
          {
            key: 'firstName',
          },
          {
            key: 'lastName',
          },
          {
            key: 'username',
          },
          {
            key: 'email',
          },
          {
            key: 'phoneNumber',
          },
          {
            key: 'confirmPassword',
          },
          {
            key: 'password',
          },
        ];
        break;
    }
    const registerEvent = [
      {
        event: 'success',
        method: 'postAPI',
        params: {
          apiUrl: registrationAPI,
          bodyParams: bodyParams,
          noAccessToken: true,
        },
        events: [
          {
            event: 'success',
            method: 'showToast',
            params: {
              type: 'information',
              message:
                'Registration successful. Please sign in with your credentials.',
            },
          },
          {
            event: 'success',
            method: 'goToPage',
            params: {
              urlPath: '/login',
            },
            type: '',
          },
          {
            event: 'failure',
            method: 'showErrorToast',
            params: {
              message: 'Registration not successful. Please try again.',
            },
          },
        ],
      },
    ];
    this.store.dispatch({
      type: '[Events] Process events',
      events: registerEvent,
    });
  }
}
