import { Model } from '@hash8io/core-web';
import { Actions } from './actions';
import { TileLayout } from './layout';
import { ReportMenu } from './reports';
import { Title } from './title';
import { Toolbar } from './toolbar';

export const SnapshotNew: Model.SnapshotModel = {
  title: Title,
  toolbar: Toolbar,
  layout: TileLayout,
  config: {
    cards: {
      details: {
        header: {
          title: {
            label: 'Tank Details',
            iconUrl: 'info',
          },
          actions: [Actions[6], Actions[1]],
        },
        content: [
          {
            type: 'row',
            props: [
              {
                type: 'text',
                props: {
                  type: 'text',
                  label: 'Last Reported Time',
                  value: '06/08/2022 03:10 AM (EST)',
                  key: 'lastReportedTime',
                  align: 'column',
                },
              },
              {
                type: 'text',
                props: {
                  type: 'tag',
                  key: 'Image',
                  label: '',
                  options: {
                    color: 'green',
                  },
                  value: '',
                  align: 'column',
                },
              },
            ],
          },
          {
            type: 'text',
            props: {
              type: 'tag',
              key: 'status',
              label: 'Reporting Status',
              options: {
                color: 'green',
              },
              value: 'Reporting',
              align: 'column',
            },
          },

          {
            type: 'text',
            props: {
              type: 'text',
              label: 'Location',
              value: '2101 Richland Avenue, Houston, Texas 77077, USA',
              key: 'location',
              align: 'column',
            },
          },
          // {
          //   type: 'grid',
          //   props: {
          //     reportUUID: '300',
          //     showToolbar: false,
          //   },
          // },
        ],
        footer: undefined,
        options: {
          showTitle: true,
          showSearch: false,
          showSettings: false,
          showBreadcrumb: false,
          actionOptions: {
            showLabel: false,
            stroked: false,
          },
        },
        type: Model.CardType.LeftRight,
      },
      properties: {
        header: {
          title: {
            label: 'Tank Properties',
            iconUrl: 'description',
          },
          actions: [Actions[3]],
        },
        content: [
          {
            type: 'text',
            props: {
              label: 'Usage Type',
              value: 'Injection',
            },
          },
          {
            type: 'text',
            props: {
              type: 'tag',
              key: 'tankShape',
              label: 'Tank Shape',
              value: 'Vertical Cylinder',
              options: {
                color: 'green',
              },
            },
          },
          {
            type: 'text',
            props: {
              type: 'text',
              label: 'Tank Height',
              value: '174 inches',
            },
          },
          {
            type: 'text',
            props: {
              type: 'text',
              label: 'Tank Diameter',
              value: '142.63 inches',
            },
          },
          {
            type: 'text',
            props: {
              type: 'text',
              label: 'Tank Capacity',
              value: '45557.83 litres',
            },
          },
          {
            type: 'text',
            props: {
              type: 'text',
              label: 'Liquid Type',
              value: 'WATER',
            },
          },
          {
            type: 'text',
            props: {
              type: 'text',
              label: 'Specific Gravity',
              value: '1',
            },
          },
        ],
      },
      status: {
        header: {
          title: {
            label: 'Tank Status',
            iconUrl: 'description',
          },
          actions: [
            {
              label: 'Change Tank Settings',
              targetUrl: '/tank-history',
              visible: true,
              clickable: true,
              type: Model.MenuType.Link,
              iconUrl: 'layers',
              configApiUrl: '',
              apiUrl: '',
              hasSubMenu: false,
              isPrimaryAction: false,
            },
          ],
        },
        content: [
          {
            type: 'row',
            props: [
              {
                type: 'kpi',
                props: {
                  label: 'Liquid Level',
                  value: '0.06 inches',
                },
              },
              {
                type: 'kpi',
                props: {
                  label: 'Liquid Volume',
                  value: '15.70 litres',
                },
              },
              {
                type: 'kpi',
                props: {
                  label: 'Liquid Volume',
                  value: '15.70 litres',
                },
              },
              {
                type: 'kpi',
                props: {
                  label: 'Temperature',
                  value: '13• C',
                },
              },
            ],
          },
          {
            type: 'grid',
            props: {
              reportUUID: '400',
              showToolbar: false,
            },
          },
        ],
        footer: undefined,
        options: {
          showTitle: true,
          showSearch: false,
          actionOptions: {
            showLabel: false,
            stroked: false,
          },
        },
        type: Model.CardType.TopBottom,
      },
      deviceDetails: {
        header: {
          title: {
            label: 'Device Details',
            iconUrl: 'memory',
          },
          actions: [
            {
              label: 'Deactivate Device',
              targetUrl: '/netusage',
              visible: true,
              clickable: true,
              type: Model.MenuType.Link,
              iconUrl: 'edit',
              configApiUrl: '',
              apiUrl: '',
              hasSubMenu: false,
            },
          ],
        },
        content: [
          // {
          //   type: 'breadcrumb',
          //   props: {
          //     breadcrumbs: [
          //       {
          //         label: 'My Organizations',
          //         targetUrl: '/myorganization',
          //         value: '',
          //       },
          //       {
          //         label: 'Organization',
          //         targetUrl: '/organization',
          //         value: 'Autocavo',
          //       },
          //     ],
          //   },
          // },

          {
            type: 'text',
            props: {
              label: 'Device ID',
              value: '01778378SKYC42F',
            },
          },
          {
            type: 'text',
            props: {
              label: 'Device Model',
              value: 'TiQ.115',
            },
          },
          // {
          //   type: 'grid',
          //   props: {
          //     reportUUID: 100,
          //     showToolbar: false,
          //   },
          // },

          {
            type: 'text',
            props: {
              label: 'Sensor Offset',
              value: '1',
            },
          },
          {
            type: 'text',
            props: {
              label: 'Battery',
              value: '17.5 v',
            },
          },
        ],
        options: {
          showTitle: true,
          showSearch: false,
          showActions: false,
          actionOptions: {
            showLabel: false,
            stroked: false,
          },
        },
      },
      visualization: {
        content: [
          {
            type: 'title',
            props: {
              title: {
                label: 'Tank Visualization Component',
                iconUrl: 'help_outline',
              },
              iconSize: 'xs',
            },
          },
          {
            type: 'tankImage',
            props: {
              tank: [
                {
                  id: 'asset-4e61ec85-f0ae-441b-9669-011dfb493433',
                  tankName: 'TA-CORR12517A',
                  lastMessageTime: '05/17/2022 11:34 AM',
                  lastKnownLocation:
                    '2101 Richland Avenue, Houston, Texas 77077, USA',
                  address: '2101 Richland Avenue, Houston, Texas 77077, USA',
                  location: '2101 Richland Avenue, Houston, Texas 77077, USA',
                  reasonCode: 'Scheduled Message',
                  tankType: 'Vertical Cylinder',
                  liquidLevel: 145.03,
                  liquidType: 'Chemical',
                  liquidCode: 'CORR12517A',
                  tagName: 'Sensor Type :Temperature, Pressure',
                  sensorType: ['Temperature', 'Pressure'],
                  liquidVolume: 38705.62,
                  availableCapacity: 24889.78,
                  capacityPercentage: '60.86',
                  liquidWeight: 33286.8,
                  volumeChange: 12.3848,
                  temperature: '51.71',
                  sensorReading: 0.29,
                  battery: 7.58,
                  capacity: 63595.39,
                  reportingStatus: 'Not Reporting',
                  deviceId: '300234066887470',
                  sensorIndex: 1,
                  deviceModel: 'TiQ.200',
                  accountName: 'CHAMPIONX',
                  organizationName: 'CHAMPIONX',
                  netDailyUsage: 1232.91,
                  tankShape: 'Vertical Cylinder',
                  tankDiameter: 238.29,
                  tankHeight: 144.25,
                  specificGravity: 0.86,
                  latitude: 29.74101,
                  longitude: -95.6119,
                  sensorOffset: 10.6339,
                  deviceType: 'TIQ-200',
                  targetInjectionRate: 40.0,
                  labels: 'Litres',
                  refillPoint: 25000,
                  injectionRateAverageFirst: 'One Day',
                  injectionRateAverageSecond: 'Three Days',
                  injectionRateAverageThird: 'Seven Days',
                  injectionRateAverageFourth: 'Fourteen Days',
                  injectionRateAverageFirstValue: 1232.91,
                  injectionRateAverageSecondValue: 680.81,
                  injectionRateAverageThirdValue: 426.98,
                  injectionRateAverageFourthValue: 224.16,
                  deltaFirst: 2982.27,
                  deltaSecond: 1602.04,
                  deltaThird: 967.46,
                  deltaFourth: 460.41,
                  daysEmpty: 56,
                  percentageOfVolume: 40,
                  batteryVoltage: 7.58,
                  movingType: 'Static Storage Tank',
                  usageType: 'Injection',
                  shapeType: 'Cylindrical',
                  highAlarmValue: '57235.43',
                  highAlarm: '365.76',
                  lowAlarmValue: '6345.71',
                  lowAlarm: '40.64',
                  tankImage:
                    'https://teezle-tankiq-v2.s3.amazonaws.com/Screen%20Shot%202020-06-23%20at%208-1592915026400.png',
                },
              ],
            },
          },
        ],
      },
      reports: {
        content: [
          {
            type: 'report-card',
            props: {
              reportNavigations: ReportMenu,
            },
          },
        ],
      },
    },
  },
  reports: ReportMenu,
};
