/* eslint-disable @typescript-eslint/no-empty-function */
/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
import {
  Component,
  Input,
  OnChanges,
  OnInit,
  SimpleChanges,
} from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { TemplateService } from '@hash8io/events-web';
import { Store } from '@ngrx/store';

@Component({
  selector: 'hash8-platform-tags-list-page',
  templateUrl: './tags-list-page.component.html',
  styleUrls: ['./tags-list-page.component.scss'],
})
export class TagsListPageComponent implements OnInit, OnChanges {
  _config: any;
  _tagStatusConfiguration!: any;

  @Input() config: any;

  constructor(
    private store: Store,
    public templateService: TemplateService,
    private route: ActivatedRoute
  ) {
    this.route.data.subscribe((data) => {
      this._config = data['config'];
      this.setTagConfiguration(this._config);
    });
  }

  ngOnInit(): void {
    if (this.config) {
      this._config = this.config;
      this.setTagConfiguration(this._config);
    }
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes['config'] && changes['config'].currentValue) {
      this._config = changes['config'].currentValue;
      this.setTagConfiguration(this._config);
    }
  }

  setTagConfiguration(config: any) {
    if (config) {
      const { organizationFilterConfig, ...gridConfig } = config;
      this._tagStatusConfiguration = Object.assign(
        {},
        this.templateService.processTemplate(gridConfig),
        { organizationFilterConfig }
      );
    }
  }

  onTagSelected(selectedTags: any) {
    if (selectedTags.length > 0) {
      const selectedTagUUID = selectedTags.map((t: any) => t.uuid);
      console.log('Send to backend api', selectedTagUUID);
    }
  }

  refreshGrid() {
    this.store.dispatch({
      type: '[Data Report] Refresh data report',
      gridName: 'tagsHierarchyDataGrid',
    });
  }
}
