﻿/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
import {
  Component,
  EventEmitter,
  Input,
  Output,
  ViewChild,
  TemplateRef,
  ViewContainerRef,
  OnInit,
  OnChanges,
  SimpleChanges,
} from '@angular/core';
import { TreeviewHeaderTemplateContext } from '../../models/tree-view-header-template-context';
import { TreeviewItemTemplateContext } from '../../models/tree-view-item-template-context';
import { TreeviewItem } from '../../models/tree-view-item';
import { TreeviewConfig } from '../../models/tree-view-config';
import { TreeViewComponent } from '../tree-view/tree-view.component';
import { TreeviewI18n } from '../../models/tree-view-i18n';
import { TemplateService } from '@hash8io/events-web';
import { DropdownTreeviewService } from './dropdown-treeview.service';
import { Store } from '@ngrx/store';

@Component({
  selector: 'hash8-platform-dropdown-treeview',
  templateUrl: './dropdown-treeview.component.html',
  styleUrls: ['./dropdown-treeview.component.scss'],
})
export class DropdownTreeviewComponent implements OnInit, OnChanges {
  @Input() buttonClass = '';
  @Input() headerTemplate!: TemplateRef<TreeviewHeaderTemplateContext>;
  @Input() itemTemplate!: TemplateRef<TreeviewItemTemplateContext>;
  @Input() items!: TreeviewItem[];
  @Input() config: TreeviewConfig;
  @Input() label = 'options';
  @Output() selectedChange = new EventEmitter<any[]>(true);
  @Output() filterChange = new EventEmitter<string>();
  @ViewChild(TreeViewComponent, { static: false })
  treeviewComponent!: TreeViewComponent;
  buttonLabel = 'Select';
  showDropdown = false;

  @Input() dropdownWidth: any = '240px';
  @Input() dropdownHeight: any = '500px';
  @ViewChild('dropContainer', { read: ViewContainerRef })
  dropContainerRef!: ViewContainerRef;

  apiUrl!: any;

  dataSource!: any;
  _previousSelection: any = null;

  treeData!: any;
  constructor(
    public i18n: TreeviewI18n,
    private defaultConfig: TreeviewConfig,
    public templateService: TemplateService,
    public dropDownTreeviewService: DropdownTreeviewService,
    private store: Store
  ) {
    this.config = this.defaultConfig;
  }

  ngOnInit(): void {
    this.buttonLabel = `${this.buttonLabel} ${this.label}`;
    this.apiUrl = this.config?.configuration?.apiUrl;
    // console.log("This.apiURl", this.apiUrl)

    if (this.apiUrl) {
      const dropdownApiUrl = this.templateService.processTemplate(
        this.config?.configuration?.apiUrl
      );

      this.dropDownTreeviewService
        .getData(dropdownApiUrl)
        .subscribe((data: any) => {
          this.treeData = data?.data;
          this.templateService.setData({
            [this.config?.configuration?.dataKey || 'organizationTree']:
              JSON.stringify(this.treeData),
          });
          this.dataSource = new TreeviewItem(data?.data[0]);
          this.items = [this.dataSource];
        });
    }
  }

  ngOnChanges(changes: SimpleChanges): void {
    // console.log(changes);
    // if(changes['showDropdown']){
    //   this.showDropdown = changes['showDropdown'].currentValue;
    // }
  }

  dropdownClick() {
    // alert('asd');
  }

  onSelectedChange(checked: any): void {
    const selectedFilterValue = checked.values;
    const orgFilterUUIDs = selectedFilterValue?.map((data: any) => data);

    if (orgFilterUUIDs.length === 0) {
      return;
    }

    this.buttonLabel = this.i18n.getText(
      this.treeviewComponent.selection,
      this.label
    );

    this.templateService.setData({
      orgFilterUUIDs: JSON.stringify(selectedFilterValue),
    });
    this.templateService.setData({
      selectedOrgFilterUUIDs: selectedFilterValue,
    });

    this.templateService.setData({
      selectedOrgFilterObj: checked.objects,
    });

    // console.log(this.templateService.getData());

    this.store.dispatch({
      type: '[Events] Process events',
      events: this.config?.configuration?.events,
    });
    // this.hierarchySelectedValue = selectedFilterValue

    this.selectedChange.emit(selectedFilterValue);

    this._previousSelection = selectedFilterValue;
  }

  onFilterChange(text: string): void {
    this.filterChange.emit(text);
  }
}
