/* eslint-disable @angular-eslint/no-empty-lifecycle-method */
import {
  Component,
  EventEmitter,
  Input,
  OnInit,
  Output,
  TemplateRef,
} from '@angular/core';
import { isNil } from 'lodash';
import { TreeviewConfig } from '../../models/tree-view-config';
import { TreeviewItem } from '../../models/tree-view-item';
import { TreeviewItemTemplateContext } from '../../models/tree-view-item-template-context';

@Component({
  selector: 'hash8-platform-tree-view-list',
  templateUrl: './tree-view-list.component.html',
  styleUrls: ['./tree-view-list.component.scss'],
})
export class TreeViewListComponent implements OnInit {
  @Input() config: TreeviewConfig;
  @Input() template!: TemplateRef<TreeviewItemTemplateContext>;
  @Input() item!: TreeviewItem;
  @Output() checkedChange = new EventEmitter<boolean>();

  constructor(private defaultConfig: TreeviewConfig) {
    this.config = this.defaultConfig;
  }

  ngOnInit(): void {}

  onCollapseExpand = () => {
    this.item.collapsed = !this.item.collapsed;
  };

  onCheckedChange = () => {
    const checked = this.item.checked;
    if (!isNil(this.item.children) && !this.config.decoupleChildFromParent) {
      this.item.children.forEach((child) => child.setCheckedRecursive(checked));
    }
    this.checkedChange.emit(checked);
  };

  onChildCheckedChange(child: TreeviewItem, checked: boolean): void {
    if (!this.config.decoupleChildFromParent) {
      let itemChecked: any = null;
      for (const childItem of this.item.children) {
        if (itemChecked === null) {
          itemChecked = childItem.checked;
        } else if (itemChecked !== childItem.checked) {
          itemChecked = undefined;
          break;
        }
      }

      if (itemChecked === null) {
        itemChecked = false;
      }

      if (this.item.checked !== itemChecked) {
        this.item.checked = itemChecked;
      }
    }
    this.checkedChange.emit(checked);
  }
}
