import { Injectable } from '@angular/core';
import { TreeviewSelection } from './tree-view-item';

@Injectable()
export abstract class TreeviewI18n {
  abstract getText(selection: TreeviewSelection, label?: string): string;
  abstract getAllCheckboxText(): string;
  abstract getFilterPlaceholder(): string;
  abstract getFilterNoItemsFoundText(): string;
  abstract getTooltipCollapseExpandText(isCollapse: boolean): string;
}

@Injectable()
export class DefaultTreeviewI18n extends TreeviewI18n {
  label!: string;
  getText(selection: TreeviewSelection, label: string): string {
    this.label = label;
    if (selection.uncheckedItems.length === 0) {
      if (selection.checkedItems.length > 0) {
        return `${this.getAllCheckboxText()} ${label}`;
      } else {
        return '';
      }
    }

    switch (selection.checkedItems.length) {
      case 0:
        return `Select ${label}`;
      case 1:
        return selection.checkedItems[0].text;
      default:
        return `${selection.checkedItems.length} ${label} selected`;
    }
  }

  getAllCheckboxText(): string {
    return 'All';
  }

  getFilterPlaceholder(): string {
    return 'Filter';
  }

  getFilterNoItemsFoundText(): string {
    return `No ${this.label || 'options'} found`;
  }

  getTooltipCollapseExpandText(isCollapse: boolean): string {
    return isCollapse ? 'Expand' : 'Collapse';
  }
}
