import { ModuleWithProviders, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { TreeViewComponent } from './components/tree-view/tree-view.component';
import { TreeViewListComponent } from './components/tree-view-list/tree-view-list.component';
import { SharedModule } from '@hash8io/core-web';
import { TreeviewConfig } from './models/tree-view-config';
import { DefaultTreeviewI18n, TreeviewI18n } from './models/tree-view-i18n';
import {
  DefaultTreeviewEventParser,
  TreeviewEventParser,
} from './helpers/treeview-event-parser';
import { TreeviewPipe } from './helpers/treeview.pipe';
import { DropdownTreeviewComponent } from './components/dropdown-treeview/dropdown-treeview.component';
import { OverlayModule } from '@angular/cdk/overlay';
import { PortalModule } from '@angular/cdk/portal';
import { DropdownDirective } from './directives/dropdown.directive';
import { DropdownMenuDirective } from './directives/dropdown-menu.directive';
import { DropdownToggleDirective } from './directives/dropdown-toggle.directive';

@NgModule({
  imports: [CommonModule, SharedModule, OverlayModule, PortalModule],
  exports: [
    TreeViewComponent,
    TreeViewListComponent,
    DropdownTreeviewComponent,
    TreeviewPipe,
  ],
  declarations: [
    TreeViewComponent,
    TreeViewListComponent,
    DropdownTreeviewComponent,
    TreeviewPipe,
    DropdownDirective,
    DropdownMenuDirective,
    DropdownToggleDirective,
    DropdownTreeviewComponent,
  ],
})
export class TreeViewModule {
  static forRoot(): ModuleWithProviders<TreeViewModule> {
    return {
      ngModule: TreeViewModule,
      providers: [
        TreeviewConfig,
        { provide: TreeviewI18n, useClass: DefaultTreeviewI18n },
        { provide: TreeviewEventParser, useClass: DefaultTreeviewEventParser },
      ],
    };
  }
}
